Package: funMoDisco
Type: Package
Title: Motif Discovery in Functional Data
Version: 1.1.5
Date: 2026-04-21
Authors@R: c(
    person(given = c("Marzia", "Angela"), family = "Cremona", role = "aut"),
    person(given = "Francesca", family = "Chiaromonte", role = "aut"),
    person(given = "Jacopo", family = "Di Iorio", role = c("aut", "cre"), 
           email = "jacopo.di.iorio@emory.edu"),
    person(given = "Niccolò", family = "Feresini", role = "aut"),
    person(given = "Riccardo", family = "Lazzarini", role = "aut"))
Author: Marzia Angela Cremona [aut],
  Francesca Chiaromonte [aut],
  Jacopo Di Iorio [aut, cre],
  Niccolò Feresini [aut],
  Riccardo Lazzarini [aut]
Maintainer: Jacopo Di Iorio <jacopo.di.iorio@emory.edu>
Description: Efficiently implementing two complementary methodologies for discovering motifs in functional data: ProbKMA and FunBIalign. 
    Cremona and Chiaromonte (2023) "Probabilistic K-means with Local Alignment for Clustering and Motif Discovery in Functional Data" <doi:10.1080/10618600.2022.2156522> is a probabilistic K-means algorithm that leverages local alignment and fuzzy clustering to identify recurring patterns (candidate functional motifs) across and within curves, allowing different portions of the same curve to belong to different clusters. It includes a family of distances and a normalization to discover various motif types and learns motif lengths in a data-driven manner. It can also be used for local clustering of misaligned data.
    Di Iorio, Cremona, and Chiaromonte (2023) "funBIalign: A Hierarchical Algorithm for Functional Motif Discovery Based on Mean Squared Residue Scores" <doi:10.48550/arXiv.2306.04254> applies hierarchical agglomerative clustering with a functional generalization of the Mean Squared Residue Score to identify motifs of a specified length in curves. This deterministic method includes a small set of user-tunable parameters.
    Both algorithms are suitable for single curves or sets of curves. The package also includes a flexible function to simulate functional data with embedded motifs, allowing users to generate benchmark datasets for validating and comparing motif discovery methods.
License: GPL (>= 2)
Suggests: knitr, rmarkdown, tinytest, kableExtra, R.rsp
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: Rcpp (>= 1.0.12),dendextend,fastcluster,
        fda,ggtext,methods,purrr,scales,
        utils,class,combinat,data.table,stringr
Depends: R (>= 3.5.0),dplyr,parallel,ggplot2,progress
LinkingTo: Rcpp, RcppArmadillo
SystemRequirements: C++20
LazyData: true
VignetteBuilder: R.rsp
BuildVignettes: true
Language: en-US
NeedsCompilation: yes
Packaged: 2026-04-22 00:33:44 UTC; jqd5830
Repository: CRAN
Date/Publication: 2026-04-22 02:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-22 04:58:35 UTC; unix
Archs: funMoDisco.so.dSYM

Package: funMoDisco
Type: Package
Title: Motif Discovery in Functional Data
Version: 1.1.5
Date: 2026-04-21
Authors@R: c(
    person(given = c("Marzia", "Angela"), family = "Cremona", role = "aut"),
    person(given = "Francesca", family = "Chiaromonte", role = "aut"),
    person(given = "Jacopo", family = "Di Iorio", role = c("aut", "cre"), 
           email = "jacopo.di.iorio@emory.edu"),
    person(given = "Niccolò", family = "Feresini", role = "aut"),
    person(given = "Riccardo", family = "Lazzarini", role = "aut"))
Author: Marzia Angela Cremona [aut],
  Francesca Chiaromonte [aut],
  Jacopo Di Iorio [aut, cre],
  Niccolò Feresini [aut],
  Riccardo Lazzarini [aut]
Maintainer: Jacopo Di Iorio <jacopo.di.iorio@emory.edu>
Description: Efficiently implementing two complementary methodologies for discovering motifs in functional data: ProbKMA and FunBIalign. 
    Cremona and Chiaromonte (2023) "Probabilistic K-means with Local Alignment for Clustering and Motif Discovery in Functional Data" <doi:10.1080/10618600.2022.2156522> is a probabilistic K-means algorithm that leverages local alignment and fuzzy clustering to identify recurring patterns (candidate functional motifs) across and within curves, allowing different portions of the same curve to belong to different clusters. It includes a family of distances and a normalization to discover various motif types and learns motif lengths in a data-driven manner. It can also be used for local clustering of misaligned data.
    Di Iorio, Cremona, and Chiaromonte (2023) "funBIalign: A Hierarchical Algorithm for Functional Motif Discovery Based on Mean Squared Residue Scores" <doi:10.48550/arXiv.2306.04254> applies hierarchical agglomerative clustering with a functional generalization of the Mean Squared Residue Score to identify motifs of a specified length in curves. This deterministic method includes a small set of user-tunable parameters.
    Both algorithms are suitable for single curves or sets of curves. The package also includes a flexible function to simulate functional data with embedded motifs, allowing users to generate benchmark datasets for validating and comparing motif discovery methods.
License: GPL (>= 2)
Suggests: knitr, rmarkdown, tinytest, kableExtra, R.rsp
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: Rcpp (>= 1.0.12),dendextend,fastcluster,
        fda,ggtext,methods,purrr,scales,
        utils,class,combinat,data.table,stringr
Depends: R (>= 3.5.0),dplyr,parallel,ggplot2,progress
LinkingTo: Rcpp, RcppArmadillo
SystemRequirements: C++20
LazyData: true
VignetteBuilder: R.rsp
BuildVignettes: true
Language: en-US
NeedsCompilation: yes
Packaged: 2026-04-22 00:33:44 UTC; jqd5830
Repository: CRAN
Date/Publication: 2026-04-22 02:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:00:59 UTC; unix
Archs: funMoDisco.so.dSYM

# Generated by roxygen2: do not edit by hand

export(.check_fits)
export(.diss_d0_d1_L2)
export(.domain)
export(.find_min_diss)
export(.find_occurrences)
export(.generate_coefficients)
export(.mapply_custom)
export(.resample)
export(.select_domain)
export(.transform_list)
export(.transform_to_matrix)
export(ProbKMA)
export(add_error_to_motif)
export(add_motif)
export(cluster_candidate_motifs)
export(cluster_candidate_motifs_plot)
export(compare_nodes)
export(discoverMotifs)
export(filter_candidate_motifs)
export(find_recommended_path)
export(generate_background_curve)
export(generate_curve_vector)
export(get_accolites)
export(get_minidend)
export(get_parents)
export(get_path_complete)
export(initialChecks)
export(motifSimulationBuilder)
export(motifs_search)
export(motifs_search_plot)
export(padding)
export(plot_motifs)
export(probKMA_plot)
export(probKMA_silhouette_filter)
export(probKMA_silhouette_plot)
export(probKMA_wrap)
export(recommend_node)
export(to_motifDiscovery)
exportClasses(motifSimulation)
exportMethods(generateCurves)
exportMethods(plot_motifs)
exportMethods(to_motifDiscovery)
import(Rcpp)
import(dendextend)
import(dplyr)
import(ggplot2)
import(grDevices)
import(graphics)
import(methods)
import(parallel)
importFrom(Rcpp,evalCpp)
importFrom(class,knn)
importFrom(combinat,combn)
importFrom(data.table,as.data.table)
importFrom(data.table,setnames)
importFrom(dplyr,"%>%")
importFrom(fastcluster,hclust)
importFrom(fda,create.bspline.basis)
importFrom(fda,eval.fd)
importFrom(fda,fd)
importFrom(fda,smooth.basis)
importFrom(ggtext,element_markdown)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(progress,progress_bar)
importFrom(purrr,is_empty)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,density)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,var)
importFrom(utils,modifyList)
importFrom(utils,tail)
useDynLib(funMoDisco, .registration = TRUE)

#' @title Filter Motifs from probKMA Results Based on Silhouette and Size Thresholds
#'
#' @description This function filters the motifs identified by the `probKMA` algorithm based on a threshold for the average silhouette index and a minimum size criterion. Motifs that meet or exceed both the silhouette index threshold and the minimum number of curves (size) are retained. The function returns a cleaned version of the input data, including the filtered motifs, their derivatives, and associated matrices.
#'
#' @param probKMA_results A list representing the output of the `probKMA` function with `return_options = TRUE`. This output includes the motifs, dissimilarity matrices, and membership matrices required for filtering.
#' @param silhouette A list output from the `probKMA_silhouette` function, which provides silhouette scores for each motif.
#' @param sil_threshold A numeric value representing the threshold for the average silhouette index. Motifs with a silhouette index greater than or equal to this value will be retained. Default is 0.5.
#' @param size_threshold An integer representing the minimum number of curves (size) in a motif cluster. Motifs with a cluster size greater than or equal to this value will be retained. Default is 2.
#' 
#' @return A list containing the filtered results:
#' \item{V0_clean}{Filtered motifs.}
#' \item{V1_clean}{Derivatives of the filtered motifs.}
#' \item{D}{Dissimilarity matrix.}
#' \item{D_clean}{Filtered dissimilarity matrix after cleaning motifs.}
#' \item{P}{Membership matrix.}
#' \item{P_clean}{Filtered membership matrix after cleaning motifs.}
#' \item{c}{Filtered minimum motif lengths.}
#' \item{K}{Vector containing the number of motifs repeated by the filtered motifs.}
#'
#' @export
probKMA_silhouette_filter <- function(probKMA_results,silhouette,sil_threshold=0.5,size_threshold=2){
  index_sil=which(silhouette$silhouette_average>=sil_threshold)
  index_size=which(colSums(probKMA_results$P_clean)>=size_threshold)
  index=intersect(index_sil,index_size)
  if(length(index)==0)
    return(NULL)
  return(list(V0_clean=probKMA_results$V0_clean[index],V1_clean=probKMA_results$V1_clean[index],
              D=probKMA_results$D[,index],D_clean=probKMA_results$D_clean[,index],P=probKMA_results$P[,index],P_clean=probKMA_results$P_clean[,index],
              c=probKMA_results$c[index],K=rep(probKMA_results$K,length(index))))
}

#' @title Get Complete Paths from a Dendrogram
#'
#' @description
#' This function computes recommended paths from a dendrogram structure using parallel processing. 
#' It utilizes the `find_recommended_path` function to identify optimal paths based on a minimum 
#' cardinality constraint, distributing the computation across multiple worker nodes.
#'
#' @param minidend A dendrogram structure from which to derive paths. This should be created from 
#'                 a hierarchical clustering result.
#' @param window_data A data frame or matrix containing the data associated with the nodes in the dendrogram. 
#'                    This data is used for path recommendations.
#' @param min_card An integer specifying the minimum number of leaves (or nodes) that must be present 
#'                  in a path for it to be considered valid.
#' @param worker_number An integer representing the number of worker nodes to be used for parallel processing.
#'
#' @return A list where each element contains the recommended paths for the corresponding node in the dendrogram. 
#'         Each path includes information about the nodes and their associated scores.
#'
#' @details
#' The function creates a cluster of worker nodes, loads necessary libraries, and exports required 
#' variables and functions to each worker. It then applies the `find_recommended_path` function 
#' in parallel to the leaves of the provided dendrogram, gathering results into a single list. 
#' Finally, the cluster is stopped, and the results are returned.
#'
#' @export
get_path_complete <- function(minidend, window_data, min_card,worker_number){

  # Set up a cluster
  cl <- makeCluster(worker_number)
  
  # Load necessary libraries on each worker
  clusterEvalQ(cl, {
    library(dplyr)
    library(data.table)
    library(Rcpp) # ADDED
  })
  
  # Export necessary variables and functions to the cluster (partition_leaces and get_nodes_xy from dendextend)
  clusterExport(cl, 
                c("find_recommended_path",
                  "window_data",
                  "min_card",
                  "partition_leaves",
                  "get_nodes_xy",
                  "get_parents", # ADDED
                  "recommend_node"), # ADDED
                envir = environment())
  
  # Use parLapply to run in parallel
  all_paths <- parLapply(cl,
                         minidend,
                         find_recommended_path, 
                         window_data = window_data,
                         min_card = min_card)
  
  # Stop the cluster
  stopCluster(cl)
  
  return(all_paths)
}

globalVariables(c("pos", "window_data", "probKMA_results", "silhouette", "y", "depth_order", 
                  "xmin", "xmax", "initial_number", "SNR", "instance", "motif_id", 
                  "sd_noise", "start_break_pos", "Y0_inters_k","createWindow", "createDistance", "deleteV", "fMSR_adj", "ProbKMA", "sd_noise"))

#' @title Find Recommended Path in a Tree Structure
#'
#' @description
#' This function identifies a recommended path in a tree structure based on user-defined minimum criteria. 
#' It filters nodes based on the number of leaves and calculates scores for potential recommendations.
#'
#' @param minidend A tree structure representing the hierarchy of nodes, 
#'                 from which leaves will be derived for analysis.
#' @param window_data A matrix or data frame containing data associated with each node.
#' @param min_card An integer specifying the minimum number of leaves that must be present in a node 
#'                  for it to be considered for recommendation.
#'
#' @return A list containing:
#'         - \code{seed_path_info}: A data frame summarizing the valid seed nodes, their number of leaves, 
#'         heights, and recommended nodes.
#'         - \code{seed_path_list}: A list containing the names of elements for each node in the seed path.
#'         - \code{score_path_list}: A list of scores adjusted for each node in the seed path.
#'         - \code{recommended_node_labels}: A list of labels for the recommended nodes.
#'         - \code{recommended_node_scores}: A numeric vector containing scores for the recommended nodes.
#'
#' @details
#' The function follows these steps:
#' 1. Extracts the leaves and heights of each node in the tree.
#' 2. Filters nodes based on the minimum number of leaves specified by \code{min_card}.
#' 3. Identifies nodes that are parents of the filtered nodes, marking them for deletion.
#' 4. For each seed node, calculates crossing points with other seed nodes.
#' 5. Calculates adjusted scores for nodes in the seed path using the C++ function \code{fMSR_adj}.
#' 6. Recommends nodes based on calculated scores and returns a summary of results.
#'
#' @import dplyr
#' @importFrom data.table setnames
#' @importFrom Rcpp evalCpp
#' @import dendextend
#' @export
#' 
find_recommended_path <- function(minidend, window_data, min_card){
  
  # get leaves for each node
  node_list <- minidend %>% partition_leaves()
  # get number of leaves in each node as a vector
  node_leaves  <- node_list %>%
    lapply(length) %>%
    unlist() %>%
    data.table::as.data.table() %>%
    data.table::setnames('node_leaves')
  
  # get branches heights
  node_heights <- minidend %>%
    get_nodes_xy() %>%
    data.table::as.data.table() %>%
    data.table::setnames(c('x','y'))
  
  node_heights <- node_heights[,'y']
  
  # bind together on a dataframe with deep-first search order
  temp <- cbind(node_leaves, node_heights) %>%
    mutate(depth_order = 1:length(node_leaves)) # deep-first search order
  
  # generate for each nodes (using function get_parents)
  node_parents <- lapply(node_list, get_parents, node_list = node_list )
  
  # minimum cardinality (minimum number of node leaves - selected by the user)
  min_card <- min_card
  colnames(temp) <- c('node_leaves','y','depth_order')
  # add column interesting according to the number of leaves and the min_card
  temp <- temp %>%
    filter(node_leaves >= min_card) %>%
    dplyr::arrange(node_leaves, y)
  
  # no node with at least min_card branches -> EXIT (return list with NULL)
  if(dim(temp)[1] == 0){
    return(NULL)
  }
  
  # find the nodes that need to be deleted because parents of minimal ones
  delete_nodes <- c()
  for(i in 1:dim(temp)[1]){
    node_index   <- temp$depth_order[i] # focus node
    delete_now   <- node_parents[[node_index]] # all parents nodes of the focus
    delete_nodes <- c(delete_nodes,
                      delete_now[delete_now != node_index])
  }
  
  # seed nodes
  temp <- temp %>%
    filter(!(depth_order %in% unique(delete_nodes)))
  
  # crossing points between seed nodes
  seed_parents <- node_parents[temp$depth_order]
  all_parents  <- seed_parents %>% unlist() %>% unique()
  
  seed_path <- list()
  for(i in 1:length(seed_parents)){
    my_parent      <- seed_parents[[i]]
    the_rest       <- seed_parents[-i] %>% unlist() %>% unique() %>% sort()
    if(!(is.null(the_rest))){
      cross_point    <- intersect(my_parent, the_rest) %>% max()
      seed_path[[i]] <- my_parent[my_parent > cross_point]
    }else{
      seed_path[[i]] <- my_parent
    }
  }
  
  # add column with parents till the new crossing
  temp$parents <- lapply(seed_path, toString) %>% unlist()
  
  # get the list of elements for each node in the seed_path
  seed_path_list <- lapply(seed_path, function(x){node_list[x]}) # names of elements
  
  
  cppFunction('double fMSR_adj(NumericMatrix& mat) {
  
  int nrow = mat.nrow();
  int ncol = mat.ncol();
  const arma::mat matRef(mat.begin(), nrow, ncol,false,true);
  if(!nrow) // degenerate case with zero curves
    return 0.0;
  
  double score = arma::accu(arma::square(
                 (matRef.each_col() - arma::mean(matRef, 1)).each_row() - arma::mean(matRef,0)
                 + arma::mean(arma::vectorise(matRef))))/static_cast<double>(nrow * ncol);
  if (nrow > 2) 
  {
    score /= arma::as_scalar(arma::prod(arma::regspace<arma::vec>(2, 1, nrow).transform(
      [](double k){return k * k / (k * k - 1);})));
  }
  return score;
}',depends="RcppArmadillo",includes="#include <ranges>",plugins="cpp20")
  
  # get the list of h-score adjusted for each node in the seed_path
  score_path_list <- lapply(seed_path, function(x){
    lapply(x, function(x){window_data[node_list[[x]],] %>% fMSR_adj()})}
  )
  
  recommendation <- lapply(score_path_list, recommend_node) %>% unlist()
  temp$recommended <- recommendation
  
  # get information about recommended nodes (labels and scores)
  recommended_node_labels <- list()
  recommended_node_scores  <- c()
  for(i in 1:length(seed_path_list)){
    recommended_index <- recommendation[i]
    recommended_node_labels[[i]] <- seed_path_list[[i]][[recommended_index]]
    recommended_node_scores[i]   <- score_path_list[[i]][[recommended_index]]
  }
  
  
  res <- list("seed_path_info"  = temp,
              "seed_path_list"  = seed_path_list,
              "score_path_list" = score_path_list,
              "recommended_node_labels" = recommended_node_labels,
              "recommended_node_scores" = recommended_node_scores
  )
  return(res)
}

#' @title Generate Functional Curves with Embedded Motifs
#'
#' @description
#' The `generateCurves` function is designed to create synthetic functional data by embedding predefined motifs into background curves. This is particularly useful for testing and benchmarking motif discovery and clustering algorithms in functional data analysis. By allowing the incorporation of various noise types and controlled motif placements, the function provides a flexible framework for simulating realistic scenarios where motifs may or may not be noisy.
#'
#' The function supports two types of noise addition:
#' \itemize{
#'   \item \strong{Pointwise Noise}: Adds noise directly to the data points of the curves, simulating random fluctuations or measurement errors.
#'   \item \strong{Coefficient Noise}: Perturbs the coefficients of the basis functions used to represent the curves, allowing for smoother variations and controlled distortions.
#' }
#'
#' Additionally, `generateCurves` allows for the specification of vertical shifts to motifs, enabling the simulation of motifs appearing at different baseline levels within the curves. The function ensures that all generated subcurves meet the minimum motif length requirement, maintaining the integrity of the embedded motifs.
#'
#' This function is integral to the `funMoDisco` package's motif simulation capabilities, providing users with the ability to create complex functional datasets tailored to their specific research or testing needs.
#'
#' @param object An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object encapsulates all necessary parameters and configurations for curve and motif generation (mandatory).
#' @param noise_type A character string specifying the type of noise to add to the curves. Acceptable values are `'pointwise'` for adding noise directly to data points or `'coeff'` for perturbing the coefficients of the basis functions (mandatory).
#' @param noise_str A list detailing the structure and magnitude of the noise to be added for each motif. 
#'   \itemize{
#'     \item If `noise_type` is `'pointwise'`, `noise_str` should contain vectors or matrices indicating the noise level for each motif.
#'     \item If `noise_type` is `'coeff'`, `noise_str` should include individual values or vectors representing the noise to be applied to the coefficients.
#'   }
#'   This parameter allows fine-grained control over the noise characteristics applied to each motif (mandatory).
#' @param seed_background An integer value setting the seed for the random number generator used in background curve generation. This ensures reproducibility of the background curves. Default is `777`.
#' @param only_der A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `FALSE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `TRUE`.
#' @param coeff_min_shift A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.
#' @param coeff_max_shift A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.
#'
#' @return 
#' A list containing the following components:
#' \itemize{
#'   \item \strong{basis}: The basis functions used to represent the curves.
#'   \item \strong{background}: A list containing the coefficients and the background curves without any motifs.
#'   \item \strong{no_noise}: A list containing the coefficients and the background curves with embedded motifs but without added noise.
#'   \item \strong{with_noise}: A list containing the noise structure and the curves with embedded motifs and added noise.
#'   \item \strong{SNR}: A list of Signal-to-Noise Ratio (SNR) metrics calculated for each motif within each curve, useful for assessing the quality of motif embedding.
#' }
#'
#' @examples
#' \donttest{
#' # Example 0: Special case with no motifs
#' mot_len <- 100
#' mot_details <- NULL  # or list()
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
#' curves <- funMoDisco::generateCurves(builder)
#'
#' # Example 1: Set the motif position and add pointwise noise
#' # Define motif positions and their respective curves
#' motif_str <- rbind.data.frame(
#'   c(1, 1, 20),
#'   c(2, 1, 2),
#'   c(2, 7, 1),
#'   c(2,17,1)
#' )
#' names(motif_str) <- c("motif_id", "curve", "start_break_pos")
#'
#' # Define motif details
#' mot1 <- list(
#'   "len" = mot_len, 
#'   "coeffs" = NULL, 
#'   "occurrences" = motif_str %>% filter(motif_id == 1)
#' )
#' mot2 <- list(
#'   "len" = mot_len, 
#'   "coeffs" = NULL, 
#'   "occurrences" = motif_str %>% filter(motif_id == 2)
#' )
#' mot_details <- list(mot1, mot2)
#'
#' # Define noise structure for pointwise noise
#' noise_str <- list(
#'   rbind(rep(2, 100), rep(c(rep(0.1, 50), rep(2, 50)), 1)),
#'   rbind(rep(0.0, 100), rep(0.5, 100))
#' )
#'
#' # Build the simulation object
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')
#'
#' # Generate curves with pointwise noise
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str = noise_str)
#'
#' # Example 2: Set the motif position and add coefficient noise
#' # Define noise structure for coefficient noise
#' noise_str <- list(c(0.1, 1.0, 5.0), c(0.0, 0.0, 0.0))
#'
#' # Generate curves with coefficient noise without vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
#'
#' # Example 3: Random motif positions and add pointwise noise
#' mot1 <- list(
#'   "len" = mot_len,
#'   "coeffs" = NULL,
#'   "occurrences" = 5
#' )
#' mot2 <- list(
#'   "len" = mot_len,
#'   "coeffs" = NULL,
#'   "occurrences" = 6
#' )
#' mot_details <- list(mot1, mot2)
#'
#' # Define noise structure for pointwise noise
#' noise_str <- list(
#'   rbind(rep(2, 100)),
#'   rbind(rep(0.5, 100))
#' )
#'
#' # Build the simulation object
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')
#'
#' # Generate curves with pointwise noise and vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str, only_der = FALSE)
#'
#' # Example 4: Random motif positions and add coefficient noise
#' # Define noise structure for coefficient noise
#' noise_str <- list(c(0.1, 5.0, 10.0), c(0.1, 5.0, 10.0))
#'
#' # Generate curves with coefficient noise and vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
#' }
setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL,seed_background = NULL,
                                      only_der = FALSE, coeff_min_shift = -10,coeff_max_shift = 10) {
  standardGeneric("generateCurves")
})


#' @title Generate Functional Curves with Embedded Motifs
#'
#' @description
#' The `generateCurves` function is designed to create synthetic functional data by embedding predefined motifs into background curves. This is particularly useful for testing and benchmarking motif discovery and clustering algorithms in functional data analysis. By allowing the incorporation of various noise types and controlled motif placements, the function provides a flexible framework for simulating realistic scenarios where motifs may or may not be noisy.
#'
#' The function supports two types of noise addition:
#' \itemize{
#'   \item \strong{Pointwise Noise}: Adds noise directly to the data points of the curves, simulating random fluctuations or measurement errors.
#'   \item \strong{Coefficient Noise}: Perturbs the coefficients of the basis functions used to represent the curves, allowing for smoother variations and controlled distortions.
#' }
#'
#' Additionally, `generateCurves` allows for the specification of vertical shifts to motifs, enabling the simulation of motifs appearing at different baseline levels within the curves. The function ensures that all generated subcurves meet the minimum motif length requirement, maintaining the integrity of the embedded motifs.
#'
#' This function is integral to the `funMoDisco` package's motif simulation capabilities, providing users with the ability to create complex functional datasets tailored to their specific research or testing needs.
#'
#' @param object An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object encapsulates all necessary parameters and configurations for curve and motif generation (mandatory).
#' @param noise_type A character string specifying the type of noise to add to the curves. Acceptable values are `'pointwise'` for adding noise directly to data points or `'coeff'` for perturbing the coefficients of the basis functions (mandatory).
#' @param noise_str A list detailing the structure and magnitude of the noise to be added for each motif. 
#'   \itemize{
#'     \item If `noise_type` is `'pointwise'`, `noise_str` should contain vectors or matrices indicating the noise level for each motif.
#'     \item If `noise_type` is `'coeff'`, `noise_str` should include individual values or vectors representing the noise to be applied to the coefficients.
#'   }
#'   This parameter allows fine-grained control over the noise characteristics applied to each motif (mandatory).
#' @param seed_background An integer value setting the seed for the random number generator used in background curve generation. This ensures reproducibility of the background curves. Default is `777`.
#' @param only_der A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `TRUE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `FALSE`.
#' @param coeff_min_shift A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `TRUE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.
#' @param coeff_max_shift A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `TRUE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.
#'
#' @return 
#' A list containing the following components:
#' \itemize{
#'   \item \strong{basis}: The basis functions used to represent the curves.
#'   \item \strong{background}: A list containing the coefficients and the background curves without any motifs.
#'   \item \strong{no_noise}: A list containing the coefficients and the background curves with embedded motifs but without added noise.
#'   \item \strong{with_noise}: A list containing the noise structure and the curves with embedded motifs and added noise.
#'   \item \strong{SNR}: A list of Signal-to-Noise Ratio (SNR) metrics calculated for each motif within each curve, useful for assessing the quality of motif embedding.
#' }
#'
#' @export 
#' 
#' @importFrom purrr is_empty
#' @examples
#' \donttest{
#' # Example 0: Special case with no motifs
#' mot_len <- 100
#' mot_details <- NULL  # or list()
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
#' curves <- funMoDisco::generateCurves(builder)
#'
#' # Example 1: Set the motif position and add pointwise noise
#' # Define motif positions and their respective curves
#' motif_str <- rbind.data.frame(
#'   c(1, 1, 20),
#'   c(2, 1, 2),
#'   c(2, 7, 1),
#'   c(2,17,1)
#' )
#' names(motif_str) <- c("motif_id", "curve", "start_break_pos")
#'
#' # Define motif details
#' mot1 <- list(
#'   "len" = mot_len, 
#'   "coeffs" = NULL, 
#'   "occurrences" = motif_str %>% filter(motif_id == 1)
#' )
#' mot2 <- list(
#'   "len" = mot_len, 
#'   "coeffs" = NULL, 
#'   "occurrences" = motif_str %>% filter(motif_id == 2)
#' )
#' mot_details <- list(mot1, mot2)
#'
#' # Define noise structure for pointwise noise
#' noise_str <- list(
#'   rbind(rep(2, 100), rep(c(rep(0.1, 50), rep(2, 50)), 1)),
#'   rbind(rep(0.0, 100), rep(0.5, 100))
#' )
#'
#' # Build the simulation object
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')
#'
#' # Generate curves with pointwise noise
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str = noise_str)
#'
#' # Example 2: Set the motif position and add coefficient noise
#' # Define noise structure for coefficient noise
#' noise_str <- list(c(0.1, 1.0, 5.0), c(0.0, 0.0, 0.0))
#'
#' # Generate curves with coefficient noise without vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
#'
#' # Example 3: Random motif positions and add pointwise noise
#' mot1 <- list(
#'   "len" = mot_len,
#'   "coeffs" = NULL,
#'   "occurrences" = 5
#' )
#' mot2 <- list(
#'   "len" = mot_len,
#'   "coeffs" = NULL,
#'   "occurrences" = 6
#' )
#' mot_details <- list(mot1, mot2)
#'
#' # Define noise structure for pointwise noise
#' noise_str <- list(
#'   rbind(rep(2, 100)),
#'   rbind(rep(0.5, 100))
#' )
#'
#' # Build the simulation object
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')
#'
#' # Generate curves with pointwise noise and vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str, only_der = FALSE)
#'
#' # Example 4: Random motif positions and add coefficient noise
#' # Define noise structure for coefficient noise
#' noise_str <- list(c(0.1, 5.0, 10.0), c(0.1, 5.0, 10.0))
#'
#' # Generate curves with coefficient noise and vertical shifts
#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
#' }
setMethod("generateCurves", "motifSimulation", function(object,
                                                        noise_type = NULL,
                                                        noise_str = NULL,
                                                        seed_background = NULL,
                                                        only_der = FALSE,
                                                        coeff_min_shift = -10,
                                                        coeff_max_shift = 10){
  
  breaks = lapply(rep(object@len, length.out = object@N), seq, from = 0, by = object@dist_knots) # generate N lists with equally spaced nodes 'dist_knots' from 0 to 'len'
  basis = lapply(breaks, function(breaks_i) create.bspline.basis(norder = object@norder, breaks = breaks_i)) # nbasis = norder + nbreaks - 2 = norder + interior_knots,
  len_motifs <- unlist(lapply(object@mot_details, function(x){x$len}))
  
  # if the seed is user defined (not null) then set the seed
  set.seed(seed_background)
  
  # loop for each curve
  fd_curves <- NULL
  if(is.null(noise_type)){
    noise_type <- "coeff" # to generate background curves without noise
    noise_str  <- list(0)
  }
  if(is.null(noise_type) && !is.null(noise_str)) {
    stop("\'noise_str'\ must be specified because \'noise_type\' is not null")
  }
  if(is.null(noise_str) && !is.null(noise_type)) {
    stop("\'noise_type'\ must be specified because \'noise_str\' is not null")
  }
  if((is.null(noise_type) && !is_empty(object@motifs_in_curves))) {
    stop("\'noise_type\' must be choosen between \'coeff\' and \'pointwise\'")
  } 

  
  if(noise_type == 'coeff') {
    
    tryCatch({
      # Attempt to coerce the input to a numeric vector
      noise_str <- lapply(noise_str, function(error){return(as.numeric(error))})
    }, error = function(e) {
      stop("\'noise_str'\ cannot be vectorized.")
    })
    
    if(length(noise_str) != length(object@mot_details)) {
      stop("\'noise_str\' must have the same length of \'mot_details\'") 
    }
    
    fd_curves <- mapply(function(motifs_in_curves_i,len_i,basis_i,j,len_motifs){
      list_coeff_error <- NULL
      if(is.numeric(object@distribution)) {
        or_coeff = sample(object@distribution,size = len_i,replace = TRUE)
      } else if(object@distribution == 'unif'){
        or_coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # coefficients of the curve = degree of freedom are initialized uniformly
      } else if(object@distribution == 'beta') { 
        or_coeff = object@coeff_min + rbeta(len_i,0.45,0.45)*(object@coeff_max - object@coeff_min) # coefficients of the curve = degree of freedom are initialized uniformly
      } else {
        stop('Wrong \'distrib\'')
      }
      coeff <- or_coeff
      fda_no_error = fda::fd(coef = coeff, basisobj = basis_i)
      or_y_no_error <- generate_curve_vector(fda_no_error)
      fda_with_error <- NULL
      or_y <- NULL
      shifted_coeff <- NULL
      #set.seed(seed_motif)
      if(!is.null(motifs_in_curves_i)) {
        pos_coeff_motifs = unlist(mapply(
          function(a,b) seq(a) + b,
          rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1, # number of motifs coefficients for each selected motif
          motifs_in_curves_i$starting_coeff_pos - 1,SIMPLIFY=FALSE)) # Calculating the position of the coefficients of the motifs within the coefficients of the curves
        shifted_coeff <- rep(runif(length(motifs_in_curves_i$motif_id), min = coeff_min_shift, max = coeff_max_shift),rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1) 
        
        list_coeff_error <- lapply(1:length(noise_str[[1]]),function(z) {
          if(only_der == FALSE) {
            coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
              sd_noise <- noise_str[[id]][z]
              message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
              object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)}))
          } else { 
            coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
              sd_noise <- noise_str[[id]][z]
              message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
              object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)})) + shifted_coeff
          }
          # For each chosen coefficient add a uniform number and a gaussian noise
          return(coeff)
        })
        if(only_der == FALSE) {
          coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
            object@mot_details[[id]]$coeffs}))
        } else {
          coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
            object@mot_details[[id]]$coeffs})) + shifted_coeff
        }
        fda_with_motif <- fda::fd(coef = coeff, basisobj = basis_i)
        or_y_motif <- generate_curve_vector(fda_with_motif)
        
        fda_with_error <- Map(fda::fd,list_coeff_error,MoreArgs = list(basisobj=basis_i)) # Fitting curves using such coefficients and basis
        or_y <- Map(generate_curve_vector,fda_with_error)
        num_motifs <- length(motifs_in_curves_i$motif_id)
        SNR <- vector("list",length(noise_str[[1]]))
        for (k in seq_along(noise_str[[1]])) {
          SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
          SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
          
          # Loop over each motif
          for (n in seq_along(motifs_in_curves_i$motif_id)) {
            start_break <- motifs_in_curves_i$starting_coeff_pos[n]
            start_point <- (start_break - 1) * object@dist_knots
            end_point   <- start_point + len_motifs[motifs_in_curves_i$motif_id[n]]
            
            # Calculate variance for SNR numerator and denominator
            SNR_num <- rbind(SNR_num,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y_motif[start_point:end_point])))
            SNR_den <- rbind(SNR_den,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y[[k]][start_point:end_point] - or_y_motif[start_point:end_point])))
          }
          # Calculate SNR in decibels for the k-th error
          SNR[[k]] <- SNR_num
          SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
        }
        return(list(basis = basis_i,
                    background = list(or_coeff = or_coeff,no_error_y = or_y_no_error),
                    no_noise = list(or_coeff = coeff,motif_y = or_y_motif),
                    with_noise = list(noise_structure = noise_str,noise_y = or_y),
                    SNR = SNR))
      }
      
      return(list(basis = basis_i,
                  background = list(or_coeff = or_coeff,no_error_y = or_y_no_error)))
    },object@motifs_in_curves,object@norder-1+rep(object@len/object@dist_knots,length.out=object@N),basis,1:object@N,MoreArgs = list(len_motifs),SIMPLIFY=FALSE)
    
  }else if(noise_type == 'pointwise'){
    
    if(length(noise_str) != length(object@mot_details)) {
      stop("\'noise_str\' must have the same length of \'mot_details\'") 
    }
    
    fd_curves <- lapply(1:object@N, function(x){
      coeff <- NULL
      len_i <- object@norder - 1 + object@len/object@dist_knots
      if(is.numeric(object@distribution)) {
        coeff = sample(object@distribution, size = len_i, replace = TRUE)
      }
      else if(object@distribution=='unif'){
        coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # If we don't have motifs we sample len_i coefficients uniformely
      }else if(object@distribution=='beta'){
        coeff = object@coeff_min + rbeta(len_i, 0.45, 0.45)*(object@coeff_max - object@coeff_min) 
      }else {
        stop('Wrong \'distrib\'')
      }
      generate_background_curve(object@len, object@dist_knots, object@norder, coeff, add_noise = TRUE)
    })
    
    #set.seed(seed_motif)
    curve_ids <- unique(unlist(lapply(object@mot_details,function(mot){mot$occurrences$curve})))
    for(j in curve_ids){
      message(paste("\033[34m --- Adding motifs to curve", j, "\033[0m"))
      temp_curve <- fd_curves[[j]]
      temp_pattern <- do.call(rbind,lapply(object@mot_details,function(mot_details){(mot_details$occurrences %>% filter(curve == j)) %>% dplyr::select(motif_id, start_break_pos)}))
      temp_len <- do.call(rbind,lapply(unique(apply(temp_pattern,1,function(row){row[1]})),function(k){data.frame("motif_id" = k,"len" = object@mot_details[[k]]$len)}))
      temp_weights <- lapply(as.character(unique(apply(temp_pattern, 1, function(row) { row[1] }))), 
                             function(k) { 
                               object@mot_details[[as.numeric(k)]]$coeffs
                             })
      names(temp_weights) <- as.character(unique(apply(temp_pattern, 1, function(row) { row[1] })))
      fd_curves[[j]] <- add_motif(base_curve  = temp_curve,
                                  mot_pattern = temp_pattern,
                                  mot_len     = temp_len,
                                  dist_knots  = object@dist_knots,
                                  mot_order   = object@norder,
                                  mot_weights = temp_weights,
                                  noise_str   = noise_str,
                                  only_der = only_der,
                                  coeff_min_shift = coeff_min_shift,
                                  coeff_max_shift = coeff_max_shift)
    }
    fd_curves <- .transform_list(fd_curves,noise_str)
  } else {
    stop("\'noise_type\' must be choosen between \'coeff\' and \'pointwise\'")
  }
  return(fd_curves = fd_curves)
})

# # PREVIOUS VERSION
# setMethod("generateCurves", "motifSimulation", function(object, noise_type = NULL, noise_str = NULL, seed_background = NULL, seed_motif = NULL,
#                                                         only_der = FALSE, coeff_min_shift = -10, coeff_max_shift = 10) {
#   breaks = lapply(rep(object@len, length.out = object@N), seq, from = 0, by = object@dist_knots) # generate N lists with equally spaced nodes 'dist_knots' from 0 to 'len'
#   basis = lapply(breaks, function(breaks_i) create.bspline.basis(norder = object@norder, breaks = breaks_i)) # nbasis = norder + nbreaks - 2 = norder + interior_knots,
#   len_motifs <- unlist(lapply(object@mot_details, function(x){x$len}))
# 
#   # if the seed is user defined (not null) then set the seed
#   if(!is.null(seed_background)){
#     set.seed(seed_background)
#   }
# 
#   # loop for each curve
#   fd_curves <- NULL
#   if(is.null(noise_type) && !is.null(noise_str)) {
#     stop("\'noise_str'\ must be specified because \'noise_type\' is not null")
#   }
#   if(is.null(noise_str) && !is.null(noise_type)) {
#     stop("\'noise_type'\ must be specified because \'noise_str\' is not null")
#   }
#   if((is.null(noise_type) && !is_empty(object@motifs_in_curves))) {
#     stop("\'noise_type\' must be choosen between \'coeff\' and \'pointiwse\'")
#   }
#   if(is.null(noise_type)) {
#     noise_type <- "pointwise" # to generate background curves without noise
#   }
#   if(noise_type == 'coeff') {
#     tryCatch({
#       # Attempt to coerce the input to a numeric vector
#       noise_str <- lapply(noise_str, function(error){return(as.numeric(error))})
#     }, error = function(e) {
#       stop("\'noise_str'\ cannot be vectorized.")
#     })
#     if(length(noise_str) != length(object@mot_details)) {
#      stop("\'noise_str\' must have the same length of \'mot_details\'")
#     }
#     fd_curves <- mapply(function(motifs_in_curves_i,len_i,basis_i,j,len_motifs){
#       list_coeff_error <- NULL
#       if(is.numeric(object@distribution)) {
#           or_coeff = sample(object@distribution,size = len_i,replace = TRUE)
#       } else if(object@distribution == 'unif'){
#           or_coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # coefficients of the curve = degree of freedom are initialized uniformly
#       } else if(object@distribution == 'beta') {
#           or_coeff = object@coeff_min + rbeta(len_i,0.45,0.45)*(object@coeff_max - object@coeff_min) # coefficients of the curve = degree of freedom are initialized uniformly
#       } else {
#         stop('Wrong \'distrib\'')
#       }
#       coeff <- or_coeff
#       fda_no_error = fda::fd(coef = coeff, basisobj = basis_i)
#       or_y_no_error <- generate_curve_vector(fda_no_error)
#       fda_with_error <- NULL
#       or_y <- NULL
#       shifted_coeff <- NULL
#       #set.seed(seed_motif)
#       if(!is.null(motifs_in_curves_i)) {
#         pos_coeff_motifs = unlist(mapply(
#           function(a,b) seq(a) + b,
#           rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1, # number of motifs coefficients for each selected motif
#           motifs_in_curves_i$starting_coeff_pos - 1,SIMPLIFY=FALSE)) # Calculating the position of the coefficients of the motifs within the coefficients of the curves
#         shifted_coeff <- rep(runif(length(motifs_in_curves_i$motif_id), min = coeff_min_shift, max = coeff_max_shift),rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1)
# 
#         list_coeff_error <- lapply(1:length(noise_str[[1]]),function(z) {
#           if(only_der == FALSE) {
#           coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
#             sd_noise <- noise_str[[id]][z]
#             message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
#             object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)}))
#           } else {
#             coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
#               sd_noise <- noise_str[[id]][z]
#               message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
#               object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)})) + shifted_coeff
#           }
#           # For each chosen coefficient add a uniform number and a gaussian noise
#           return(coeff)
#         })
#       if(only_der == FALSE) {
#         coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
#                                   object@mot_details[[id]]$coeffs}))
#       } else {
#         coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
#                                     object@mot_details[[id]]$coeffs})) + shifted_coeff
#       }
#       fda_with_motif <- fda::fd(coef = coeff, basisobj = basis_i)
#       or_y_motif <- generate_curve_vector(fda_with_motif)
# 
#       fda_with_error <- Map(fda::fd,list_coeff_error,MoreArgs = list(basisobj=basis_i)) # Fitting curves using such coefficients and basis
#       or_y <- Map(generate_curve_vector,fda_with_error)
#       num_motifs <- length(motifs_in_curves_i$motif_id)
#       SNR <- vector("list",length(noise_str[[1]]))
#       for (k in seq_along(noise_str[[1]])) {
#         SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
#         SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
# 
#         # Loop over each motif
#         for (n in seq_along(motifs_in_curves_i$motif_id)) {
#           start_break <- motifs_in_curves_i$starting_coeff_pos[n]
#           start_point <- (start_break - 1) * object@dist_knots
#           end_point   <- start_point + len_motifs[motifs_in_curves_i$motif_id[n]]
# 
#           # Calculate variance for SNR numerator and denominator
#           SNR_num <- rbind(SNR_num,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y_motif[start_point:end_point])))
#           SNR_den <- rbind(SNR_den,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y[[k]][start_point:end_point] - or_y_motif[start_point:end_point])))
#         }
#         # Calculate SNR in decibels for the k-th error
#         SNR[[k]] <- SNR_num
#         SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
#       }
#       return(list(basis = basis_i,
#                   background = list(or_coeff = or_coeff,no_error_y = or_y_no_error),
#                   no_noise = list(or_coeff = coeff,motif_y = or_y_motif),
#                   with_noise = list(noise_structure = noise_str,noise_y = or_y),
#                   SNR = SNR))
#       }
# 
#       return(list(basis = basis_i,
#                   background = list(or_coeff = or_coeff,no_error_y = or_y_no_error)))
#     },object@motifs_in_curves,object@norder-1+rep(object@len/object@dist_knots,length.out=object@N),basis,1:object@N,MoreArgs = list(len_motifs),SIMPLIFY=FALSE)
# 
#   }else if(noise_type == 'pointwise'){
#     if(length(noise_str) != length(object@mot_details)) {
#       stop("\'noise_str\' must have the same length of \'mot_details\'")
#     }
#     fd_curves <- lapply(1:object@N, function(x){
#       coeff <- NULL
#       len_i <- object@norder - 1 + object@len/object@dist_knots
#       if(is.numeric(object@distribution)) {
#         coeff = sample(object@distribution, size = len_i, replace = TRUE)
#       }
#       else if(object@distribution=='unif'){
#         coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # If we don't have motifs we sample len_i coefficients uniformely
#       }else if(object@distribution=='beta'){
#         coeff = object@coeff_min + rbeta(len_i, 0.45, 0.45)*(object@coeff_max - object@coeff_min)
#       }else {
#         stop('Wrong \'distrib\'')
#       }
#       generate_background_curve(object@len, object@dist_knots, object@norder, coeff, add_noise = TRUE)
#     })
# 
#     #set.seed(seed_motif)
#     curve_ids <- unique(unlist(lapply(object@mot_details,function(mot){mot$occurrences$curve})))
#     for(j in curve_ids){
#       message(paste("\033[34m --- Adding motifs to curve", j, "\033[0m"))
#       temp_curve <- fd_curves[[j]]
#       temp_pattern <- do.call(rbind,lapply(object@mot_details,function(mot_details){(mot_details$occurrences %>% filter(curve == j)) %>% dplyr::select(motif_id, start_break_pos)}))
#       temp_len <- do.call(rbind,lapply(unique(apply(temp_pattern,1,function(row){row[1]})),function(k){data.frame("motif_id" = k,"len" = object@mot_details[[k]]$len)}))
#       temp_weights <- lapply(as.character(unique(apply(temp_pattern, 1, function(row) { row[1] }))),
#                              function(k) {
#                                object@mot_details[[as.numeric(k)]]$coeffs
#                              })
#       names(temp_weights) <- as.character(unique(apply(temp_pattern, 1, function(row) { row[1] })))
#       fd_curves[[j]] <- add_motif(base_curve  = temp_curve,
#                                   mot_pattern = temp_pattern,
#                                   mot_len     = temp_len,
#                                   dist_knots  = object@dist_knots,
#                                   mot_order   = object@norder,
#                                   mot_weights = temp_weights,
#                                   noise_str   = noise_str,
#                                   only_der = only_der,
#                                   coeff_min_shift = coeff_min_shift,
#                                   coeff_max_shift = coeff_max_shift)
#     }
#     fd_curves <- .transform_list(fd_curves,noise_str)
#   } else {
#     stop("\'noise_type\' must be choosen between \'coeff\' and \'pointwise\'")
#   }
#   return(fd_curves = fd_curves)
# })

#' @title Plot Embedded Motifs in Functional Curves
#'
#' @description
#' The `plot_motifs` function visualizes the results generated by the `generateCurves` function. It provides comprehensive plots of functional curves with embedded motifs, allowing users to inspect the placement and characteristics of each motif within the curves. This visualization is crucial for validating the correctness of motif embedding and for gaining insights into the distribution and variability of motifs across different curves.
#'
#' The function supports saving the generated plots to a specified directory, facilitating the creation of reports or the sharing of visual results. By plotting motifs in distinct colors or styles, `plot_motifs` ensures that overlapping motifs and their respective curves remain distinguishable, enhancing the clarity and interpretability of the visualizations.
#'
#' This plotting utility is an essential tool within the `funMoDisco` package, aiding users in the exploratory analysis of simulated functional data and the assessment of motif detection algorithms.
#'
#' @param object An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object contains all necessary parameters and configurations used during curve and motif generation (mandatory).
#' @param name Name of the output file.
#' @param curves The output list from the `generateCurves` function, containing the generated functional curves with embedded motifs. This parameter provides the data to be visualized (mandatory).
#' @param path A character string specifying the directory path where the generated plots will be saved. The function will save the plots in this directory, allowing for organized storage and easy access to visual results (mandatory).
#'
#' @return 
#' The function does not return any value but generates and saves plots of the functional curves with embedded motifs in the specified directory. Each plot visually represents the motifs within the curves, aiding in the qualitative assessment of motif embedding.
#'
#' @export
#' 
#' @importFrom stats setNames
#' @importFrom ggtext element_markdown
#'
#' @examples
#' \donttest{
#' # Example: Plotting motifs in generated curves
#' # Assume 'builder' has been created and 'curves' have been generated using generateCurves
#' 
#' mot_details <- NULL
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details = mot_details)
#' curves <- funMoDisco::generateCurves(builder)
#'
#' # Specify the directory to save plots
#' plots_name <- "plots_1"  
#'
#' # Generate and save the plots
#' funMoDisco::plot_motifs(builder, curves, plots_name, path = tempdir())
#' }
setGeneric("plot_motifs", function(object,curves,name,path) 
  standardGeneric("plot_motifs")
)

#' @title Plot Embedded Motifs in Functional Curves
#'
#' @description
#' The `plot_motifs` function visualizes the results generated by the `generateCurves` function. It provides comprehensive plots of functional curves with embedded motifs, allowing users to inspect the placement and characteristics of each motif within the curves. This visualization is crucial for validating the correctness of motif embedding and for gaining insights into the distribution and variability of motifs across different curves.
#'
#' The function supports saving the generated plots to a specified directory, facilitating the creation of reports or the sharing of visual results. By plotting motifs in distinct colors or styles, `plot_motifs` ensures that overlapping motifs and their respective curves remain distinguishable, enhancing the clarity and interpretability of the visualizations.
#'
#' This plotting utility is an essential tool within the `funMoDisco` package, aiding users in the exploratory analysis of simulated functional data and the assessment of motif detection algorithms.
#'
#' @param object An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object contains all necessary parameters and configurations used during curve and motif generation (mandatory).
#' @param curves The output list from the `generateCurves` function, containing the generated functional curves with embedded motifs. This parameter provides the data to be visualized (mandatory).
#' @param name Name of the output file.
#' @param path A character string specifying the directory path where the generated plots will be saved. The function will save the plots in this directory, allowing for organized storage and easy access to visual results (mandatory).
#'
#' @return 
#' The function does not return any value but generates and saves plots of the functional curves with embedded motifs in the specified directory. Each plot visually represents the motifs within the curves, aiding in the qualitative assessment of motif embedding.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Example: Plotting motifs in generated curves
#' # Assume 'builder' has been created and 'curves' have been generated using generateCurves
#' mot_details <- NULL
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
#' curves <- funMoDisco::generateCurves(builder)
#'
#' # Specify the directory to save plots
#' plots_name <- "plots_1"  
#'
#' # Generate and save the plots
#' funMoDisco::plot_motifs(builder, curves, plots_name, path = tempdir())
#' }
setMethod("plot_motifs","motifSimulation",
          function(object, curves, name, path) {
            
  # Ensure the path is absolute
  if (!grepl("^(/|[A-Za-z]:\\\\)", path)) {  
    path <- file.path(getwd(), path)  # Convert to absolute path
  }
  
  # Create the directory if it does not exist
  if (!dir.exists(path)) {
    dir.create(path, recursive = TRUE)
  }
  
  # Ensure the filename has a .pdf extension
  if (!grepl("\\.pdf$", name)) {     
    name <- paste0(name, ".pdf")   
  }
  
  # Construct the output file path
  output_file <- file.path(path, name)
            
  # Open a PDF device with the correct path
  pdf(file = output_file, width = 8, height = 6) 
  for (k in seq_along(curves)) {
    if(purrr::is_empty(object@motifs_in_curves)) {
      curve_data_no_error <- data.frame(
        t = seq(0, curves[[k]]$basis$rangeval[2]-1),
        x = curves[[k]]$background$no_error_y
      )
      names(curve_data_no_error) <- c("t", "x")
      
      p <- ggplot() +
        # Plot the main curve in gray30
        geom_line(data = curve_data_no_error, aes(x = t, y = x, color = 'background_curve'), linewidth = 0.5) +
        
        scale_color_manual(
          values = c('background_curve' = scales::alpha('gray30', 0.90))
        ) +
        
        # Title
        labs(
          title = paste0('<b><span style="color:#0073C2;"> Curve ', k, '</span></b>'),
          x = "t",
          y = "x"
        ) +
        
        # Clean theme with subtle grid
        theme_minimal(base_size = 15) +
        theme(
          plot.title = element_markdown(size = 20, face = "bold", hjust = 0.5, margin = margin(b = 10)),
          axis.title = element_text(size = 14, margin = margin(t = 10)),
          axis.text = element_text(size = 12),
          legend.position = "none", 
          plot.margin = margin(15, 15, 15, 15),  # Margini intorno al grafico
          panel.grid.major = element_line(color = "gray90"),
          panel.grid.minor = element_blank()
        ) 
      
      print(p)
    }else if(!is.null(curves[[k]]$with_noise)) {
      curve_data_no_error <- data.frame(
        t = seq(0, curves[[k]]$basis$rangeval[2]-1),
        x = curves[[k]]$background$no_error_y,
        z = curves[[k]]$no_noise$motif_y)
      names(curve_data_no_error) <- c("t","x","z")
 
      curve_data_error <- NULL
      curve_data_error <- data.frame(
        t = seq(0, curves[[k]]$basis$rangeval[2]-1),
        x = curves[[k]]$with_noise$noise_y)
      names(curve_data_error) <- c("t",paste0("x",seq(length(curves[[k]]$with_noise$noise_y))))
      
      motif_lines <- mapply(function(id_motif, pos_motif, instance) {
        motif_t = seq((pos_motif - 1) * object@dist_knots,
                      (pos_motif - 1) * object@dist_knots + object@mot_details[[id_motif]]$len)
        motif_x = lapply(curves[[k]]$with_noise$noise_y,function(curve){return(curve[motif_t + 1])})
        
        return(lapply(motif_x,function(motif){ data.frame(t = motif_t, x = motif, motif_id = factor(paste(id_motif, instance, sep = "_")),
                                                          initial_number = stringr::str_extract(as.character(id_motif), "^[^_]+"),
                                                          xmin = (pos_motif - 1) * object@dist_knots,
                                                          xmax = (pos_motif - 1) * object@dist_knots + object@mot_details[[id_motif]]$len)}))
      }, object@motifs_in_curves[[k]]$motif_id, object@motifs_in_curves[[k]]$starting_coeff_pos, seq_along(object@motifs_in_curves[[k]]$motif_id), SIMPLIFY = FALSE)
      
      motif_colors <- c( "1" = "red", "2" = "blue", "3" = "darkgreen", "4" = "orange",
                         "5" = "purple", "6" = "cyan", "7" = "magenta", "8" = "brown",
                         "9" = "pink", "10" = "grey")
      motif_colors <- rep(motif_colors,length.out = length(object@mot_details))
      if( length(object@mot_details) > 10 )
        attr(motif_colors,"names")[11:length(object@mot_details)] <- as.character(as.integer(attr(motif_colors,"names")[11:length(object@mot_details)]) + 10)
  
      max_dataframes <- max(sapply(motif_lines, function(sublist) length(sublist)))
      # Initialize a list to store results
      motif_data <- vector("list", max_dataframes)
      for (i in seq_len(max_dataframes)) {
        # Extract data frames at the i-th level
        dataframes_at_level_i <- lapply(motif_lines, function(sublist) sublist[[i]])
        # Combine the extracted data frames
        motif_data[[i]] <- bind_rows(dataframes_at_level_i)
        names(motif_data[[i]]) <- c("t", "x", "motif_id", "initial_number", "xmin", "xmax")
      }
      p <- lapply(1:length(motif_data), function(j) {
        # Create motif_id labels
        motif_labels <- paste("motif_id:", unique(motif_data[[j]]$initial_number))
        pic <- ggplot() +
          # Plot the main curve in gray30
          geom_line(data = curve_data_no_error, aes(x = t, y = x, color = 'background_curve'), linewidth = 0.5) +
          # Plot the curve with motifs in gold
          geom_line(data = curve_data_no_error, aes(x = t, y = z, color = 'zero_noise_curve'), linewidth = 0.5) +
          # Plot the error curve
          geom_line(data = curve_data_error, aes_string(x = "t", y = paste0("x", j)), color = "black", linewidth = 0.5) +
          # Add shaded rectangles for motif positions
          geom_rect(data = motif_data[[j]], 
                    aes(xmin = xmin, xmax = xmax, ymin = -Inf, ymax = Inf, fill = factor(initial_number)), 
                    alpha = 0.005) +
          # Add SNR text on top of each rectangle
          geom_text(data = curves[[k]]$SNR[[j]], 
                    aes(x = (xmin + xmax) / 2, y = Inf, 
                        label = paste("SNR:", round(SNR, 3))),
                    vjust = 1.5, color = "black", size = 3.5) +
          # Plot motifs with distinct colors
          geom_line(data = motif_data[[j]], aes(x = t, y = x, color = factor(initial_number), group = motif_id), linewidth = 1.0) + 
          # Add color and fill scales with custom labels for motif_id
          scale_color_manual(
            values = c('background_curve' = scales::alpha('gray30', 0.15), 
                       'zero_noise_curve' = 'gold', 
                       motif_colors),
            labels = c('background_curve' = 'background_curve', 
                       'zero_noise_curve' = 'zero_noise_curve', 
                       setNames(motif_labels, unique(motif_data[[j]]$initial_number)))
          ) +
          scale_fill_manual(
            values = motif_colors, 
            labels = setNames(motif_labels, unique(motif_data[[j]]$initial_number))
          ) +
          # Title
          labs(
            title = paste0('<b><span style="color:#0073C2;">Curve ', k, ' - type_error ', 
                           ifelse(j %% length(curves[[k]]$with_noise$noise_y) == 0, 
                                  length(curves[[k]]$with_noise$noise_y), j %% length(curves[[k]]$with_noise$noise_y)), 
                           '</span></b>'), 
            x = "t", 
            y = "x"
          ) +
          # Clean theme with subtle grid
          theme_minimal(base_size = 15) +
          theme(
            plot.title = element_markdown(size = 20, face = "bold", hjust = 0.5, margin = margin(b = 10)),
            axis.title = element_text(size = 14, margin = margin(t = 10)),
            axis.text = element_text(size = 12),
            legend.text = element_text(size = 12),
            legend.position = "right",
            legend.box.margin = margin(10, 10, 10, 10),
            plot.margin = margin(15, 15, 15, 15),
            panel.grid.major = element_line(color = "gray90"),
            panel.grid.minor = element_blank()
          ) +
          guides(
            color = guide_legend(ncol = 1, byrow = TRUE, title = NULL),
            fill = "none",
          )
        
        return(pic)
      })
      
      Map(print, p)
    }
  }
  if(!purrr::is_empty(object@motifs_in_curves)) {
    curves_data_noise <- vector("list",max_dataframes)
    for(error_n in 1:max_dataframes) {
      temp <- list()
      for(motif_id in 1:length(object@mot_details)) {
        for(curve_k in 1:length(object@motifs_in_curves)) {
          motif_instance <- 1
          if(!is.null(object@motifs_in_curves[[curve_k]])) {
            for(z in 1:length(object@motifs_in_curves[[curve_k]]$motif_id)) {
              if(object@motifs_in_curves[[curve_k]]$motif_id[z] == motif_id) {
                # Calcola l'intervallo della curva
                start <- (object@motifs_in_curves[[curve_k]]$starting_coeff_pos[z] - 1) * object@dist_knots + 1
                
                # Crea un identificatore unico per ogni istanza
                instance_id <- paste0("curve_", curve_k, "_", motif_instance)
                motif_instance <- motif_instance + 1
                
                curve_y_noise <- curves[[curve_k]]$with_noise$noise_y[[error_n]][start:(start + object@mot_details[[motif_id]]$len - 1)]
                # Creazione di una sequenza x per il plotting
                x <- seq_along(curve_y_noise)
                
                # Aggiungi i dati alla lista
                temp[[length(temp) + 1]] <- data.frame(x = x, y = curve_y_noise, id = motif_id, instance = instance_id) 
              }
            }
          }
        } 
      }
      curves_data_noise[[error_n]] <- temp
    }
  
    # Combina i dati in un unico dataframe
    curves_df <- lapply(curves_data_noise,bind_rows)
    # Ottieni la lista unica degli ID
    unique_ids <- unique(curves_df[[1]]$id)
    
    # Compute the mean motif for each error
    motif_y_means <- vector("list", length(unique_ids))
    index <- 0
    # Iterate over each unique ID
    for (id in unique_ids) {
      index <- index + 1
      # Iterate through each sublist in builder@mot_details for the current id
      for (i in object@mot_details[[index]]$occurrences$curve %>% unique()) {
        curve <- curves[[i]]
        motif_in_curve_i <- object@motifs_in_curves[[i]]
        for(z in 1:length(motif_in_curve_i$motif_id))
        {
          if(motif_in_curve_i$motif_id[z] == id) {
            start <- (motif_in_curve_i$starting_coeff_pos[z] - 1) * object@dist_knots + 1
            # Compute pointwise means across all curves in the subcurve list
            
            motif_y_means[[index]] <- curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
            
            # MODIFIED BY ME: this code was computing the mean of the noisy occurrences
            # if (is.null(motif_y_means[[index]])) {
            #   motif_y_means[[index]] <- curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
            # } else {
            #   motif_y_means[[index]] <- motif_y_means[[index]] + curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
            # }
          }
        }
      }
      #motif_y_means[[index]] <- data.frame(x = seq_along(motif_y_means[[index]]), y = motif_y_means[[index]] / length(object@mot_details[[index]]$occurrences$curve %>% unique()))
      motif_y_means[[index]] <- data.frame(x = seq_along(motif_y_means[[index]]), y = motif_y_means[[index]] %>% unique())
      names(motif_y_means)[index] <- id
    }
    
    # Loop per plottare ogni ID separatamente su pagine diverse
    for (id in unique_ids) {
      for(error_n in 1:max_dataframes) {
        # Filtra i dati per il singolo ID
        plot_data <- curves_df[[error_n]] %>% filter(id == !!id)
        
        p <- ggplot() + 
          geom_line(data = plot_data, aes(x = x, y = y, color = instance), size = 1, linetype = "longdash") + 
          geom_line(data = motif_y_means[[as.character(id)]], aes(x = x, y = y, color = "black"), size = 2, linetype = "solid") + 
          scale_color_manual(
            values = c("motif_y_means" = "black", setNames(rainbow(length(unique(plot_data$instance))), unique(plot_data$instance))),
            labels = c("motif_y_means" = paste0("motif ", id), unique(plot_data$instance))
          ) + 
          labs(
            title = paste0('<b><span style="color:#0073C2;">Motif ', id,'</span></b>'),
            subtitle = paste0("Mean of the no noise motif <b>", id, "</b> with type error <b>", error_n, "</b>"),
            x = "x",
            y = "t",
            color = "Mot. occurrences"
          ) + 
          theme_minimal(base_size = 15) + 
          theme(
            plot.title = element_markdown(size = 20, face = "bold", hjust = 0.5, margin = margin(b = 10)),
            plot.subtitle = element_markdown(size = 16, hjust = 0.5, margin = margin(b = 10)),  # Styled subtitle
            axis.title = element_text(size = 14, margin = margin(t = 10)),
            axis.text = element_text(size = 12),
            legend.text = element_text(size = 12),
            legend.position = "right",
            legend.box.margin = margin(10, 10, 10, 10),
            plot.margin = margin(15, 15, 15, 15),
            panel.grid.major = element_line(color = "gray90"),
            panel.grid.minor = element_blank()
          ) + 
          guides(
            color = guide_legend(ncol = 1, byrow = TRUE, title = "Mot. occurrences"),
            fill = "none"
          )
        print(p)
      }
    }
  }
  dev.off()
})

#' @title to_motifDiscovery
#' @description Transforms the result of generateCurves into a format suitable for discoverMotifs
#' @param curves A list coming from the generateCurves function.
#' @return A list containing all curves formatted to be suitable for input into the discoverMotifs function.
#' @export
#' @examples
#' \donttest{
#' N <- 20 # number of curves
#' len <- 300 # length of the curves
#' mot_details <- NULL # or list()
#'
#' builder <- motifSimulationBuilder(N = N, len = len, mot_details = NULL)
#' curves <- funMoDisco::generateCurves(builder)
#' formatted_curves <- to_motifDiscovery(curves)
#' }
setGeneric("to_motifDiscovery", function(curves) 
  standardGeneric("to_motifDiscovery")
)

#' @title to_motifDiscovery
#' @description Transforms the result of generateCurves into a format suitable for discoverMotifs
#' @param curves A list coming from the generateCurves function.
#' @return A list containing all curves formatted to be suitable for input into the discoverMotifs function.
#' @export
#' @examples
#' \donttest{
#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details = NULL)
#' curves <- funMoDisco::generateCurves(builder)
#' formatted_curves <- to_motifDiscovery(curves)
#' }
setMethod("to_motifDiscovery",c(curves = "list"), function(curves) {
  
  if(purrr::is_empty(curves)) {
    warning("\'curves' object is empty. Return \'NULL\'")
    return(NULL)
  }
  
  n <- 1
  for(i in 1:length(curves)) {
    if("with_noise" %in% names(curves[[i]])) {
      n <- length(curves[[i]]$with_noise$noise_y)
      break
    }
  }
 
  result <- vector("list", n)  
  for(i in 1:n) {
    result[[i]] <- setNames(lapply(curves, function(curve) {  
      if("with_noise" %in% names(curve)) {
        return(curve$with_noise$noise_y[[i]])
      } else {
        return(curve$background$no_error_y)
      }
    }), paste0("c", seq_along(curves)))  
  }
  
  result <- setNames(result, paste0("Noise_type_", seq_len(n)))
  
  return(result)
})

setClassUnion("charOrNum", c("character", "numeric"))

#' @title motifSimulationS4Class
#' 
#' @description 
#' The `motifSimulation` class is an S4 class designed for simulating functional data with embedded motifs. This class is essential for modeling various aspects of the data, such as the number of curves, motif details, curve characteristics, and knot-based spline definitions. It allows users to generate realistic synthetic functional data to test and benchmark motif discovery algorithms like ProbKMA and funBIalign.
#' 
#' @slot N A numeric value representing the number of curves to be simulated. 
#' This parameter controls the number of functional curves generated in the simulation.
#' 
#' @slot mot_details A list containing details of the motifs to be embedded within the curves. 
#' Each motif detail specifies attributes such as motif shape, location, and frequency of occurrence across the simulated curves.
#' 
#' @slot motifs_in_curves A list specifying which curves contain motifs and the positions where they are embedded. 
#' This slot allows precise control over the placement of motifs in the generated curves, enabling flexible motif-to-curve assignments.
#' 
#' @slot distribution A character string or numeric value representing the distribution of the weights for the motifs. 
#' This slot defines the distribution used to generate the weight coefficients for the motifs, influencing their amplitude in the simulated data. 
#' Accepted values include any distribution supported in R or a custom vector provided by the user.
#' 
#' @slot dist_knots A numeric value indicating the distance between knots in the spline representation of the curves. 
#' This parameter is crucial for defining the smoothness of the generated curves and the precision of the spline-based motif embedding process.
#' 
#' @slot len A numeric value representing the length of the generated curves. 
#' This parameter determines the number of time points or the granularity of the data, allowing for high-resolution simulations.
#' 
#' @slot norder A numeric value specifying the order of the B-spline used in the functional data representation. 
#' Higher-order splines provide smoother representations of the curves, while lower-order splines offer more flexible curve shapes.
#' 
#' @slot coeff_min A numeric value specifying the minimum coefficient value for the spline-based curve generation. 
#' This controls the lower bound of the weight coefficients applied to the basis functions, influencing the range of curve amplitudes.
#' 
#' @slot coeff_max A numeric value specifying the maximum coefficient value for the spline-based curve generation. 
#' This controls the upper bound of the weight coefficients applied to the basis functions, setting the maximum amplitude for the curves.
#' 
#' @slot min_dist_motifs A numeric value indicating the minimum distance between motifs in the simulated curves. 
#' This ensures that motifs are not placed too closely, preserving their distinctiveness and reducing overlap during the simulation.
#' 
#' @export 
setClass("motifSimulation",
         slots = list(
           N = "numeric",            # Number of curves
           mot_details = "list",     # Details of motifs 
           motifs_in_curves = "list",# Details of curves
           distribution = "charOrNum",  # distribution of the weights
           dist_knots = "numeric",   # Distance between knots 
           len = "numeric",          # Length of the curve or curves
           norder = "numeric",       # Order of spline 
           coeff_min = "numeric",    # Minimum coefficient value
           coeff_max = "numeric",    # Maximum coefficient value
           min_dist_motifs = "numeric" # Minimum distance between motifs
         ))

#' @title Filter Candidate Motifs
#' 
#' @description
#' Filters the candidate motifs based on a specified threshold for the average silhouette index and 
#' a threshold for the size of the curves in the motif. This function is useful for refining the set 
#' of candidate motifs by removing those that do not meet the defined criteria.
#' 
#' @param find_candidate_motifs_results Output from the \code{find_candidate_motif} function, which contains the results of the motif discovery process.
#' @param sil_threshold A numeric value representing the threshold for the average silhouette index. 
#'                      Values should be between -1 and 1.
#' @param size_threshold An integer representing the threshold for the size of the motif, defined as the number of curves in the cluster. 
#'                       Should be at least 1.
#' @param K A vector containing the numbers of motifs that must be considered. 
#'          This should be a subset of \code{find_candidate_motifs_results$K}.
#' @param c A vector with the minimum motif lengths that must be considered. 
#'          This should be a subset of \code{find_candidate_motifs_results$c}.
#' 
#' @return A list containing the filtered candidate motifs and some ProbKMA options:
#' \item{V0_clean}{A vector of candidate motifs after filtering.}
#' \item{V1_clean}{A vector of derived candidate motifs after filtering.}
#' \item{D_clean}{A matrix of distances of candidate motifs from the curves after filtering.}
#' \item{P_clean}{A matrix of probabilities of membership of the candidate motifs after filtering.}
#' \item{Y0}{The original input data for the first curve.}
#' \item{Y1}{The original input data for the derivative of the curve.}
#' \item{diss}{The dissimilarity matrix used in motif discovery.}
#' \item{alpha}{The weight parameter for the Sobolev distance.}
#' \item{w}{Weights for the dissimilarity index across different dimensions.}
#' \item{max_gap}{Maximum allowable gap between curves.}
#' 
#' @details
#' This function checks the provided thresholds for silhouette indices and motif sizes, 
#' ensuring they are valid before filtering the motifs. If any parameters are invalid, 
#' default values from the \code{find_candidate_motifs_results} are used.
#' 
#' The function then loads the necessary data files for each combination of motif numbers and sizes, 
#' filtering them using the silhouette criteria. The resulting motifs are sorted by their length 
#' in descending order, and the filtered results are returned in a structured list.
#' 
#' @export
filter_candidate_motifs <- function(find_candidate_motifs_results,sil_threshold=0.5,size_threshold=2,
                                    K=find_candidate_motifs_results$K,c=find_candidate_motifs_results$c){
  
  ### check input #############################################################################################
  # check sil_threshold
  if(length(sil_threshold)!=1)
    stop('sil_threshold not valid.')
  if((sil_threshold<(-1))|(sil_threshold>1))
    stop('sil_threshold should be a number bewteen -1 and 1.')
  # check size_threshold
  if(length(size_threshold)!=1)
    stop('size_threshold not valid.')
  if(size_threshold%%1!=0)
    stop('size_threshold should be an integer.')
  if(size_threshold<1)
    stop('size_threshold should be at least 1.')
  # check K
  if(TRUE %in% !(K %in% find_candidate_motifs_results$K)){
    warning('K is not a subset of find_candidate_motifs_results$K. Using default K.')
    K=find_candidate_motifs_results$K
  }
  # check c
  if(TRUE %in% !(c %in% find_candidate_motifs_results$c)){
    warning('c is not a subset of find_candidate_motifs_results$c. Using default c.')
    c=find_candidate_motifs_results$c
  }
  
  ### filter motifs ###########################################################################################  
  n_init=find_candidate_motifs_results$n_init
  name=find_candidate_motifs_results$name
  motifs=lapply(K,
                function(K){
                  lapply(c,
                         function(c){
                           lapply(seq_len(n_init),
                                  function(i){
                                    load(paste0(name,"K",K,"_c",c,'/random',i,'.RData'))
                                    motifs=probKMA_silhouette_filter(probKMA_results,silhouette,sil_threshold,size_threshold)
                                    return(motifs)
                                  })
                         })
                })
  motifs=unlist(unlist(motifs,recursive=FALSE),recursive=FALSE)
  if(is.null(unlist(motifs)))
    stop("No motif present after filtering. Please re-run the function with less stringent parameters.")
  V0_clean=unlist(lapply(motifs,function(motifs) motifs$V0_clean),recursive=FALSE)
  V_clean_length=unlist(lapply(V0_clean,length))
  index=order(V_clean_length,decreasing=TRUE) # order from the longest to the shortest
  V0_clean=V0_clean[index]
  V1_clean=unlist(lapply(motifs,function(motifs) motifs$V1_clean),recursive=FALSE)[index]
  D_clean=Reduce(cbind,lapply(motifs,function(motifs) motifs$D_clean))[,index]
  P_clean=Reduce(cbind,lapply(motifs,function(motifs) motifs$P_clean))[,index]
  c=Reduce(c,lapply(motifs,function(motifs) motifs$c))[index]
  K=Reduce(c,lapply(motifs,function(motifs) motifs$K))[index]
  
  ### output ##################################################################################################
  load(paste0(name,"K",K[1],"_c",c[1],'/random',1,'.RData'))
  return(list(V0_clean=V0_clean,V1_clean=V1_clean,D_clean=as.matrix(D_clean),P_clean=as.matrix(P_clean),c=c,K=K,
              Y0=probKMA_results$Y0,Y1=probKMA_results$Y1,
              diss=probKMA_results$diss,alpha=probKMA_results$alpha,w=probKMA_results$w,max_gap=probKMA_results$max_gap))
}

#' @title Custom mapply Function for Parallel Processing
#'
#' @description
#' A wrapper function that conditionally applies either the classical `mapply` function 
#' or `parallel::clusterMap` based on whether a cluster object is provided. 
#' If the number of clusters is not null, it applies `clusterMap` for parallel execution; 
#' otherwise, it defaults to the classical `mapply`.
#'
#' @param cl A cluster object created by `makeCluster`. If `NULL`, the classical `mapply` function is used.
#' @param FUN A function to apply to the arguments.
#' @param ... Arguments to be passed to `FUN`. These should be vectors of equal length.
#' @param MoreArgs A list of additional arguments to be passed to `FUN`.
#' @param SIMPLIFY A logical value indicating whether to simplify the result if possible. Default is `TRUE`.
#' @param USE.NAMES A logical value indicating whether to use names from the first argument. Default is `TRUE`.
#'
#' @return A vector or list containing the results of applying the function `FUN` to the provided arguments. 
#'         The output type depends on the value of `SIMPLIFY`.
#'
#' @details
#' This function is useful for switching between parallel and non-parallel execution 
#' based on the availability of a cluster, allowing for more flexible and efficient code.
#'
#' @author Marzia Angela Cremona & Francesca Chiaromonte
#'
#' @export
.mapply_custom <- function(cl,FUN,...,MoreArgs=NULL,SIMPLIFY=TRUE,USE.NAMES=TRUE){
  if(is.null(cl)){
    mapply(FUN,...,MoreArgs=MoreArgs,SIMPLIFY=SIMPLIFY,USE.NAMES=USE.NAMES)
  }else{
    parallel::clusterMap(cl,FUN,...,MoreArgs=MoreArgs,SIMPLIFY=SIMPLIFY,USE.NAMES=USE.NAMES)
  }
}

#' Recommend Node Function
#'
#' @title Recommend Node from a Numeric Vector
#'
#' @description
#' This function analyzes a numeric vector and recommends an index based on the characteristics of the vector. 
#' If the vector is strictly increasing, it suggests stopping before the maximum growth rate. 
#' If the vector is not strictly increasing, it recommends the index of the minimum value.
#'
#' @param node A numeric vector representing scores or metrics. Must contain at least two elements.
#'
#' @return An integer index indicating the recommended node based on the analysis of the input vector.
#'
#' @details
#' - If the input vector has fewer than two elements, the function defaults to returning an index of 1.
#' - The function first checks if the input is a numeric vector. If not, an error is thrown.
#' - The function calculates the differences between successive elements and identifies whether the vector is strictly increasing.
#' - In the case of a strictly increasing vector, it returns the index of the maximum growth rate.
#' - For non-increasing vectors, it returns the index of the minimum value.
#'
#' @export
recommend_node <- function(node){
  xx <- unlist(node)
  
  if(length(xx) >= 2){
    diff_xx <- diff(xx)
    if(all(diff(xx)>0)){ # always increasing h-score: stop before maximum growth rate
      return((diff_xx/xx[1:length(xx)-1]) %>% which.max()) # or +1
    }else{ #also decreasing: stop minimum hscore
      return(which.min(xx))
    }
  }else{
    return(1)
  }
}

#' @title Plot Motif Search Results
#'
#' @description
#' The `motifs_search_plot` function visualizes the results obtained from the `motifs_search` function.
#' It generates plots for detected motifs across multiple dimensions, displaying both the motifs and their
#' corresponding derivative curves (if available). Users can filter motifs based on frequency thresholds
#' and choose to display either all motifs or the top `n` motifs. Additionally, the function provides an
#' option to plot all underlying curves with colored motifs highlighted.
#'
#' @param motifs_search_results A list containing the output from the `motifs_search` function. This
#'   includes elements such as `V0`, `V1`, `V_frequencies`, `Y0`, `Y1`, `V_length`, `V_occurrences`,
#'   `V_mean_diss`, and `R_motifs`, which store information about the detected motifs and their properties.
#'
#' @param ylab A character string specifying the label for the y-axis in the plots. This label will be
#'   appended with the dimension number to create individual titles for each plot. Default is an empty
#'   string (`''`).
#'
#' @param freq_threshold An integer indicating the minimum frequency a motif must have to be included
#'   in the plots. Only motifs with a frequency equal to or greater than `freq_threshold` will be
#'   visualized. Default value is `5`.
#'
#' @param top_n Determines how many motifs to plot based on their frequency. If set to `'all'`, all
#'   motifs meeting the `freq_threshold` will be plotted. If an integer is provided, only the top
#'   `top_n` motifs with the highest frequencies will be displayed. Default is `'all'`.
#'
#' @param plot_curves A logical value indicating whether to plot all underlying curves with colored
#'   motifs highlighted. If `TRUE`, the function overlays motifs on the curves for better visualization.
#'   Default is `TRUE`.
#'
#'
#' @return
#' The function does not return a value but generates plots visualizing the motifs and their occurrences
#' across different dimensions. It creates separate plots for each dimension and includes legends for
#' easy identification of motifs.
#'
#' @details
#' The `motifs_search_plot` function performs the following steps:
#' \enumerate{
#'   \item Validates input parameters, ensuring that the frequency threshold and `top_n` are appropriate.
#'   \item Selects motifs that meet the frequency threshold and, if specified, limits the number of motifs
#'     to the top `n`.
#'   \item For each dimension, it plots the motif centers (`V0`) and their derivatives (`V1`, if available).
#'   \item If `plot_curves` is `TRUE`, it overlays the motifs on the original curves, highlighting them with
#'     distinct colors.
#' }
#'
#' @importFrom dplyr %>%
#' @importFrom data.table as.data.table setnames
#' @export
#' 
#' 
#' 
motifs_search_plot <- function(motifs_search_results,
                               ylab = ylab,
                               freq_threshold = 5,
                               top_n = 'all',
                               plot_curves = TRUE) {
  # Plot the results of motifs_search.
  #   motifs_search_results: output of motifs_search function.
  #   freq_threshold: plot only motifs with frequency at least equal to freq_threshold.
  #   top_n: if 'all', plot all motifs found. If top_n is an integer, then all top top_n motifs are plotted.
  #   plot_curves: if TRUE, plot all the curves with coloured motifs.
  
  
  or_data <- motifs_search_results$Y0 # original data
  der_data <- motifs_search_results$Y1 # original data
  d       <- ncol(or_data[[1]]) # dimensional
  N       <- length(motifs_search_results$Y0) # Number of curves
  index_plot <- which(motifs_search_results$V_frequencies >= freq_threshold)
  if (top_n != 'all') {
    if (length(index_plot) > top_n)
      index_plot = index_plot[seq_len(top_n)]
  }
  
  K  = length(index_plot)
  V0 = motifs_search_results$V0[index_plot] # interesting motifs
  V1 = motifs_search_results$V1[index_plot] # interesting motifs DER
  V_dom = lapply(V0, function(v)
    rowSums(!is.na(v)) != 0)
  V_length = motifs_search_results$V_length[index_plot] # motif length
  V_occurrences = motifs_search_results$V_occurrences[index_plot] # motif occurrences
  V_frequencies = motifs_search_results$V_frequencies[index_plot] # number of motif occurrences
  V_mean_diss = motifs_search_results$V_mean_diss[index_plot] # motif diss
  R_motifs = motifs_search_results$R_motifs[index_plot] # motif radius
  n_motifs <- V0 %>% length()
  
  # all occurrences as data frame
  temp <- lapply(seq_along(V_occurrences), function(x) {
    df <- V_occurrences[[x]]
    cbind(df, motif = x)
  })
  
  all_occurrences <- do.call(rbind.data.frame, temp)
  
  # The plot is composed by
  # - overview (motif centers); (DONE)
  # - overview (derivative); (DONE)
  # - single motif (all curvers) + single motif insight V0 and V1; (DONE)
  # - curve perspective; (TO ADD)
  
  # Palette fissa per tutti i motif
  motif_colors <- rainbow(K)
  
  # OVERVIEW PLOTS ------
  layout(matrix(c(seq_len(d), rep(d + 1, d)), ncol = 2), widths = c(7, 1))
  
  # Motif centers
  lapply(seq_len(d), function(j) {
    par(mar = c(3, 4, 4, 2) + 0.1)
    plot(
      V0[[1]][, j],
      type = 'l',
      col = motif_colors,
      lwd = 5,
      lty = 1,
      main = paste0("Motif centers - Dimension: ", j),
      xlim = c(1, max(V_length)),
      ylab = ylab,
      ylim = c(min(unlist(V0)), max(unlist(V0)))
    )
    mapply(
      function(v, k)
        points(
          v[, j],
          type = 'l',
          col = motif_colors[k + 1],
          lwd = 5,
          lty = 1
        ),
      V0[-1],
      seq_len(K - 1)
    )
    par(mar = c(0, 0, 0, 0))
    return(NULL)
  })
  plot.new()
  legend(
    'left',
    paste('motif', seq_len(K)),
    col = motif_colors,
    lwd = 7,
    lty = 1,
    bty = "n",
    xpd = TRUE
  )
  
  # Motif centers - derivative
  if (!is.null(V1[[1]])) {
    layout(matrix(c(seq_len(d), rep(d + 1, d)), ncol = 2), widths = c(7, 1))
    lapply(seq_len(d), function(j) {
      par(mar = c(3, 4, 4, 2) + 0.1)
      plot(
        V1[[1]][, j],
        type = 'l',
        col = motif_colors,
        lwd = 5,
        lty = 1,
        main = paste("Motif centers - Dimension: ", j, ' - derivative'),
        xlim = c(1, max(V_length)),
        ylab = ylab,
        ylim = c(min(unlist(V1)), max(unlist(V1)))
      )
      mapply(
        function(v, k)
          points(
            v[, j],
            type = 'l',
            col = motif_colors[k + 1],
            lwd = 5,
            lty = 1
          ),
        V1[-1],
        seq_len(K - 1)
      )
      par(mar = c(0, 0, 0, 0))
      return(NULL)
    })
    plot.new()
    legend(
      'left',
      paste('motif', seq_len(K)),
      col = motif_colors,
      lwd = 7,
      lty = 1,
      bty = "n",
      xpd = TRUE
    )
  }
  
  
  # LOOP SUI MOTIFS ------
  for (k in 1:K) {
    motif_occurrences <- V_occurrences[[k]]
    obs_mot_k <- nrow(motif_occurrences)
    unique_curves <- unique(motif_occurrences[, 1])
    curve_colors <- rainbow(length(unique_curves))
    
    layout(matrix(c(seq_len(d), rep(d + 1, d)), ncol = 2), widths = c(7, 1))
    mot_color <- motif_colors[k]
    
    lapply(seq_len(d), function(j) {
      par(mar = c(3, 4, 4, 2) + 0.1)
      
      # Palette fissa per le curve coinvolte
      #unique_curves <- unique(motif_occurrences[, 1])
      #curve_colors <- rainbow(length(unique_curves))
      
      # PLOT COLORATO
      par(mar = c(5, 4, 3, 2), oma = c(0, 0, 0, 7))
      
      # PLOT BASE
      plot(
        or_data[[1]],
        xlab = "",
        ylab = ylab,
        col = scales::alpha("gray30", 0.30),
        lwd = 1.5,
        type = "l",
        main = paste0(
          "Motif ",
          k,
          " - Dimension: ",
          j,
          "\n",
          "Number of instances: ",
          obs_mot_k,
          ifelse(is.null(R_motifs), "", paste0(" - Motif Radius: ", round(R_motifs[k], 3)))
        )
      )
      
      # ALTRE CURVE
      for (curve_i in 2:length(or_data)) {
        lines(or_data[[curve_i]],
              col = scales::alpha("gray30", 0.30),
              lwd = 1.5)
      }
      
      # CURVE CON ALMENO UN'OCCORRENZA
      for (i in seq_along(unique_curves)) {
        lines(or_data[[unique_curves[i]]],
              col = curve_colors[i],
              lwd = 1.5,
              lty = 2)
      }
      
      # MOTIF + RETTANGOLO TRASPARENTE
      for (i in seq_len(obs_mot_k)) {
        curve_id <- motif_occurrences[i, 1]
        shift <- motif_occurrences[i, 2]
        motif_length <- length(motifs_search_results$V0[[k]])
        x_vals <- shift:(shift + motif_length)
        y_vals <- or_data[[curve_id]][x_vals]
        
        # rettangolo trasparente dello stesso colore del motif
        rgb_vals <- col2rgb(mot_color) / 255
        rect(
          min(x_vals),
          par("usr")[3],
          max(x_vals),
          par("usr")[4],
          col = rgb(rgb_vals[1], rgb_vals[2], rgb_vals[3], alpha = 0.05),
          border = NA
        )
        
        # linea sopra
        lines(x_vals, y_vals, col = mot_color, lwd = 2.5)
      }
      
      # LEGENDA
      par(xpd = NA)
      legend(
        par("usr")[2] + diff(par("usr")[1:2]) * 0.02,
        par("usr")[4],
        legend = c(paste0("curve ", unique_curves), paste0("motif_", k)),
        col = c(curve_colors, mot_color),
        lty = c(rep(2, length(
          unique_curves
        )), 1),
        lwd = c(rep(1.5, length(
          unique_curves
        )), 3),
        bty = "n",
        cex = 0.8
      )
    })
    
    # FOCUSED PLOT
    lapply(seq_len(d), function(j) {
      par(mar = c(5, 4, 3, 2), oma = c(0, 0, 0, 7))
      
      plot(
        V0[[k]],
        type = 'l',
        col = mot_color,
        lwd = 2.5,
        xlab = "",
        ylab = ylab,
        main = paste0(
          "Motif ",
          k,
          " - Dimension: ",
          j,
          "\n",
          "Number of instances: ",
          obs_mot_k,
          ifelse(is.null(R_motifs), "", paste0(
            " - Motif Radius: ", round(R_motifs[k], 3)
          ))
        )
      )
      
      for (curve_i in seq_len(nrow(motif_occurrences))) {
        curve_id <- motif_occurrences[curve_i, 1]
        shift <- motif_occurrences[curve_i, 2]
        motif_length <- length(V0[[k]])
        x_vals <- shift:(shift + motif_length)
        y_vals <- or_data[[curve_id]][x_vals]
        lines(y_vals, col = curve_colors[curve_i], lwd = 1)
      }
      
      # linea del motif sopra le altre
      lines(V0[[k]], col = mot_color, lwd = 2.5)
      
      # LEGENDA
      par(xpd = NA)
      legend(
        par("usr")[2] + diff(par("usr")[1:2]) * 0.02,
        par("usr")[4],
        legend = c(paste0("curve ", unique_curves), paste0("motif_", k)),
        col = c(curve_colors, mot_color),
        lty = c(rep(2, length(
          unique_curves
        )), 1),
        lwd = c(rep(1.5, length(
          unique_curves
        )), 3),
        bty = "n",
        cex = 0.8
      )
    })
    
    # FOCUSED PLOT V1
    lapply(seq_len(d), function(j) {
      par(mar = c(5, 4, 3, 2), oma = c(0, 0, 0, 7))
      
      plot(
        V1[[k]],
        type = 'l',
        col = mot_color,
        lwd = 2.5,
        xlab = "",
        ylab = ylab,
        main = paste0(
          "Motif ",
          k,
          " - Dimension: ",
          j,
          "\n",
          "Number of instances: ",
          obs_mot_k,
          ifelse(is.null(R_motifs), "", paste0(
            " - Motif Radius: ", round(R_motifs[k], 3)
          ))
        )
      )
      
      for (curve_i in seq_len(nrow(motif_occurrences))) {
        curve_id <- motif_occurrences[curve_i, 1]
        shift <- motif_occurrences[curve_i, 2]
        motif_length <- length(V1[[k]])
        x_vals <- shift:(shift + motif_length)
        y_vals <- der_data[[curve_id]][x_vals]
        lines(y_vals, col = curve_colors[curve_i], lwd = 1)
      }
      
      # linea del motif sopra le altre
      lines(V1[[k]], col = mot_color, lwd = 2.5)
      
      # LEGENDA
      par(xpd = NA)
      legend(
        par("usr")[2] + diff(par("usr")[1:2]) * 0.02,
        par("usr")[4],
        legend = c(paste0("curve ", unique_curves), paste0("motif_", k)),
        col = c(curve_colors, mot_color),
        lty = c(rep(2, length(
          unique_curves
        )), 1),
        lwd = c(rep(1.5, length(
          unique_curves
        )), 3),
        bty = "n",
        cex = 0.8
      )
    })
  }
  
  # plot every single curve
  if (plot_curves == TRUE) {
    for (f in 1:N) {
      lapply(seq_len(d), function(j) {
        par(mar = c(5, 4, 3, 2), oma = c(0, 0, 0, 7))
        
        plot(
          or_data[[f]],
          type = 'l',
          col = scales::alpha("gray30", 0.30),
          lwd = 1.5,
          ylab = ylab,
          xlab = "",
          main = paste0("Curve ", f, " - Dimention: ", j)
        )
        
        # check if they have a motif, add it
        occ_inside <- all_occurrences[which(all_occurrences$curve == f), ]
        if (nrow(occ_inside) > 0) {
          for (occ_i in seq_len(nrow(occ_inside))) {
            temp_mot <- occ_inside$motif[occ_i]
            shift <- occ_inside[occ_i, 2]
            motif_length <- length(V0[[temp_mot]])
            x_vals <- shift:(shift + motif_length)
            y_vals <- or_data[[f]][x_vals]
            # plot line
            lines(x_vals, y_vals, col = motif_colors[temp_mot], lwd = 2)
          }
          
          # LEGENDA
          motifs_here <- unique(occ_inside$motif)
          
          par(xpd = NA)
          legend(
            par("usr")[2] + diff(par("usr")[1:2]) * 0.02,
            par("usr")[4],
            legend = paste0("motif_", motifs_here),
            col = motif_colors[motifs_here],
            lwd = 2,
            bty = "n",
            cex = 0.8
          )
        }
      })
    }
  }
  return()
}

#'OLD VERSION 
# motifs_search_plot <- function(motifs_search_results, ylab = '',freq_threshold = 5, top_n = 'all', plot_curves = TRUE, transformed = FALSE){
#   # Plot the results of motifs_search.
#   # motifs_search_results: output of motifs_search function.
#   # freq_threshold: plot only motifs with frequency at least equal to freq_threshold.
#   # top_n: if 'all', plot all motifs found. If top_n is an integer, then all top top_n motifs are plotted.
#   # plot_curves: if TRUE, plot all the curves with coloured motifs.
#   
#   ### check input ############################################################################################
#   # check freq_threshold
#   oldpar <- par(no.readonly = TRUE)
#   # Ensure the original settings are restored when the function exits
#   on.exit(par(oldpar))
#   if(max(motifs_search_results$V_frequencies) < freq_threshold)
#     stop('There are no motifs with frequency at least equal to freq_threshold.')
#   # check top_n
#   if(top_n != 'all'){
#     if(length(top_n) != 1)
#       stop('top_n not valid.')
#     if(top_n%%1 != 0)
#       stop('top_n should be an integer.')
#     if(top_n < 1)
#       stop('top_n should be at least 1.')
#   }
#   
#   ### select motifs to plot ##################################################################################
#   d = ncol(motifs_search_results$Y0[[1]])
#   N = length(motifs_search_results$Y0)
#   index_plot = which(motifs_search_results$V_frequencies>=freq_threshold)
#   if(top_n != 'all'){
#     if(length(index_plot) > top_n)
#       index_plot = index_plot[seq_len(top_n)]
#   }
#   K  = length(index_plot)
#   V0 = motifs_search_results$V0[index_plot]
#   V1 = motifs_search_results$V1[index_plot]
#   V_dom = lapply(V0,function(v) rowSums(!is.na(v))!=0)
#   V_length = motifs_search_results$V_length[index_plot]
#   V_occurrences = motifs_search_results$V_occurrences[index_plot]
#   V_frequencies = motifs_search_results$V_frequencies[index_plot]
#   V_mean_diss = motifs_search_results$V_mean_diss[index_plot]
#   R_motifs = motifs_search_results$R_motifs[index_plot]
#   
#   ### plot motifs ############################################################################################
#   layout(matrix(c(seq_len(d),rep(d+1,d)),ncol=2),widths=c(7,1))
#   lapply(seq_len(d),
#          function(j){
#            par(mar=c(3,4,4,2)+0.1)
#            plot(V0[[1]][,j],type='l',col=rainbow(K),lwd=5,lty=1,main=paste0(ylab,"-","Dimension:",j),xlim=c(1,max(V_length)),
#                 ylab=ylab,ylim=c(min(unlist(V0)),max(unlist(V0))))
#            mapply(function(v,k) points(v[,j],type='l',col=rainbow(K)[k+1],lwd=5,lty=1,ylab=ylab),
#                   V0[-1],seq_len(K-1))
#            par(mar=c(0,0,0,0))
#            return()})
#   plot.new()
#   legend('left',paste('motif',seq_len(K)),col=rainbow(K),lwd=7,lty=1,bty="n",xpd=TRUE)
#   if(!is.null(V1[[1]])){
#     layout(matrix(c(seq_len(d),rep(d+1,d)),ncol=2),widths=c(7,1))
#     lapply(seq_len(d),
#            function(j){
#              par(mar=c(3,4,4,2)+0.1)
#              plot(V1[[1]][,j],type='l',col=rainbow(K),lwd=5,lty=1,main=paste(ylab,"-","Dimension:",j,'derivative'),xlim=c(1,max(V_length)),
#                   ylab = ylab, ylim = c(min(unlist(V1)),max(unlist(V1))))
#              mapply(function(v,k) points(v[,j],type='l',col=rainbow(K)[k+1],lwd=5,lty=1,ylab=ylab),
#                     V1[-1],seq_len(K-1))
#              par(mar=c(0,0,0,0))
#              return()})
#     plot.new()
#     legend('left',paste('motif',seq_len(K)),col=rainbow(K),lwd=7,lty=1,bty="n",xpd=TRUE)
#   }
#   
#   ### plot motifs with matched curves ########################################################################
#   if(is.null(V1[[1]])){
#     mapply(function(v,v_dom,v_occurrences,v_frequencies,k,R_motif){
#       Y_inters_k=mapply(function(y,s_k_i,v_dom){
#         v_len=length(v_dom)
#         Y_inters_k=as.matrix(as.matrix(y[s_k_i-1+seq_len(v_len),])[v_dom,])
#         return(Y_inters_k)},  # c('curve','shift','diss')
#         motifs_search_results$Y0[v_occurrences[,1]],v_occurrences[,2],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
#       layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(7,1))
#       Y0_diff_k=lapply(Y_inters_k,
#                        function(Y0_inters_k){
#                          y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
#                          y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
#                          y0_diff=y0_max-y0_min
#                          return(y0_diff)
#                        })
#       lapply(seq_len(d),
#              function(j){
#                par(mar=c(3,4,4,2)+0.1)
#                y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y_inters_k))
#                if(transformed){
#                  y_plot[v_dom,]=Reduce('cbind',
#                                        mapply(function(Y_inters_k, Y_diff_k) {
#                                          y0_min=min(Y_inters_k[,j])
#                                          y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
#                                          y0_const = (Y_diff_k[j] == 0)
#                                          y0_norm[,y0_const] = 0.5
#                                          return(y0_norm)},
#                                          Y_inters_k, Y0_diff_k, SIMPLIFY=FALSE) )
#                } else {
#                  y_plot[v_dom,]=Reduce('cbind',lapply(Y_inters_k,function(Y_inters_k) Y_inters_k[,j]))
#                }
#                matplot(y_plot,type='l',col=v_occurrences[,1]+1,lwd=round(-4/R_motif*v_occurrences[,3]+5,2),
#                        lty=1,ylab=ylab,main=paste0('Motif ',k,' (',v_frequencies,' occurrences) - ',ylab,"-","Dimension:",j))
#                points(v[,j],type='l',col='black',lwd=7,lty=1)
#                par(mar=c(0,0,0,0))
#                plot.new()
#                legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
#              })
#       return()},V0,V_dom,V_occurrences,V_frequencies,seq_len(K),R_motifs)
#   }else{
#     mapply(function(v0,v1,v_dom,v_occurrences,v_frequencies,k,R_motif){
#       Y0_inters_k=mapply(
#         function(y,s_k_i,v_dom){
#           v_len=length(v_dom)
#           Y_inters_k=as.matrix(as.matrix(y[s_k_i-1+seq_len(v_len),])[v_dom,])
#           return(Y_inters_k)},
#         motifs_search_results$Y0[v_occurrences[,1]],v_occurrences[,2],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
#       Y1_inters_k=mapply(
#         function(y,s_k_i,v_dom){
#           v_len=length(v_dom)
#           Y_inters_k=as.matrix(as.matrix(y[s_k_i-1+seq_len(v_len),])[v_dom,])
#           return(Y_inters_k)},
#         motifs_search_results$Y1[v_occurrences[,1]],v_occurrences[,2],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
#       layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(7,1))
#       Y0_diff_k=lapply(Y0_inters_k,
#                        function(Y0_inters_k){
#                          y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
#                          y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
#                          y0_diff=y0_max-y0_min
#                          return(y0_diff)
#                        })
#       lapply(seq_len(d),
#              function(j){
#                par(mar=c(3,4,4,2)+0.1)
#                y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y0_inters_k))
#                if(transformed){
#                  y_plot[v_dom,]=Reduce('cbind',
#                                        mapply(function(Y_inters_k, Y_diff_k) {
#                                          y0_min=min(Y_inters_k[,j])
#                                          y0_norm = t( (t(Y_inters_k[,j]) - y0_min[j]) / Y_diff_k[j] )
#                                          y0_const = (Y_diff_k[j] == 0)
#                                          y0_norm[,y0_const] = 0.5
#                                          return(y0_norm)},
#                                          Y0_inters_k, Y0_diff_k, SIMPLIFY=FALSE) 
#                  )
#                } else {
#                  y_plot[v_dom,]=Reduce('cbind',lapply(Y0_inters_k,function(Y_inters_k) Y_inters_k[,j]))
#                }
#                matplot(y_plot,type='l',col=v_occurrences[,1]+1,lwd=round(-4/R_motif*v_occurrences[,3]+5,2),
#                        lty=1,ylab=ylab,main=paste0('Motif ',k,' (',v_frequencies,' occurrences) - ',ylab,"-","Dimension:",j))
#                points(v0[,j],type='l',col='black',lwd=7,lty=1)
#                par(mar=c(0,0,0,0))
#                plot.new()
#                legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
#              })
#       lapply(seq_len(d),
#              function(j){
#                par(mar=c(3,4,4,2)+0.1)
#                y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y1_inters_k))
#                if(transformed){
#                  y_plot[v_dom,]=Reduce('cbind',
#                                        mapply(function(Y1_inters_k, Y_diff_k) {
#                                          y1_norm = t( t(Y1_inters_k[,j])/ Y_diff_k[j] )
#                                          y0_const = (Y_diff_k[j] == 0)
#                                          y1_norm[,y0_const] = 0
#                                          return(y1_norm)},
#                                          Y1_inters_k, Y0_diff_k, SIMPLIFY=FALSE)
#                  ) 
#                } else {
#                  y_plot[v_dom,]=Reduce('cbind',lapply(Y1_inters_k,function(Y_inters_k) Y_inters_k[,j]))
#                }               
#                matplot(y_plot,type='l',col=v_occurrences[,1]+1,lwd=round(-4/R_motif*v_occurrences[,3]+5,2),
#                        lty=1,ylab=ylab,main=paste0('Motif ',k,' (',v_frequencies,' occurrences) - ',ylab,"-","Dimension:",j,' derivative'))
#                points(v1[,j],type='l',col='black',lwd=7,lty=1)
#                par(mar=c(0,0,0,0))
#                plot.new()
#                legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
#              })
#       return()},V0,V1,V_dom,V_occurrences,V_frequencies,seq_len(K),R_motifs)
#   }
#   
#   ### plot curves with motifs ################################################################################
#   if(plot_curves){
#     if(is.null(motifs_search_results$Y1[[1]])){
#       mapply(function(y0,i){
#         s_i=lapply(V_occurrences,function(occurrences) occurrences[occurrences[,1]==i,2])
#         motifs_in_curve=rep(seq_len(K),unlist(lapply(s_i,length)))
#         s_i=unlist(s_i)
#         Y_inters_k=mapply(function(v_dom,s_i_k,y){
#           v_len=length(v_dom)
#           Y_inters_k=matrix(NA,nrow=length(v_dom),ncol=d)
#           Y_inters_k[v_dom,]=as.matrix(as.matrix(y[s_i_k-1+seq_len(v_len),])[v_dom,])
#           return(Y_inters_k)},
#           V_dom[motifs_in_curve],s_i,MoreArgs=list(y0),SIMPLIFY=FALSE)
#         layout(matrix(c(seq_len(d),rep(d+1,d)),ncol=2),widths=c(7,1))
#         lapply(seq_len(d),
#                function(j){
#                  par(mar=c(3,4,4,2)+0.1)
#                  plot(y0[,j],type='l',main=paste('Region',i,'-',ylab,"-","Dimension:",j),ylab=ylab)
#                  for(k in seq_along(motifs_in_curve)){
#                    lines(s_i[k]-1+seq_len(V_length[motifs_in_curve[k]]),Y_inters_k[[k]][,j],col=rainbow(K)[motifs_in_curve[k]],lwd=5)
#                    rect(s_i[k], min(y0[,j],na.rm=TRUE)-10, tail(s_i[k]-1+seq_len(V_length[motifs_in_curve[k]]), n=1), max(y0[,j],na.rm=TRUE)+10,
#                         border = scales::alpha(rainbow(K)[motifs_in_curve[k]], 0.05), col = scales::alpha(rainbow(K)[motifs_in_curve[k]], 0.05))
#                  }
#                })
#         plot.new()
#         if(length(motifs_in_curve)==0){
#           legend_text=''
#         }else{
#           legend_text=paste('motif',unique(motifs_in_curve))
#         }
#         legend('left',legend_text,col=rainbow(K)[unique(motifs_in_curve)],lwd=7,lty=1,bty="n",xpd=TRUE,title='Motifs')
#         return()},motifs_search_results$Y0,seq_len(N))
#     }else{
#       mapply(function(y0,y1,i){
#         s_i=lapply(V_occurrences,function(occurrences) occurrences[occurrences[,1]==i,2])
#         motifs_in_curve=rep(seq_len(K),unlist(lapply(s_i,length)))
#         s_i=unlist(s_i)
#         Y0_inters_k=mapply(function(v_dom,s_i_k,y){
#           v_len=length(v_dom)
#           Y_inters_k=matrix(NA,nrow=length(v_dom),ncol=d)
#           Y_inters_k[v_dom,]=as.matrix(as.matrix(y[s_i_k-1+seq_len(v_len),])[v_dom,])
#           return(Y_inters_k)},
#           V_dom[motifs_in_curve],s_i,MoreArgs=list(y0),SIMPLIFY=FALSE)
#         Y1_inters_k=mapply(function(v_dom,s_i_k,y){
#           v_len=length(v_dom)
#           Y_inters_k=matrix(NA,nrow=length(v_dom),ncol=d)
#           Y_inters_k[v_dom,]=as.matrix(as.matrix(y[s_i_k-1+seq_len(v_len),])[v_dom,])
#           return(Y_inters_k)},
#           V_dom[motifs_in_curve],s_i,MoreArgs=list(y1),SIMPLIFY=FALSE)
#         layout(matrix(c(seq_len(d),rep(d+1,d)),ncol=2),widths=c(7,1))
#         lapply(seq_len(d),
#                function(j){
#                  par(mar=c(3,4,4,2)+0.1)
#                  plot(y0[,j],type='l',main=paste('Region',i,'-',ylab,"-","Dimension:",j),ylab=ylab,xlab='')
#                  for(k in seq_along(motifs_in_curve)){
#                    lines(s_i[k]-1+seq_len(V_length[motifs_in_curve[k]]),Y0_inters_k[[k]][,j],col=rainbow(K)[motifs_in_curve[k]],lwd=5)
#                  }
#                })
#         plot.new()
#         if(length(motifs_in_curve)==0){
#           legend_text=''
#         }else{
#           legend_text=paste('motif',unique(motifs_in_curve))
#         }
#         legend('left',legend_text,col=rainbow(K)[unique(motifs_in_curve)],lwd=7,lty=1,bty="n",xpd=TRUE,title='Motifs')
#         lapply(seq_len(d),
#                function(j){
#                  par(mar=c(3,4,4,2)+0.1)
#                  plot(y1[,j],type='l',main=paste('Region',i,'-',paste(ylab,"-","Dimension:",j,'derivative')),ylab=ylab)
#                  for(k in seq_along(motifs_in_curve)){
#                    lines(s_i[k]-1+seq_len(V_length[motifs_in_curve[k]]),Y1_inters_k[[k]][,j],col=rainbow(K)[motifs_in_curve[k]],lwd=5)
#                  }
#                })
#         plot.new()
#         if(length(motifs_in_curve)==0){
#           legend_text=''
#         }else{
#           legend_text=paste('motif',unique(motifs_in_curve))
#         }
#         legend('left',legend_text,col=rainbow(K)[unique(motifs_in_curve)],lwd=7,lty=1,bty="n",xpd=TRUE,title='Motifs')
#         return()},motifs_search_results$Y0,motifs_search_results$Y1,seq_len(N))
#     }
#   }
#   
#   return()
# }

#' @title Generate Minimum Dendrogram from Hierarchical Clustering
#'
#' @description
#' This function generates a minimum dendrogram by performing hierarchical clustering on an input distance matrix. 
#' The function identifies an optimal height cut based on the largest gap in cluster heights and returns the resulting 
#' dendrogram, allowing for further analysis of clustering structures.
#'
#' @param adj_fMSR A numeric matrix or a distance object representing the dissimilarity matrix 
#'                  computed from functional data. This matrix is used to perform hierarchical clustering.
#'
#' @return A dendrogram object representing the clustered data. The dendrogram is cut at the identified height,
#'         resulting in a tree structure that can be used for further analysis or visualization.
#'
#' @details
#' The function performs the following steps:
#' 1. Uses `fastcluster::hclust` to perform hierarchical clustering on the provided dissimilarity matrix using 
#'    the "complete" method.
#' 2. Extracts the heights of the clusters and identifies the largest gap in heights to determine the optimal 
#'    cut position for the dendrogram.
#' 3. Cuts the dendrogram at the identified height and returns the lower part of the cut.
#'
#' @importFrom fastcluster hclust
#' @export
get_minidend <- function(adj_fMSR){
  # generate dendrogram
  total_hc <- fastcluster::hclust(adj_fMSR, method = "complete")
  total_dend <- as.dendrogram(total_hc) # transform in dendrogram
  
  # Identify height_cut
  # It is made by ordering all the heights and identifying the largest gap.
  # The height cut corresponds to the mean between the elements generating the gap
  cut_position <- sort(total_hc$height) %>% diff() %>% which.max() # total_hc$height is filled with the cluster distance for each step
  height_cut <- sort(total_hc$height)[c(cut_position, cut_position + 1)] %>%
    mean()
  
  # Generating minidend
  minidend <- cut(total_dend, h = height_cut)$lower
  return(minidend)
}

#' @title Get Parent Nodes from a Given Node
#'
#' @description
#' This function identifies the parent nodes of a given node in a hierarchical structure. 
#' It checks which nodes in the provided list contain the specified node, thereby determining 
#' its parent nodes.
#'
#' @param node A vector representing a single node whose parents are to be found. 
#'              It is expected to contain the elements that may be present in the parent nodes.
#' @param node_list A list of vectors, where each vector represents a node in the hierarchical structure. 
#'                  The function will search through these nodes to find parents of the specified node.
#'
#' @return A numeric vector containing the indices of the parent nodes in the `node_list` that include the specified node.
#'
#' @details
#' The function works by applying a logical check across the `node_list`. It returns the indices of all nodes 
#' that contain all elements of the specified node vector. If no parent nodes are found, the function will return 
#' an empty integer vector.
#'
#' @export
get_parents <- function(node, node_list){
  which(lapply(node_list, function(x){all(node %in% x)}) %>% unlist())
}

#' @title Pad a Matrix to a Specified Number of Rows
#'
#' @description 
#' This function pads a matrix with `NA` values to ensure that the matrix reaches a specified number of rows (`maxLen`). 
#' If the matrix has fewer rows than the desired length, additional rows of `NA` are appended to the matrix. 
#' Otherwise, the matrix is returned unchanged.
#'
#' @param dataMatrix A matrix to be padded. Each row represents a data point.
#' @param maxLen An integer specifying the desired number of rows in the matrix.
#'
#' @details 
#' The function compares the number of rows in `dataMatrix` with `maxLen`. If the matrix has fewer rows, 
#' it pads the matrix with `NA` values until it reaches the specified number of rows. 
#' No changes are made if `dataMatrix` has rows equal to or greater than `maxLen`.
#'
#' @return A matrix padded with `NA` values up to the specified number of rows. 
#' If `dataMatrix` already has the desired number of rows or more, the original matrix is returned.
#'
#' @export
padding <- function(dataMatrix, maxLen) 
{
  actual_len <- nrow(dataMatrix)
  if (actual_len < maxLen) 
  {
    extension_len <- maxLen - actual_len
    extension <- matrix(NA, nrow = extension_len, ncol = ncol(dataMatrix))
    full_mat <- rbind(dataMatrix, extension)
    return(full_mat)
  }else{return(dataMatrix)}
}

#' @title Motif Search in Curves
#'
#' @description
#' The `motifs_search` function identifies and ranks motifs within a set of curves based on their 
#' frequencies and dissimilarity measures. It processes candidate motifs clustered from hierarchical 
#' clustering results, selects optimal motifs within each cluster, and determines their occurrences 
#' in the original curves. The function supports parallel processing to enhance computational efficiency 
#' and offers flexibility in handling different dissimilarity metrics and motif selection criteria.
#'
#' @param cluster_candidate_motifs_results A list containing the output from the `cluster_candidate_motifs` 
#'   function. This list must include elements such as:
#'   \describe{
#'     \item{Y0}{A list of matrices representing the original curves.}
#'     \item{Y1}{A list of matrices representing the derivatives of the curves (if applicable).}
#'     \item{V0_clean}{A list of candidate motifs derived from `Y0`.}
#'     \item{V1_clean}{A list of candidate motifs derived from `Y1` (if applicable).}
#'     \item{D_clean}{A matrix of dissimilarity measures between motifs and curves.}
#'     \item{P_clean}{A matrix indicating positive matches (e.g., presence of motifs in curves).}
#'     \item{hclust_res}{A hierarchical clustering object obtained from `hclust`.}
#'     \item{R_all}{A numeric value representing the global radius used for dendrogram cutting.}
#'     \item{w}{A numeric vector of weights for the dissimilarity index across different dimensions.}
#'     \item{transformed}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
#'     \item{max_gap}{A numeric value defining the maximum allowed gap in distances for cluster separation.}
#'     \item{k_knn}{An integer specifying the number of neighbors for K-Nearest Neighbors classification.}
#'     \item{votes_knn_Rm}{A numeric value defining the probability threshold for KNN-based radius determination.}
#'     \item{c}{A numeric vector specifying the minimum number of overlapping elements required for motif validation.}
#'   }
#' 
#' @param R_all A numeric value representing the global radius used to cut the dendrogram, ensuring that 
#'   clusters are at least twice this radius apart. This parameter defines the grouping of motifs into clusters.
#'
#' @param R_m A numeric vector containing group-specific radii used to identify motif occurrences within each cluster. 
#'   The length of this vector must match the number of clusters obtained by cutting the dendrogram at a height of `2 * R_all`. 
#'   If `NULL`, the function automatically determines `R_m` for each group based on the distances between motifs 
#'   within the same cluster and all curves.
#'
#' @param different_R_m_finding A logical value indicating whether to use a different radius (`R_m_finding`) 
#'   for finding motif occurrences compared to the initial radius (`R_m`). If `TRUE`, `R_m_finding` is used; 
#'   otherwise, `R_m` is employed. This allows for separate tuning of motif occurrence detection.
#'
#' @param R_m_finding A numeric vector containing group-specific radii used specifically for finding motif occurrences 
#'   when `different_R_m_finding` is set to `TRUE`. The length of this vector must match the number of clusters obtained 
#'   by cutting the dendrogram at a height of `2 * R_all`. If `NULL`, `R_m_finding` is determined automatically for 
#'   each group based on distances between motifs within the same cluster and all curves.
#'
#' @param use_real_occurrences A logical value indicating whether to compute real occurrences of candidate motifs 
#'   within the curves. If `TRUE`, the function calculates actual frequencies and mean dissimilarities for motif 
#'   selection, providing more accurate results at the cost of increased computation time. If `FALSE`, it uses 
#'   approximate frequencies and mean dissimilarities for faster execution. Defaults to `FALSE`.
#'
#' @param length_diff A numeric value specifying the minimum percentage difference in length required among 
#'   motifs within the same group to retain multiple motifs. This parameter ensures diversity in motif selection 
#'   by preventing motifs of similar lengths from being selected simultaneously. It is defined as a percentage 
#'   relative to the length of the most frequent motif. Defaults to `Inf`, meaning no additional motifs are selected 
#'   based on length differences.
#'
#' @param worker_number An integer indicating the number of CPU cores to utilize for parallel processing. 
#'   By default, the function uses one less than the total number of available cores (`detectCores() - 1`). 
#'   Setting `worker_number = 1` forces the function to run sequentially without parallelization. If `NULL`, 
#'   the function automatically determines the optimal number of workers based on the system's available cores.
#'
#' @return A list containing:
#'   \describe{
#'     \item{V0}{A list of selected motifs derived from `Y0`.}
#'     \item{V1}{A list of selected motifs derived from `Y1` (if applicable).}
#'     \item{V_length}{A numeric vector representing the real lengths of the selected motifs.}
#'     \item{V_occurrences}{A list detailing the occurrences of each selected motif within the curves.}
#'     \item{V_frequencies}{A numeric vector indicating the real frequencies of each selected motif.}
#'     \item{V_mean_diss}{A numeric vector representing the average dissimilarity of each selected motif.}
#'     \item{Y0}{A list of matrices corresponding to the original curves, as provided in `cluster_candidate_motifs_results`.}
#'     \item{Y1}{A list of matrices corresponding to the derivatives of the curves (if applicable), as provided in `cluster_candidate_motifs_results`.}
#'     \item{R_motifs}{A numeric vector containing the radii associated with each selected motif.}
#'   }
#'
#' @details
#' The `motifs_search` function operates through the following steps:
#' \enumerate{
#'   \item **Parallelization Setup**: Determines the number of worker cores to use based on `worker_number`. If `worker_number > 1`, 
#'     it initializes a cluster for parallel processing.
#'   \item **Input Preparation**: Depending on the dissimilarity metric (`d0_L2`, `d1_L2`, or `d0_d1_L2`), it prepares the data structures 
#'     `Y` and `V` for processing.
#'   \item **Dendrogram Cutting**: Cuts the hierarchical clustering dendrogram at a height of `2 * R_all` to define clusters of motifs.
#'   \item **Radius Determination**: If `R_m` or `R_m_finding` is not provided, the function calculates these radii for each cluster 
#'     based on motif distances and K-Nearest Neighbors (KNN) classification.
#'   \item **Candidate Motif Selection**: Depending on `use_real_occurrences`, the function either computes real occurrences and uses 
#'     actual frequencies and mean dissimilarities to select motifs, or it uses approximate measures for faster processing.
#'   \item **Motif Filtering**: Within each cluster, motifs are ranked based on their frequency and mean dissimilarity. Additional motifs 
#'     can be selected if their lengths differ sufficiently from the most frequent motif, as defined by `length_diff`.
#'   \item **Output Compilation**: The selected motifs and their associated properties are compiled into a comprehensive list for further analysis or visualization.
#' }
#'
#' @import parallel
#' @importFrom dplyr %>%
#' @importFrom data.table as.data.table setnames
#' @importFrom class knn
#' @importFrom fastcluster hclust
#' @importFrom Rcpp evalCpp
#' @export
motifs_search <- function(cluster_candidate_motifs_results,
                          R_all=cluster_candidate_motifs_results$R_all,R_m=NULL,different_R_m_finding=FALSE,R_m_finding=NULL,
                          use_real_occurrences=FALSE,length_diff=Inf,worker_number=NULL){
  # Find occurrences of the candidate motifs in the curves and sort them according to their frequencies and radius.
  # In each group (as defined by cutting the dendrogram at high 2*Rall), we choose the motif
  # with highest frequency and lower mean dissimilarity (the one ranking best in both dimensions).
  # Additional motifs can be chosen in a group, if their lengths differ enough from the length of the first motif chosen.
  # A candidate motif matches a piece of curve if their dissimilarity is less than the corresponding R_m.
  # cluster_candidate_motifs_results: output of cluster_candidate_motifs function.
  # R_all: global radius, used to cut the dendrogram (requiring groups to be more than 2*Rall apart).
  # R_m: vector with group-specific radii, used to find motif occurrences. The length of the vector must match the number of clusters
  #      obtained cutting the dendrogram at height 2*Rall. If NULL, Rm is determined in each group (based on distances between motifs of the same group and all curves).
  # different_R_m_finding: if TRUE, find the final occurrences using the R_m_finding radius instead of the R_m
  # R_m_finding: vector with group-specific radii, used to find motif occurrences if different_R_m_finding= TRUE. The length of the vector must match the number of clusters
  #      obtained cutting the dendrogram at height 2*Rall. If NULL, Rm for finding is determined in each group (based on distances between motifs of the same group and all curves).
  # use_real_occurrences: if TRUE, find occurrences for all candidate motifs and uses real frequency and mean dissimilarity to choose motifs
  #                       in groups (more accurate, but time consuming). Otherwise, uses approximate frequency and mean dissimilarity (default).
  # length_diff: minimum difference in length among motifs of the same group, required in ordered to keep more than one motif, in % of the most frequent motif.
  # worker_number: number of CPU cores to be used for parallelization (default number of CPU cores). If worker_number=1, the function is run sequentially.
  
  ### set parallel jobs ###################################################################################
  core_number <- detectCores()
  # check worker number
  if(!is.null(worker_number)){
    if(!is.numeric(worker_number)){
      warning('Worker number not valid. Selecting default number.')
      worker_number=NULL
    }else{
      if((worker_number%%1!=0)|(worker_number<1)|(worker_number>core_number)){
        warning('Worker number not valid. Selecting default number.')
        worker_number=NULL
      }
    }
  }
  if(is.null(worker_number))
    worker_number <- core_number-1
  rm(core_number)
  if(worker_number>1){
    cl_search=makeCluster(worker_number,timeout=60*60*24*30)
    on.exit(stopCluster(cl_search))
  }else{
    cl_search=NULL
  }
  
  ### prepare input data ##################################################################################
  if(cluster_candidate_motifs_results$diss=='d0_L2'){
    alpha=0
    use0=TRUE
    use1=FALSE
    Y=lapply(cluster_candidate_motifs_results$Y0,function(y0) list(y0=y0,y1=NULL))
    V=lapply(cluster_candidate_motifs_results$V0_clean,function(v0) list(v0=v0,v1=NULL))
  }
  if(cluster_candidate_motifs_results$diss=='d1_L2'){
    alpha=1
    use0=FALSE
    use1=TRUE
    Y=lapply(cluster_candidate_motifs_results$Y1,function(y1) list(y0=NULL,y1=y1))
    V=lapply(cluster_candidate_motifs_results$V1_clean,function(v1) list(v0=NULL,v1=v1))
  }
  if(cluster_candidate_motifs_results$diss=='d0_d1_L2'){
    alpha=cluster_candidate_motifs_results$alpha
    use0=TRUE
    use1=TRUE
    Y=mapply(function(y0,y1) list(y0=y0,y1=y1),cluster_candidate_motifs_results$Y0,cluster_candidate_motifs_results$Y1,SIMPLIFY=FALSE)
    V=mapply(function(v0,v1) list(v0=v0,v1=v1),cluster_candidate_motifs_results$V0_clean,cluster_candidate_motifs_results$V1_clean,SIMPLIFY=FALSE)
  }
  w=cluster_candidate_motifs_results$w
  transformed=cluster_candidate_motifs_results$transformed
  max_gap=cluster_candidate_motifs_results$max_gap
  d=ncol(cluster_candidate_motifs_results$Y0[[1]])
  N=nrow(cluster_candidate_motifs_results$D)
  K=ncol(cluster_candidate_motifs_results$D)
  V_dom=lapply(cluster_candidate_motifs_results$V0,function(v) rowSums(!is.na(v))!=0)
  V_length=unlist(lapply(V_dom,length))
  
  ### cut dendrogram and check group-specific radius Rm ##################################################
  V_hclust=cutree(cluster_candidate_motifs_results$hclust_res,h=2*R_all) # cut at high 2*R_all
  n_hclust=max(V_hclust)
  
  # re-compute or check group-specific radius Rm and Rm for finding
  if(is.null(R_m)){
    R_m=rep(NA,n_hclust)
    for(i_hclust in seq_len(n_hclust)){
      index_i=which(V_hclust==i_hclust) # motifs in cluster i
      V_length_i=V_length[index_i]
      V_P_i=as.matrix(cluster_candidate_motifs_results$P_clean[,index_i])
      V_D_i=as.matrix(cluster_candidate_motifs_results$D_clean[,index_i])
      
      dataR=data.frame(P=as.vector(V_P_i),D=as.vector(V_D_i))
      if(max(dataR$D[dataR$P==1])<=min(dataR$D[dataR$P==0])){
        R_m[i_hclust]=max(dataR$D[dataR$P==1])
      }else{
        D_new=seq(0,max(V_D_i),length.out=10000)
        pred_knn=class::knn(train=dataR$D,test=as.matrix(D_new),cl=dataR$P,k=cluster_candidate_motifs_results$k_knn,prob=TRUE)
        R_m[i_hclust]=D_new[which(ifelse(pred_knn==1,attributes(pred_knn)$prob,1-attributes(pred_knn)$prob)<=cluster_candidate_motifs_results$votes_knn_Rm)[1]]
      }
    }
  }
  if(length(R_m)!=n_hclust)
    stop(paste0('The length of the vector R_m must match the number of clusters: ',n_hclust))
  if(is.null(R_m_finding)){
    R_m_finding=rep(NA,n_hclust)
    for(i_hclust in seq_len(n_hclust)){
      index_i=which(V_hclust==i_hclust) # motifs in cluster i
      V_length_i=V_length[index_i]
      V_P_i=as.matrix(cluster_candidate_motifs_results$P_clean[,index_i])
      V_D_i=as.matrix(cluster_candidate_motifs_results$D_clean[,index_i])
      
      dataR=data.frame(P=as.vector(V_P_i),D=as.vector(V_D_i))
      if(max(dataR$D[dataR$P==1])<=min(dataR$D[dataR$P==0])){
        R_m_finding[i_hclust]=max(dataR$D[dataR$P==1])
      }else{
        D_new=seq(0,max(V_D_i),length.out=10000)
        pred_knn=class::knn(train=dataR$D,test=as.matrix(D_new),cl=dataR$P,k=cluster_candidate_motifs_results$k_knn,prob=TRUE)
        R_m_finding[i_hclust]=D_new[which(ifelse(pred_knn==1,attributes(pred_knn)$prob,1-attributes(pred_knn)$prob)<=cluster_candidate_motifs_results$votes_knn_Rm_finding)[1]]
      }
    }
  }
  if(length(R_m_finding)!=n_hclust)
    stop(paste0('The length of the vector R_m_finding must match the number of clusters: ',n_hclust))
  
  ### select candidate motifs and find occurrences #######################################################
  if(use_real_occurrences){
    ### find candidate motifs in the curves ####################################################################
    c_k=floor(V_length*(1-max_gap))
    c_k[c_k<cluster_candidate_motifs_results$c]=cluster_candidate_motifs_results$c[c_k<cluster_candidate_motifs_results$c]
    V_R_m=R_m[V_hclust]
    V_R_m_finding=R_m_finding[V_hclust]
    # find occurrences
    V_occurrences=.mapply_custom(cl_search,.find_occurrences,
                                 V,V_R_m,c_k,MoreArgs=list(Y=Y,alpha=alpha,w=w,use0,use1,transformed),SIMPLIFY=FALSE)
    not_null=which(!unlist(lapply(V_occurrences,is.null)))
    V=V[not_null]
    V_length=V_length[not_null]
    V_R_m=V_R_m[not_null]
    V_R_m_finding=V_R_m_finding[not_null]
    V_hclust=V_hclust[not_null]
    
    ### select candidate motifs in each group ##############################################################
    select=c()
    for(i_hclust in seq_len(n_hclust)){
      index_i=which(V_hclust==i_hclust) # motifs in cluster i
      
      V_frequencies_i=unlist(lapply(V_occurrences[index_i],nrow)) # real frequency
      V_mean_diss_i=unlist(lapply(V_occurrences[index_i],function(x) mean(x[,3]))) # real average distance (actual radius)
      # order based of frequency and average distance
      V_order_i=order(rank(-V_frequencies_i)+rank(V_mean_diss_i)) # sum of ranks in the two dimensions
      V_frequencies_i=V_frequencies_i[V_order_i]
      V_mean_diss_i=V_mean_diss_i[V_order_i]
      index_i_ordered=index_i[V_order_i]
      V_length_i=V_length[index_i_ordered]
      
      # select motifs to keep
      keep=rep_len(TRUE,length(index_i))
      for(i in seq_len(length(index_i))){
        if(keep[i]){
          select=c(select,index_i_ordered[i])
          keep[i]=FALSE
          # motifs with length different enough from length of selected motif
          length_diff_enough=keep&((V_length_i<(V_length_i[i]*(1-length_diff)))|(V_length_i>(V_length_i[i]*(1+length_diff))))
          keep[!length_diff_enough]=FALSE
        }
      }
    }
    V=V[select]
    V_length=V_length[select]
    V_R_m=V_R_m[select]
    V_R_m_finding=V_R_m_finding[select]
    
    if(!different_R_m_finding){
      V_occurrences=V_occurrences[select]
      V_frequencies=unlist(lapply(V_occurrences,nrow)) # real frequency
      V_mean_diss=unlist(lapply(V_occurrences,function(x) mean(x[,3]))) # real average distance (actual radius)
      # sort final motifs based of frequency and average distance
      V_order=order(rank(-V_frequencies)+rank(V_mean_diss)) # sum of ranks in the two dimensions
      V=V[V_order]
      V_occurrences=V_occurrences[V_order]
      V_length=V_length[V_order]
      V_R_m=V_R_m[V_order]
      V_R_m_finding=V_R_m_finding[V_order]
      V_frequencies=V_frequencies[V_order]
      V_mean_diss=V_mean_diss[V_order]
      
      index_final=not_null[select][V_order]
    }else{
      ### find candidate motifs in the curves ####################################################################
      c_k=floor(V_length*(1-max_gap))
      c_k[c_k<cluster_candidate_motifs_results$c[select]]=cluster_candidate_motifs_results$c[select][c_k<cluster_candidate_motifs_results$c[select]]
      # find occurrences
      V_occurrences=.mapply_custom(cl_search,.find_occurrences,
                                   V,V_R_m_finding,c_k,MoreArgs=list(Y=Y,alpha=alpha,w=w,use0,use1,transformed),SIMPLIFY=FALSE)
      not_null=which(!unlist(lapply(V_occurrences,is.null)))
      V=V[not_null]
      V_occurrences=V_occurrences[not_null]
      V_length=V_length[not_null]
      V_R_m=V_R_m[not_null]
      V_R_m_finding=V_R_m_finding[not_null]
      V_frequencies=unlist(lapply(V_occurrences,nrow)) # real frequency
      V_mean_diss=unlist(lapply(V_occurrences,function(x) mean(x[,3]))) # real average distance (actual radius)
      # sort final motifs based of frequency and average distance
      V_order=order(rank(-V_frequencies)+rank(V_mean_diss)) # sum of ranks in the two dimensions
      V=V[V_order]
      V_occurrences=V_occurrences[V_order]
      V_length=V_length[V_order]
      V_R_m=V_R_m[V_order]
      V_R_m_finding=V_R_m_finding[V_order]
      V_frequencies=V_frequencies[V_order]
      V_mean_diss=V_mean_diss[V_order]
      
      index_final=select[not_null][V_order]
    }
  }else{
    ### select candidate motifs in each group ##############################################################
    select=c()
    for(i_hclust in seq_len(n_hclust)){
      index_i=which(V_hclust==i_hclust) # motifs in cluster i
      V_D_i=as.matrix(cluster_candidate_motifs_results$D_clean[,index_i])
      
      V_frequencies_approx_i=colSums(V_D_i<=R_m[i_hclust]) # approximate frequency
      V_mean_diss_approx_i=apply(V_D_i,2,function(x) mean(x[x<=R_m[i_hclust]])) # approximate average distance (actual radius)
      # order based of frequency and average distance
      V_order_i=order(rank(-V_frequencies_approx_i)+rank(V_mean_diss_approx_i)) # sum of ranks in the two dimensions
      V_frequencies_approx_i=V_frequencies_approx_i[V_order_i]
      V_mean_diss_approx_i=V_mean_diss_approx_i[V_order_i]
      index_i_ordered=index_i[V_order_i]
      V_length_i=V_length[index_i_ordered]
      
      # select motifs to keep
      keep=rep_len(TRUE,length(index_i))
      for(i in seq_len(length(index_i))){
        if(keep[i]){
          select=c(select,index_i_ordered[i])
          keep[i]=FALSE
          # motifs with length different enough from length of selected motif
          length_diff_enough=keep&((V_length_i<(V_length_i[i]*(1-length_diff)))|(V_length_i>(V_length_i[i]*(1+length_diff))))
          keep[!length_diff_enough]=FALSE
        }
      }
    }
    V=V[select]
    V_length=V_length[select]
    V_R_m=R_m[V_hclust[select]]
    V_R_m_finding=R_m_finding[V_hclust[select]]
    
    ### find candidate motifs in the curves ####################################################################
    c_k=floor(V_length*(1-max_gap))
    c_k[c_k<cluster_candidate_motifs_results$c[select]]=cluster_candidate_motifs_results$c[select][c_k<cluster_candidate_motifs_results$c[select]]
    # find occurrences
    if(different_R_m_finding){
      V_occurrences=.mapply_custom(cl_search,.find_occurrences,
                                   V,V_R_m_finding,c_k,MoreArgs=list(Y=Y,alpha=alpha,w=w,use0,use1,transformed),SIMPLIFY=FALSE)
    }else{
      V_occurrences=.mapply_custom(cl_search,.find_occurrences,
                                   V,V_R_m,c_k,MoreArgs=list(Y=Y,alpha=alpha,w=w,use0,use1,transformed),SIMPLIFY=FALSE)
    }
    not_null=which(!unlist(lapply(V_occurrences,is.null)))
    V=V[not_null]
    V_occurrences=V_occurrences[not_null]
    V_length=V_length[not_null]
    V_R_m=V_R_m[not_null]
    V_R_m_finding=V_R_m_finding[not_null]
    V_frequencies=unlist(lapply(V_occurrences,nrow)) # real frequency
    V_mean_diss=unlist(lapply(V_occurrences,function(x) mean(x[,3]))) # real average distance (actual radius)
    # sort final motifs based of frequency and average distance
    V_order=order(rank(-V_frequencies)+rank(V_mean_diss)) # sum of ranks in the two dimensions
    V=V[V_order]
    V_occurrences=V_occurrences[V_order]
    V_length=V_length[V_order]
    V_R_m=V_R_m[V_order]
    V_R_m_finding=V_R_m_finding[V_order]
    V_frequencies=V_frequencies[V_order]
    V_mean_diss=V_mean_diss[V_order]
    
    index_final=select[not_null][V_order]
  }
  
  ### output ##################################################################################################
  if(cluster_candidate_motifs_results$diss=='d0_L2'){
    V0=lapply(V,function(v) v$v0)
    V1=cluster_candidate_motifs_results$V1_clean[index_final]
  }else if(cluster_candidate_motifs_results$diss=='d1_L2'){
    V0=cluster_candidate_motifs_results$V0_clean[index_final]
    V1=lapply(V,function(v) v$v1)
  }else{
    V0=lapply(V,function(v) v$v0)
    V1=lapply(V,function(v) v$v1)
  }
  if(different_R_m_finding){
    R_motifs=V_R_m_finding
  }else{
    R_motifs=V_R_m
  }
  return(list(V0=V0,V1=V1,
              V_length=V_length,V_occurrences=V_occurrences,V_frequencies=V_frequencies,V_mean_diss=V_mean_diss,
              Y0=cluster_candidate_motifs_results$Y0,Y1=cluster_candidate_motifs_results$Y1,R_motifs=R_motifs))
}

#' @title Wrapper for the Probabilistic K-means Algorithm (ProbKMA)
#'
#' @description This function serves as a wrapper for the Probabilistic K-means Algorithm (ProbKMA) to cluster functional data. It handles preprocessing, parameter setup, and execution of the core algorithm, returning the results along with silhouette analysis to assess the clustering quality.
#'
#' @param Y0 A matrix of functional data for the first set of observations.
#' @param Y1 A matrix of functional data for the second set of observations.
#' @param P0 A matrix representing the initial membership probabilities.
#' @param S0 A matrix representing the initial shift parameters.
#' @param standardize A logical value indicating whether to standardize the data. Default is `FALSE`.
#' @param c_max Maximum number of motifs to extract. Default is `Inf`.
#' @param iter_max Maximum number of iterations for the algorithm. Default is 1000.
#' @param iter4elong Number of iterations for elongation. Default is 10.
#' @param trials_elong Number of trials for elongation. Default is 10.
#' @param return_options A logical value indicating whether to return additional options. Default is `TRUE`.
#' @param alpha A numeric value representing the weighting parameter. Default is 0.
#' @param max_gap Maximum allowable gap between motifs. Default is 0.2.
#' @param quantile Quantile to be used for cleaning. Default is 0.25.
#' @param stopCriterion Stopping criterion for the algorithm, can be 'max' or other specified values. Default is 'max'.
#' @param tol Tolerance for convergence. Default is 1e-8.
#' @param tol4elong Tolerance for elongation iterations. Default is 1e-3.
#' @param max_elong Maximum elongation allowed. Default is 0.5.
#' @param deltaJK_elong Increment for the elongation. Default is 0.05.
#' @param iter4clean Number of iterations for the cleaning process. Default is 50.
#' @param tol4clean Tolerance for the cleaning process. Default is 1e-4.
#' @param m Parameter controlling the clustering behavior. Default is 2.
#' @param w Weighting parameter for the dissimilarity measure. Default is 1.
#' @param seed Random seed for reproducibility. Default is 1.
#' @param K Number of motifs to extract. Default is 2.
#' @param c Minimum motif length. Default is 40.
#' @param quantile4clean Quantile used for the cleaning process. Default is 1/K.
#' @param exe_print A logical value indicating whether to print execution details. Default is `FALSE`.
#' @param set_seed A logical value indicating whether to set the random seed. Default is `FALSE`.
#' @param diss Dissimilarity measure to be used. Default is 'd0_2'.
#' @param transformed A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.
#' @param V_init Initial values for the motifs. Default is `NULL`.
#' @param align A logical value indicating whether to align the curves. Default is `TRUE`.
#' @param n_threads Number of threads to use for parallel processing. Default is 1.
#'
#' @return A list containing:
#' \item{probKMA_results}{A list of results from the ProbKMA algorithm, including processed functional data and model parameters.}
#' \item{silhouette_results}{Results from silhouette analysis, indicating the quality of the clustering.}
#'
#' @export
probKMA_wrap <- function(Y0 = NULL,Y1 = NULL,P0 = matrix(),S0 = matrix(),
                         standardize= FALSE,c_max = Inf,iter_max = 1000,
                         iter4elong = 10,trials_elong = 10,return_options = TRUE,
                         alpha = 0,max_gap = 0.2,quantile = 0.25, stopCriterion = 'max', 
                         tol = 1e-8, tol4elong = 1e-3, max_elong = 0.5, deltaJK_elong = 0.05, 
                         iter4clean = 50, tol4clean = 1e-4,m = 2,w = 1, seed = 1, 
                         K = 2, c = 40, quantile4clean = 1/K, exe_print = FALSE,
                         set_seed = FALSE,diss = 'd0_2', 
                         transformed = FALSE, V_init = NULL,align = TRUE,n_threads = 1){
  params = list(standardize=standardize,c_max = c_max,iter_max = iter_max,
                iter4elong = iter4elong,trials_elong = trials_elong,
                return_options = return_options, alpha = alpha,
                max_gap = max_gap,quantile = quantile, 
                stopCriterion = stopCriterion, tol = tol, 
                tol4elong = tol4elong, max_elong = max_elong, 
                deltaJK_elong = deltaJK_elong, iter4clean = iter4clean, 
                tol4clean = tol4clean,quantile4clean = quantile4clean, 
                m = m, w = w, seed = seed, K = K, c = c, exe_print = exe_print,
                set_seed = set_seed, 
                transformed = transformed, V_init = V_init,n_threads = n_threads) 
  checked_data <- initialChecks(Y0,Y1,P0,S0,params,diss,V_init)
  params <- checked_data$Parameters
  
  data <- checked_data$FuncData
  string_diss <- ifelse(alpha == 0 || alpha == 1,"L2","H1")
  prok <- NULL
  if (!is.null(data$V_init)) {
    prok <- new(ProbKMA, data$Y, params, data$P0, data$S0, string_diss, data$V_init)
  } else {
    prok <- new(ProbKMA, data$Y, params, data$P0, data$S0, string_diss)
  }
  probKMA_results_1 = list(Y0 = data$Y$Y0,
                           Y1 = data$Y$Y1,
                           diss = diss, w = w, 
                           alpha = alpha, 
                           V_init = data$V_init, 
                           transformed = transformed)
  
  probKMA_results_2 = prok$probKMA_run() 
  sil <- prok$compute_silhouette(align)
  
  rm(params)
  rm(data)
  rm(prok)
  return(list(probKMA_results = c(probKMA_results_1,probKMA_results_2), silhouette_results = sil))

}

#' @title Create motifSimulation Object
#' @description It represents the constructor of the S4 class 'motifSimulation'..
#' @param N An integer specifying the number of background curves to be generated (mandatory).
#' @param len An integer specifying the length of the background curves (mandatory).
#' @param mot_details A list outlining the definitions of the motifs to be included. Each motif is characterized by its length, a set of coefficients that may be optionally specified, and the number of occurrences. These occurrences can be indicated either by specific positions within the curves or by a total count. In the latter case, the algorithm will randomly position the motifs throughout the curves (mandatory).
#' @param norder An integer specifying the order of the B-splines (default = 3).
#' @param coeff_min Additive coefficients to be incorporated into the generation of coefficients for the background curves (default = -15).
#' @param coeff_max Additive coefficients to be incorporated into the generation of coefficients for the background curves (default = 15).
#' @param dist_knots An integer specifying the distance between two consecutive knots (default = 10).
#' @param min_dist_motifs An integer specifying the minimum distance between two consecutive motifs embedded in the same curve (default = 'norder' * 'dist_knots').
#' @param distribution A character string specifying the distribution from which the coefficients of the background curves are generated. You can choose between a uniform distribution or a beta distribution. Alternatively, you can pass a vector representing the empirical distribution from which you wish to sample (default = "beta").
#' @param motif_seed Set the seed for motif b-spline coefficients generation (default = NULL, i.e., random) 
#' @return An object of class motifSimulation
#' @export
#' 
#' @importFrom stats start
#' @examples
#' \donttest{
#' mot_len <- 100
#' motif_str <- rbind.data.frame(c(1, 1, 20),
#'                               c(2, 1, 2), 
#'                               c(1, 3, 1),
#'                               c(1, 2, 1),
#'                               c(1, 2, 15),
#'                               c(1, 4, 1),
#'                               c(2, 5, 1),
#'                               c(2, 7, 1),
#'                               c(2,17,1))
#' 
#' names(motif_str) <- c("motif_id", "curve","start_break_pos")
#' 
#' mot1 <- list("len" = mot_len, #length
#'              "coeffs" = NULL, # weights for the motif
#'              "occurrences" = motif_str %>% filter(motif_id == 1))
#' 
#' mot2 <- list("len" = mot_len,
#'              "coeffs" = NULL,
#'              "occurrences" = motif_str %>% filter(motif_id == 2))
#' 
#' mot_details <- list(mot1,mot2)
#' 
#' # MATRIX ERROR 
#' noise_str <- list(rbind(rep(2, 100)),
#'                   rbind(rep(0.0, 100)))
#' 
#' builder <- funMoDisco::motifSimulationBuilder(N = 20,len = 300, mot_details,
#'                                         distribution = 'beta')
#' }
motifSimulationBuilder <- function(N,
                                   len,
                                   mot_details,
                                   norder = 3,
                                   coeff_min = -15,
                                   coeff_max = 15,
                                   dist_knots = 10,
                                   min_dist_motifs = NULL,
                                   distribution = 'beta',
                                   motif_seed = NULL) {
  ########################## CHECKS #################################
  
  # set minimum distance between two consecutive motifs
  if(is.null(min_dist_motifs)) {
    min_dist_motifs <- norder * dist_knots
  }
  
  # check N, dist_knots and len
  if((N%%1 != 0) | (N < 1))
    stop('Invalid \'N\'.')
  
  # case with no motifs
  if(length(mot_details) == 0) {
    
    mot1 <- list(
       "len" = 10, # motif length
       "coeffs" = NULL, # B-spline weight
       "occurrences" = 1 # occurrences
    )
    
   mot_details <- list(mot1)
  }
    
  # if(length(mot_details) == 0) {
  #   return(new("motifSimulation",N = N,
  #              mot_details = list(),
  #              motifs_in_curves = list(),
  #              distribution = distribution,
  #              dist_knots=dist_knots,len=len,norder=norder,
  #              coeff_min=coeff_min,coeff_max=coeff_max,
  #              min_dist_motifs=min_dist_motifs))
  # }
  
  if(( dist_knots%%1 != 0) | (dist_knots < 1))
    stop('Invalid \'dist_knots\'.')
  
  if(TRUE %in% ((len %% 1 != 0) | (len < 1) | (sum(len %% dist_knots) > 0)))
    stop('Invalid \'len\'.')
  
  mot_details <- lapply(mot_details, 
                        function(mot){
                          names(mot) <- c("len","coeffs","occurrences")
                          if(is.data.frame(mot[[3]]) || is.matrix(mot[[3]])) { 
                            names(mot[[3]]) <- c("motif_id", "curve","start_break_pos") }
                          return(mot)
                        }
  )
  
  # check if the coeff for the motifs are defined
  weights_defined <- sapply(mot_details, 
                            function(x) !is.null(x$coeffs))
  
  # stop if some are defined and others are not
  if (!(all(weights_defined) || all(!weights_defined))) {
    stop("Inconsistent coefficients field: some elements have it defined, others do not.")
  }
  
  # get motifs length
  len_motifs <- sapply(mot_details,
                       function(x){x$len},
                       simplify = TRUE)
  
  # get number of motifs
  nmotifs <- length(mot_details)
  
  # if all the motifs have coefficients defined we can use them
  if(TRUE %in% weights_defined) {
    for(i in 1:nmotifs){
      nbasis <- len_motifs[i]/dist_knots + norder - 1
      mot_details[[i]]$coeffs <- mot_details[[i]]$coeffs[1:nbasis]
    }
  }
  
  # check nmotifs and len_motifs
  if( (nmotifs %% 1 != 0) | (nmotifs < 1) )
    stop('Invalid \'nmotifs\'.')
  
  if(TRUE %in% ((len_motifs %% 1 != 0) | (len_motifs < 1) | (sum(len_motifs %% dist_knots) > 0) | (TRUE %in% (len_motifs > len))))
    stop('Invalid \'len_motifs\'.')
  
  if (FALSE %in% unlist(lapply(mot_details,
                               function(x) {
                                 if (all(weights_defined)) {
                                   x$len == (length(x$coeffs) - norder + 1)*dist_knots
                                 } else {
                                   TRUE
                                 }
                               }
  )
  )
  ) {
    stop("The length of the motifs is incompatible")
  }
  
  # check min_dist_motifs
  if((min_dist_motifs < norder*dist_knots) | (min_dist_motifs %% dist_knots > 0))
    stop('Invalid \'min_dist_motifs\'.')
  # check freq_motifs and convert it into a list with the frequencies for the different motifs in the different curves
  
  freq_motifs_vector <- sapply(mot_details,
                               function(mot_i){
                                 ifelse(is.data.frame(mot_i$occurrences),
                                        dim(mot_i$occurrences['curve'])[1],NA)
                               }
  )
  
  is_appearance_defined <- !(NA %in% freq_motifs_vector)
  if(sum(is.na(freq_motifs_vector)) != 0 && sum(is.na(freq_motifs_vector)) != nmotifs) {
    stop("Inconsistent appearance field: some elements have it defined, others do not.")
  }
  
  if(is_appearance_defined) {
    freq_motifs_vector <- freq_motifs_vector[!is.na(freq_motifs_vector)]
    if(TRUE %in% ((freq_motifs_vector%%1!=0) | (freq_motifs_vector<1) | (sum(norder-1+rep(len/dist_knots,length.out=N)-2*(norder-1)) < (sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_vector,length.out=nmotifs)) + max(0,sum(rep(freq_motifs_vector, length.out = nmotifs))-N)*(min_dist_motifs/dist_knots-norder+1)))))
      stop('Invalid \'freq_motifs\'.')
    
    motif_str_list <- lapply(mot_details, function(x) {
      df <- x$occurrences
      len <- x$len
      df$len <- len  # Add the length to each motif dataframe
      return(df)
    })
    motif_str_new <- do.call(rbind, motif_str_list)
    
    # Convert columns to numeric if necessary
    motif_str_new[] <- lapply(motif_str_new, as.numeric)
    
    # Add columns for start and end positions
    motif_str_new <- motif_str_new %>%
      mutate(
        start = (start_break_pos - 1) * dist_knots,
        end = start + len
      )
    # Apply the check for each curve
    valid_curves <- mapply(function(sub_data) {
      .check_fits(sub_data, min_dist_motifs = min_dist_motifs,len = len)
    },
    split(motif_str_new, motif_str_new$curve), 
    SIMPLIFY = TRUE)
    
    # Check if all curves have valid motifs
    if(FALSE %in% valid_curves) {
      stop('Invalid position of the knots.')
    }
  }
  
  freq_check=1
  it=0
  # Compute the frequency(if not defined) of the motifs inside the curves 
  if(!is_appearance_defined) {
    while((sum(freq_check)>0) && (it<10000)){
      it=it+1
      freq_motifs <- matrix(data = 0,nrow = N,ncol = nmotifs)
      for(motif_j in 1:nmotifs) {
        curves <- as.vector(sample(N,mot_details[[motif_j]]$occurrences,replace=TRUE))
        for (curve in curves) {
          freq_motifs[curve, motif_j] <- freq_motifs[curve, motif_j] + 1
        }
      }
      freq_motifs=split(freq_motifs,rep(1:N,nmotifs))# construct N list(for each curve). Each list_j contains how many times the motif_i appears in the curve j
      freq_check=mapply(function(freq_motifs_i,len_i) ((len_i-2*(norder-1))<(sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_i,length.out=nmotifs))+(sum(freq_motifs_i)-1)*(min_dist_motifs/dist_knots-norder+1))),
                        freq_motifs,norder-1+rep(len/dist_knots,length.out=N))
    }
    if(it == 10000)
      stop('Tried 10.000 random configurations of motif frequencies in the different curves, unable to find a valid configuration. Please select lower \'freq_motifs\'.')
  } else {
    freq_motifs <- matrix(data = 0,nrow = N,ncol = nmotifs)
    for(motif_j in 1:nmotifs) {
      for(curve_i in 1:N){
        count <- sum(mot_details[[motif_j]]$occurrences$curve == curve_i)
        freq_motifs[curve_i, motif_j] <-  freq_motifs[curve_i, motif_j] + count
      }
    }
    freq_motifs=split(freq_motifs,rep(1:N,nmotifs))
    ifelse(TRUE %in% mapply(function(freq_motifs_i,len_i) ((len_i-2*(norder-1))<(sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_i,length.out=nmotifs))+(sum(freq_motifs_i)-1)*(min_dist_motifs/dist_knots-norder+1))),
                            freq_motifs,norder-1+rep(len/dist_knots,length.out=N)),stop('Please select lower \'freq_motifs\'.'),"")
  }
  
  ########################## POSITION ASSIGNMENT ###############################
  # randomly assign motif positions in curves if they are not already assigned
  # At this point we only have the coefficients of the motifs
  motifs_in_curves <- vector("list", N)
  if (!is_appearance_defined) {
    # Initialize appearance for each motif
    for (mot in 1:nmotifs) {
      mot_details[[mot]]$occurrences <- data.frame(motif_id = integer(0),
                                                   curve = integer(0),
                                                   start_break_pos = integer(0),
                                                   coeff_pos = integer(0))
    }
    
    motifs_in_curves <- mapply(function(freq_motifs_i, len_i, curve_i) {
      if (sum(freq_motifs_i) == 0) {
        return(NULL) # No motifs embedded in this curve
      }
      
      id_motifs <- .resample(rep(seq_along(freq_motifs_i), freq_motifs_i))
      len_elements <- c(norder - 1, rep(len_motifs / dist_knots + norder - 1, length.out = nmotifs)[id_motifs] + c(rep((min_dist_motifs / dist_knots - norder + 1), sum(freq_motifs_i) - 1), 0), norder - 1)
      gaps_tot <- len_i - sum(len_elements) # number of free coefficients
      gaps <- diff(c(0, sort(sample(gaps_tot + sum(freq_motifs_i), sum(freq_motifs_i))))) - 1
      coeff_pos <- cumsum(len_elements[seq_along(id_motifs)]) + 1 + cumsum(gaps) # Starting coefficient
      
      for (mot in unique(id_motifs)) {
        indices <- which(id_motifs == mot)
        df <- data.frame(
          motif_id = mot,
          curve = rep(curve_i, length(indices)),
          start_break_pos = coeff_pos[indices] # coincide with the coeffs position
        )
        mot_details[[mot]]$occurrences <<- rbind(mot_details[[mot]]$occurrences, df)
      }
      
      return(list(motif_id=id_motifs,starting_coeff_pos=coeff_pos)) # returns N list(one for each curve) with the id of the motif embedded and the starting position
    }, freq_motifs, norder - 1 + rep(len / dist_knots, length.out = N), 1:N,SIMPLIFY = FALSE)
  } else {
    splited_curves <- split(motif_str_new, motif_str_new$curve)
    names(motifs_in_curves) <- as.character(1:N)
    for (name in as.character(1:N)) {
      if (name %in% names(splited_curves)) {
        motifs_in_curves[[name]] <- list(motif_id = splited_curves[[name]]$motif_id,starting_coeff_pos = splited_curves[[name]]$start_break_pos)
      }
    }
  }
  
  ########################## WEIGHTS GENERATION #################################
  # if weights are not provided, randomly generate them according to the distribution
  # user can set seed (motif_seed) to ensure the same motif coefficients
  
  if(all(!weights_defined)) {
    
    # set the seed
    if(is.null(motif_seed)){
      my_motif_seed <- vector("list", length(mot_details))
    } else {
      my_motif_seed <- rep(motif_seed, len = length(mot_details)) + seq(0, length(mot_details) -1)
    }
    
    mot_details <- mapply(.generate_coefficients, 
                          mot_details,
                          motif_seed = my_motif_seed,
                          MoreArgs = list(distrib = distribution,
                                          dist_knots = dist_knots,
                                          norder = norder, 
                                          coeff_min = coeff_min,
                                          coeff_max = coeff_max),
                          SIMPLIFY = FALSE)
  }
  
  ########################## RETURN #################################
  return(new("motifSimulation",
             N = N,
             mot_details = mot_details,
             motifs_in_curves = motifs_in_curves,
             distribution = distribution,
             dist_knots=dist_knots,len=len,norder=norder,
             coeff_min=coeff_min,coeff_max=coeff_max,
             min_dist_motifs=min_dist_motifs)
  )
}


# PREVIOUS VERSION ------
# motifSimulationBuilder <- function(N,len,mot_details,norder = 3,
#                                    coeff_min=-15,coeff_max=15,
#                                    dist_knots=10,min_dist_motifs=NULL,
#                                    distribution = 'beta') {
#   ########################## CHECKS #################################
#   if(is.null(min_dist_motifs)) {
#     min_dist_motifs <- norder * dist_knots
#   }
#   # check N, dist_knots and len
#   if((N%%1!=0)|(N<1))
#     stop('Invalid \'N\'.')
#   if(length(mot_details) == 0) {
#     return(new("motifSimulation",N = N,
#                mot_details = list(),
#                motifs_in_curves = list(),
#                distribution = distribution,
#                dist_knots=dist_knots,len=len,norder=norder,
#                coeff_min=coeff_min,coeff_max=coeff_max,
#                min_dist_motifs=min_dist_motifs))
#   }
#   if((dist_knots%%1!=0)|(dist_knots<1))
#     stop('Invalid \'dist_knots\'.')
#   if(TRUE %in% ((len%%1!=0)|(len<1)|(sum(len%%dist_knots)>0)))
#     stop('Invalid \'len\'.')
#   mot_details <- lapply(mot_details,function(mot){names(mot) <- c("len","coeffs","occurrences")
#                                                   if(is.data.frame(mot[[3]]) || is.matrix(mot[[3]])) { 
#                                                     names(mot[[3]]) <- c("motif_id", "curve","start_break_pos") }
#                                                   return(mot)})
# 
#   weights_defined <- sapply(mot_details, function(x) !is.null(x$coeffs))
#   if (!(all(weights_defined) || all(!weights_defined))) {
#     stop("Inconsistent coefficients field: some elements have it defined, others do not.")
#   }
#   
#   len_motifs <- sapply(mot_details,function(x){x$len},simplify = TRUE)
#   nmotifs <- length(mot_details)
#   if(TRUE %in% weights_defined) {
#     for(i in 1:nmotifs){
#       nbasis <- len_motifs[i]/dist_knots + norder - 1
#       mot_details[[i]]$coeffs <- mot_details[[i]]$coeffs[1:nbasis]
#       
#     }
#   }
#   # check nmotifs and len_motifs
#   if((nmotifs%%1!=0)|(nmotifs<1))
#     stop('Invalid \'nmotifs\'.')
#   if(TRUE %in% ((len_motifs%%1!=0)|(len_motifs<1)|(sum(len_motifs%%dist_knots)>0)|(TRUE %in% (len_motifs>len))))
#     stop('Invalid \'len_motifs\'.')
#   
#   if (FALSE %in% unlist(lapply(mot_details, function(x) {
#     if (all(weights_defined)) {
#       x$len == (length(x$coeffs)-norder+1)*dist_knots
#     } else {
#       TRUE
#     }
#   }))) {
#     stop("The length of the motifs is incompatible")
#   }
#   # check min_dist_motifs
#   if((min_dist_motifs < norder*dist_knots)|(min_dist_motifs%%dist_knots>0))
#     stop('Invalid \'min_dist_motifs\'.')
#   # check freq_motifs and convert it into a list with the frequencies for the different motifs in the different curves
#   
#   freq_motifs_vector <- sapply(mot_details,function(mot_i){ifelse(is.data.frame(mot_i$occurrences),dim(mot_i$occurrences['curve'])[1],NA)})
#   is_appearance_defined <- !(NA %in% freq_motifs_vector)
#   if(sum(is.na(freq_motifs_vector)) != 0 && sum(is.na(freq_motifs_vector)) != nmotifs) {
#     stop("Inconsistent appearance field: some elements have it defined, others do not.")
#   }
#   if(is_appearance_defined) {
#     freq_motifs_vector <- freq_motifs_vector[!is.na(freq_motifs_vector)]
#     if(TRUE %in% ((freq_motifs_vector%%1!=0)|(freq_motifs_vector<1)|(sum(norder-1+rep(len/dist_knots,length.out=N)-2*(norder-1))<(sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_vector,length.out=nmotifs))+max(0,sum(rep(freq_motifs_vector, length.out = nmotifs))-N)*(min_dist_motifs/dist_knots-norder+1)))))
#       stop('Invalid \'freq_motifs\'.')
#     motif_str_list <- lapply(mot_details, function(x) {
#       df <- x$occurrences
#       len <- x$len
#       df$len <- len  # Add the length to each motif dataframe
#       return(df)
#     })
#     motif_str_new <- do.call(rbind, motif_str_list)
#     
#     # Convert columns to numeric if necessary
#     motif_str_new[] <- lapply(motif_str_new, as.numeric)
#     
#     # Add columns for start and end positions
#     motif_str_new <- motif_str_new %>%
#       mutate(
#         start = (start_break_pos - 1) * dist_knots,
#         end = start + len
#       )
#     # Apply the check for each curve
#     valid_curves <- mapply(function(sub_data) {
#                             .check_fits(sub_data, min_dist_motifs = min_dist_motifs,len = len)
#                           },
#                           split(motif_str_new, motif_str_new$curve), 
#                           SIMPLIFY = TRUE)
#     
#     # Check if all curves have valid motifs
#     if(FALSE %in% valid_curves) {
#       stop('Invalid position of the knots.')
#     }
#   }
#   
#   freq_check=1
#   it=0
#   # Compute the frequency(if not defined) of the motifs inside the curves 
#   if(!is_appearance_defined) {
#     while((sum(freq_check)>0)&&(it<10000)){
#       it=it+1
#       freq_motifs <- matrix(data = 0,nrow = N,ncol = nmotifs)
#       for(motif_j in 1:nmotifs) {
#         curves <- as.vector(sample(N,mot_details[[motif_j]]$occurrences,replace=TRUE))
#         for (curve in curves) {
#           freq_motifs[curve, motif_j] <- freq_motifs[curve, motif_j] + 1
#         }
#       }
#       freq_motifs=split(freq_motifs,rep(1:N,nmotifs))# construct N list(for each curve). Each list_j contains how many times the motif_i appears in the curve j
#       freq_check=mapply(function(freq_motifs_i,len_i) ((len_i-2*(norder-1))<(sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_i,length.out=nmotifs))+(sum(freq_motifs_i)-1)*(min_dist_motifs/dist_knots-norder+1))),
#                         freq_motifs,norder-1+rep(len/dist_knots,length.out=N))
#     }
#     if(it==10000)
#       stop('Tried 10.000 random configurations of motif frequencies in the different curves, unable to find a valid configuration. Please select lower \'freq_motifs\'.')
#   }else {
#     freq_motifs <- matrix(data = 0,nrow = N,ncol = nmotifs)
#     for(motif_j in 1:nmotifs) {
#       for(curve_i in 1:N){
#         count <- sum(mot_details[[motif_j]]$occurrences$curve == curve_i)
#         freq_motifs[curve_i, motif_j] <-  freq_motifs[curve_i, motif_j] + count
#       }
#     }
#     freq_motifs=split(freq_motifs,rep(1:N,nmotifs))
#     ifelse(TRUE %in% mapply(function(freq_motifs_i,len_i) ((len_i-2*(norder-1))<(sum(rep(len_motifs/dist_knots+norder-1,length.out=nmotifs)*rep(freq_motifs_i,length.out=nmotifs))+(sum(freq_motifs_i)-1)*(min_dist_motifs/dist_knots-norder+1))),
#                             freq_motifs,norder-1+rep(len/dist_knots,length.out=N)),stop('Please select lower \'freq_motifs\'.'),"")
#   }
#   
#   ########################## POSITION ASSIGNMENT ###############################
#   # randomly assign motif positions in curves if they are not already assigned
#   # At this point we only have the coefficients of the motifs
#   motifs_in_curves <- vector("list", N)
#   if (!is_appearance_defined) {
#     # Initialize appearance for each motif
#     for (mot in 1:nmotifs) {
#       mot_details[[mot]]$occurrences <- data.frame(motif_id = integer(0),
#                                                   curve = integer(0),
#                                                   start_break_pos = integer(0),
#                                                   coeff_pos = integer(0))
#     }
#     
#     motifs_in_curves <- mapply(function(freq_motifs_i, len_i, curve_i) {
#       if (sum(freq_motifs_i) == 0) {
#         return(NULL) # No motifs embedded in this curve
#       }
#       
#       id_motifs <- .resample(rep(seq_along(freq_motifs_i), freq_motifs_i))
#       len_elements <- c(norder - 1, rep(len_motifs / dist_knots + norder - 1, length.out = nmotifs)[id_motifs] + c(rep((min_dist_motifs / dist_knots - norder + 1), sum(freq_motifs_i) - 1), 0), norder - 1)
#       gaps_tot <- len_i - sum(len_elements) # number of free coefficients
#       gaps <- diff(c(0, sort(sample(gaps_tot + sum(freq_motifs_i), sum(freq_motifs_i))))) - 1
#       coeff_pos <- cumsum(len_elements[seq_along(id_motifs)]) + 1 + cumsum(gaps) # Starting coefficient
#       
#       for (mot in unique(id_motifs)) {
#         indices <- which(id_motifs == mot)
#         df <- data.frame(
#           motif_id = mot,
#           curve = rep(curve_i, length(indices)),
#           start_break_pos = coeff_pos[indices] # coincide with the coeffs position
#         )
#         mot_details[[mot]]$occurrences <<- rbind(mot_details[[mot]]$occurrences, df)
#       }
#       
#       return(list(motif_id=id_motifs,starting_coeff_pos=coeff_pos)) # returns N list(one for each curve) with the id of the motif embedded and the starting position
#     }, freq_motifs, norder - 1 + rep(len / dist_knots, length.out = N), 1:N,SIMPLIFY = FALSE)
#   } else {
#     splited_curves <- split(motif_str_new, motif_str_new$curve)
#     names(motifs_in_curves) <- as.character(1:N)
#     for (name in as.character(1:N)) {
#       if (name %in% names(splited_curves)) {
#         motifs_in_curves[[name]] <- list(motif_id = splited_curves[[name]]$motif_id,starting_coeff_pos = splited_curves[[name]]$start_break_pos)
#       }
#     }
#   }
#   
#   ########################## WEIGHTS GENERATION #################################
#   # if weights are not provided, randomly generate them according to the distribution
#   if(all(!weights_defined)) {
#     mot_details <- mapply(.generate_coefficients, mot_details,
#                           MoreArgs = list(distrib = distribution,
#                                           dist_knots = dist_knots,
#                                           norder = norder, 
#                                           coeff_min = coeff_min,
#                                           coeff_max = coeff_max,
#                           ),
#                           SIMPLIFY = FALSE)
#   }
#   
#   ########################## RETURN #################################
#   return(new("motifSimulation",N = N,
#              mot_details = mot_details,
#              motifs_in_curves = motifs_in_curves,
#              distribution = distribution,
#              dist_knots=dist_knots,len=len,norder=norder,
#              coeff_min=coeff_min,coeff_max=coeff_max,
#              min_dist_motifs=min_dist_motifs))
# }

#' @title Functional Motif Discovery
#'
#' @description
#' The `discoverMotifs` function facilitates the discovery of recurring patterns, or motifs, within functional data by employing two sophisticated algorithms: 
#' \code{ProbKMA} (Probabilistic K-means with Local Alignment) and \code{funBIalign}. 
#' These algorithms are designed to identify and cluster functional motifs across multiple curves, leveraging advanced clustering and alignment techniques to handle complex data structures.
#'
#' \code{ProbKMA} integrates probabilistic clustering with local alignment strategies, enabling the detection of motifs that exhibit variability in both shape and position across different curves. 
#' This method is particularly adept at handling noisy data and motifs that may appear at varying scales or locations within the curves.
#'
#' On the other hand, \code{funBIalign} utilizes hierarchical clustering based on mean squared residue scores to uncover motifs. 
#' This approach effectively captures the additive nature of functional motifs, considering both portion-specific adjustments and time-varying components to accurately identify recurring patterns.
#'
#' By providing a flexible interface that accommodates different clustering paradigms, `discoverMotifs` empowers users to perform robust motif discovery tailored to their specific data characteristics and analytical requirements. 
#' Whether opting for the probabilistic and alignment-focused \code{ProbKMA} or the hierarchical and residue-based \code{funBIalign}, users can leverage these methods to extract meaningful and interpretable motifs from their functional datasets.
#'
#' @details
#' The `discoverMotifs` function dynamically switches between two advanced motif discovery algorithms based on the user's specification. 
#' Each algorithm employs distinct strategies to identify and cluster motifs within functional data, offering flexibility and adaptability to various analytical scenarios.
#'
#' @section Theoretical Background for ProbKMA:
#' 
#' \code{ProbKMA} is inspired by methodologies prevalent in bioinformatics, particularly those involving local alignment techniques extended from high-similarity seeds. 
#' This algorithm combines fuzzy clustering approaches with local alignment strategies to effectively minimize a generalized least squares functional. 
#' The minimization process can incorporate both the levels and derivatives of the curves through a Sobolev-based distance metric, enhancing the algorithm's sensitivity to both shape and rate changes in the data.
#' 
#' Throughout its iterative process, \code{ProbKMA} refines motif centers, membership probabilities, and alignment shifts, making it highly effective for capturing complex motif structures and motifs distributed across multiple curves. 
#' This ensures that the discovered motifs are both representative and robust against variations and noise within the functional data.
#'
#' @section Theoretical Background for funBIalign:
#' 
#' \code{funBIalign} models functional motifs as an additive combination of motif means, portion-specific adjustments, and time-varying components. 
#' The algorithm constructs a hierarchical dendrogram utilizing the generalized mean squared residue score (fMSR) to identify candidate motifs across curves.
#' 
#' A critical aspect of \code{funBIalign} is its post-processing step, which filters out redundant motifs and refines the final selection to ensure that only the most significant and representative motifs are retained. 
#' This hierarchical approach allows for a nuanced identification of motifs, capturing both broad and subtle patterns within the data.
#'
#' @section Common Parameters:
#' The following parameters are common to both \code{ProbKMA} and \code{funBIalign} algorithms:
#' \describe{
#'   \item{\code{Y0}}{A list containing \code{N} vectors (for univariate curves) or \code{N} matrices (for multivariate curves) representing the functional data. Each curve is evaluated on a uniform grid, ensuring consistency across the dataset.}
#'   \item{\code{method}}{A character string specifying the motif discovery algorithm to use. Acceptable values are \code{"ProbKMA"} for Probabilistic K-means with Local Alignment and \code{"funBIalign"} for Functional Bi-directional Alignment.}
#'   \item{\code{stopCriterion}}{A character string indicating the convergence criterion for the selected algorithm. For \code{ProbKMA}, options include \code{"max"}, \code{"mean"}, or \code{"quantile"} based on the Bhattacharyya distance between memberships in successive iterations. For \code{funBIalign}, options are \code{"fMRS"} (functional Mean Squared Residue) or \code{"Variance"} to guide the ranking of motifs.}
#'   \item{\code{name}}{A character string specifying the name of the output directory where results will be saved. This facilitates organized storage and easy retrieval of analysis results.}
#'   \item{\code{plot}}{A logical value indicating whether to generate and save plots of the discovered motifs and clustering results. When set to \code{TRUE}, visualizations are produced to aid in the qualitative assessment of the motif discovery process.}
#'   \item{\code{worker_number}}{An integer specifying the number of CPU cores to utilize for parallel computations. By default, the function uses the total number of available cores minus one, optimizing computational efficiency without overloading the system.}
#' }
#'
#' @section ProbKMA Options:
#' The following parameters are specific to the \code{ProbKMA} algorithm:
#' \describe{
#'   \item{\code{K}}{An integer or vector specifying the number of motifs to be discovered. It can be a single integer for uniform motif discovery or a vector for specifying different numbers of motifs.}
#'   \item{\code{c}}{An integer or vector indicating the minimum motif lengths. This ensures that each discovered motif meets a specified minimum length requirement, maintaining the integrity of motif structures.}
#'   \item{\code{c_max}}{An integer or vector specifying the maximum motif lengths, allowing control over the upper bounds of motif sizes to prevent excessively long motifs.}
#'   \item{\code{diss}}{A character string defining the dissimilarity measure to use. Possible values include \code{"d0_L2"}, \code{"d1_L2"}, and \code{"d0_d1_L2"}, which determine how the algorithm quantifies differences between motifs based on level and derivative information.}
#'   \item{\code{alpha}}{A numeric value between 0 and 1 that serves as a weight parameter between \code{d0_L2} and \code{d1_L2} when using \code{d0_d1_L2}. An \code{alpha} of 0 emphasizes \code{d0_L2}, while an \code{alpha} of 1 emphasizes \code{d1_L2}, allowing for balanced consideration of both metrics.}
#'   \item{\code{w}}{A numeric vector specifying the weight for the dissimilarity index across different dimensions. All values must be positive, enabling the algorithm to prioritize certain dimensions over others based on their relative importance.}
#'   \item{\code{m}}{A numeric value greater than 1 that acts as the weighting exponent in the least-squares functional method. This parameter influences the sensitivity of the algorithm to differences in motif alignment and membership probabilities.}
#'   \item{\code{iter_max}}{An integer specifying the maximum number of iterations allowed for the algorithm to converge. This prevents excessive computation time by limiting the number of optimization steps.}
#'   \item{\code{quantile}}{A numeric value representing the quantile probability used when \code{stopCriterion} is set to \code{"quantile"}. This determines the threshold for convergence based on the distribution of Bhattacharyya distances.}
#'   \item{\code{tol}}{A numeric value specifying the tolerance level for convergence. The algorithm stops iterating if the change in the stop criterion falls below this threshold, ensuring precise and stable convergence.}
#'   \item{\code{iter4elong}}{An integer indicating the number of iterations after which motif elongation is performed. If set to a value greater than \code{iter_max}, no elongation is performed. Motif elongation allows the algorithm to extend motifs to better fit the data.}
#'   \item{\code{tol4elong}}{A numeric value defining the tolerance on the Bhattacharyya distance for motif elongation. This parameter controls how much the objective function can increase during elongation, ensuring that motif extensions do not degrade the overall fit.}
#'   \item{\code{max_elong}}{A numeric value representing the maximum elongation allowed in a single iteration, expressed as a percentage of the motif length. This prevents excessive extension of motifs in any single step.}
#'   \item{\code{trials_elong}}{An integer specifying the number of elongation trials (equispaced) on each side of the motif in a single iteration. Multiple trials enhance the robustness of motif elongation by exploring various extension possibilities.}
#'   \item{\code{deltaJK_elong}}{A numeric value indicating the maximum relative increase in the objective function permitted during motif elongation. This ensures that elongation steps contribute positively to the motif fitting process.}
#'   \item{\code{max_gap}}{A numeric value defining the maximum gap allowed in each alignment as a percentage of the motif length. This parameter controls the allowable discontinuity between aligned motifs, maintaining coherence in motif placement.}
#'   \item{\code{iter4clean}}{An integer specifying the number of iterations after which motif cleaning is performed. If set to a value greater than \code{iter_max}, no cleaning is performed. Motif cleaning removes redundant or poorly fitting motifs to refine the final motif set.}
#'   \item{\code{tol4clean}}{A numeric value representing the tolerance on the Bhattacharyya distance for motif cleaning. This parameter determines the threshold for identifying and removing redundant motifs during the cleaning process.}
#'   \item{\code{quantile4clean}}{A numeric value specifying the dissimilarity quantile used for motif cleaning. This quantile determines which motifs are considered sufficiently dissimilar to be retained in the final set.}
#'   \item{\code{return_options}}{A logical value indicating whether to return the options passed to the \code{ProbKMA} method. When set to \code{TRUE}, users receive detailed information about the algorithm's configuration, facilitating transparency and reproducibility.}
#'   \item{\code{Y1}}{A list of derivative curves used if the dissimilarity measure \code{"d0_d1_L2"} is selected. These derivatives enhance the algorithm's ability to capture both shape and rate changes in the functional data.}
#'   \item{\code{P0}}{An initial membership matrix (N x K), where N is the number of curves and K is the number of clusters. If set to \code{NULL}, a random matrix is generated, initiating the probabilistic clustering process.}
#'   \item{\code{S0}}{An initial shift warping matrix (N x K). If set to \code{NULL}, a random matrix is generated to initialize the alignment process, allowing motifs to adapt to variations in the data.}
#'   \item{\code{n_subcurves}}{An integer specifying the number of splitting subcurves used when the number of curves is equal to one. This parameter allows the algorithm to handle single-curve datasets by dividing them into manageable segments for motif discovery.}
#'   \item{\code{sil_threshold}}{A numeric value representing the threshold to filter candidate motifs based on their silhouette scores. This ensures that only motifs with sufficient clustering quality are retained in the final results.}
#'   \item{\code{set_seed}}{A logical value indicating whether to set a random seed for reproducibility. When set to \code{TRUE}, the function initializes the random number generator to ensure consistent results across multiple runs.}
#'   \item{\code{seed}}{An integer specifying the random seed used for initialization when \code{set_seed} is \code{TRUE}. This parameter guarantees reproducibility of the clustering and alignment processes.}
#'   \item{\code{exe_print}}{A logical value determining whether to print execution details for each iteration. When set to \code{TRUE}, users receive real-time feedback on the algorithm's progress, aiding in monitoring and debugging.}
#'   \item{\code{V_init}}{A list of motif sets provided as specific initializations for clustering rather than using random initializations. The `V_init` parameter allows users to provide a set of motifs as starting points for the algorithm, instead of relying on random initialization. If `n_init` is specified as greater than the number of motifs given in `V_init`, the remaining initializations will be randomly generated. For example, if `n_init = 10` but only 5 motif sets are given in `V_init`, the algorithm will use these 5 initializations and generate an additional 5 randomly.}
#'   \item{\code{transformed}}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
#'   \item{\code{n_init_motif}}{The number of initial motif sets from `V_init` to be used directly as starting points in clustering. If `n_init_motif` is set to a value larger than the number of motifs provided in `V_init`, additional initializations will be generated randomly to meet the specified number. For example, if `n_init = 10` and `n_init_motif = 5` with only 3 motif sets in `V_init`, the algorithm will use these 3 sets and generate 7 additional random initializations.}
#' }
#'
#' @section funBIalign Options:
#' The following parameters are specific to the \code{funBIalign} algorithm:
#' \describe{
#'   \item{\code{portion_len}}{An integer specifying the length of curve portions to align. This parameter controls the granularity of alignment, allowing the algorithm to focus on specific segments of the curves for motif discovery.}
#'   \item{\code{min_card}}{An integer representing the minimum cardinality of motifs, i.e., the minimum number of motif occurrences required for a motif to be considered valid. This ensures that only motifs with sufficient representation across the dataset are retained.}
#'   \item{\code{cut_off}}{A double that specifies the number of top-ranked motifs to keep based on the ranking criteria, facilitating focused visualization of the most significant motifs.
#'                         In particular, all motifs that rank below the cut_off are retained.}
#' }
#'
#' @param Y0 A list containing N vectors (for univariate curves) or N matrices (for multivariate curves) representing the functional data.
#' @param method A character string specifying the motif discovery algorithm to use. Acceptable values are "ProbKMA" for Probabilistic K-means with Local Alignment and "funBIalign" for Functional Bi-directional Alignment.
#' @param stopCriterion A character string indicating the convergence criterion for the selected algorithm.
#' @param name A character string specifying the name of the output directory where results will be saved.
#' @param plot A logical value indicating whether to generate and save plots of the discovered motifs and clustering results.
#' @param probKMA_options A list of options specific to the ProbKMA algorithm.
#' @param funBIalign_options A list of options specific to the funBIalign algorithm.
#' @param worker_number An integer specifying the number of CPU cores to utilize for parallel computations.
#'
#' @return 
#' A list containing the discovered motifs and their corresponding statistics, tailored to the selected method:
#' \describe{
#'   \item{\code{motifs}}{A list of identified motifs, each containing the motif's representative curve, membership probabilities, and alignment information.}
#'   \item{\code{statistics}}{Detailed statistics for each motif, including measures such as silhouette scores, variance explained, and other relevant metrics that quantify the quality and significance of the discovered motifs.}
#'   \item{\code{parameters}}{The final parameters and configurations used during the motif discovery process, providing transparency and facilitating reproducibility of the results.}
#'   \item{\code{plots}}{If \code{plot = TRUE}, this component contains the generated plots visualizing the motifs and their distribution across the functional data.}
#' }
#'
#' @examples
#' \donttest{
#' # Example 1: Discover motifs using ProbKMA
#' 
#' # Define dissimilarity measure and weight parameter
#' diss <- 'd0_d1_L2'
#' alpha <- 0.5
#' 
#' # Define number of motifs and their minimum lengths
#' K <- c(2, 3)
#' c <- c(61, 51)
#' n_init <- 3
#' 
#' # Load simulated data
#' data("simulated200")
#' 
#' # Perform motif discovery using ProbKMA
#' results <- funMoDisco::discoverMotifs(
#'   Y0 = simulated200$Y0,
#'   method = "ProbKMA",
#'   stopCriterion = "max",
#'   name = tempdir(),
#'   plot = FALSE,
#'   probKMA_options = list(
#'     Y1 = simulated200$Y1,
#'     K = K,
#'     c = c,
#'     n_init = n_init,
#'     diss = diss,
#'     alpha = alpha
#'   ),
#'   worker_number = 1
#' )
#'
#' # Example 2: Discover motifs using funBIalign
#' results_funbialign <- funMoDisco::discoverMotifs(
#'   Y0 = simulated200$Y0,
#'   method = "funBIalign",
#'   stopCriterion = "fMSR",
#'   name = tempdir(),
#'   plot = FALSE,
#'   funBIalign_options = list(
#'     portion_len = 60,
#'     min_card = 3,
#'     cut_off = 1.0
#'   ),
#'   worker_number = 1
#'   ) 
#' 
#' }
#'
#' @seealso 
#' \strong{ProbKMA}: 
#' \href{https://arxiv.org/pdf/1808.04773}{Probabilistic K-means with Local Alignment} \cr
#' \strong{funBIalign}: 
#' \href{https://arxiv.org/pdf/2306.04254}{Hierarchical Clustering with Mean Squared Residue Scores}.
#'
#' @importFrom grDevices pdf dev.off rainbow
#' @importFrom progress progress_bar
#' 
#' @export
discoverMotifs <- function(Y0,method,stopCriterion,name,plot,
                         probKMA_options = list(),
                         funBIalign_options = list(portion_len = NULL,min_card = NULL,cut_off=NULL),
                         worker_number = NULL){
  
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  ### set parallel jobs #############################################################################
  core_number <- parallel::detectCores()
  # check worker number
  if(!is.null(worker_number)){
    if(!is.numeric(worker_number)){
      warning('Worker number not valid. Selecting default number.')
      worker_number=NULL
    }else{
      if((worker_number%%1!=0)|(worker_number<1)|(worker_number>core_number)){
        warning('Worker number not valid. Selecting default number.')
        worker_number=NULL
      }
    }
  } else {
    worker_number <- core_number - 1 
  }
  rm(core_number)
  
  if(method == 'ProbKMA')
  {
    
    pb <- progress_bar$new(
      format = "Progress [:bar] :percent | Elapsed: :elapsed | ETA: :eta",
      total = 5,  
      width = 50
    )
    
    default_probKMA_options <- list(K=NULL,c=NULL,diss="d0_L2",alpha=0,
                                    Y1=NULL,P0=matrix(),S0=matrix(),
                                    n_init=10,return_options=TRUE,names_var='x(t)',
                                    standardize=FALSE,c_max=Inf,
                                    iter_max=1e3,iter4elong=100,
                                    trials_elong=201,max_gap=0.2,
                                    quantile=0.25,tol=1e-8,
                                    tol4elong=1e-3,max_elong=0.5,
                                    deltaJK_elong=0.05,iter4clean=50,
                                    tol4clean=1e-4,quantile4clean=0.5,
                                    m=2,w=1,transformed = FALSE,silhouette_align=FALSE,
                                    n_subcurves = 10,sil_threshold=0.9,
                                    seed = 1,exe_print = FALSE,set_seed = FALSE,
                                    V_init=NULL, n_init_motif = 0)
    probKMA_options <- modifyList(default_probKMA_options, probKMA_options)
    Y1 <- probKMA_options$Y1
    P0 <- probKMA_options$P0
    S0 <- probKMA_options$S0
    K <- probKMA_options$K
    c <- probKMA_options$c
    n_init <- probKMA_options$n_init
    diss <- probKMA_options$diss
    alpha <- probKMA_options$alpha
    return_options <- probKMA_options$return_options
    names_var <- probKMA_options$names_var
    seed <- probKMA_options$seed
    exe_print <- probKMA_options$exe_print
    set_seed <- probKMA_options$set_seed
    V_init <- probKMA_options$V_init
    n_init_motif <- probKMA_options$n_init_motif
    
    ### check input #############################################################################################
    # check name
    if(!is.character(name))
      stop('name should be a string.')
    if(grepl(' ',name)){
      warning('name contains spaces. Changing spacing in underscores.')
      name=gsub(" ","_",name,fixed=TRUE)
    }
    if (substr(name, nchar(name), nchar(name)) != "/") {
      name <- paste0(name, "/")
    }
    files = list.files(name)
    if('find_candidate_motifs_results.RData' %in% files) {
      # candidate motifs already present, load them
      load(paste0(name,'find_candidate_motifs_results.RData'))
    } else {
      
    ### check on n_init_motif (to be checked) ############
    if(n_init_motif > n_init)
    {
      warning('n_init_motif greater than n_init: setting n_init_motif = n_init')
      n_init_motif = n_init
    }
    if(n_init_motif%%1!=0)
      stop('Number of initializations n_init_motif should be an integer.')
    if(n_init_motif<0)
      stop('Number of initializations n_init_motif should be positive.')
    ######################################################
    
    # check required input
    if(missing(K))
      stop('K must be specified')
    if(missing(c))
      stop('c must be specified')
    # check K
    if(!is.vector(K))
      stop('K should be a vector.')
    # check c
    if(!is.vector(c))
      stop('c should be a vector.')
    # check n_init
    if(n_init%%1!=0)
      stop('Number of initializations n_init should be an integer.')
    if(n_init<1)
      stop('Number of initializations n_init should be at least 1.')
    
    
    #### new check V_init #########################################
    d = ncol(Y0[[1]]) 
    
    if(n_init_motif > 0)
    {
      for(i in 1:length(K))
      {
        for(j in 1:length(c))
        {
          if(length(V_init[[i]][[j]]) < n_init_motif) 
          {
            stop('More initial motifs needed')
          }
          else
          {
            if(length(V_init[[i]][[j]]) > n_init_motif)
            {
              warning('Initial motifs passed more than necessary. Resize V_init to the correct size!')
              V_init[[i]][[j]] <- V_init[[i]][[j]][1:n_init_motif]
            }
            for(e in 1:(n_init-n_init_motif))
            {
              V_init[[i]][[j]] <- append(V_init[[i]][[j]], list(NULL))
            }
            for(k in 1:n_init_motif)
            {
              if(length(V_init[[i]][[j]][[k]])!= K[i])
              {
                stop('Number of initial motifs differs from K')
              }
              else
              {
                for(h in 1:K[i])
                {
                  if(nrow(V_init[[i]][[j]][[k]][[h]]$v0)!= c[j] || ncol(V_init[[i]][[j]][[k]][[h]]$v0)!= d)
                  {
                    stop('Uncorrect dimensions of initial motifs provided')
                  }
                  else
                  {
                    if(probKMA_options$diss == "d0_d1_L2" || probKMA_options$diss == "d1_L2")
                    {
                      if(nrow(V_init[[i]][[j]][[k]][[h]]$v1)!= c[j] || ncol(V_init[[i]][[j]][[k]][[h]]$v0)!= d)
                      {
                        stop('Uncorrect dimensions of initial motifs provided')
                      }
                    }
                    V_init_bool = TRUE
                  }
                }
              }
            }
          }
        }
      }
    }
    
    ##################################
    # check Y0
    if(missing(Y0))
      stop('Y0 must be specified.')
    
    if(!is.null(Y0))
      if(!is.list(Y0))
        stop('Y0 should be a list of vectors or matrices.')
    
    if((FALSE %in% lapply(Y0,is.matrix))&&(FALSE %in% lapply(Y0,is.vector)))
      stop('Y0 should be a list of vectors or matrices.')
    
    N = length(Y0) # number of curves
    if(N!=1 && N<5)
      stop('More curves y_i(x) needed.')
    
    Y0 = lapply(Y0,as.matrix)
    # set return_options=TRUE
    if(!is.null(probKMA_options$return_options)){
      if(probKMA_options$return_options==FALSE){
        warning('Setting return_option=TRUE')
        probKMA_options$return_options=TRUE
      }
    }
   
    arguments = list(Y0 = Y0, Y1 = Y1, P0 = P0, S0 = S0, 
                     standardize = probKMA_options$standardize, c_max = probKMA_options$c_max,
                     iter_max = probKMA_options$iter_max, iter4elong = probKMA_options$iter4elong, 
                     trials_elong = probKMA_options$trials_elong, return_options = return_options,
                     alpha = alpha, max_gap = probKMA_options$max_gap, 
                     quantile = probKMA_options$quantile, stopCriterion = stopCriterion, tol = probKMA_options$tol,
                     tol4elong =probKMA_options$tol4elong, 
                     max_elong = probKMA_options$max_elong, deltaJK_elong = probKMA_options$deltaJK_elong,
                     iter4clean = probKMA_options$iter4clean, 
                     tol4clean = probKMA_options$tol4clean, m = probKMA_options$m, w = probKMA_options$w,
                     seed = seed, K = NULL, c = NULL, 
                     quantile4clean = probKMA_options$quantile4clean,
                     exe_print = exe_print, set_seed = set_seed, 
                     diss = diss, transformed = probKMA_options$transformed, V_init = NULL,
                     align = probKMA_options$silhouette_align,
                     n_threads = worker_number)
      
    if(worker_number>1){
        cl_find=parallel::makeCluster(worker_number,timeout=60*60*24*30)
        parallel::clusterExport(cl_find,c('name','names_var',
                                          'probKMA_plot','probKMA_silhouette_plot',
                                          '.mapply_custom','.diss_d0_d1_L2','.domain',
                                          'initialChecks',
                                          '.select_domain','.find_min_diss',
                                          'probKMA_wrap','arguments'),envir=environment()) 
        parallel::clusterCall(cl_find, function() {
          combinat::combn
        })
        on.exit(parallel::stopCluster(cl_find))
      }else{
        cl_find=NULL
      }
    pb$update(0.1)
    
    ### run probKMA ##########################################################################################
    i_c_K = expand.grid(seq_len(n_init),c,K)
    vector_seed = seq(1,length(i_c_K$Var1))
    if(n_init_motif > 0 && V_init_bool)
    {
      V_init_unlist = unlist(unlist(V_init,recursive=FALSE),recursive=FALSE)
      results=tryCatch({.mapply_custom(cl_find,function(K,c,i,v_init,small_seed){
        dir.create(paste0(name,"K",K,"_c",c),showWarnings=FALSE,recursive = TRUE)
        files=list.files(paste0(name,"K",K,"_c",c))
        if(paste0('random',i,'.RData') %in% files){
          load(paste0(name,"K",K,"_c",c,'/random',i,'.RData')) 
          message("K",K,"_c",c,'_random',i,' loaded')
          return(list(probKMA_results=probKMA_results,
                      time=time,silhouette=silhouette))
        }else{
          iter=iter_max=1
          message("K",K,"_c",c,'_random',i,' running')
          while(iter==iter_max){
            start=proc.time()
            if(arguments$set_seed)
            {
              arguments$seed = small_seed
              set.seed(small_seed)
            }
            small_seed = small_seed + 1
            arguments$K = K
            arguments$c = c
            arguments$quantile4clean = 1/K
            arguments$V_init = v_init
            results = do.call(probKMA_wrap,arguments)
            probKMA_results = results[[1]]
            silhouette_results = results[[2]]
            end=proc.time()
            time=end-start
            iter=probKMA_results$iter
            iter_max=arguments$iter_max
            if(iter==iter_max)
              warning('Maximum number of iteration reached. Re-starting.')
          }
          if(plot) {
          transformed=probKMA_results$transformed
          # RANDOM PLOT
          #pdf(paste0(name,"K",K,"_c",c,'/random',i,'.pdf'),width=20,height=10)
          #probKMA_plot(probKMA_results,ylab=names_var,plot = plot,cleaned=FALSE, transformed = arguments$transformed)
          #dev.off()
          pdf(paste0(name,"K",K,"_c",c,'/random',i,'clean.pdf'),width=20,height=10)
          probKMA_plot(probKMA_results,
                       ylab = names_var,
                       sil_avg = silhouette_results[[4]],
                       plot = plot,
                       cleaned=TRUE,
                       transformed = arguments$transformed) 
          dev.off()
          
          # RANDOM SILHOUETTE PLOT
          #pdf(paste0(name,"K",K,"_c",c,'/random',i,'silhouette.pdf'),width=7,height=10)
          ##silhouette = probKMA_silhouette_plot(silhouette_results,K,plot = plot)
          dev.off()
          } else {
            silhouette = probKMA_silhouette_plot(silhouette_results,K,plot = FALSE)
          }
          save(probKMA_results, time, silhouette,
               file = paste0(name, "K", K, "_c", c, '/random', i, '.RData')
               )
          return(list(probKMA_results =probKMA_results,
                      time=time,silhouette=silhouette))
        }
      },i_c_K[,3],i_c_K[,2],i_c_K[,1],V_init_unlist,vector_seed,SIMPLIFY=FALSE)},error = function(e){
        stop(e$message)
      })
    }
    if(n_init_motif == 0 || !V_init_bool)
    {
      if(N == 1) {
        
        .split_curve_randomly <- function(curve, expected_motif_length, n_subcurves, randomness_factor = 0.3) {
          length_curve <- nrow(curve)  # Number of observations in the curve
          
          # Step 1: Randomly generate candidate split points
          candidate_splits <- sort(sample(seq(expected_motif_length, length_curve - expected_motif_length, 1), n_subcurves * 2))
          
          # Step 2: Adjust split points with randomness while respecting the minimum length requirement
          split_points <- c(1)  # Start point
          last_split <- 1
          
          for (i in candidate_splits) {
            # Check if we can add a split while respecting minimum length
            if (i - last_split >= expected_motif_length) {
              segment <- curve[last_split:i, ]
              valid_indices <- which(!is.na(as.matrix(segment)[, 1]))  # Indices of valid values
              
              if (length(valid_indices) > 0) {
                valid_segments <- split(valid_indices, cumsum(c(1, diff(valid_indices) != 1)))  # Split into contiguous valid segments
                valid_segment <- FALSE
                for (seg in valid_segments) {
                  if (length(seg) >= expected_motif_length) {
                    valid_segment <- TRUE
                    break
                  }
                }
                # Introduce randomness in adding split points
                if (valid_segment && runif(1) < randomness_factor) {
                  split_points <- c(split_points, i)
                  last_split <- i
                }
              }
            }
          }
          
          # Add the last segment if it meets the minimum length requirement
          if (length_curve - last_split >= expected_motif_length) {
            split_points <- c(split_points, length_curve)
          } else {
            split_points[length(split_points)] <- length_curve
          }
          
          split_points <- unique(split_points)
          
          # Step 3: Adjust the number of split points to match n_subcurves
          if (length(split_points) - 1 > n_subcurves) {
            # Too many splits, reduce by merging closest split points
            while (length(split_points) - 1 > n_subcurves) {
              gaps <- diff(split_points)
              min_gap_index <- which.min(gaps)
              split_points <- split_points[-(min_gap_index + 1)]  # Remove the smaller split
            }
          } else if (length(split_points) - 1 < n_subcurves) {
            # Too few splits, add new split points in the largest gaps between existing points
            additional_splits_needed <- n_subcurves - (length(split_points) - 1)
            while (additional_splits_needed > 0) {
              gaps <- diff(split_points)
              max_gap_index <- which.max(gaps)
              new_split <- round(mean(split_points[max_gap_index:(max_gap_index + 1)]))
              
              if (new_split - split_points[max_gap_index] >= expected_motif_length &&
                  split_points[max_gap_index + 1] - new_split >= expected_motif_length) {
                split_points <- sort(c(split_points, new_split))
                additional_splits_needed <- additional_splits_needed - 1
              } else {
                stop("Failed to find valid split points. Ensure each subcurve meets the minimum length requirement.")
              }
            }
          }
          
          # Step 4: Create subcurves based on valid split points
          subcurves <- list()
          for (i in 1:(length(split_points) - 1)) {
            subcurve <- curve[split_points[i]:split_points[i + 1], ]
            if (nrow(as.matrix(subcurve)) > 0) {
              subcurves[[paste0("c", i)]] <- subcurve
            }
          }
          
          return(list(subcurves = subcurves, split_points = split_points))
        }
        
        n_subcurves <- probKMA_options$n_subcurves 
        if(arguments$diss == 'd0_d1_L2') {
          split_results <- lapply(seq_len(n_init),function(null){.split_curve_randomly(arguments$Y0[[1]], max(c),n_subcurves, randomness_factor = 0.3)})
          Y0_subcurves <- lapply(split_results,function(splitted_curve){splitted_curve[[1]]})
          Y1_subcurves <- lapply(seq_len(n_init),function(j){
                            temp <- list()
                            for (i in seq_along(split_results$split_points)[-length(split_results$split_points)])
                              temp[[i]] <- split_results[[j]][[1]][pos[i]:pos[i + 1]]
                            return(temp)
                             })
        }
        else if(arguments$diss == 'd0_L2') {
          split_results <- lapply(seq_len(n_init),function(null){.split_curve_randomly(arguments$Y0[[1]], max(c),n_subcurves, randomness_factor = 0.3)})
          Y0_subcurves <- lapply(split_results,function(splitted_curve){splitted_curve[[1]]})
          Y1_subcurves <- vector("list",n_init)
        }
        else if(arguments$diss == 'd1_L2') {
          Y0_subcurves <- vector("list",n_init)
          split_results <- lapply(seq_len(n_init),function(null){.split_curve_randomly(arguments$Y1[[1]], max(c),n_subcurves, randomness_factor = 0.3)})
          Y1_subcurves <- lapply(split_results,function(splitted_curve){splitted_curve[[1]]})
        }
      results=tryCatch({.mapply_custom(cl_find, function(K,c,i,Y0_curve,Y1_curve,small_seed){ 
        dir.create(paste0(name,"K",K,"_c",c),showWarnings=TRUE,recursive = TRUE)
        files=list.files(paste0(name,"K",K,"_c",c))
        if(paste0('random',i,'.RData') %in% files){
          load(paste0(name,"K",K,"_c",c,'/random',i,'.RData'))
          message(paste("\033[34mK", K, "_c", c, "_random", i, "loaded\033[0m"))
          return(list(probKMA_results=probKMA_results,
                      time=time,silhouette=silhouette))
        }else{
          iter=iter_max=1
          message(paste("\033[34mK", K, "_c", c, "_random", i, "running\033[0m"))
          if(arguments$set_seed)
          {
            arguments$seed = small_seed
            set.seed(small_seed)
          }
          while(iter==iter_max){
            start=proc.time()
            small_seed = small_seed + 1
            arguments$Y0 <- Y0_curve
            arguments$Y1 <- Y1_curve
            arguments$K = K
            arguments$c = c
            arguments$quantile4clean = 1/K
            results = do.call(probKMA_wrap,arguments)
            probKMA_results = results[[1]]
            silhouette_results = results[[2]]
            end=proc.time()
            time=end-start
            iter=probKMA_results$iter
            iter_max=arguments$iter_max
            
            if(iter==iter_max)
              warning('Maximum number of iteration reached. Re-starting.')
          }
          if(plot) {
          # pdf(paste0(name,"K",K,"_c",c,'/random',i,'.pdf'),width=20,height=10)
          # probKMA_plot(probKMA_results,
          #                     plot = TRUE,
          #                     ylab=names_var,
          #                     sil_avg = silhouette_results[[4]],
          #                     cleaned=FALSE,
          #                     transformed = arguments$transformed)
          # dev.off()
          pdf(paste0(name,"K",K,"_c",c,'/random',i,'clean.pdf'),width=20,height=10)
          probKMA_plot(probKMA_results,
                              plot = TRUE,
                              ylab=names_var,
                              sil_avg = silhouette_results[[4]],
                              cleaned=TRUE,
                              transformed = arguments$transformed)
          dev.off()

          #pdf(paste0(name,"K",K,"_c",c,'/random',i,'silhouette.pdf'),width=7,height=10)
          silhouette = probKMA_silhouette_plot(silhouette_results,K,plot = plot)
          #dev.off()
          } else {
            silhouette = probKMA_silhouette_plot(silhouette_results,K,plot = FALSE)
          }

          probKMA_results["V_init"] <- NULL # delete V_init components
          save(probKMA_results,time,silhouette,
               file=paste0(name,"K",K,"_c",c,'/random',i,'.RData'))
           return(list(probKMA_results=probKMA_results,
                      time=time,silhouette=silhouette))
         }
      },i_c_K[,3],i_c_K[,2],i_c_K[,1],Y0_subcurves,Y1_subcurves,vector_seed,SIMPLIFY=FALSE)},error = function(e){
        stop(e$message)
      })
      } else {
      results = tryCatch({.mapply_custom(cl_find, function(K,c,i,small_seed){ 
        dir.create(paste0(name,"K",K,"_c",c),showWarnings=TRUE,recursive = TRUE)
        files=list.files(paste0(name,"K",K,"_c",c))
        if(paste0('random',i,'.RData') %in% files){
          load(paste0(name,"K",K,"_c",c,'/random',i,'.RData'))
          message(paste("\033[34mK", K, "_c", c, "_random", i, "loaded\033[0m"))
          return(list(probKMA_results=probKMA_results,
                      time=time,silhouette=silhouette))
        }else{
          iter=iter_max=1
          message(paste("\033[34mK", K, "_c", c, "_random", i, "running\033[0m"))
          if(arguments$set_seed)
          {
            arguments$seed = small_seed
            set.seed(small_seed)
          }
          while(iter==iter_max){
            start=proc.time()
            small_seed = small_seed + 1
            arguments$K = K
            arguments$c = c
            arguments$quantile4clean = 1/K
            results = do.call(probKMA_wrap,arguments)
            probKMA_results = results[[1]]
            silhouette_results = results[[2]]
            end=proc.time()
            time=end-start
            iter=probKMA_results$iter
            iter_max=arguments$iter_max
            
            if(iter==iter_max)
              warning('Maximum number of iteration reached. Re-starting.')
          }
          if(plot) {
            # pdf(paste0(name,"K",K,"_c",c,'/random',i,'.pdf'),width=20,height=10)
            # probKMA_plot(probKMA_results,
            #              plot = TRUE,
            #              ylab=names_var,
            #              sil_avg = silhouette_results[[4]],
            #              cleaned=FALSE,
            #              transformed = arguments$transformed)
            # dev.off()
            pdf(paste0(name,"K",K,"_c",c,'/random',i,'clean.pdf'),width=20,height=10)
            probKMA_plot(probKMA_results,
                         plot = TRUE,
                         ylab=names_var,
                         sil_avg = silhouette_results[[4]],
                         cleaned=TRUE,
                         transformed = arguments$transformed)
            dev.off()
            
            #pdf(paste0(name,"K",K,"_c",c,'/random',i,'silhouette.pdf'),width=7,height=10)
            silhouette = probKMA_silhouette_plot(silhouette_results, K, plot = plot)
            #dev.off()
          } else {
            silhouette = probKMA_silhouette_plot(silhouette_results,K,plot = FALSE)
          }
          
          probKMA_results["V_init"] <- NULL # delete V_init components
          save(probKMA_results,time,silhouette,
               file=paste0(name,"K",K,"_c",c,'/random',i,'.RData'))
          return(list(probKMA_results=probKMA_results,
                      time=time,silhouette=silhouette))
        }
      },i_c_K[,3],i_c_K[,2],i_c_K[,1],vector_seed,SIMPLIFY=FALSE)},error = function(e){
        stop(e$message)
      })
    }
    }

    results=split(results,list(factor(i_c_K[,2],c),factor(i_c_K[,3],K)))
    results=split(results,rep(K,each=length(c)))
    
    ### plot silhouette average #################################################################################
    silhouette_average_sd=lapply(results,
                                 function(results){
                                   silhouette_average=lapply(results,
                                                             function(results){
                                                               silhouette_average=numeric(n_init)
                                                               silhouette_sd=numeric(n_init)
                                                               for(i in seq_len(n_init)){
                                                                 silhouette_average[i]=mean(results[[i]]$silhouette$silhouette_average)
                                                                 silhouette_sd[i]=sd(results[[i]]$silhouette$silhouette_average)
                                                               }
                                                               return(cbind(silhouette_average,silhouette_sd))
                                                             })
                                 })
    if(plot){
      pdf(paste0(name,'silhouette.pdf'),width=7,height=5)
      for(i in seq_along(K)){
        silhouette_average_plot=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) average_sd[order(average_sd[,1],decreasing=TRUE),1])),ncol=length(c))
        silhouette_sd_plot=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) average_sd[order(average_sd[,1],decreasing=TRUE),2])),ncol=length(c))
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        matplot(matrix(rep(1:n_init,length(c))+rep(seq(-0.1,0.1,length.out=length(c)),each=n_init),ncol=length(c)),
                silhouette_average_plot,type='b',pch=16,lty=1,col=1+seq_along(c),ylim=c(0,1),xaxt='n',
                xlab='',ylab='Silhouette average',main=paste0('K=',K[i]))
        shift=seq(-0.1,0.1,length.out=length(c))
        for(ii in seq_along(c)){
          segments(1:n_init+shift[ii],silhouette_average_plot[,ii]-silhouette_sd_plot[,ii],
                   1:n_init+shift[ii],silhouette_average_plot[,ii]+silhouette_sd_plot[,ii],col=ii+1)
          segments(1:n_init+shift[ii]-0.1,silhouette_average_plot[,ii]+silhouette_sd_plot[,ii],
                   1:n_init+shift[ii]+0.1,silhouette_average_plot[,ii]+silhouette_sd_plot[,ii],col=ii+1)
          segments(1:n_init+shift[ii]-0.1,silhouette_average_plot[,ii]-silhouette_sd_plot[,ii],
                   1:n_init+shift[ii]+0.1,silhouette_average_plot[,ii]-silhouette_sd_plot[,ii],col=ii+1)
          text(1:n_init+shift[ii],silhouette_average_plot[,ii]-silhouette_sd_plot[,ii],
               silhouette_order[,ii],col=ii+1,pos=1,cex=0.7)
        }
        legend('bottomleft',legend=paste0('c=',c),col=1+seq_along(c),lty=1)
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
      }
      dev.off()
    }
    ### plot processing time ####################################################################################
    times=lapply(results,
                 function(results){
                   times=lapply(results,
                                function(results)
                                  times=unlist(lapply(results,function(results) results$time[3])))
                 })
    if(plot){
      #pdf(paste0(name,'times.pdf'),width=7,height=5)
      y_max=max(unlist(times))*1.2
      times_plot=vector('list',length(K))
      for(i in seq_along(K)){
        times_plot[[i]]=Reduce(cbind,
                               lapply(seq_along(c),
                                      function(j)
                                        as.matrix(times[[i]][[j]][order(silhouette_average_sd[[i]][[j]][,1],decreasing=TRUE)]))
        )
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        matplot(matrix(rep(1:n_init,length(c))+rep(seq(-0.1,0.1,length.out=length(c)),each=n_init),ncol=length(c)),
                times_plot[[i]],type='b',pch=16,lty=1,col=1+seq_along(c),ylim=c(0,y_max),xaxt='n',
                xlab='',ylab='Time',main=paste0('K=',K[i]))
        shift=seq(-0.1,0.1,length.out=length(c))
        for(ii in seq_along(c)){
          text(1:n_init+shift[ii],times_plot[[i]][,ii],silhouette_order[,ii],col=ii+1,pos=1,cex=0.7)
        }
        legend('topleft',legend=paste0('c=',c),col=1+seq_along(c),lty=1)
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
      }
      for(i in seq_along(K)){
        boxplot(times_plot[[i]],col=1+seq_along(c),names=paste0('c=',c),ylim=c(0,y_max),
                xlab='',ylab='Times',main=paste0('K=',K[i]))
      }
      #dev.off()
    }
    
    ### plot dissimilarities ####################################################################################
    if(plot){
      D=lapply(results,
               function(results){
                 D=lapply(results,
                          function(results){
                            D=lapply(results,
                                     function(results){
                                       D=as.vector(results$probKMA_results$D)
                                     })
                          })
               })
      #pdf(paste0(name,'dissimilarities.pdf'),width=7,height=5)
      y_max=max(unlist(D))
      for(i in seq_along(K)){
        D_plot=matrix(unlist(D[[i]]),ncol=length(c))
        boxplot(D_plot,col=1+seq_along(c),names=paste0('c=',c),ylim=c(0,y_max),
                xlab='',ylab='Dissimilarity',main=paste0('K=',K[i]))
      }
      for(i in seq_along(K)){
        for(j in seq_along(c)){
          silhouette_order=order(silhouette_average_sd[[i]][[j]][,1],decreasing=TRUE)
          D_plot=D[[i]][[j]][silhouette_order]
          boxplot(D_plot,col=j+1,ylim=c(0,y_max),names=silhouette_order,
                  xaxt='n',ylab='Dissimilarity',main=paste0('K=',K[i],'   c=',c[j]))
          axis(1,1:n_init,labels=rep('',n_init))
          mtext("Init",side=1,line=1)
        }
      }
      #dev.off()
      D_clean=lapply(results,
                     function(results){
                       D_clean=lapply(results,
                                      function(results){
                                        D_clean=lapply(results,
                                                       function(results){
                                                         D_clean=as.vector(results$probKMA_results$D_clean)
                                                       })
                                      })
                     })
      pdf(paste0(name,'dissimilarities_clean.pdf'),width=7,height=5)
      y_max=max(unlist(D_clean))
      for(i in seq_along(K)){
        D_plot=matrix(unlist(D_clean[[i]]),ncol=length(c))
        boxplot(D_plot,col=1+seq_along(c),names=paste0('c=',c),ylim=c(0,y_max),
                xlab='',ylab='Dissimilarity',main=paste0('K=',K[i]))
      }
      for(i in seq_along(K)){
        for(j in seq_along(c)){
          silhouette_order=order(silhouette_average_sd[[i]][[j]][,1],decreasing=TRUE)
          D_plot=D_clean[[i]][[j]][order(silhouette_average_sd[[i]][[j]][,1],decreasing=TRUE)]
          boxplot(D_plot,col=j+1,ylim=c(0,y_max),names=silhouette_order,
                  xaxt='n',ylab='Dissimilarity',main=paste0('K=',K[i],'   c=',c[j]))
          axis(1,1:n_init,labels=rep('',n_init))
          mtext("Init",side=1,line=1)
        }
      }
      dev.off()
    }
    
    ### plot motif lengths ######################################################################################
    if(plot){
      motif_length=mapply(function(results){
        motif_length=mapply(function(results){
          motif_length=as.matrix(Reduce(cbind,lapply(results,
                                                     function(results){
                                                       unlist(lapply(results$probKMA_results$V0,nrow))
                                                     })))
          return(as.matrix(motif_length))
        },results,SIMPLIFY=FALSE)
      },results,SIMPLIFY=FALSE)
      #pdf(paste0(name,'lengths.pdf'),width=7,height=5)
      motif_length_plot=lapply(motif_length,
                               function(motif_length){
                                 lapply(motif_length,
                                        function(motif_length){
                                          motif_length=motif_length+matrix(rnorm(nrow(motif_length)*n_init,mean=0,sd=0.05),ncol=n_init)
                                        })
                               })
      ymax=max(unlist(motif_length_plot))
      for(i in seq_along(K)){
        plot(1:n_init,type="n",xaxt='n',xlab='',ylim=c(-1,ymax),ylab='Motif lengths',main=paste0('K=',K[i]))
        abline(h=c,col=1+seq_along(c))
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
        shift=seq(-0.1,0.1,length.out=length(c))
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        for(ii in seq_along(c)){
          points(rep(1:n_init+shift[ii],each=K[i]),
                 motif_length_plot[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],col=ii+1,pch=8)
          text(rep(1:n_init+shift[ii],each=K[i]),motif_length[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],
               rep(silhouette_order[,ii],each=K[i]),col=ii+1,pos=1,cex=0.7)
        }
        legend('bottomleft',legend=paste0('c=',c),col=1+seq_along(c),pch=8)
      }
      #dev.off()
      
      motif_clean_length=mapply(function(results){
        motif_length=mapply(function(results){
          motif_length=as.matrix(Reduce(cbind,lapply(results,
                                                     function(results){
                                                       unlist(lapply(results$probKMA_results$V0_clean,nrow))
                                                     })))
          return(as.matrix(motif_length))
        },results,SIMPLIFY=FALSE)
      },results,SIMPLIFY=FALSE)
      pdf(paste0(name,'lengths_clean.pdf'),width=7,height=5)
      motif_length_plot=lapply(motif_clean_length,
                               function(motif_length){
                                 lapply(motif_length,
                                        function(motif_length){
                                          motif_length=motif_length+matrix(rnorm(nrow(motif_length)*n_init,mean=0,sd=0.05),ncol=n_init)
                                        })
                               })
      ymax=max(unlist(motif_length_plot))
      for(i in seq_along(K)){
        plot(1:n_init,type="n",xaxt='n',xlab='',ylim=c(-1,ymax),ylab='Motif lengths',main=paste0('K=',K[i]))
        abline(h=c,col=1+seq_along(c))
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
        shift=seq(-0.1,0.1,length.out=length(c))
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        for(ii in seq_along(c)){
          points(rep(1:n_init+shift[ii],each=K[i]),
                 motif_length_plot[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],col=ii+1,pch=8)
          text(rep(1:n_init+shift[ii],each=K[i]),motif_clean_length[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],
               rep(silhouette_order[,ii],each=K[i]),col=ii+1,pos=1,cex=0.7)
        }
        legend('bottomleft',legend=paste0('c=',c),col=1+seq_along(c),pch=8)
      }
      dev.off()
      
      #pdf(paste0(name,'lengths_perc.pdf'),width=7,height=5)
      motif_length_perc=mapply(function(results){
        motif_length=mapply(function(results){
          motif_length=as.matrix(Reduce(cbind,lapply(results,
                                                     function(results){
                                                       unlist(lapply(results$probKMA_results$V0,nrow))/results$probKMA_results$c*100
                                                     })))
          return(as.matrix(motif_length))
        },results,SIMPLIFY=FALSE)
      },results,SIMPLIFY=FALSE)
      motif_length_plot=lapply(motif_length_perc,
                               function(motif_length){
                                 lapply(motif_length,
                                        function(motif_length){
                                          motif_length=motif_length+matrix(rnorm(nrow(motif_length)*n_init,mean=0,sd=0.05),ncol=n_init)
                                        })
                               })
      ymax=max(unlist(motif_length_plot))
      for(i in seq_along(K)){
        plot(1:n_init,type="n",xaxt='n',xlab='',ylim=c(-1,ymax),ylab='Motif lengths (% of minimum length)',main=paste0('K=',K[i]))
        abline(h=100)
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
        shift=seq(-0.1,0.1,length.out=length(c))
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        for(ii in seq_along(c)){
          points(rep(1:n_init+shift[ii],each=K[i]),
                 motif_length_plot[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],col=ii+1,pch=8)
          text(rep(1:n_init+shift[ii],each=K[i]),motif_length_perc[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],
               rep(silhouette_order[,ii],each=K[i]),col=ii+1,pos=1,cex=0.7)
        }
        legend('bottomleft',legend=paste0('c=',c),col=1+seq_along(c),pch=8)
      }
      #dev.off()
      
      pdf(paste0(name,'lengths_clean_perc.pdf'),width=7,height=5)
      motif_clean_length_perc=mapply(function(results){
        motif_length=mapply(function(results){
          motif_length=as.matrix(Reduce(cbind,lapply(results,
                                                     function(results){
                                                       unlist(lapply(results$probKMA_results$V0_clean,nrow))/results$probKMA_results$c*100
                                                     })))
          return(as.matrix(motif_length))
        },results,SIMPLIFY=FALSE)
      },results,SIMPLIFY=FALSE)
      motif_length_plot=lapply(motif_clean_length_perc,
                               function(motif_length){
                                 lapply(motif_length,
                                        function(motif_length){
                                          motif_length=motif_length+matrix(rnorm(nrow(motif_length)*n_init,mean=0,sd=10^-1),ncol=n_init)
                                        })
                               })
      ymax=max(unlist(motif_length_plot))
      for(i in seq_along(K)){
        plot(1:n_init,type="n",xaxt='n',xlab='',ylim=c(-1,ymax),ylab='Motif lengths (% of minimum length)',main=paste0('K=',K[i]))
        abline(h=100)
        axis(1,1:n_init,labels=rep('',n_init))
        mtext("Init",side=1,line=1)
        shift=seq(-0.1,0.1,length.out=length(c))
        silhouette_order=matrix(unlist(lapply(silhouette_average_sd[[i]],function(average_sd) order(average_sd[,1],decreasing=TRUE))),ncol=length(c))
        for(ii in seq_along(c)){
          points(rep(1:n_init+shift[ii],each=K[i]),
                 motif_length_plot[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],col=ii+1,pch=8)
          text(rep(1:n_init+shift[ii],each=K[i]),motif_clean_length_perc[[i]][[ii]][,order(silhouette_average_sd[[i]][[ii]][,1],decreasing=TRUE)],
               rep(silhouette_order[,ii],each=K[i]),col=ii+1,pos=1,cex=0.7)
        }
        legend('bottomleft',legend=paste0('c=',c),col=1+seq_along(c),pch=8)
      }
      dev.off()
    }
    
    ### output ##################################################################################################
    find_candidate_motifs_results <- list(name = name,
                                          K = K,
                                          c = c,
                                          n_init = n_init,
                                          silhouette_average_sd = silhouette_average_sd,
                                          times = times)
    
    save(find_candidate_motifs_results, file = paste0(name,'find_candidate_motifs_results.RData'))
    }
    ### filter candidate motifs based on silhouette average and size
    silhouette_average = Reduce(rbind, Reduce(rbind, find_candidate_motifs_results$silhouette_average_sd))[ , 1] # retrieve silhouette average for all candidate motifs
    pb$update(0.5)
    filter_candidate_motifs_results = filter_candidate_motifs(find_candidate_motifs_results,
                                                              sil_threshold = quantile(silhouette_average, probKMA_options$sil_threshold),
                                                              size_threshold = 2)
    ### cluster candidate motifs based on their distance and select radii
    pb$update(0.75)
    cluster_candidate_motifs_results = cluster_candidate_motifs(filter_candidate_motifs_results,
                                                                motif_overlap = 0.6,
                                                                worker_number = worker_number)
    cluster_candidate_motifs_results$transformed = probKMA_options$transformed
    ### plot cluster candidate motifs results
    pdf(paste0(name,'FMD_clustering_candidate_motifs.pdf'), height = 12, width = 9)
    cluster_candidate_motifs_plot(cluster_candidate_motifs_results, ask = FALSE)
    dev.off()
    ### search selected motifs
    cluster_candidate_motifs_results$Y0 <- Y0
    cluster_candidate_motifs_results$Y1 <- Y1
    
    pb$update(0.9)
    motifs_search_results = motifs_search(cluster_candidate_motifs_results,
                                          use_real_occurrences = FALSE, length_diff = +Inf,
                                          worker_number = worker_number)
    
    ### plot FMD results (NB: no threshold of frequencies of motif found!)
    pdf(paste0(name,'FMD_results.pdf'), height = 7, width = 17)
    motifs_search_plot(motifs_search_results, ylab = 'x(t)', freq_threshold = 1)
    dev.off()
    
    save(find_candidate_motifs_results, silhouette_average, filter_candidate_motifs_results,
         cluster_candidate_motifs_results, motifs_search_results,
         file=paste0(name,'FMD_results.RData'))
    pb$update(1.0)
    return(list(find_candidate_motifs_results = find_candidate_motifs_results,
                silhouette_average = silhouette_average,
                filter_candidate_motifs_results = filter_candidate_motifs_results,
                cluster_candidate_motifs_results = cluster_candidate_motifs_results,
                motifs_search_results = motifs_search_results))
  }else if(method=="funBIalign"){ # funBIalign --------
    
    if(length(funBIalign_options) != 3) {
      stop('\'funBIalign_options\' must contain: \'portion_len\',\'min_card\',\'cut_off\'.')
    }
    
    portion_len <- funBIalign_options$portion_len # portion_len
    min_card <- funBIalign_options$min_card # minimum cardinality
    cut_off <- funBIalign_options$cut_off # quality threshold
    
    # check required input
    if(missing(portion_len) || is.null(portion_len))
      stop('portion_len attribute must be specified')
    
    if(missing(min_card) || is.null(min_card))
      stop('min_card attribute must be specified')
    
    pb <- progress_bar$new(
      format = "Progress [:bar] :percent | Elapsed: :elapsed | ETA: :eta",
      total = 5,  
      width = 50
    )
    
    pb$update(0.1)
    #compute maximum length
    maxLen <- max(sapply(Y0, nrow))
    # pad a Matrix to a Specified Number of Rows using the function padding (in "padding.R")
    full_data <- sapply(Y0, padding, maxLen, simplify = FALSE)
    
    # transform list into a matrix where each curve is on a row
    full_data <- t(sapply(full_data, cbind))
    
    # prepare output elements
    window_data <- NULL
    list_of_recommendations_ordered <- NULL
    vec_of_scores_ordered <- NULL
    
    # check if the resulting file exists.
    rds_files <- list.files(path = paste0(name),
                            pattern = "\\.rds$",
                            full.names = TRUE)
    
    # create 
    if(length(rds_files) > 1){
      rds_files_stats <- lapply(rds_files,
                                function(x){
                                  len <- as.integer(sub(".*portion_len_(\\d+)_.*", "\\1", x))
                                  card <- as.integer(sub(".*min_card_(\\d+)\\.rds$", "\\1", x))
                                  c("len" = len, "card" = card)
                                })
      
      rds_files_stats <- do.call(rbind,
                                 rds_files_stats) %>% as.data.frame()
    } else {
      rds_files_stats <- data.frame("len"  = 0,
                                    "card" = 0) # empty data frame
    }
    
    
    # if it does not exist then...
    if(all(rds_files_stats$len == portion_len & rds_files_stats$card == min_card) == FALSE){
      
      # there are no files with the same portion_len
      if(all(rds_files_stats$len != portion_len)){
        # create new directory
        dir.create(paste0(name),showWarnings=FALSE)
      
        # createWindow with RCpp
    cppFunction('
    Rcpp::List createWindow(Rcpp::NumericMatrix& data,
                          unsigned int portion_len,
                          unsigned int worker_number){
  
  const unsigned int totdim = data.ncol();
  const unsigned int totobs = data.nrow();
  const unsigned int totrows = (totdim - portion_len + 1) * totobs;
  const unsigned int totportion = totdim - portion_len + 1;
  
  // set the size for data structure
  const arma::mat dataRef(data.begin(), totobs, totdim,false,true);
  Rcpp::NumericMatrix windowData(totrows,portion_len);
  arma::mat windowDataRef(windowData.begin(),totrows,portion_len,false,true);
  std::vector<std::string> window_rownames(totrows);
  
  if(totobs <= 1) // we have a single curve
  {
    // fill in my data 
#ifdef _OPENMP
#pragma omp parallel for num_threads(worker_number)
#endif
    for(arma::uword i = 0; i < totrows; ++i)
    {
      windowDataRef.row(i) = dataRef.cols(i,i + portion_len - 1); //Each peace of curve is stored in a row
      window_rownames[i] = "1_" + std::to_string(i + 1) + "_" + std::to_string(i + portion_len);         
    }
  }
  else // we have multiple curves
  {
    // fill in my data
#ifdef _OPENMP
#pragma omp parallel for collapse(2) num_threads(worker_number) schedule(static)
#endif
      for(arma::uword k = 0; k < totobs; ++k)
      {
        for (arma::uword i = 0; i < totportion; ++i)
        {
          windowDataRef.row(totportion * k + i) = dataRef(k,arma::span(i,i + portion_len - 1));
          window_rownames[i + k * totportion] = std::to_string(k+1) + "_" + std::to_string(i + 1) + "_" + std::to_string(i + portion_len); 
        }
                
      }
  }
  return Rcpp::List::create(windowData,window_rownames);
}',depends = "RcppArmadillo")
  
    # step 1
    window_data_list <- createWindow(full_data, portion_len, worker_number) 
    window_data <- window_data_list[[1]] # data
    rownames(window_data) <- window_data_list[[2]] # row names added
    pb$update(0.3)
    
    # set it as a matrix and omit rows with NAs
    window_data <- na.omit(window_data) 
    
    numerosity <- rep(dim(full_data)[2] - portion_len + 1,times = dim(full_data)[1])
    removed_rows <- setdiff(window_data_list[[2]],rownames(window_data))
    curve_indices <- as.numeric(gsub("^(\\d+)_.*", "\\1", removed_rows))
    tab <- table(curve_indices)
    numerosity[as.numeric(names(tab))] <- numerosity[as.numeric(names(tab))] - tab
    
    rm(window_data_list)
    rm(removed_rows)
    rm(tab)
  
    # createDistance function
  cppFunction('
  Rcpp::NumericMatrix createDistance(Rcpp::NumericMatrix& windowData,
                                     const arma::vec& numerosity,
                                     unsigned int worker_number){
  
  int outrows = windowData.nrow();
  int outcols = windowData.ncol();
  bool isSingle = (numerosity.size() == 1);
  
  double outcols_inv = 1.0 / static_cast<double>(outcols);
  const arma::mat windowDataRef(windowData.begin(),outrows,outcols,false,true);
  const arma::colvec& vsum = arma::sum(windowDataRef,1) * outcols_inv;
  Rcpp::NumericMatrix result(outrows,outrows);
  arma::mat scoreData(result.begin(),outrows,outrows,false,true);
  
#ifdef _OPENMP
#pragma omp parallel for num_threads(worker_number) schedule(static)
#endif
  for (arma::uword i = 1; i < outrows; ++i) { // for any functional observation
    const arma::rowvec& x_i = windowDataRef.row(i); // curve i 
    for (arma::uword j = 0; j < i; ++j) {
      
      // Precompute common terms
      const arma::rowvec& crossSum = x_i + windowDataRef.row(j);;
      const double commonTerm = arma::accu(crossSum) * (outcols_inv * 0.5);
      
      // fill the lower part
      scoreData(i,j) = arma::accu(arma::square(
                                   x_i - vsum[i] - crossSum/2
                                   + commonTerm))*outcols_inv;
    }
  }
  
  // Modify accolites 
  double M = static_cast<double>(*std::max_element(scoreData.begin(),scoreData.end())) + 1000.0; // very large distance for accolites
  int overlap = static_cast<int>(std::floor(outcols / 2.0)); // number of right/left accolites
  
  // Single curve
  if (isSingle) {
    for (int i = 0; i < outrows-1; ++i) {
      // check for right accolites
      arma::uword start = i+1;
      arma::uword end = std::min<int>(i+overlap,outrows-1);
      scoreData(arma::span(start,end),i) += M;
    }
    
    // Multiple curves
  } else {
    int until_here = 0;
    int numerosity_size = numerosity.size();
    for(int j = 0; j <numerosity_size;++j)
    {
      int num = numerosity[j] - 1;
      int loop_end = until_here + num;
#ifdef _OPENMP
#pragma omp parallel for num_threads(worker_number) schedule(static)
#endif
      for(int i = until_here; i < loop_end ;++i)
      {
        arma::uword start = i+1;
        arma::uword end = std::min<int>(std::min<int>(i+overlap ,i + (num--)),outrows-1);
        scoreData(arma::span(start,end),i) += M;
      }
      until_here += numerosity[j];
    }
  }
  return Rcpp::wrap(result);
}',depends = "RcppArmadillo")

    ## STEP 2 -----
    # step 2: compute fMRS-based dissimilarity matrix
    D_fmsr <- createDistance(window_data, numerosity, worker_number) # compute
    pb$update(0.5)
    rownames(D_fmsr) <- colnames(D_fmsr) <- rownames(window_data) # update rownames and colnames
    
    # STEP 3 -----
    # step 3: get the sub-trees (tree_s)
    minidend  <- get_minidend(as.dist(D_fmsr))
    
    # case where I am just changing the min_card
    } else if(any(rds_files_stats$len == portion_len)) { # I can reuse the minidend
        
      file_id <- which(rds_files_stats$len == portion_len)[1]
      resFunBiMeta <- readRDS(rds_files[file_id]) 
      
      minidend <- resFunBiMeta$minidend
      window_data <- resFunBiMeta$window_data
    }   
    
    # step 3: identify seeds and corresponding families
    # this part depends on min_card (it should be redone when min_card is changed)
    all_paths <- get_path_complete(minidend,
                                   window_data, 
                                   min_card = min_card,
                                   worker_number = worker_number)
    all_paths <- all_paths[!(lapply(all_paths, is.null) %>% unlist())] 
    pb$update(0.65)
    
    # step 3: get recommended nodes and their info (cardinality and score)
    # collect all recommended nodes (as an array of portion ids)
    all_recommended_labels <- lapply(all_paths, function(x){x$recommended_node_labels})
    list_of_recommendations <- lapply(rapply(
      all_recommended_labels, enquote, how='unlist'),
      eval)
    
    # get recommended node cardinality
    vec_of_card <- lapply(list_of_recommendations, length) %>% unlist()
    
    # get vector of adjusted fMSR
    vec_of_scores <- lapply(all_paths, function(x){x$recommended_node_scores}) %>% unlist()
    
    # get list of candidates
    list_of_candidates <- lapply(list_of_recommendations,
                                 function(x){
                                   window_data[x,]
                                 })
    
    # create the saved (as an .rds) object
    resFunBiMeta = list(window_data = window_data, # original window_data
                        minidend = minidend, # minidend so that you can reuse it with different min_card
                        list_of_recommendations = list_of_recommendations, # list of portion IDs for each motif
                        vec_of_scores = vec_of_scores, # vec of scores for each motif
                        vec_of_card = vec_of_card, # vec of cardinalities
                        list_of_candidates = list_of_candidates) # list of candidate motifs (matrices)
    
    # save the COMPLETE results: unordered and unfiltered.
    #saveRDS(resFunBiMeta,file = paste0(name,'/resFunBi.rds'))
    saveRDS(resFunBiMeta,
            file = paste0(name,
            '/completeFunBIalign_portion_len_',
            portion_len,
            '_min_card_', min_card,
            '.rds')) 
    
    # Now that the file is generated we can proceed with STEP 4: post-processing
    } else if(any(rds_files_stats$len == portion_len & rds_files_stats$card == min_card) == TRUE){ # if the file DOES exist then I just do step 4
      
      # get the data with same portion_len and min_card
      file_id <- which((rds_files_stats$len == portion_len) & (rds_files_stats$card == min_card))
      resFunBiMeta <- readRDS(rds_files[file_id]) 
      
      # get all the info
      window_data <- resFunBiMeta$window_data
      minidend <- resFunBiMeta$minidend
      list_of_recommendations <-  resFunBiMeta$list_of_recommendations
      vec_of_scores <- resFunBiMeta$vec_of_scores
      vec_of_card <-  resFunBiMeta$vec_of_card
      list_of_candidates <- resFunBiMeta$list_of_candidates
    }
    
    ## STEP 4 -----
    # STEP 4: post-processing and rearranging results using different criteria
    pb$update(0.8)
    if (stopCriterion == "variance") { # "variance"
      ### variance ----
      motif_var <- lapply(list_of_recommendations, 
                          function(x){
                            temp <- window_data[x,]
                            temp_mean <- temp %>% colMeans()
                            ((temp - temp_mean)^2 %>% sum())/(nrow(temp))
                          }) %>% unlist()
      
      var_order <- motif_var %>% order(decreasing = TRUE)
      
      vec_of_scores_ordered <- vec_of_scores[var_order]
      vec_of_card_ordered <- vec_of_card[var_order]
      list_of_candidates_ordered <- list_of_candidates[var_order]
      list_of_recommendations_ordered <- list_of_recommendations[var_order]
      
    }else if(stopCriterion == "hybrid"){ # hybrid
      ### hybrid: ranks sum -----
      hy_order <- (rank(-vec_of_card) + rank(vec_of_scores)) %>% order()
      
      vec_of_scores_ordered <- vec_of_scores[hy_order]
      vec_of_card_ordered <- vec_of_card[hy_order]
      list_of_candidates_ordered <- list_of_candidates[hy_order]
      list_of_recommendations_ordered <- list_of_recommendations[hy_order]
      
    }else if(stopCriterion == "fMSR"){ #fMSR (default)
      ### adjusted fMSR -----
      best_order <- vec_of_scores %>% order()
      
      vec_of_scores_ordered <- vec_of_scores[best_order]
      vec_of_card_ordered <- vec_of_card[best_order]
      list_of_candidates_ordered <- list_of_candidates[best_order]
      list_of_recommendations_ordered <- list_of_recommendations[best_order]
    }
    
    #for every recommended motif, compute all its accolites
    all_accolites <- lapply(list_of_recommendations_ordered,
                            function(x){
                              lapply(x, get_accolites, window_data, portion_len, FALSE) %>% unlist()
                            })
    
    # Starting from the top, we compare each motif to those with higher rank. If all portions of
    # are acolytes to portions of an higher ranking motif, we filter it out; 
    # otherwise we retain it.

    # we identify the ones to delete
    cppFunction('Rcpp::IntegerVector deleteV(const Rcpp::List& list_of_recommendations_ordered,
                                             const Rcpp::List&  all_accolites,
                                             const Rcpp::NumericVector& vec_of_scores_ordered){
  int n = list_of_recommendations_ordered.size();
  Rcpp::IntegerVector del;
  
  for (int i = 1; i < n; ++i) { // compare every motif (from the second one)
    const Rcpp::CharacterVector& node_1 = list_of_recommendations_ordered[i];
    for (int j = 0; j < i; ++j) { // to the other higher ranked nodes
      const Rcpp::CharacterVector& node_2 = list_of_recommendations_ordered[j];
      if (node_1.size() <= node_2.size()) {
        const Rcpp::CharacterVector& accolites_2 = all_accolites[j];
        if (vec_of_scores_ordered(i) > vec_of_scores_ordered(j) && 
            Rcpp::is_true(Rcpp::all(Rcpp::in(node_1, accolites_2)))){
          del.push_back(i+1);
          break;
        }
      }
    }
  }
  
  return del;
  
}',depends="RcppArmadillo")
    
    delete <- deleteV(list_of_recommendations_ordered,
                      all_accolites,
                      vec_of_scores_ordered)
    
    # we delete the accolytes and we order the remaining ones
    vec_of_scores_ordered <- vec_of_scores_ordered[-delete]
    vec_of_card_ordered <- vec_of_card_ordered[-delete]
    list_of_candidates_ordered <- list_of_candidates_ordered[-delete]
    list_of_recommendations_ordered <- list_of_recommendations_ordered[-delete]
    
    if(!is.null(cut_off)) {
      valid_pos <- which(vec_of_scores_ordered < cut_off)
      
      vec_of_scores_ordered <- vec_of_scores_ordered[valid_pos]
      vec_of_card_ordered <- vec_of_card_ordered[valid_pos]
      list_of_candidates_ordered <- list_of_candidates_ordered[valid_pos] 
      list_of_recommendations_ordered <- list_of_recommendations_ordered[valid_pos]
      
      if(is.null(vec_of_scores_ordered))
        stop("the value of \'cut_off\' is to high. No motifs discovered. Provide a lower value")
    }
    pb$update(0.9)
    # Creating some plot ----
    ## Plot the data and highlight the motif occurrences in red -----
    if(plot)
    {
      pdf(paste0(name,
                 '/FunBIalign_portion_len_', portion_len,
                 '_min_card_', min_card,
                 '_plot_', stopCriterion,
                 '.pdf'),
          width=10,
          height=5)
      
      layout(matrix(1:2,ncol=2,byrow=TRUE),widths=c(8.5,1))
      for(q in 1:length(list_of_recommendations_ordered)){
        temp_motif <- list_of_recommendations_ordered[[q]]
        lots_in_motif <- lapply(temp_motif,
                                function(x){
                                  strsplit(x, '_') %>% 
                                    unlist() %>%
                                    as.numeric()}) %>% 
          unlist() %>%
          matrix(ncol=3, byrow=T)
        current_curves <- unique(as.numeric(gsub("[^0-9]", "", substr(temp_motif, 1, 2))))
        title   <- paste0("Number of instances: ", length(temp_motif),
                          " - adj fMSR:  ", vec_of_scores_ordered[q] %>% round(3))
        par(mar = c(5, 4, 2, 2) + 0.1)
        matplot(t(full_data), ylab='', xlab='',
                lwd=1.5,lty = 1, type = 'l', main = title,col=alpha('gray30',0.15))
        matplot(matrix(full_data[current_curves,],ncol=length(current_curves),byrow = TRUE), ylab='', xlab='',
                lwd=1.5,lty = 5, type = 'l', main = title,col=rainbow(length(current_curves)),add=TRUE)
        rect(lots_in_motif[,2], min(full_data,na.rm = TRUE)-10, lots_in_motif[,3], max(full_data,na.rm = TRUE) +10,
             border = alpha("firebrick3", 0.05), col = alpha("firebrick3", 0.05))
      
        lapply(1:nrow(lots_in_motif), function(k) {
          matplot(lots_in_motif[k, 2]:lots_in_motif[k, 3],
                  full_data[lots_in_motif[k, 1], lots_in_motif[k, 2]:lots_in_motif[k, 3]],
                  type = 'l', add = TRUE, col = 'red', lwd = 2.5)
          #box(col = "grey40", lwd = 2)
          #axis(1, at = seq(1, length(Y0), by = 12), col = "grey40", col.ticks = "grey40", col.axis = "grey60", cex.axis = 1.5)
          #axis(2, col = "grey40", col.ticks = "grey40", col.axis = "grey60", cex.axis = 1.5)
        })
        par(mar=c(0,0,0,0))
        plot.new()
        legend("topright", 
               legend = c(paste0("c",current_curves),paste0('motif_',q)),
               col = c(rainbow(length(current_curves)),'red'),lwd=c(rep(1,length(current_curves)),4), lty = 1,cex=0.75)
      }
      layout(matrix(1:2,ncol=2,byrow=TRUE),widths=c(5,1))
      ## Plot only the motif -----
      for(q in 1:length(list_of_recommendations_ordered)){
        par(mar = c(5, 4, 2, 2) + 0.1)
        temp_motif <- list_of_recommendations_ordered[[q]]
        current_curves <- as.numeric(gsub("[^0-9]", "", substr(temp_motif, 1, 2)))
        plot_me <- t(window_data[temp_motif,])
        motifMean <- rowMeans(plot_me)
        title   <- paste0("Number of occurrences: ", dim(plot_me)[2],
                          " - adj fMSR:  ", vec_of_scores_ordered[q] %>% round(3))
        
        matplot(plot_me, ylab='', xlab='',type='l',
                col=seq_len(length(temp_motif))+1,lwd=2,lty=5, main = title)
        lines(motifMean,col='black',lwd=5,lty=1)
        par(mar=c(0,0,0,0))
        plot.new()
        legend('topright',
               legend=c(paste0('c',current_curves),'motif center'),
               col=c(seq_len(length(temp_motif))+1,'black'),lwd=c(rep(2,length(temp_motif)),5),lty=c(rep(5,length(temp_motif)),1),
               xpd=TRUE,cex=0.75)
        #box(col="grey40", lwd = 2)
        #axis(1, at=1:portion_len, labels = 1:portion_len, col="grey40", col.ticks="grey40", col.axis="grey60", cex.axis=1.5)
        #axis(2, col="grey40", col.ticks="grey40", col.axis="grey60", cex.axis=1.5)
      }
      dev.off()
    }
    pb$update(1.0)
    resFunBi = list(window_data = window_data,
                    list_of_recommendations_ordered = list_of_recommendations_ordered,
                    vec_of_scores_ordered = vec_of_scores_ordered,
                    vec_of_card_ordered = vec_of_card_ordered,
                    list_of_candidates_ordered = list_of_candidates_ordered)
    
    return(resFunBi)
  } else {
    stop('\'method\' not found. It must be choosen between \'probKMA\' and \'funBIalign\'')
  }
}

#' @title Plot the Results of probKMA
#'
#' @description
#' The `probKMA_plot` function visualizes the results obtained from the `probKMA` analysis. It generates a series of plots
#' including motif memberships across different curves, the progression of the objective function over iterations,
#' and the Bhattacharyya distance between memberships. Depending on the parameters, it can plot both original
#' and cleaned motifs across multiple dimensions, providing insights into the embedding and characteristics of
#' identified motifs.
#'
#' @param probKMA_results A list containing the output from the `probKMA` function. This list should include elements such as:
#'   \describe{
#'     \item{Y0}{A list of matrices representing the original curves.}
#'     \item{Y1}{A list of matrices representing the derivatives of the curves (if applicable).}
#'     \item{V0}{A list of motifs.}
#'     \item{V1}{A list of derived motifs (if applicable).}
#'     \item{P}{A matrix indicating motif memberships across curves.}
#'     \item{S}{A matrix indicating motif start positions in curves.}
#'     \item{S_clean}{A matrix indicating cleaned motif start positions (if applicable).}
#'     \item{P_clean}{A matrix indicating cleaned motif memberships (if applicable).}
#'     \item{V0_clean}{A list of cleaned motifs (if applicable).}
#'     \item{V1_clean}{A list of derived cleaned motifs (if applicable).}
#'     \item{iter}{An integer indicating the number of iterations performed.}
#'     \item{J_iter}{A numeric vector recording the objective function value at each iteration.}
#'     \item{BC_dist_iter}{A numeric vector recording the Bhattacharyya distance between memberships at each iteration.}
#'   }
#'
#' @param plot A logical flag indicating whether to produce the plots. If `TRUE`, the function generates all relevant plots. If `FALSE`, no plots are produced.
#'
#' @param ylab A character vector of length `d`, providing labels for the y-axis in each dimension. Defaults to an empty string (`''`) for all dimensions.
#'
#' @param sil_avg A numeric vector containing the average silhouette scores for each embedded motif. This parameter is used to annotate the plots with silhouette information. Defaults to `NULL`, meaning no silhouette scores are displayed.
#'
#' @param cleaned A logical value indicating whether to plot only the cleaned motifs (`TRUE`) or all motifs (`FALSE`). When set to `TRUE`, the function highlights motifs that have been cleaned based on predefined criteria. Defaults to `FALSE`.
#'
#' @param transformed A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.
#'
#' @return 
#' The function generates a series of plots visualizing:
#' \itemize{
#'   \item \strong{Motifs with Matched Curves}: Displays the original curves with embedded motifs overlaid. If `cleaned = TRUE`, only cleaned motifs are highlighted.
#'   \item \strong{Memberships}: Shows bar plots representing the membership scores of each motif across all curves.
#'   \item \strong{Objective Function}: Plots the progression of the objective function (`J_iter`) over iterations to illustrate convergence.
#'   \item \strong{Bhattacharyya Distance}: Plots the Bhattacharyya distance (`BC_dist_iter`) between memberships over iterations to assess similarity.
#' }
#' No value is returned; the function is used solely for its side effects of generating visualizations.
#'
#' @details
#' The `probKMA_plot` function performs the following operations:
#' \enumerate{
#'   \item **Motif Visualization**:
#'     \itemize{
#'       \item Plots the original curves (`Y0`) with embedded motifs (`V0`). If derivatives (`Y1` and `V1`) are available, additional plots are generated for them.
#'       \item When `cleaned = TRUE`, the function highlights only the cleaned motifs (`V0_clean` and `V1_clean`), providing a clearer view of significant motifs.
#'       \item Utilizes color coding and legends to differentiate between different motifs and their instances.
#'     }
#'   \item **Memberships**:
#'     \itemize{
#'       \item Generates bar plots showing the membership scores (`P` or `P_clean`) of each motif across all curves.
#'       \item Provides a visual representation of how strongly each motif is associated with different curves.
#'     }
#'   \item **Objective Function and Bhattacharyya Distance**:
#'     \itemize{
#'       \item Plots the objective function (`J_iter`) over the iterations to demonstrate the optimization process.
#'       \item Plots the Bhattacharyya distance (`BC_dist_iter`) to measure the similarity between motif memberships across iterations.
#'     }
#' }
#' 
#' The function is designed to handle multiple dimensions (`d`) and can accommodate both original and derivative data if provided. 
#' It also supports the visualization of cleaned motifs, which are motifs that have been refined based on specific criteria to ensure quality and relevance.
#'
#' @export
probKMA_plot <- function(probKMA_results, plot, ylab='',sil_avg=NULL,cleaned = FALSE, transformed = FALSE){
  d=ncol(probKMA_results$Y0[[1]])
  N=nrow(probKMA_results$P)
  K=ncol(probKMA_results$P)
  V_dom=lapply(probKMA_results$V0,function(v) rowSums(!is.na(v))!=0)
  S_k=split(probKMA_results$S,rep(seq_len(K),each=N))
  P_k=split(probKMA_results$P,rep(seq_len(K),each=N))
  
  ### plot motifs with matched curves #######################################################################
  if(cleaned){
    S_clean_k=split(probKMA_results$S_clean,rep(seq_len(K),each=N))
    P_clean_k=split(probKMA_results$P_clean,rep(seq_len(K),each=N))
    S_clean_i=split(probKMA_results$S_clean,seq_len(nrow(probKMA_results$S_clean)))
    P_clean_i=split(probKMA_results$P_clean,seq_len(nrow(probKMA_results$S_clean)))
    has_a_motif <- sapply(P_clean_i,function(p_i){sum(p_i)!=0}) # at least one motif embedded
    if(is.null(probKMA_results$V1[[1]])){
      NULL
      #### Plot curves all together with the embedded motifs -------------
      # colorful plot (deleted)
      # mapply(function(v0,v_dom,s_k,p_clean_k,k)
      #   {
      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      #   keep=which(p_clean_k==1)
      #   # matrix extension (eventually)
      #   full_mat <- probKMA_results$Y0
      #   # max len of the curves
      #   maxLen <-  max(sapply(full_mat, nrow))
      #   # Extended full matrix
      #   full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
      #   # Join
      #   full_mat <- do.call(cbind, full_mat)
      #   dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
      #   # Extract keep submatrix
      #   column <- numeric(length(keep) * d)
      #   for (i in 1:length(keep)) {
      #     start_col <- (keep[i] - 1) * d + 1
      #     end_col <- start_col + d - 1
      #     column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
      #   }
      #   # Plot curves all together 
      #   lapply(seq_len(d),
      #          function(j){
      #            par(mar=c(3,4,4,2)+0.1)
      #            matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
      #            univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add=TRUE)
      #            for (i in 1:ncol(dom_sequence)) {
      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
      #            }
      #            title(sub = paste('Curves & ', 'Motif:', k, ' - Dimension:', j))
      #            par(mar=c(0,0,0,0))
      #            plot.new()
      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
      #          })
      #   return()},probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
      # 
      #### for each curve plot the embedded motifs  ############################################################################################
      # single line plot
      
      # mapply(function(curve_i,s_i,p_clean_i,i){
      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      #   lapply(seq_len(d),
      #          function(j)
      #            {
      #             par(mar=c(3,4,4,2)+0.1)
      #             univariate_mat <- curve_i[,j]
      #             matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
      #                     main=paste0('c',i,' - Dimension:',j),
      #                     #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
      #                     xlab='domain')
      #             # plot motifs with different colors 
      #             col_i <- 1
      #             col <- rainbow(sum(p_clean_i))
      #             lapply(seq_len(K),function(k)
      #              {
      #               if(p_clean_i[k]==1)
      #               {
      #                 univariate_motif <- probKMA_results$V0_clean[[k]][,j]
      #                 v_dom <- V_dom[[k]]
      #                 dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
      #                 if(transformed)
      #                 {
      #                   max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
      #                   min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
      #                   univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
      #                 }
      #                 lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
      #                 rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                                 border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
      #                 col_i <<- col_i + 1
      #               }
      #               })
      #               par(mar=c(0,0,0,0))
      #               plot.new()
      #               legend('left',legend=paste0('motif',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
      #               })
      #   return()},probKMA_results$Y0[ has_a_motif] # consider curves with at least one embedded motif
      #             ,S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
      # mapply(function(v,v_dom,s_k,p_clean_k,k){
      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      #   keep=which(p_clean_k==1)
      #   Y_inters_k=mapply(
      #     function(y,s_k_i,v_dom){
      #       v_len=length(v_dom)
      #       d=ncol(y)
      #       y_len=nrow(y)
      #       index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
      #       Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
      #                        matrix(y[index[index<=y_len],],ncol=d),
      #                        matrix(NA,nrow=sum(index>y_len),ncol=d))
      #       return(Y_inters_k)},
      #     probKMA_results$Y0[keep],s_k[keep],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
      #   Y0_diff_k=lapply(Y_inters_k,
      #                    function(Y0_inters_k){
      #                      y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
      #                      y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
      #                      y0_diff=y0_max-y0_min
      #                      return(y0_diff)
      #                    })
      #   lapply(seq_len(d),
      #          function(j){
      #            par(mar=c(3,4,4,2)+0.1)
      #            y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y_inters_k))
      #            if(transformed){
      #              y_plot[v_dom,]=Reduce('cbind',
      #                                    mapply(function(Y_inters_k, Y_diff_k) {
      #                                      y0_min=min(Y_inters_k[,j])
      #                                      y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
      #                                      y0_const = (Y_diff_k[j] == 0)
      #                                      y0_norm[,y0_const] = 0.5
      #                                      return(y0_norm)},
      #                                      Y_inters_k, Y0_diff_k, SIMPLIFY=FALSE) ) 
        #          }else{
        #            y_plot[v_dom,]=Reduce('cbind',lapply(Y_inters_k,function(Y_inters_k) Y_inters_k[,j]))
        #          }
        #          
        #          matplot(y_plot,type='l',col=seq_len(N)+1,lwd=1,lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
        #          points(v[,j],type='l',col='black',lwd=7,lty=1)
        #          par(mar=c(0,0,0,0))
        #          plot.new()
        #          legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
        #        })
        # return()},
        # probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
    }else{
      print("working on it")
      ##### Plot curves and derivatives all together with the embedded motifs  ############################################################################################
      # colorful plot with everything together
      
      # mapply(function(v0,v1,v_dom,s_k,p_clean_k,k)
      # {
      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      #   keep=which(p_clean_k==1)
      #   # matrix extension (eventually)
      #   full_mat <- probKMA_results$Y0
      #   full_mat_dev <- probKMA_results$Y1
      #   # max len of the curves
      #   maxLen <-  max(sapply(full_mat, nrow))
      #   # Extended full matrix
      #   full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
      #   full_mat_dev <- sapply(full_mat_dev, padding, maxLen,simplify = FALSE)
      #   # Join
      #   full_mat <- do.call(cbind, full_mat)
      #   full_mat_dev <- do.call(cbind, full_mat_dev)
      #   dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
      #   # Extract "keep" submatrix
      #   column <- numeric(length(keep) * d)
      #   for (i in 1:length(keep)) {
      #     start_col <- (keep[i] - 1) * d + 1
      #     end_col <- start_col + d - 1
      #     column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
      #   }
      #   # Plot curves all together 
      #   lapply(seq_len(d),
      #          function(j){
      #            par(mar=c(3,4,4,2)+0.1)
      #            matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
      #            univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
      #            for (i in 1:ncol(dom_sequence)) {
      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
      #            }
      #            title(sub = paste('Curves & ', 'Motif_', k, ' - Dimension:', j))
      #            par(mar=c(0,0,0,0))
      #            plot.new()
      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
      #          })
      #   # Plot Derivatives all together 
      #   lapply(seq_len(d),
      #          function(j){
      #            par(mar=c(3,4,4,2)+0.1)
      #            matplot(full_mat_dev[,seq(j,ncol(full_mat_dev),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
      #                    main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
      #            univariate_mat <- as.matrix(as.matrix(full_mat_dev[,column])[,seq(j,length(keep),by = d)])
      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
      #                    main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
      #            for (i in 1:ncol(dom_sequence)) {
      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
      #            }
      #            title(sub = paste('Derivatives & ', 'Motif:', k, ' - Dimension:', j))
      #            par(mar=c(0,0,0,0))
      #            plot.new()
      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
      #          })
      #   
      #   return()},probKMA_results$V0_clean,probKMA_results$V1_clean,
      #             V_dom,S_clean_k,P_clean_k,seq_len(K))
      
    ##### for each curve plot the embedded motifs  ############################################################################################
    # for each curve the motif highlighted
      
      # mapply(function(curve_i,dev_i,s_i,p_clean_i,i){
      # #plot curve with embedded motif 
      # layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      # lapply(seq_len(d),
      #        function(j)
      #        {
      #          par(mar=c(3,4,4,2)+0.1)
      #          univariate_mat <- curve_i[,j]
      #          matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
      #                  main=paste0('c',i,' - Dimension:',j),
      #                  #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
      #                  xlab='domain')
      #          # plot motifs with different colors 
      #          col_i <- 1
      #          col <- rainbow(sum(p_clean_i))
      #          lapply(seq_len(K),function(k)
      #          {
      #            if(p_clean_i[k]==1)
      #            {
      #              univariate_motif <- probKMA_results$V0_clean[[k]][,j]
      #              v_dom <- V_dom[[k]]
      #              dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
      #              if(transformed)
      #              {
      #                max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
      #                min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
      #                univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
      #              }
      #              lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
      #              rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                   border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
      #              col_i <<- col_i + 1
      #            }
      #          })
      #          par(mar=c(0,0,0,0))
      #          plot.new()
      #          legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
      #        })
      
      #plot derivative with embedded motif 
      #for each derivative curve the motif highlighted
      
      # layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
      # lapply(seq_len(d),
      #        function(j)
      #        {
      #          par(mar=c(3,4,4,2)+0.1)
      #          univariate_mat_curve <- curve_i[,j]
      #          univariate_mat <- dev_i[,j]
      #          matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
      #                  main=paste0('c',i,' - Dimension:',j),
      #                  ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
      #                  xlab='domain')
      #          # plot motifs with different colors 
      #          col_i <- 1
      #          col <- rainbow(sum(p_clean_i))
      #          lapply(seq_len(K),function(k)
      #          {
      #            if(p_clean_i[k]==1)
      #            {
      #              univariate_motif <- probKMA_results$V1_clean[[k]][,j]
      #              v_dom <- V_dom[[k]]
      #              dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
      #              if(transformed)
      #              {
      #                max_num = max(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
      #                min_num = min(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
      #                univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num)
      #              }
      #              lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
      #              rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
      #                   border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
      #              col_i <<- col_i + 1
      #            }
      #          })
      #          par(mar=c(0,0,0,0))
      #          plot.new()
      #          legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
      #        })
      # 
      # return()},probKMA_results$Y0[ has_a_motif], # consider curves with at least one embedded motif
      #           probKMA_results$Y1[ has_a_motif],
      #           S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
    
      mapply(function(v0,v1,v_dom,s_k,p_clean_k,k){
        keep=which(p_clean_k==1)
        Y0_inters_k=mapply(
          function(y,s_k_i,v_dom){
            v_len=length(v_dom)
            d=ncol(y)
            y_len=nrow(y)
            index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
            Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
                             matrix(y[index[index<=y_len],],ncol=d),
                             matrix(NA,nrow=sum(index>y_len),ncol=d))
            return(Y_inters_k)},
          probKMA_results$Y0[keep],s_k[keep],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
        Y1_inters_k=mapply(
          function(y,s_k_i,v_dom){
            v_len=length(v_dom)
            d=ncol(y)
            y_len=nrow(y)
            index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
            Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
                             matrix(y[index[index<=y_len],],ncol=d),
                             matrix(NA,nrow=sum(index>y_len),ncol=d))
            return(Y_inters_k)},
          probKMA_results$Y1[keep],s_k[keep],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
        Y0_diff_k=lapply(Y0_inters_k,
                         function(Y0_inters_k){
                           y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
                           y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
                           y0_diff=y0_max-y0_min
                           return(y0_diff)
                         })
        lapply(seq_len(d),
               function(j){
                 par(mar=c(3,4,4,2)+0.1)
                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y0_inters_k))
                 if(transformed){
                   y_plot[v_dom,]=Reduce('cbind',
                                         mapply(function(Y_inters_k, Y_diff_k) {
                                           y0_min=min(Y_inters_k[,j])
                                           y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
                                           y0_const = (Y_diff_k[j] == 0)
                                           y0_norm[,y0_const] = 0.5
                                           return(y0_norm)},
                                           Y0_inters_k, Y0_diff_k, SIMPLIFY=FALSE) )
                 }else{
                   y_plot[v_dom,]=Reduce('cbind',lapply(Y0_inters_k,function(Y_inters_k) Y_inters_k[,j]))}
                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=1,lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
                 points(v0[,j],type='l',col='black',lwd=7,lty=1)
                 par(mar=c(0,0,0,0))
                 plot.new()
                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
               })
        lapply(seq_len(d),
               function(j){
                 par(mar=c(3,4,4,2)+0.1)
                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y1_inters_k))
                 if(transformed){
                   y_plot[v_dom,]=Reduce('cbind',
                                         mapply(function(Y1_inters_k, Y_diff_k) {
                                           y1_norm = t( t(Y1_inters_k[,j])/ Y_diff_k[j] )
                                           y0_const = (Y_diff_k[j] == 0)
                                           y1_norm[,y0_const] = 0
                                           return(y1_norm)},
                                           Y1_inters_k, Y0_diff_k, SIMPLIFY=FALSE)
                   ) 
                 }else{
                   y_plot[v_dom,]=Reduce('cbind',lapply(Y1_inters_k,function(Y_inters_k) Y_inters_k[,j]))}
                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=1,lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j],' derivative'))
                 points(v1[,j],type='l',col='black',lwd=7,lty=1)
                 par(mar=c(0,0,0,0))
                 plot.new()
                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
               })
        return()},
        probKMA_results$V0_clean,probKMA_results$V1_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
    }
    
  }
  else
  {
    if(is.null(probKMA_results$V1[[1]])){
      mapply(function(v,v_dom,s_k,p_k,k){
        Y_inters_k=mapply(function(y,s_k_i,v_dom){
          v_len=length(v_dom)
          d=ncol(y)
          y_len=nrow(y)
          index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
          Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
                           matrix(y[index[index<=y_len],],ncol=d),
                           matrix(NA,nrow=sum(index>y_len),ncol=d))
          return(Y_inters_k)},
          probKMA_results$Y0,s_k,MoreArgs=list(v_dom),SIMPLIFY=FALSE)
        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
        Y0_diff_k=lapply(Y_inters_k,
                         function(Y0_inters_k){
                           y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
                           y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
                           y0_diff=y0_max-y0_min
                           return(y0_diff)
                         })
        lapply(seq_len(d),
               function(j){
                 par(mar=c(3,4,4,2)+0.1)
                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y_inters_k))
                 if(transformed){
                   y_plot[v_dom,]=Reduce('cbind',
                                         mapply(function(Y_inters_k, Y_diff_k) {
                                           y0_min=min(Y_inters_k[,j])
                                           y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
                                           y0_const = (Y_diff_k[j] == 0)
                                           y0_norm[,y0_const] = 0.5
                                           return(y0_norm)},
                                           Y_inters_k, Y0_diff_k, SIMPLIFY=FALSE) )
                 }else{
                   y_plot[v_dom,]=Reduce('cbind',lapply(Y_inters_k,function(Y_inters_k) Y_inters_k[,j]))}
                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=round(5*p_k,2),lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
                 points(v[,j],type='l',col='black',lwd=7,lty=1)
                 par(mar=c(0,0,0,0))
                 plot.new()
                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
               })
        return()},
        probKMA_results$V0,V_dom,S_k,P_k,seq_len(K))
    }else{
      mapply(function(v0,v1,v_dom,s_k,p_k,k){
        Y0_inters_k=mapply(function(y,s_k_i,v_dom){
          v_len=length(v_dom)
          d=ncol(y)
          y_len=nrow(y)
          index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
          Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
                           matrix(y[index[index<=y_len],],ncol=d),
                           matrix(NA,nrow=sum(index>y_len),ncol=d))
          return(Y_inters_k)},
          probKMA_results$Y0,s_k,MoreArgs=list(v_dom),SIMPLIFY=FALSE)
        Y1_inters_k=mapply(function(y,s_k_i,v_dom){
          v_len=length(v_dom)
          d=ncol(y)
          y_len=nrow(y)
          index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
          Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
                           matrix(y[index[index<=y_len],],ncol=d),
                           matrix(NA,nrow=sum(index>y_len),ncol=d))
          return(Y_inters_k)},
          probKMA_results$Y1,s_k,MoreArgs=list(v_dom),SIMPLIFY=FALSE)
        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
        Y0_diff_k=lapply(Y0_inters_k,
                         function(Y0_inters_k){
                           y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
                           y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
                           y0_diff=y0_max-y0_min
                           return(y0_diff)
                         })
        lapply(seq_len(d),
               function(j){
                 par(mar=c(3,4,4,2)+0.1)
                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y0_inters_k))
                 if(transformed){
                   y_plot[v_dom,]=Reduce('cbind',
                                         mapply(function(Y_inters_k, Y_diff_k) {
                                           y0_min=min(Y_inters_k[,j])
                                           y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
                                           y0_const = (Y_diff_k[j] == 0)
                                           y0_norm[,y0_const] = 0.5
                                           return(y0_norm)},
                                           Y0_inters_k, Y0_diff_k, SIMPLIFY=FALSE) )
                 }else{
                   y_plot[v_dom,]=Reduce('cbind',lapply(Y0_inters_k,function(Y_inters_k) Y_inters_k[,j]))}
                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=round(5*p_k,2),lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
                 points(v0[,j],type='l',col='black',lwd=7,lty=1)
                 par(mar=c(0,0,0,0))
                 plot.new()
                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
               })
        lapply(seq_len(d),
               function(j){
                 par(mar=c(3,4,4,2)+0.1)
                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y1_inters_k))
                 if(transformed){
                   y_plot[v_dom,]=Reduce('cbind',
                                         mapply(function(Y1_inters_k, Y_diff_k) {
                                           y1_norm = t( t(Y1_inters_k[,j])/ Y_diff_k[j] )
                                           y0_const = (Y_diff_k[j] == 0)
                                           y1_norm[,y0_const] = 0
                                           return(y1_norm)},
                                           Y1_inters_k, Y0_diff_k, SIMPLIFY=FALSE)
                   ) 
                 }else{
                   y_plot[v_dom,]=Reduce('cbind',lapply(Y1_inters_k,function(Y_inters_k) Y_inters_k[,j]))}
                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=round(5*p_k,2),lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j],' derivative'))
                 points(v1[,j],type='l',col='black',lwd=7,lty=1)
                 par(mar=c(0,0,0,0))
                 plot.new()
                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
               })
        return()},
        probKMA_results$V0,probKMA_results$V1,V_dom,S_k,P_k,seq_len(K))
    }
  }
  
  layout(matrix(1:(2*d), ncol=2, byrow=TRUE), widths=c(8.5,1))
  ### plot motifs ############################################################################################
  if(cleaned){
    lapply(seq_len(d),
           function(j){
             par(mar=c(3,4,4,2)+0.1)
             motif_length=unlist(lapply(probKMA_results$V0_clean,nrow))
             plot(probKMA_results$V0_clean[[1]][,j],type='l',col=2,lwd=7,lty=1,main=ylab[j],xlim=c(1,max(motif_length)),
                  ylab=ylab[j],ylim=c(min(unlist(probKMA_results$V0_clean),na.rm=TRUE),max(unlist(probKMA_results$V0_clean),na.rm=TRUE)))
             mapply(function(v,k) points(v[,j],type='l',col=k+2,lwd=7,lty=1,ylab=ylab),
                    probKMA_results$V0_clean[-1],seq_len(K-1))
             par(mar=c(0,0,0,0))
             plot.new()
             legend('left',paste('motif_',seq_len(K)),col=seq_len(K)+1,lwd=7,lty=1,bty="n",xpd=TRUE)
             return()})
  }else{
    lapply(seq_len(d),
           function(j){
             par(mar=c(3,4,4,2)+0.1)
             motif_length=unlist(lapply(probKMA_results$V0,nrow))
             plot(probKMA_results$V0[[1]][,j],type='l',col=2,lwd=7,lty=1,main=ylab[j],xlim=c(1,max(motif_length)),
                  ylab=ylab[j],ylim=c(min(unlist(probKMA_results$V0),na.rm=TRUE),max(unlist(probKMA_results$V0),na.rm=TRUE)))
             mapply(function(v,k) points(v[,j],type='l',col=k+2,lwd=7,lty=1,ylab=ylab),
                    probKMA_results$V0[-1],seq_len(K-1))
             par(mar=c(0,0,0,0))
             plot.new()
             legend('left',paste('motif_',seq_len(K)),col=seq_len(K)+1,lwd=7,lty=1,bty="n",xpd=TRUE)
             return()})
  }
  if(!is.null(probKMA_results$V1[[1]])){
    layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
    if(cleaned){
      lapply(seq_len(d),
             function(j){
               par(mar=c(3,4,4,2)+0.1)
               motif_length=unlist(lapply(probKMA_results$V1_clean,nrow))
               plot(probKMA_results$V1_clean[[1]][,j],type='l',col=2,lwd=7,lty=1,main=paste0(ylab[j],' derivative'),xlim=c(1,max(motif_length)),
                    ylab=ylab[j],ylim=c(min(unlist(probKMA_results$V1_clean),na.rm=TRUE),max(unlist(probKMA_results$V1_clean),na.rm=TRUE)))
               mapply(function(v,k) points(v[,j],type='l',col=k+2,lwd=7,lty=1,ylab=ylab),
                      probKMA_results$V1_clean[-1],seq_len(K-1))
               par(mar=c(0,0,0,0))
               plot.new()
               legend('left',paste('motif_',seq_len(K)),col=seq_len(K)+1,lwd=7,lty=1,bty="n",xpd=TRUE)
               return()})
    }else{
      lapply(seq_len(d),
             function(j){
               par(mar=c(3,4,4,2)+0.1)
               motif_length=unlist(lapply(probKMA_results$V1,nrow))
               plot(probKMA_results$V1[[1]][,j],type='l',col=2,lwd=7,lty=1,main=paste0(ylab[j],' derivative'),xlim=c(1,max(motif_length)),
                    ylab=ylab[j],ylim=c(min(unlist(probKMA_results$V1),na.rm=TRUE),max(unlist(probKMA_results$V1),na.rm=TRUE)))
               mapply(function(v,k) points(v[,j],type='l',col=k+2,lwd=7,lty=1,ylab=ylab),
                      probKMA_results$V1[-1],seq_len(K-1))
               par(mar=c(0,0,0,0))
               plot.new()
               legend('left',paste('motif_',seq_len(K)),col=seq_len(K)+1,lwd=7,lty=1,bty="n",xpd=TRUE)
               return()})
    }
  }
  
  ### plot memberships #######################################################################################
  par(mfrow=c(K,1),mar=c(3,4,4,2)+0.1)
  if(cleaned){
    mapply(function(p_k,p_clean_k,k){
      col=rep('lightgray',N)
      col[p_clean_k==1]='gray35'
      barplot(p_k,names.arg=seq_len(N),col=col,las=2,ylim=c(0,1),ylab='memberships',main=paste('Motif_',k))
      legend('left',paste('p_clean==1'),col='gray35',pch=15,cex=2,bty="n",xpd=TRUE)
    },P_k,P_clean_k,seq_len(K))
  }else{
    mapply(function(p_k,k){
      barplot(p_k,names.arg=seq_len(N),col='gray',las=2,ylim=c(0,1),ylab='memberships',main=paste('Motif_',k))
    },P_k,seq_len(K))
  }
  
  ### plot objective function and Bhattacharyya distance #####################################################
  par(mfrow=c(1,1))
  plot(seq_len(probKMA_results$iter),probKMA_results$J_iter,type='l',xlab='iter',ylab='objective function',
       main=paste0('Objective function Jm:',round(tail(probKMA_results$J_iter,n=1)),4),lwd=2)
  points(seq_len(probKMA_results$iter), probKMA_results$J_iter,
         col = 'red', pch = 8, cex = 2,lwd=3)
  for (i in 1:probKMA_results$iter) {
    segments(i,0.0,i,probKMA_results$J_iter[i], col = "red",lwd=0.65)
  }
  
  plot(seq_len(probKMA_results$iter),probKMA_results$BC_dist_iter,type='l',xlab='iter',ylab='distance between memberships',
       main=paste0('Bhattacharyya distance between memberships',round(tail(probKMA_results$BC_dist_iter,n=1)),4),lwd=2)
  points(seq_len(probKMA_results$iter), probKMA_results$BC_dist_iter,
         col = 'red', pch = 8, cex = 2,lwd=3)
  for (i in 1:probKMA_results$iter) {
    segments(i,0.0,i,probKMA_results$BC_dist_iter[i], col = "red",lwd=0.65)
  }
  return()
}  

#' @title Find Minimum Dissimilarity
#' 
#' @description
#' Finds the shift warping that minimizes dissimilarity between multidimensional curves. 
#' The function operates on curves represented by two sets of data, each containing multiple dimensions.
#' 
#' @param y A list containing two matrices:
#'        - \code{y0}: The first set of curve values \(y(x)\).
#'        - \code{y1}: The second set of curve values \(y'(x)\).
#'        Each matrix should have \(d\) columns corresponding to the dimensions.
#' @param v A list containing two matrices:
#'        - \code{v0}: The first set of curve values \(v(x)\).
#'        - \code{v1}: The second set of curve values \(v'(x)\).
#'        Each matrix should have \(d\) columns corresponding to the dimensions.
#' @param alpha A numeric weight coefficient that balances the contributions of 
#'               the L2 norms of the two curve sets. 
#' @param w A numeric vector of weights for the dissimilarity index across different dimensions.
#'          All weights must be positive (\(w > 0\)).
#' @param c_k An integer specifying the minimum length of the intersection of the supports 
#'             of the shifted \(y\) and \(v\).
#' @param d An integer indicating the dimensionality of the curves.
#' @param use0 A logical value indicating whether to use the first component of the curves (i.e., \(y0\) and \(v0\)).
#' @param use1 A logical value indicating whether to use the second component of the curves (i.e., \(y1\) and \(v1\)).
#' @param transform_y A logical value indicating whether to normalize \(y\) to the range \([0,1]\) 
#'                    before calculating the distance.
#' @param transform_v A logical value indicating whether to normalize \(v\) to the range \([0,1]\) 
#'                    before calculating the distance.
#' 
#' @return A numeric vector containing:
#'         - The optimal shift that minimizes the dissimilarity.
#'         - The minimum dissimilarity value found.
#' 
#' @details
#' This function computes the shift warping between the provided multidimensional curves by 
#' examining various shifts and calculating the corresponding dissimilarity. The user can control 
#' which components of the curves to include in the calculation and whether to normalize the data.
#' 
#' The function returns both the optimal shift and the minimal dissimilarity, which can be used 
#' to assess the similarity between the two sets of curves under the specified constraints.
#' 
#' @author Marzia Angela Cremona & Francesca Chiaromonte
#' @export
.find_min_diss <- function(y,v,alpha,w,c_k,d,use0,use1,transform_y=FALSE,transform_v=FALSE){
  v_dom=.domain(v,use0)
  v=.select_domain(v,v_dom,use0,use1)
  v_len=length(v_dom)
  y_len=unlist(lapply(y,nrow))[1]
  s_rep=(1-(v_len-c_k)):(y_len-v_len+1+(v_len-c_k))
  y_rep=lapply(s_rep,
               function(i){
                 index=i-1+seq_len(v_len)
                 y_rep_i=list(y0=NULL,y1=NULL)
                 if(use0){
                   y_rep_i$y0=rbind(matrix(NA,nrow=sum(index<=0),ncol=d),
                                    as.matrix(y[[1]][index[(index>0)&(index<=y_len)],]),
                                    matrix(NA,nrow=sum(index>y_len),ncol=d))
                 }
                 if(use1){
                   y_rep_i$y1=rbind(matrix(NA,nrow=sum(index<=0),ncol=d),
                                    as.matrix(y[[2]][index[(index>0)&(index<=y_len)],]),
                                    matrix(NA,nrow=sum(index>y_len),ncol=d))
                 }
                 y_rep_i=.select_domain(y_rep_i,v_dom,use0,use1)
                 return(y_rep_i)
               })
  length_inter=unlist(lapply(y_rep,
                             function(y_rep_i){
                               if(use0)
                                 return(sum((!is.na(y_rep_i$y0[,1]))))
                               return(sum((!is.na(y_rep_i$y1[,1]))))
                             }))
  valid=length_inter>=c_k
  if(sum(valid)==0){
    valid[length_inter==max(length_inter)]=TRUE
  }
  s_rep=s_rep[valid]
  y_rep=y_rep[valid]
  d_rep=unlist(mapply(.diss_d0_d1_L2,y_rep,MoreArgs=list(v,w,alpha,transform_y,transform_v)))
  return(c(s_rep[which.min(d_rep)],min(d_rep)))
}

#' Transform Input to Matrix
#'
#' @title Transform to Matrix
#'
#' @description
#' This function transforms a numeric vector into a matrix. If the input is a single number, it creates a 1x1 matrix. 
#' If the input is a vector, it creates a 1xN matrix. If the input is already a matrix, it returns it unchanged.
#'
#' @param input A numeric vector or matrix to be transformed.
#'
#' @return A matrix representation of the input.
#'
#' @details
#' - If the input is neither a numeric vector nor a matrix, an error is raised.
#'
#' @export
.transform_to_matrix <- function(input) {
  if (is.vector(input)) {
    if (length(input) == 1) {
      # Se l'input è un singolo numero, trasformalo in una matrice 1x1
      result <- matrix(input, nrow = 1, ncol = 1)
    } else {
      # Se l'input è un vettore, trasformalo in una matrice 1xN
      result <- matrix(input, nrow = 1, ncol = length(input))
    }
  } else if (is.matrix(input)) {
    # Se l'input è già una matrice, non fare nulla
    result <- input
  } else {
    stop("Input must be numeric or a matrix")
  }
  return(result)
}

#' Transform List Structure
#'
#' @title Transform List Structure
#'
#' @description
#' This function modifies a list of structures by checking for the presence of a "with_noise" field. 
#' If absent, it creates a new sub-list containing relevant fields.
#'
#' @param lst A list containing elements to be transformed.
#' @param noise_str A string indicating the noise structure to apply.
#'
#' @return The transformed list with updated structures.
#'
#' @export
.transform_list <- function(lst,noise_str) {
  # Looping the list
  for (i in seq_along(lst)) {
    # check"with_noise" field
    if (!"with_noise" %in% names(lst[[i]])) {
      # Create a new sub-list
      background <- list(or_coeff = lst[[i]]$or_coeff,    
                         no_error_y = lst[[i]]$no_error_y)
      lst[[i]] <- list(
        basis = lst[[i]]$basis,
        background = background)
    }
  }
  return(lst)
}

#' Generate Curve Vector
#'
#' @title Generate Curve Vector from FD Object
#'
#' @description
#' This function generates a curve vector from a functional data (fd) object, with a specified step size and optional derivative.
#'
#' @param fd_curve A functional data object from which to generate the curve.
#' @param step_by A numeric value indicating the step size for generating the curve.
#' @param Lfdobj A numeric value indicating the order of the derivative to compute. Default is 0 (no derivative).
#'
#' @return A numeric vector representing the generated curve.
#'
#' @export
generate_curve_vector <- function(fd_curve, step_by = 1, Lfdobj = 0){
  # from an fd object generate the curve as a vector of len fixed by the fd_obj
  # with step equal to step_by. It generates the derivative Lfdobj
  x <- seq(0, fd_curve$basis$rangeval[2] - 1, by = step_by)
  y <- eval.fd(x, fd_curve, Lfdobj = Lfdobj)  # return the evaluation of the Lfdobj derivative
  return(y)
}


#' Add Error to Motif
#'
#' @title Add Additive Error to Motif
#'
#' @description
#' This function adds an additive noise to a given motif based on specified parameters. The noise is generated as a percentage of the standard deviation of the motif.
#'
#' @param or_y A numeric vector representing the original motif values.
#' @param noise_str A string or numeric vector indicating the structure of the noise to be added.
#' @param start_point A numeric vector indicating the starting points of motifs.
#' @param end_point A numeric vector indicating the ending points of motifs.
#' @param k An integer representing the current iteration or motif index.
#'
#' @return A numeric vector of the motif values with added noise.
#'
#' @importFrom utils tail
#' @importFrom stats rnorm sd var
#' @importFrom fda create.bspline.basis fd smooth.basis eval.fd
#' @export
add_error_to_motif <- function(or_y, noise_str, start_point, end_point,k){
  err_y <- or_y
  # --- additive noise
  # noise to be added as a percentage of sd(motif) with mean 0
  error_str_temp <- noise_str
  for(n in seq_along(start_point)) {
    if(length(start_point[n]:end_point[n]) - 1 > length(noise_str)) {
      
      warning(paste("error structure on row",k,"is long",length(noise_str),
                    "but the motif is long",length(start_point[n]:end_point[n]) - 1,". -- Extension"))
      # Last value
      last_value <- tail(noise_str, 1)
      
      # Extention 
      error_str_temp <- c(noise_str, rep(last_value, length(start_point[n]:end_point[n]) - length(noise_str)))
    }else if(length(start_point[n]:end_point[n]) - 1 < length(noise_str)) {
      
      warning(paste("error structure on row",k,"is long",length(noise_str),
                    "but the motif is long",length(start_point[n]:end_point[n]) - 1,". -- Truncation"))
      #Truncation
      error_str_temp <- noise_str[1:length(start_point[n]:end_point[n])]
    }
    
    noise_coeff <- rnorm(length(start_point[n]:end_point[n]) - 1,
                         mean = 0,
                         sd = sd(or_y[start_point[n]:end_point[n]])*error_str_temp)
  # add noise
    err_y[(start_point[n]+1):end_point[n]] <- err_y[(start_point[n]+1):end_point[n]] + noise_coeff
  }
  return(err_y)
}

#' Generate Background Curve
#'
#' @title Generate Background Curve
#'
#' @description
#' This function generates a background curve using B-splines with specified parameters, including knots, order, and weights. 
#' Optionally, additive noise can be added to the curve.
#'
#' @param len An integer indicating the length of the curve to generate.
#' @param dist_knots A numeric value indicating the distance between knots.
#' @param norder An integer specifying the order of the B-spline.
#' @param weights A numeric vector containing the coefficients for the B-spline.
#' @param add_noise A logical value indicating whether to add Gaussian noise to the curve.
#'
#' @return A list containing the generated basis, coefficients, and curve values with or without noise.
#'
#' @export
generate_background_curve <- function(len, dist_knots, norder, weights, add_noise){
  
  # create knots and generate the corresponding b-spline basis for every curve
  breaks <- seq(from = 0, len, by = dist_knots)
  basis  <- create.bspline.basis(norder = norder, breaks = breaks)
  
  if(length(weights) >= basis$nbasis){
    or_coeff <- weights[1:(basis$nbasis)]
  } else {
    stop('A longer vector of "weights" is required')
  }
  
  # create fd object
  fd_curve <- fd(or_coeff, basis)
  # create the curve vector with no noise
  or_y <- generate_curve_vector(fd_curve = fd_curve)
  
  if(add_noise == TRUE){
    or_y <- or_y + rnorm(n = length(or_y), mean = 0, sd = 0.01)
  }
  
  # create list with:
  res <- list("basis" = basis,
              "or_coeff" = or_coeff,
              "no_error_y" = or_y # curve as vector with no error
  ) 
  return(res)
}

#' Add Motif to Base Curve
#'
#' @title Add Motif to Base Curve
#'
#' @description
#' This function adds specified motifs to a base curve, adjusting coefficients and applying noise as needed. It also computes the 
#' signal-to-noise ratio (SNR) for the motifs.
#'
#' @param base_curve A list representing the base curve structure with coefficients and basis.
#' @param mot_pattern A data frame containing information about the motif patterns to add.
#' @param mot_len A matrix specifying the lengths of the motifs.
#' @param dist_knots A numeric value indicating the distance between knots for the motifs.
#' @param mot_order An integer specifying the order of the B-spline for the motifs.
#' @param mot_weights A list of numeric vectors containing weights for each motif.
#' @param noise_str A list of structures defining the noise to be added to motifs.
#' @param only_der A logical value indicating whether to add only derivatives.
#' @param coeff_min_shift A numeric value for the minimum shift to apply to coefficients.
#' @param coeff_max_shift A numeric value for the maximum shift to apply to coefficients.
#'
#' @return A list containing the updated base curve, background information, motifs with and without noise, and SNR data.
#'
#' @export
add_motif <- function(base_curve, mot_pattern, mot_len, dist_knots, mot_order, mot_weights, noise_str,
                      only_der, coeff_min_shift, coeff_max_shift){
  # create knots and generate the corresponding b-spline basis for every curve
  max_len <- max(mot_len[2])
  mot_breaks <- seq(from = 0,max_len,by = dist_knots)
  mot_basis  <- create.bspline.basis(norder = mot_order, breaks = mot_breaks)
  
  mot_coeff <- vector("list")
  for(j in names(mot_weights)) {
    if(length(mot_weights[[j]]) >= floor(mot_len[mot_len[,1]==as.numeric(j),2] / dist_knots) + 1 + mot_order - 2) { #nbreaks + norder - 2
      mot_coeff[[j]] <- mot_weights[[j]][1:(floor(mot_len[mot_len[,1]==as.numeric(j),2] / dist_knots) + 1 + mot_order - 2)]
    }
    else('A longer vector of "weights" is required')
  }
  # add info about motif: id, starting break or point, ending break or point
  base_curve_coeff <- base_curve$or_coeff
  base_y <- generate_curve_vector(fd_curve = fd(base_curve_coeff, base_curve$basis))
  for(i in names(mot_weights)){
    start_break <- mot_pattern[mot_pattern[,1]==as.numeric(i), "start_break_pos"] # select the first coefficient 
    end_break   <- start_break + floor(mot_len[mot_len[,1]==as.numeric(i),2] / dist_knots) + mot_order - 2 
    for (n in seq_along(start_break)) {
      base_curve_coeff[start_break[n]:end_break[n]] <- mot_coeff[[i]]
      if(only_der == TRUE) {
        base_curve_coeff[start_break[n]:end_break[n]] <- base_curve_coeff[start_break[n]:end_break[n]] + rep(runif(1,min=coeff_min_shift,max=coeff_max_shift),length(mot_coeff[[i]])) 
      }
    }
  }
  
  # Errorless curve
  # create fd object
  # Convert base_curve_coeff and basis to fd object
  fd_curve <- fd(base_curve_coeff, base_curve$basis)
  
  # Generate curve vector with no noise
  or_y <- generate_curve_vector(fd_curve = fd_curve)
  
  background <- list("or_coeff" = base_curve$or_coeff,
                     "no_error_y" = base_y)
  no_error_res <- list("or_coeff" = base_curve_coeff,
                       "motif_y" = or_y)
  # Error curve
  # add extra error on motif
  err_y_mat <- list()
  noise_str <- lapply(noise_str,.transform_to_matrix)
  SNR <- vector("list",nrow(noise_str[[1]]))
  for(k in 1:nrow(noise_str[[1]])) {
    err_y <- or_y
    SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
    SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
    for(i in names(mot_weights)){
      error_str_i_k <- noise_str[[as.numeric(i)]][k,]
      start_break <- mot_pattern[mot_pattern[,1]==as.numeric(i), "start_break_pos"] # select the first coefficient 
      start_point <- (start_break-1)*dist_knots
      end_point   <- start_point + mot_len[mot_len[,1]==as.numeric(i),"len"]
      err_y <- add_error_to_motif(err_y, error_str_i_k, start_point, end_point,k)
      for(n in seq_along(start_point)) {
        SNR_num <- rbind(SNR_num,data.frame(xmin =start_point[n],xmax = end_point[n],SNR = var(no_error_res$motif_y[start_point[n]:end_point[n]])))
      }
    }
    # and then smooth it again
    mot_breaks <- seq(from = 0, length(err_y), by = dist_knots)
    basisobj = create.bspline.basis(norder = mot_order, breaks = mot_breaks)
    #plot(basisobj)
    ys = smooth.basis(argvals = 1:(length(err_y)),
                      y = err_y,
                      fdParobj = basisobj) # smooth again given the error on data(pointwise)
    # smoothed again after adding error
    yy <- eval.fd(1:(length(err_y)), ys$fd)
    for(i in names(mot_weights)) {
      start_break <- mot_pattern[mot_pattern[,1]==as.numeric(i), "start_break_pos"] # select the first coefficient 
      start_point <- (start_break-1)*dist_knots
      end_point   <- start_point + mot_len[mot_len[,1]==as.numeric(i),"len"]
      for(n in seq_along(start_point)) {
        SNR_den <- rbind(SNR_den,data.frame(xmin=start_point[n],xmax=end_point[n],SNR=var(yy[start_point[n]:end_point[n]] - no_error_res$motif_y[start_point[n]:end_point[n]])))
      }
    }
    SNR[[k]] <- SNR_num
    SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
    err_y_mat[[k]] <- yy
  }

  error_res <- list("noise_structure" = noise_str,
                    "noise_y" = err_y_mat) 
  res <- list("basis" = base_curve$basis,
              "background" = background,
              "no_noise" = no_error_res,
              "with_noise" = error_res,
              "SNR" = SNR) 
  # create list with 
  return(res)
}

#' Generate Coefficients for Motif
#'
#' @title Generate Coefficients
#'
#' @description
#' This function generates a vector of coefficients for a motif based on the specified distribution. 
#' It can sample coefficients from a numeric vector, a uniform distribution, or a beta distribution.
#'
#' @param motif_i A list representing the motif structure that includes the length (`len`).
#' @param distrib A string or numeric vector indicating the distribution from which to generate coefficients. 
#'                Accepted values are "unif", "beta", or a numeric vector.
#' @param dist_knots A numeric value indicating the distance between knots.
#' @param norder An integer specifying the order of the B-spline.
#' @param coeff_min A numeric value indicating the minimum coefficient value for uniform or beta distributions.
#' @param coeff_max A numeric value indicating the maximum coefficient value for uniform or beta distributions.
#' @param motif_seed An integer value setting the seed for the random number generator used in motif generation. This ensures reproducibility of the motif embedding process. Default is `43213`.
#'
#' @return A modified motif structure containing the generated coefficients.
#'
#' @details
#' - For uniform distribution, coefficients are generated within the specified range defined by `coeff_min` and `coeff_max`.
#' - For beta distribution, coefficients are scaled to the desired range using the specified minimum and maximum values.
#'
#' @importFrom stats runif rbeta
#' 
#' @export
.generate_coefficients <- function(motif_i, distrib, dist_knots, norder, coeff_min, coeff_max, motif_seed) {
  # Calculate the length of the coefficients vector
  l <- motif_i$len / dist_knots + norder - 1
  # set seed
  set.seed(motif_seed) # NEW UPDATE
  if(is.numeric(distrib)) {
    motif_i$coeffs <- sample(distrib, size = l,replace = TRUE)
  }
  else if (distrib == "unif") {
    # Generate coefficients from a uniform distribution
    motif_i$coeffs <- runif(l, min = coeff_min, max = coeff_max)
  } else if (distrib == "beta") {
    # Generate coefficients from a beta distribution and scale to the desired range
    motif_i$coeffs <- coeff_min + rbeta(l, shape1 = 0.45, shape2 = 0.45) * (coeff_max - coeff_min)
  } else {
    stop("Wrong 'distrib': ", distrib)
  }
  return(motif_i)
}

# PREVIOUS VERSION
# .generate_coefficients <- function(motif_i, distrib, dist_knots, norder, coeff_min, coeff_max) {
#   # Calculate the length of the coefficients vector
#   l <- motif_i$len / dist_knots + norder - 1
#   if(is.numeric(distrib)) {
#     motif_i$coeffs <- sample(distrib, size = l,replace = TRUE)
#   }
#   else if (distrib == "unif") {
#     # Generate coefficients from a uniform distribution
#     motif_i$coeffs <- runif(l, min = coeff_min, max = coeff_max)
#   } else if (distrib == "beta") {
#     # Generate coefficients from a beta distribution and scale to the desired range
#     motif_i$coeffs <- coeff_min + rbeta(l, shape1 = 0.45, shape2 = 0.45) * (coeff_max - coeff_min)
#   } else {
#     stop("Wrong 'distrib': ", distrib)
#   }
#   return(motif_i)
# }


#' Check Fits for Motifs
#'
#' @title Check Fits
#'
#' @description
#' This function checks if motifs fit within a specified curve length and whether there are overlaps between motifs. 
#' It returns TRUE if all conditions are met and FALSE otherwise.
#'
#' @param df A data frame containing motif information with columns `start` and `end` indicating the positions of motifs.
#' @param min_dist_motifs A numeric value specifying the minimum distance required between motifs.
#' @param len A numeric value representing the total length of the curve.
#'
#' @return A logical value indicating whether the motifs fit within the curve and do not overlap.
#'
#' @export
.check_fits <- function(df,min_dist_motifs,len) {
  df <- df[order(df$start), ]
  # Check for overlaps
  for (i in seq_len(nrow(df) - 1)) {
    if (df$end[i] + min_dist_motifs > df$start[i + 1]) {
      return(FALSE)
    }
  }
  
  # Check if the last motif fits within the curve length
  if (nrow(df) > 0 && df$end[nrow(df)] > len) {
    return(FALSE)
  }
  
  return(TRUE)
}

#' Resample Vector
#'
#' @title Resample Vector
#'
#' @description
#' This function resamples a vector by randomly selecting indices. The number of samples can be specified.
#'
#' @param x A vector to be resampled.
#' @param ... Additional arguments passed to the sampling function.
#'
#' @return A resampled vector.
#'
#' @export
.resample <- function(x, ...) x[sample.int(length(x), ...)]

#' @title Find Occurrences of a Motif
#' 
#' @description
#' Finds occurrences of a specified motif in a set of curves, where the dissimilarity is lower than a specified threshold \( R \). 
#' The function compares the motif against curves represented in multiple dimensions and returns the details of matching occurrences.
#' 
#' @param v A list containing two matrices:
#'          - \code{v0}: The first set of motif values.
#'          - \code{v1}: The second set of motif values.
#'          Each matrix should have \( d \) columns corresponding to the dimensions.
#' @param Y A list of \( N \) lists, each containing two matrices:
#'          - \code{Y0}: The first set of curve values.
#'          - \code{Y1}: The second set of curve values.
#'          Each matrix should have \( d \) columns corresponding to the dimensions.
#' @param R A numeric value representing the maximum allowed dissimilarity.
#' @param alpha A numeric value that serves as a weight coefficient between the L2 norms 
#'               of the two sets of motifs when using the dissimilarity function \code{diss_d0_d1_L2}.
#' @param w A numeric vector of weights for the dissimilarity index across different dimensions.
#'          All weights must be positive (\( w > 0 \)).
#' @param c_k An integer specifying the minimum length of the intersection of the supports 
#'             of the shifted motif and the curves.
#' @param use0 A logical value indicating whether to use the first component of the curves (i.e., \( Y0 \) and \( v0 \)).
#' @param use1 A logical value indicating whether to use the second component of the curves (i.e., \( Y1 \) and \( v1 \)).
#' @param transformed A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.
#' 
#' @return A matrix with three columns:
#'         - \code{curve}: The ID of the curve where the motif was found.
#'         - \code{shift}: The optimal shift at which the motif occurs.
#'         - \code{diss}: The dissimilarity value associated with the match.
#'         If no occurrences are found, an empty matrix is returned.
#' 
#' @details
#' The function systematically checks each curve in the provided list against the specified motif 
#' to identify positions where the dissimilarity does not exceed the defined threshold \( R \). 
#' It handles multidimensional curves and can selectively consider different components of the motifs and curves.
#' 
#' @author Marzia Angela Cremona & Francesca Chiaromonte
#' @export
.find_occurrences <- function(v,Y,R,alpha,w,c_k,use0,use1,transformed=FALSE){
  v_dom=.domain(v,use0)
  v_len=length(v_dom)
  SD_motif=lapply(Y,
                  function(y){
                    y_len=unlist(lapply(y,nrow))[1]
                    s_rep=seq_len(y_len-v_len+1)
                    y_rep=lapply(s_rep,
                                 function(i){
                                   y_rep=list(y0=NULL,y1=NULL)
                                   if(use0)
                                     y_rep$y0=as.matrix(y$y0[i-1+seq_len(v_len),])
                                   if(use1)
                                     y_rep$y1=as.matrix(y$y1[i-1+seq_len(v_len),])
                                   y_rep=.select_domain(y_rep,v_dom,use0,use1)
                                 })
                    valid=unlist(lapply(lapply(y_rep,.domain,use0),sum))>=c_k
                    s_rep=s_rep[valid]
                    y_rep=y_rep[valid]
                    d_rep=unlist(lapply(y_rep,.diss_d0_d1_L2,.select_domain(v,v_dom,use0,use1),w,alpha,transformed))
                    d_rep_R=c(FALSE,d_rep<=R,FALSE)
                    diff_d_rep_R=diff(d_rep_R)
                    start=which(diff_d_rep_R==1)
                    end=which(diff_d_rep_R==(-1))-1
                    SD_motif=mapply(function(start,end){
                      index=(start:end)[which.min(d_rep[start:end])]
                      return(c(s_rep[index],d_rep[index]))
                    },start,end)
                    return(SD_motif)
                  })
  if(!is.null(unlist(SD_motif))){
    v_occurrences=cbind(rep(seq_along(SD_motif),unlist(lapply(SD_motif,length))/2),
                        matrix(unlist(SD_motif),ncol=2,byrow=TRUE))
    row.names(v_occurrences)=NULL
    colnames(v_occurrences)=c('curve','shift','diss')
  }else{
    v_occurrences=c()
  }
  return(v_occurrences)
}

#' @title Plot Silhouette Index from probKMA Results
#'
#' @description This function generates a bar plot displaying the adapted silhouette index based on the results from the `probKMA` algorithm. It visually represents the quality of the motifs identified by illustrating the average silhouette width for each motif. Additionally, it provides relevant information about the number of curves associated with each motif.
#'
#' @param silhouette_results A list containing the results from the silhouette analysis, including:
#'   - Silhouette indices for each motif.
#'   - Motif identifiers.
#'   - Curves associated with each motif.
#'   - Average silhouette widths.
#'   - Number of curves in each motif.
#' @param K An integer representing the number of motifs identified by the `probKMA` algorithm.
#' @param plot A logical value indicating whether to generate the plot. Default is `TRUE`.
#'
#' @return A list containing the following elements:
#' \item{silhouette}{A vector of silhouette indices for the motifs.}
#' \item{motifs}{A vector of motif identifiers.}
#' \item{curves}{A vector containing all curves associated with the motifs.}
#' \item{silhouette_average}{A vector of average silhouette widths for each motif.}
#'
#' @import graphics
#' @import grDevices
#' @export
#' 
probKMA_silhouette_plot <- function(silhouette_results,K,plot = TRUE){
  # Plot the adapted silhouette index on the results of probKMA.
  oldpar <- par(no.readonly = TRUE)
  # Ensure the original settings are restored when the function exits
  on.exit(par(oldpar))
  ### plot silhouette ########################################################################################
    if(plot) {
    silhouette = silhouette_results[[1]]
    Y_motifs = silhouette_results[[2]]
    curves_in_motifs = silhouette_results[[3]]
    silhouette_average = silhouette_results[[4]]
    curves_in_motifs_number = silhouette_results[[5]]
    
    
    n=length(silhouette)
    sil=rev(silhouette)
    y=barplot(sil,space=c(0,rev(diff(Y_motifs))),xlab='Silhouette index',names='',
              xlim=c(min(0,min(sil)-0.05),1.2),horiz=TRUE,las=1,mgp=c(2.5,1,0),col='lightblue')
    sapply(seq(0, 1, by = 0.2),function(h){abline(v = h, col = "firebrick3", lty = 2)})
    text(ifelse(sil>=0,sil+0.03,sil-0.03),ifelse(sil>0,y,y+0.2),labels=paste0("c",rev(unlist(curves_in_motifs))),cex=0.5)
    title(main='Silhouette plot',sub=paste("Average silhouette width:",round(mean(silhouette_average),2)),adj = 0.5)
    title(sub=paste("Average silhouette width:",round(mean(silhouette_average),2)),adj = 0.5,col.sub='red')
    mtext(paste("#curves =",n),adj=0,side = 3, line = 1,font=1,col=rgb(0,0.35,0))
    mtext(substitute("#motifs:"~K,list(K=K)),side = 3, line = 2, adj = 0,col=rgb(0,0.35,0))
    mtext(expression(paste(motif," | ",curve[j]," | avg ",s[i])),side = 3, line = 1.5, adj = 1,font=1,col=rgb(0,0.35,0))
    y=rev(y)
    for(k in seq_len(K)){
      y_k=mean(y[Y_motifs==k])
      text(1.1,y_k,paste0(k, " | ", curves_in_motifs_number[k], " | ", format(silhouette_average[k], digits = 1, nsmall = 2)),xpd=TRUE,srt=90,col='firebrick3')
    }
  }
  
  return(list(silhouette=silhouette_results[[1]],motifs=silhouette_results[[2]],curves=unlist(silhouette_results[[3]]),
              silhouette_average=silhouette_results[[4]]))
}

#' @title Dissimilarity Index for Multidimensional Curves
#' 
#' @description
#' Computes a Sobolev-type dissimilarity index for multidimensional curves based on a weighted combination of L2 norms of the function and its derivative. 
#' The distance is normalized on a common support, allowing for a comparison between curves considering both their levels and variations.
#' 
#' @param y A list of two matrices: 
#'   \itemize{
#'     \item \code{y[[1]]}: Values of the curve at points in the domain (y(x)).
#'     \item \code{y[[2]]}: Values of the derivative of the curve at the same points (y'(x)).
#'   }
#'   Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curves.
#' 
#' @param v A list of two matrices: 
#'   \itemize{
#'     \item \code{v[[1]]}: Values of the reference curve at points in the domain (v(x)).
#'     \item \code{v[[2]]}: Values of the derivative of the reference curve at the same points (v'(x)).
#'   }
#'   Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curves.
#' 
#' @param w A numeric vector of weights for the dissimilarity index in different dimensions. Each weight should be greater than 0.
#' 
#' @param alpha A numeric value (between 0 and 1) that specifies the weight coefficient between the L2 norm of the function (d0.L2) and the L2 norm of the derivative (d1.L2):
#'   \itemize{
#'     \item \code{alpha = 0}: Only the levels (d0.L2) are considered.
#'     \item \code{alpha = 1}: Only the derivative information (d1.L2) is considered.
#'     \item Values between 0 and 1 provide a mixture of both.
#'   }
#' 
#' @param transform_y A logical value indicating whether to normalize \code{y} to the range [0, 1] before applying the distance computation. Default is \code{FALSE}.
#' 
#' @param transform_v A logical value indicating whether to normalize \code{v} to the range [0, 1] before applying the distance computation. Default is \code{FALSE}.
#' 
#' @return A numeric value representing the dissimilarity index between the curves defined by \code{y} and \code{v}.
#' 
#' @details
#' The dissimilarity index is calculated based on the following Sobolev-type distance:
#' \deqn{D = (1 - \alpha) \cdot d0.L2 + \alpha \cdot d1.L2}
#' where:
#' \itemize{
#'   \item \code{d0.L2}: L2 distance considering only the levels of the curves.
#'   \item \code{d1.L2}: L2 distance considering only the derivatives of the curves.
#' }
#' 
#' The function normalizes the inputs based on the specified flags to ensure that all features are comparable.
#' 
#' @export
.diss_d0_d1_L2 <- function(y,v,w,alpha,transform_y=FALSE,transform_v=FALSE){

  y_norm=y
  v_norm=v
  
  .diss_L2 <- function(y,v,w){
    # Dissimilarity index for multidimensional curves (dimension=d).
    # L2 distance with normalization on common support.
    # y=y(x), v=v(x) for x in dom(v), matrices with d columns.
    # w: weights for the dissimilarity index in the different dimensions (w>0).
    
    sum(colSums((y-v)^2,na.rm=TRUE)/(colSums(!is.na(y)))*w)/ncol(y)
    # NB: divide for the length of the interval, not for the squared length!
  }
  
  if(transform_y){
    y0_min = apply(y[[1]], 2, min, na.rm = TRUE)
    y0_max = apply(y[[1]], 2, max, na.rm = TRUE)
    y0_diff = y0_max - y0_min
    y0_const = unlist(lapply(y0_diff, function(diff) all.equal(diff, 0) == TRUE))
    y_norm[[1]] = t( (t(y[[1]]) - y0_min) / y0_diff )
    y_norm[[1]][,y0_const]=0.5
    if(alpha != 0) {
      y_norm[[2]] = t( t(y[[2]]) / y0_diff )
      y_norm[[2]][,y0_const] = 0
    }
  } 
  if(transform_v){
    v0_min=apply(v[[1]],2,min,na.rm=TRUE)
    v0_max=apply(v[[1]],2,max,na.rm=TRUE)
    v0_diff=v0_max-v0_min
    v0_const=unlist(lapply(v0_diff,function(diff) all.equal(diff,0)==TRUE))
    v_norm[[1]]=t( (t(v[[1]]) - v0_min)/v0_diff)
    v_norm[[1]][,v0_const]=0.5
    if(alpha != 0) {
      v_norm[[2]]=t( t(v[[2]])/v0_diff)
      v_norm[[2]][,v0_const]=0
    }
  }
  
  if(alpha==0){#L2-like distance which focuses excusively on the levels
    return(.diss_L2(y_norm[[1]],v_norm[[1]],w))
  }else if(alpha==1){#L2-like pseudo distance, which uses only weak
    #derivative information
    return(.diss_L2(y_norm[[2]],v_norm[[2]],w))
  }else{#Sobolev-like distance that allows to highlight
    #more complex features of curve shapes,taking into account
    #both levels and variations
    return((1-alpha)*.diss_L2(y_norm[[1]],v_norm[[1]],w)+alpha*.diss_L2(y_norm[[2]],v_norm[[2]],w))
  }
}

#' @title Get Accolites for a Given Leaf Label
#'
#' @description
#' This function retrieves the accolites (adjacent elements) of a specified leaf label from a dataset. 
#' Accolites are defined as the elements that overlap with the specified portion length around the leaf.
#' The function can filter accolites based on their origin curve in cases where multiple curves are present.
#'
#' @param leaf_label A character string representing the label of the leaf for which to find accolites.
#' @param window_data A data frame or matrix where the row names correspond to the labels of elements,
#'                    containing the data from which accolites will be extracted.
#' @param portion_len An integer specifying the total length of the portion used to define the accolites. 
#'                    The overlap is computed as half of this length.
#' @param multiple A logical indicating whether to check for multiple curves. If TRUE, the function 
#'                 filters out accolites that do not originate from the same curve as the specified leaf label.
#'
#' @return A character vector containing the labels of the identified accolites. 
#'         If no accolites are found, the function returns an empty vector.
#'
#' @details
#' The function works as follows:
#' 1. It calculates the index of the specified leaf label within the provided `window_data`.
#' 2. Determines the range of indices representing the accolites by calculating the overlap based on `portion_len`.
#' 3. Retrieves the corresponding leaf labels from the `window_data`.
#' 4. If the `multiple` argument is TRUE, it checks if the accolites come from the same curve as the leaf label, 
#'    removing those that do not.
#'
#' @export
get_accolites <- function(leaf_label, window_data, portion_len, multiple){
  # number of overlapping elements that define accolites
  overlap <- floor(portion_len/2)
  leaf_label_curve <-  (strsplit(leaf_label, '_') %>%
                          unlist() %>%
                          as.numeric())[1]
  
  leaf_index  <- which(rownames(window_data) == leaf_label) # index in window_data
  r_accolites <- (leaf_index+1):min(leaf_index+overlap, dim(window_data)[1])  # accolites to the right
  l_accolites <- (leaf_index-1):max(leaf_index-overlap,0)  # accolites to the left
  accolites   <- c(leaf_index, l_accolites, r_accolites) # overall accolites
  
  # get the accolites
  leaf_accolites <- rownames(window_data)[accolites]
  leaf_accolites <- leaf_accolites[!is.na(leaf_accolites)] #remove NAs
  # check if they come from the same curve (multiple curves cases)
  if(multiple == TRUE){
    leaf_accolites_curve <- strsplit(leaf_accolites, '_') %>%
      unlist() %>%
      as.numeric() %>%
      matrix(ncol=3, byrow=T) %>%
      as.data.frame()
    to_delete <- which(leaf_accolites_curve$V1 != leaf_label_curve)
    if(length(to_delete) > 0){
      leaf_accolites <- leaf_accolites[-to_delete]
    }
  }
  return(leaf_accolites)
}

#' @title cluster_candidate_motifs_plot
#'
#' @description This function generates plots to visualize the results of the
#' `cluster_candidate_motifs` function. It provides insights into the
#' distances between motifs and curves, group-specific radii, and displays
#' aligned motifs for each cluster.
#'
#' @param cluster_candidate_motifs_results A list containing the output of the
#' `cluster_candidate_motifs` function, which includes clustering results and
#' distances between motifs and curves.
#' 
#' @param R_all A numeric value representing the global radius. This is used
#' to cut the dendrogram, requiring groups to be more than twice `R_all`
#' apart. The default is set to the `R_all` component of the input results.
#'
#' @param R_m A numeric vector with group-specific radii. The length of this
#' vector must match the number of clusters obtained by cutting the dendrogram
#' at height `2 * R_all`. If set to NULL, the radius `R_m` is determined
#' within each group based on the distances between motifs of the same group
#' and all curves.
#'
#' @param ask A logical value indicating whether the user should be prompted
#' before a new plot is drawn. Default is TRUE.
#'
#' @param ylab A character string specifying the label for the y-axis in the
#' plots. Default is an empty string.
#'
#' @details This function produces:
#' - Histograms of distances between all motifs and all curves, highlighting
#' the global radius.
#' - A dendrogram of motifs cut at `2 * R_all`, which allows visualization of
#' the clustering structure.
#' - Histograms of distances for motifs within specific clusters, comparing
#' motifs from curves with and without the motifs.
#' - Plots of aligned motifs for each cluster, showing their positions and
#' distributions.
#' - A scatter plot displaying the approximate average distance against the
#' approximate frequency for motifs in each cluster.
#'
#' @author Marzia Angela Cremona & Francesca Chiaromonte
#' 
#' @export
cluster_candidate_motifs_plot <- function(cluster_candidate_motifs_results,ylab='',
                                          R_all=cluster_candidate_motifs_results$R_all,R_m=NULL,ask=TRUE){
  oldpar <- par(no.readonly = TRUE)         
  # Ensure the original settings are restored when the function exits
  on.exit(par(oldpar))
  ### prepare input data ##################################################################################
  if(cluster_candidate_motifs_results$diss=='d0_L2'){
    alpha=0
    use0=TRUE
    use1=FALSE
    Y=lapply(cluster_candidate_motifs_results$Y0,function(y0) list(y0=y0,y1=NULL))
    V=lapply(cluster_candidate_motifs_results$V0_clean,function(v0) list(v0=v0,v1=NULL))
  }
  if(cluster_candidate_motifs_results$diss=='d1_L2'){
    alpha=1
    use0=FALSE
    use1=TRUE
    Y=lapply(cluster_candidate_motifs_results$Y1,function(y1) list(y0=NULL,y1=y1))
    V=lapply(cluster_candidate_motifs_results$V1_clean,function(v1) list(v0=NULL,v1=v1))
  }
  if(cluster_candidate_motifs_results$diss=='d0_d1_L2'){
    alpha=cluster_candidate_motifs_results$alpha
    use0=TRUE
    use1=TRUE
    Y=mapply(function(y0,y1) list(y0=y0,y1=y1),cluster_candidate_motifs_results$Y0,cluster_candidate_motifs_results$Y1,SIMPLIFY=FALSE)
    V=mapply(function(v0,v1) list(v0=v0,v1=v1),cluster_candidate_motifs_results$V0_clean,cluster_candidate_motifs_results$V1_clean,SIMPLIFY=FALSE)
  }
  w=cluster_candidate_motifs_results$w
  max_gap=cluster_candidate_motifs_results$max_gap
  d=ncol(cluster_candidate_motifs_results$Y0[[1]])
  N=nrow(cluster_candidate_motifs_results$D)
  K=ncol(cluster_candidate_motifs_results$D)
  V_dom=lapply(cluster_candidate_motifs_results$V0,function(v) rowSums(!is.na(v))!=0)
  V_length=unlist(lapply(V_dom,length))
  
  ### plot global distances and global radius Rall ########################################################
  par(ask=ask,mfrow=c(2,1))
  dist_breaks=diff(hist(cluster_candidate_motifs_results$D_clean[cluster_candidate_motifs_results$D_clean<quantile(cluster_candidate_motifs_results$D_clean,0.90)],plot=FALSE)$breaks[1:2])
  breaks=seq(0,ceiling(max(cluster_candidate_motifs_results$D_clean)/10)*10+dist_breaks,by=dist_breaks)
  xlim=c(0,quantile(cluster_candidate_motifs_results$D_clean,0.95))
  
  # histogram of distances between all motifs and all curves
  hist_res=hist(cluster_candidate_motifs_results$D_clean,breaks=breaks,
                probability=TRUE,xlim=xlim,xlab='Distances motif-curve',main='Distances motif-curve - All motifs')
  D_clean_density=density(cluster_candidate_motifs_results$D_clean,bw=hist_res$breaks[2])
  lines(D_clean_density$x,D_clean_density$y,lwd=2)
  abline(v=R_all,lwd=2)
  text(x=R_all,y=max(hist_res$density),labels=expression(R[all]),pos=4)
  
  # histogram of distances for all motifs, separately for curves with the motifs and for curves without the motifs
  hist_res=hist(cluster_candidate_motifs_results$D_clean[cluster_candidate_motifs_results$P_clean==1],breaks=breaks,
                probability=TRUE,xlim=xlim,xlab='Distances motif-curve',main='Distances motif-curve - All motifs')
  D_clean_density1=density(cluster_candidate_motifs_results$D_clean[cluster_candidate_motifs_results$P_clean==1],bw=breaks[2])
  lines(D_clean_density1$x,D_clean_density1$y,col='red',lwd=2)
  hist(cluster_candidate_motifs_results$D_clean[cluster_candidate_motifs_results$P_clean==0],breaks=breaks,probability=TRUE,add=TRUE)
  D_clean_density0=density(cluster_candidate_motifs_results$D_clean[cluster_candidate_motifs_results$P_clean==0],bw=breaks[2])
  lines(D_clean_density0$x,D_clean_density0$y,col='blue',lwd=2)
  abline(v=R_all,lwd=2)
  text(x=R_all,y=max(hist_res$density),labels=expression(R[all]),pos=4)
  legend('topright',legend=c('Curves with motif','Curves without motif'),lwd=2,col=c('red','blue'))
  
  ### cut dendrogram and check group-specific radius Rm ##################################################
  if(is.null(cluster_candidate_motifs_results$hclust_res)){
    V_hclust=1
  }else{
    V_hclust=dendextend::cutree(cluster_candidate_motifs_results$hclust_res,h=2*R_all) # cut at high 2*R_all
  }
  n_hclust=max(V_hclust)
  if(!is.null(cluster_candidate_motifs_results$hclust_res)){
    par(mfrow=c(1,1))
    dendr=as.dendrogram(cluster_candidate_motifs_results$hclust_res,hang=1)
    dendextend::labels_cex(dendr)=0.8
    plot(dendr,ylab='Distance motif-motif',main='Dendrogram of motifs')
    abline(h=2*R_all,col='black',lwd=2)
    text(x=1,y=2*R_all,labels=expression('2R'[all]),pos=3,offset=0.5)
    if(n_hclust>1)
      dendextend::rect.dendrogram(dendr,k=n_hclust)
  }
  
  # re-compute or check group-specific radius Rm, and plot
  if(is.null(R_m))
    R_m=rep(NA,n_hclust)
  if(length(R_m)!=n_hclust)
    stop(paste0('The length of the vector R_m must match the number of clusters: ',n_hclust))
  
  for(i_hclust in seq_len(n_hclust)){
    index_i=which(V_hclust==i_hclust) # motifs in cluster i
    V_length_i=V_length[index_i]
    V_P_i=as.matrix(cluster_candidate_motifs_results$P_clean[,index_i])
    V_D_i=as.matrix(cluster_candidate_motifs_results$D_clean[,index_i])
    
    if(is.na(R_m[i_hclust])){
      dataR=data.frame(P=as.vector(V_P_i),D=as.vector(V_D_i))
      if(max(dataR$D[dataR$P==1])<=min(dataR$D[dataR$P==0])){
        R_m[i_hclust]=max(dataR$D[dataR$P==1])
      }else{
        D_new=seq(0,max(V_D_i),length.out=10000)
        pred_knn=class::knn(train=dataR$D,test=as.matrix(D_new),cl=dataR$P,k=cluster_candidate_motifs_results$k_knn,prob=TRUE)
        R_m[i_hclust]=D_new[which(ifelse(pred_knn==1,attributes(pred_knn)$prob,1-attributes(pred_knn)$prob)<=cluster_candidate_motifs_results$votes_knn_Rm)[1]]
      }
    }
    
    # plot histograms and motifs
    par(mfrow=c(2,1))
    
    # histogram of distances between motifs in cluster i and all curves
    hist_res=hist(V_D_i,breaks=breaks,probability=TRUE,xlim=xlim,xlab='Distances motif-curve',main=paste0('Distances motif-curve - Cluster ',i_hclust))
    D_clean_density=density(V_D_i,bw=hist_res$breaks[2])
    lines(D_clean_density$x,D_clean_density$y,lwd=2)
    abline(v=R_m[i_hclust],lwd=2)
    text(x=R_m[i_hclust],y=max(hist_res$density),labels=bquote(R[.(i_hclust)]),pos=4)
    
    # histogram of distances for motifs in cluster i, separately for curves with the motifs and for curves without the motifs
    hist_res=hist(V_D_i[V_P_i==1],breaks=breaks,probability=TRUE,xlim=xlim,xlab='Distances motif-curve',main=paste0('Distances motif-curve - Cluster ',i_hclust))
    D_clean_density1=density(V_D_i[V_P_i==1],bw=breaks[2])
    lines(D_clean_density1$x,D_clean_density1$y,col='red',lwd=2)
    hist(V_D_i[V_P_i==0],breaks=breaks,probability=TRUE,add=TRUE)
    D_clean_density0=density(V_D_i[V_P_i==0],bw=breaks[2])
    lines(D_clean_density0$x,D_clean_density0$y,col='blue',lwd=2)
    abline(v=R_m[i_hclust],lwd=2)
    text(x=R_m[i_hclust],y=max(hist_res$density),labels=bquote(R[.(i_hclust)]),pos=4)
    legend('topright',legend=c('Curves with motif','Curves without motif'),lwd=2,col=c('red','blue'))
    
    par(mfrow=c(2+d,1))
    
    # plot aligned motifs
    VV_S_i=as.matrix(cluster_candidate_motifs_results$VV_S[index_i,index_i])
    V0_i=cluster_candidate_motifs_results$V0_clean[index_i]
    for(jd in seq_len(d)){
      V0_i_jd=lapply(V0_i,function(v0_i) as.matrix(v0_i[,jd]))
      plot(1,1,ylim=range(unlist(V0_i_jd)),xlim=c(min(VV_S_i[1,]),max(VV_S_i[1,]+V_length_i-1)),ylab=ylab[jd],
           xlab='Position',main=paste0('Motifs in Cluster ',i_hclust),type='n',cex.axis=1.5,cex.main=1.5,cex.lab=1.5)
      for(j in seq_along(V_length_i))
        lines(seq(VV_S_i[1,j],VV_S_i[1,j]+V_length_i[j]-1),V0_i_jd[[j]],col=j)
    }
    
    # plot alignment
    plot(1,1,ylim=c(1,length(V_length_i)),xlim=c(min(VV_S_i[1,]),max(VV_S_i[1,]+V_length_i-1)),
         ylab='Motifs',xlab='Position',main=paste0('Alignment motifs in Cluster ',i_hclust),type='n',yaxt='n',cex.axis=1.5,cex.main=1.5,cex.lab=1.5)
    for(j in seq_along(index_i)){
      lines(c(VV_S_i[1,j],VV_S_i[1,j]+V_length_i[j]-1),c(j,j),lwd=2,col=j)
      text(VV_S_i[1,j],j,labels=index_i[j],pos=2)
    }
    
    # plot
    V_mean_diss_approx_i=apply(V_D_i,2,function(x) mean(x[x<=R_m[i_hclust]])) # approximate average distance (actual radius)
    V_frequencies_approx_i=colSums(V_D_i<=R_m[i_hclust]) # approximate frequency 
    V_frequencies_approx_i_scatter=V_frequencies_approx_i+rnorm(length(V_frequencies_approx_i),sd=0.1)
    plot(V_mean_diss_approx_i,V_frequencies_approx_i_scatter,cex=V_length_i/min(V_length_i),col=seq_along(V_length_i),
         xlab='Approx average distance',ylab='Approx frequency',main=paste0('Motif in Cluster ',i_hclust),yaxt='n',cex.axis=1.5,cex.main=1.5,cex.lab=1.5)
    axis(2,at=seq(min(V_frequencies_approx_i),max(V_frequencies_approx_i)),cex.axis=1.5,cex.lab=1.5)
    text(V_mean_diss_approx_i,V_frequencies_approx_i_scatter,labels=index_i,pos=4)
  }
  return()
}

#' @title Simulated data for local clustering 
#'
#' @description
#' 18 curves of length 200 (corresponding to 201 evaluation points), each
#' containing exactly one motif of length 60 (corresponding to 61 evaluation
#' points), with noise level sigma=0.1
#' Curves 1-6, 14-16 contain one occurrence of motif 1
#' Curves 7-13, 17-18 contain one occurrence of motif 2
#' The data have been generated using a B-spline basis of order 3 and knots at
#' distance 10, following the simulation procedure presented in Cremona and
#' Chiaromonte (Comparison with non-sparse and sparse functional clustering
#' simulation section).
#' @name simulated200
#' @usage data(simulated200)
"simulated200"

#' @title Domain Length Calculation for Curves
#' 
#' @description
#' Computes the length of the domain of a curve by checking the presence of non-NA values in the specified matrix.
#' The function returns a logical vector indicating which rows contain at least one non-NA element, allowing for an evaluation of the effective length of the domain.
#' 
#' @param v A list containing two elements:
#'   \itemize{
#'     \item \code{v[[1]]}: A matrix representing the values of the curve (v(x)).
#'     \item \code{v[[2]]}: A matrix representing the values of the derivative of the curve (v'(x)).
#'   }
#'   Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curve.
#' 
#' @param use0 A logical value indicating which element of \code{v} to consider:
#'   \itemize{
#'     \item If \code{use0 = TRUE}, the function considers \code{v[[1]]} (the curve values).
#'     \item If \code{use0 = FALSE}, the function considers \code{v[[2]]} (the derivative values).
#'   }
#' 
#' @return A logical vector where each element is \code{TRUE} if at least one element in the corresponding row of the selected matrix (either \code{v[[1]]} or \code{v[[2]]}) is not NA, and \code{FALSE} otherwise.
#' 
#' @details
#' The function evaluates each row of the specified matrix (either \code{v[[1]]} or \code{v[[2]]}) to determine if there is at least one non-NA entry. This is particularly useful for assessing the effective domain of curves where some values may be missing.
#' 
#' @export
.domain <- function(v,use0){
  if(use0){
    rowSums(!is.na(v[[1]]))!=0
  }else{
    rowSums(!is.na(v[[2]]))!=0
  }
}

#' @title Compare Nodes
#'
#' @description This function compares two nodes to determine if all elements of the first node (`node_1`) are present within the accolites of the second node (`node_2`). Accolites are defined as a set of elements retrieved from the `get_accolites` function, which operates on `node_1` within a specified window of data. The comparison is successful if all elements in `node_1` are found in the accolites of `node_2`.
#'
#' @param node_1 A vector representing the first node whose elements will be compared against the accolites of `node_2`.
#' @param node_2 A vector representing the second node, from which accolites will be derived for comparison.
#'
#' @return A logical value. The function returns `TRUE` if all elements of `node_1` are found in the accolites of `node_2`. It returns `FALSE` otherwise.
#'
#' @export
compare_nodes <- function(node_1, node_2){
  accolites_1    <- lapply(node_1, get_accolites, window_data, 50) %>% unlist()
  common_elements <- (node_2 %in% accolites_1) %>% sum()
  if(length(node_1) == common_elements){
    TRUE # if the number of common elements is equal to node_1 cardinality
  }else{
    FALSE
  }
}

#' @docType class
#' @title ProbKMA Class
#' @description 
#' The `ProbKMA` class is an R wrapper for the C++ implementation of the 
#' Probabilistic K-means Algorithm (ProbKMA) with local alignment. This class 
#' facilitates local clustering of functional data and functional motif discovery, 
#' as proposed in the paper 
#' `Probabilistic K-means with local alignment for clustering and motif discovery in functional data`, 
#' authored by Marzia A. Cremona and Francesca Chiaromonte.
#' 
#' @section Constructor:
#' Create a `ProbKMA` object using the following constructor:
#' 
#' \preformatted{
#' prok <- new(ProbKMA, data$Y, data$V, params, data$P0, data$S0, "H1")
#' }
#' 
#' @return 
#' A `ProbKMA` object from the C++ ProbKMA class.
#' 
#' @section Parameters:
#' \describe{
#'   \item{Y}{A list containing functional data and possibly derivatives.}
#'   \item{params}{An instance of the Parameters class, containing algorithm settings.}
#'   \item{P0}{A matrix representing the initial membership probabilities.}
#'   \item{S0}{A matrix representing the initial shift warping parameters.}
#'   \item{diss}{A character string specifying the dissimilarity measure. Possible choices are: 
#'     \itemize{
#'       \item `'d0_L2'`
#'       \item `'d1_L2'`
#'       \item `'d0_d1_L2'`
#'     }}
#' }
#' 
#' @section Usage:
#' You can access and modify the `ProbKMA` object with the following methods:
#' \describe{
#'   \item{Getters:}{
#'     \describe{
#'       \item{prok$get_parameters()}{Returns a list of parameters.}
#'       \item{prok$get_motifs()}{Returns a list containing the motifs found.}
#'     }
#'   }
#'   \item{Setters:}{
#'     \describe{
#'       \item{prok$set_P0(P)}{Sets the membership matrix.}
#'       \item{prok$set_S0(S)}{Sets the shift warping matrix.}
#'       \item{prok$set_parameters(param)}{Sets parameters field by passing a list of parameters.}
#'     }
#'   }
#'   \item{Initialize Motifs:}{
#'     \describe{
#'       \item{prok$reinit_motifs(c, d)}{Reinitializes (empty) K motifs with dimension c_k x d.}
#'     }
#'   }
#'   \item{Run ProbKMA algorithm:}{
#'     \describe{
#'       \item{prok$probKMA_run()}{Runs the algorithm.}
#'     }
#'   }
#' }
#' 
#' @author Niccolò Feresini and Riccardo Lazzarini
#' @name ProbKMA
#' @export ProbKMA
loadModule(module = "ProbKMAModule", TRUE)

#' @title Initial Checks for ProbKMA
#' 
#' @description
#' This function performs various input checks on the parameters provided by the user to ensure they are valid 
#' for running the ProbKMA algorithm. It verifies the structure and content of the input data, including 
#' curves, derivatives, initial membership probabilities, shift matrices, and various parameters.
#' 
#' @param Y0 A list of matrices or vectors representing the curves.
#' @param Y1 A list of matrices or vectors representing the derivatives of the curves.
#' @param P0 A numeric matrix representing initial membership probabilities. Rows correspond to curves, and columns correspond to clusters.
#' @param S0 A numeric matrix representing the initial shift matrix.
#' @param params A list containing various parameters for ProbKMA, including:
#' \itemize{
#'    \item \code{standardize} Logical indicating whether to standardize the curves.
#'    \item \code{K} Number of motifs.
#'    \item \code{c} Minimum motif length.
#'    \item \code{c_max} Maximum motif length.
#'    \item \code{iter_max} Maximum number of iterations.
#'    \item \code{quantile} Quantile value for stopping criterion.
#'    \item \code{alpha} A numeric value related to the dissimilarity measure.
#'    \item \code{w} Weights used in the algorithm.
#'    \item \code{stopCriterion} Stopping criterion for the algorithm.
#'    \item \code{m} Weighting exponent.
#'    \item \code{tol} Tolerance level for stopping criteria.
#'    \item \code{iter4elong} Maximum iterations for elongation.
#'    \item \code{tol4elong} Tolerance for elongation.
#'    \item \code{max_elong} Maximum elongation allowed.
#'    \item \code{trials_elong} Number of trials for elongation.
#'    \item \code{deltaJK_elong} Threshold for elongation.
#'    \item \code{max_gap} Maximum gap allowed.
#'    \item \code{iter4clean} Number of iterations for cleaning.
#'    \item \code{tol4clean} Tolerance for cleaning.
#'    \item \code{quantile4clean} Quantile for cleaning.
#'    \item \code{return_options} Options for returning results.
#'    \item \code{seed} Seed for random number generation.
#'    \item \code{exe_print} Boolean to control printing of execution messages.
#'    \item \code{set_seed} Boolean to control whether to set a random seed.
#'    \item{\code{transformed}} A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.
#'    \item \code{n_threads} Number of threads for parallel processing.
#'  }
#' @param diss A character string indicating the type of dissimilarity measure to be used. 
#' Possible values are: \code{'d0_L2'}, \code{'d1_L2'}, \code{'d0_d1_L2'}.
#' @param V_init A list containing initial values for the clusters. If provided, it must match the expected structure based on \code{K}.
#' 
#' @return A list containing:
#' \item{FuncData}{A list of processed curves and derivatives after performing the checks.}
#' \item{Parameters}{A list of validated parameters ready for use in initializing the ProbKMA object.}
#' 
#' @export
initialChecks <- function(Y0,Y1,P0,S0,params,diss,V_init){
  ### Unpacking #############################################################################
  standardize = params$standardize
  K = params$K
  c = params$c
  c_max = params$c_max
  iter_max = params$iter_max
  quantile = params$quantile
  alpha = params$alpha
  w = params$w
  stop_criterion = params$stopCriterion
  m = params$m
  tol = params$tol
  iter4elong = params$iter4elong
  tol4elong = params$tol4elong
  max_elong = params$max_elong
  trials_elong = params$trials_elong
  deltaJk_elong = params$deltaJK_elong
  max_gap = params$max_gap
  iter4clean = params$iter4clean
  tol4clean = params$tol4clean
  quantile4clean = params$quantile4clean
  return_options = params$return_options
  seed = params$seed
  exe_print = params$exe_print
  set_seed = params$set_seed
  transformed = params$transformed
  n_threads = params$n_threads
  
  if(set_seed){
    set.seed(seed)
  }
  
  Y0_f <- function(y_i)
  {
    return(y_i[[1]])
  }
  Y1_f <- function(y_i)
  {
    return(y_i[[2]])
  }
  
  ### check input ####################################################################################
  start=proc.time()
  # check dissimilarity
  if(!(diss %in% c('d0_L2','d1_L2','d0_d1_L2')))
    stop('Dissimilarity not valid. Possible choices are: \'d0_L2\', \'d1_L2\' and \'d0_d1_L2\'.')
  # check Y0
  if(missing(Y0))
    stop('Y0 must be specified.')
  if(!is.null(Y0))
    if(!is.list(Y0))
      stop('Y0 should be a list of vectors or matrices.')
  if((FALSE %in% lapply(Y0,is.matrix))&&(FALSE %in% lapply(Y0,is.vector)))
    stop('Y0 should be a list of vectors or matrices.')
  N=length(Y0) # number of curves
  #if(N<5)
  #  stop('More curves y_i(x) needed.')
  Y0=lapply(Y0,as.matrix)
  d=unique(unlist(lapply(Y0,ncol),use.names=FALSE)) # dimension of curves
  if(length(d)>1)
    stop('Curves have not all the same dimension.')
  Y0_NA=lapply(Y0,function(y,d) sum(rowSums(!is.na(y)) %in% c(0,d))==nrow(y),d)
  if(FALSE %in% Y0_NA){
    warning('y_j(x)=NA only in some dimensions, for some x. Putting NA in all dimensions.')
    Y0=lapply(Y0,
              function(y,d){
                y[rowSums(!is.na(y))!=d,]=NA
                return(y)},
              d)
  }
  rm(Y0_NA)
  if(standardize){
    Y0_tot=Reduce(rbind,Y0)
    Y0_mean=colMeans(Y0_tot,na.rm=TRUE)
    Y0_sd=apply(Y0_tot,2,sd,na.rm=TRUE)
    rm(Y0_tot)
    Y0=lapply(Y0,function(y0,m,s) t((t(y0)-m)/s*100),m=Y0_mean,s=Y0_sd)
  }
  if(diss=='d0_d1_L2'){
    # check required input
    if(!is.numeric(alpha)){
      warning('alpha not valid. Setting alpha=0.5.')
      alpha=0.5
    }
    if((alpha<0)|(alpha>1)){
      warning('alpha not valid. Setting alpha=0.5.')
      alpha=0.5
    } else if(alpha==0){
      diss='d0_L2'
    } else if(alpha==1){
      diss='d1_L2'
    }
  }
  if(diss=='d0_L2'){
    alpha=0
    use0=TRUE
    use1=FALSE
    Y=lapply(Y0,function(y0) list(y0=y0,y1=NULL))
  }else{
    # check Y1
    if(is.null(Y1))
      stop(paste0('Y1 must be specified with diss=\'',diss,'\'.'))
    if(!is.list(Y1))
      stop('Y1 should be a list of vectors or matrices.')
    if((FALSE %in% lapply(Y1,is.matrix))&&(FALSE %in% lapply(Y1,is.vector)))
      stop('Y1 should be a list of vectors or matrices.')
    if(N!=length(Y1))
      stop('The number of curves in Y0 and derivatives Y1 should be the same.')
    Y1=lapply(Y1,as.matrix)
    d1=unique(unlist(lapply(Y1,ncol),use.names=FALSE)) # dimension of derivatives
    if(length(d1)>1)
      stop('Derivatives have not all the same dimension.')
    if(d!=d1)
      stop('The dimension of curves in Y0 and derivatives Y1 should be the same')
    Y1_NA=lapply(Y1,function(y,d) sum(rowSums(!is.na(y)) %in% c(0,d))==nrow(y),d)
    if(FALSE %in% Y1_NA){
      warning('y\'_j(x)=NA only in some dimensions, for some x. Putting NA in all dimensions.')
      Y1=lapply(Y1,
                function(y,d){
                  y[rowSums(!is.na(y))!=d,]=NA
                  return(y)},
                d)
    }
    rm(Y1_NA)
    Y0_NA=lapply(Y0,function(y) rowSums(is.na(y))!=0)
    Y1_NA=lapply(Y1,function(y) rowSums(is.na(y))!=0)
    diff_NA=mapply(function(y0,y1) y0!=y1,Y0_NA,Y1_NA)
    index_diff_NA=which(unlist(lapply(diff_NA,sum))!=0)
    if(length(index_diff_NA)>0){
      warning('y_j(x) and y\'_j(x) are not both defined, for some x. Putting NA in that case.')
      same_NA=mapply(function(y0,y1,diff_NA){
        y0[diff_NA]=NA
        y1[diff_NA]=NA
        return(list(y0,y1))
      },Y0[index_diff_NA],Y1[index_diff_NA],diff_NA[index_diff_NA])
      Y0[index_diff_NA]=same_NA[1,]
      Y1[index_diff_NA]=same_NA[2,]
    }
    rm(Y0_NA,Y1_NA,diff_NA)
    if(standardize){
      Y1=lapply(Y1,function(y1,s) t(t(y1)/s*100),s=Y0_sd)
    }
    if(diss=='d1_L2'){
      alpha=1
      use0=FALSE
      use1=TRUE
      Y=lapply(Y1,function(y1) list(y0=NULL,y1=y1))
    }
    if(diss=='d0_d1_L2'){
      use0=TRUE
      use1=TRUE
      Y=mapply(function(y0,y1) list(y0=y0,y1=y1),Y0,Y1,SIMPLIFY=FALSE)
    }
  }
  # check required input
  if(missing(K))
    stop('K must be specified')
  if(missing(c))
    stop('c must be specified')
  # check K
  if(length(K)!=1)
    stop('Number of motifs K not valid.')
  if(K%%1!=0)
    stop('Number of motifs K should be an integer.')
  if(K<1)
    stop('Number of motifs K should be at least 1.')
  # check c
  if(!(length(c) %in% c(1,K)))
    stop('Minimum motif length c not valid.')
  if(sum(c%%1!=0))
    stop('Minimum motif lengths should be integers.')
  if(sum(c<=1))
    stop('Minimum motif lengths should be at least 2.')
  
  #check V_init
  if(!is.null(V_init)){
    if(length(V_init)!=K){
      V_init=NULL
      warning('The length of the list does not represent the number of clusters. The random
         initialization will be used.')
    }
    for(i in 1:length(V_init)){
      if((nrow(V_init[[i]][[1]])!=c)&(nrow(V_init[[i]][[2]])!=c)){ 
        #use nrow instead of length so it will work in a d-dimensional case as well.
        V_init=NULL
        warning('The length of the list does not represent the number of clusters. The random
         initialization will be used.')  
      }
    }
    
  }
  
  if(!is.null(V_init)){
    V_init <- list(lapply(V_init,Y0_f), lapply(V_init,Y1_f))
  }
  
  c=rep(c,length.out=K)
  # find all intervals contained in supp(y_i) and their lengths
  Y_intervals=lapply(Y0,
                     function(y,d){
                       y_not_NA=(rowSums(!is.na(y))==d) # find supp(y)
                       intervals=which((y_not_NA[2:length(y_not_NA)]-y_not_NA[1:(length(y_not_NA)-1)])==1)+1
                       if(y_not_NA[1])
                         intervals=c(1,intervals)
                       intervals_lengths=rle(y_not_NA)$lengths[rle(y_not_NA)$values==TRUE]
                       intervals_all=data.frame(start=intervals,
                                                end=intervals+intervals_lengths-1,
                                                length=intervals_lengths)
                       return(intervals_all)
                     },d)
  # check minimum motif length compatibility
  min_length=unlist(lapply(Y_intervals,
                           function(y_intervals,c){
                             return(sum(y_intervals$length>=c))},
                           max(c)))
  if(0 %in% min_length)
    stop('Minimum motifs length is incompatible with supplied curves. Choose a smaller minimum motifs length.')
  # check c_max
  if(!(length(c_max) %in% c(1,K)))
    stop('Maximum motif length c_max not valid.')
  c_max=rep(c_max,length.out=K)
  for(k in 1:K){
    if(c_max[k]!=Inf){
      if(c_max[k]%%1!=0)
        stop('Maximum motif lengths should be integers.')
      if(c_max[k]<=1)
        stop('Maximum motif length should be at least 2.')
      # check maximum motif length compatibility
      max_length=unlist(lapply(Y_intervals,
                               function(y_intervals,c_max){
                                 return(sum(floor((1+max_gap)*y_intervals$length)>=c_max))},
                               c_max[k]))
      if(0 %in% max_length){
        warning('Maximum motif length is incompatible with supplied curves. Choosing default maximum motif length for motif ',k,'.')
        c_max[k]=Inf
      }
    }
    if(c_max[k]==Inf){
      c_max[k]=floor((1+max_gap)*min(unlist(lapply(Y_intervals,function(y_intervals) max(y_intervals$length)))))
    }
  }
  # check P0
  if(!is.null(P0)){
    if(sum(dim(P0)!=c(N,K))!=0){
      warning('Membership matrix dimensions not valid. Choosing random initial membership matrix.')
      P0=NULL
    }else{
      if(sum((P0<0)|(P0>1))){
        warning('Memberships should be non-negative and <=1. Choosing random initial membership matrix.')
        P0=NULL
      }else{
        if(sum(rowSums(P0)!=1)){
          warning('Memberships of each curve should sum to 1. Choosing random initial membership matrix.')
          P0=NULL
        }else{
          if(sum(colSums(P0)==0)){
            warning('Sum of memberships of each cluster should be positive. Choosing random initial membership matrix.')
            P0=NULL
          }
        }
      }
    }
  }
  # check S0
  if(!is.null(S0)){
    if(sum(dim(S0)!=c(N,K))!=0){
      warning('Shift warping matrix dimensions not valid. Choosing random initial shift warping matrix.')
      S0=NULL
    }else{
      Y_segments=mapply(function(y,S_i,c){
        return(mapply(function(s,c) NA %in% y[s+seq_len(c)-1],S_i,c))},
        Y0,lapply(seq_len(N),function(i) S0[i,]),MoreArgs=list(c),SIMPLIFY=TRUE)
      if(sum(Y_segments)){
        warning('Shift warping matrix not valid. Choosing random initial shift warping matrix.')
        S0=NULL
      }
    }
  }
  # check weigths
  if(!is.vector(w)|!is.numeric(w))
    stop('Weights w not valid.')
  if(!(length(w) %in% c(1,d)))
    stop('Weights w not valid.')
  if(TRUE %in% (w<=0))
    stop('Weights w not valid.')
  w=rep(w,length.out=d)
  # check weighting exponent
  if(!is.numeric(m)|(length(m)>1))
    stop('Weighting exponent m not valid.')
  if(m<=1)
    stop('Weighting exponent m should be >1')
  # check maximum number of iterations
  if(!is.numeric(iter_max)|(length(iter_max)!=1))
    stop('Maximum number of iterations iter_max not valid.')
  if(((iter_max%%1)!=0)|(iter_max<=0))
    stop('Maximum number of iterations iter_max not valid.')
  # check stop criterion
  if(!(stop_criterion %in% c('max','mean','quantile')))
    stop('Stop criterion not valid. Possible choices are: \'max\', \'mean\', \'quantile\'.')
  if(stop_criterion=='quantile')
    if((!is.numeric(quantile))||(length(quantile)!=1)||(quantile<0)||(quantile>1))
      stop('quantile should be a number in [0,1].')
  # check tolerance
  if((!is.numeric(tol))||(length(tol)!=1)||(tol<=0))
    stop('Tolerance should be a positive number.')
  # check elongation input
  if((!is.numeric(iter4elong))||(length(iter4elong)!=1))
    stop('iter4elong not valid.')
  if(((iter4elong%%1)!=0)|(iter4elong<=0))
    stop('iter4elong not valid.')
  if((!is.numeric(tol4elong))||(length(tol4elong)!=1)||(tol4elong<=0))
    stop('tol4elong should be a positive number.')
  if((!is.numeric(max_elong))||(length(max_elong)!=1)||(max_elong<=0))
    stop('max_elong should be a positive number.')
  if((!is.numeric(trials_elong))||(length(trials_elong)!=1))
    stop('trials_elong not valid.')
  if(((trials_elong%%1)!=0)|(trials_elong<=0))
    stop('trials_elong not valid.')
  if((!is.numeric(deltaJk_elong))||(length(deltaJk_elong)!=1)||(deltaJk_elong<=0))
    stop('deltaJk_elong should be a positive number.')
  if((!is.numeric(max_gap))||(length(max_gap)!=1)||(max_gap<0))
    stop('max_gap should be a non-negative number.')
  # check cleaning input
  if((!is.numeric(iter4clean))||(length(iter4clean)!=1))
    stop('iter4clean not valid.')
  if(((iter4clean%%1)!=0)|(iter4clean<=0))
    stop('iter4clean not valid.')
  if((!is.numeric(tol4clean))||(length(tol4clean)!=1)||(tol4clean<=0))
    stop('tol4clean should be a positive number.')
  if((!is.numeric(quantile4clean))||(length(quantile4clean)!=1)||(N*K*quantile4clean<K)||(quantile4clean>1))
    stop('quantile4clean not valid.')
 
  ### initialize #############################################################################################
  start=proc.time()
  if(is.null(P0)){
    # create random membership matrix, with N rows and k columns
    P0=matrix(c(runif(N*(K-1)),rep(1,N)),nrow=N,ncol=K)
    P0=as.matrix(apply(P0,1,sort))
    if(K>1)
      P0=cbind(P0[1,],Reduce('cbind',lapply(2:K,function(k) P0[k,]-P0[k-1,])))
  }
  colnames(P0)=NULL
  if(is.null(S0)){
    # create shift warping matrix, with N rows and k columns
    S0=matrix(unlist(lapply(Y_intervals,
                            function(y_intervals,c,K){
                              s0=rep(NA,K)
                              for(k in 1:K){
                                y_intervals_k=y_intervals[y_intervals$length>=c[k],]
                                y_starts=unlist(apply(y_intervals_k,1,
                                                      function(y_interval)
                                                        return(y_interval[1]:(y_interval[2]-c[k]+1))),
                                                use.names=FALSE)
                                s0[k]=sample(y_starts,1)
                              }
                              return(s0)
                            },c,K)),
              nrow=N,ncol=K,byrow=TRUE)
  }
  
  Y <- list("Y0" = lapply(Y,Y0_f),"Y1" = lapply(Y,Y1_f))

  return(list("FuncData" = list("Y"=Y,"P0"=P0,"S0"=S0,"V_init"=V_init),
              "Parameters" = list("standardize"=standardize,"K"=K,"c"=c,"c_max"=c_max,
                                  "iter_max"=iter_max,"quantile"=quantile,
                                  "stopCriterion"=stop_criterion,"m"=m,"w"=w,
                                  "alpha"=alpha,"tol"=tol,
                                  "iter4elong"=iter4elong,"tol4elong"=tol4elong,
                                  "max_elong"=max_elong,"trials_elong"=trials_elong,
                                  "deltaJK_elong"=deltaJk_elong,"max_gap"=max_gap,
                                  "iter4clean"=iter4clean,"tol4clean"=tol4clean,
                                  "quantile4clean"=quantile4clean,
                                  "seed"=seed,
                                  "return_options"=return_options,
                                  "exe_print"= exe_print,
                                  "set_seed" = set_seed,
                                  "transformed" = transformed,
                                  "n_threads" = n_threads) ) )
}

#' Select Domain of a Motif
#'
#' @title .select_domain
#'
#' @description
#' This function selects the portion of a motif that is free from NA values based on a specified domain. 
#' It allows for the selection of two matrices, `v0` and `v1`, depending on the boolean flags provided.
#'
#' @param v A list containing two elements: 
#'          \itemize{
#'            \item v[[1]]: A matrix representing \( v(x) \) with \( d \) columns.
#'            \item v[[2]]: A matrix representing \( v'(x) \) with \( d \) columns.
#'          }
#' @param v_dom A boolean vector indicating the domain for `v0` and `v1`. 
#' @param use0 A boolean value indicating whether to select `v0`. If TRUE, `v0` is selected based on `v_dom`.
#' @param use1 A boolean value indicating whether to select `v1`. If TRUE, `v1` is selected based on `v_dom`.
#'
#' @return A list containing the selected portions of `v0` and `v1` (if applicable), with dimensions adjusted accordingly.
#'
#' @author Marzia Angela Cremona & Francesca Chiaromonte
#'
#' @export
.select_domain <- function(v,v_dom,use0,use1){
  if(use0)
    v[[1]]=as.matrix(v[[1]][v_dom,])
  if(use1)
    v[[2]]=as.matrix(v[[2]][v_dom,])
  return(v)
}

#' @title Cluster Candidate Motifs
#'
#' @description This function clusters candidate motifs based on their distances and computes group-specific radii for motif clusters. It utilizes K-nearest neighbors (KNN) for determining a global radius and evaluates overlaps among motifs. The function supports parallel computation for efficiency.
#'
#' @param filter_candidate_motifs_results A list containing results from filtering candidate motifs, including various components like `Y0`, `Y1`, `V0_clean`, `V1_clean`, `D_clean`, and more, which are essential for the clustering process.
#' @param motif_overlap A numeric value representing the minimum proportion of overlap required between motifs to be considered similar (default is 0.6).
#' @param k_knn An integer specifying the number of nearest neighbors to consider when determining the global radius (default is 3).
#' @param votes_knn_Rall A numeric value indicating the threshold for KNN voting when determining the global radius (default is 0.5).
#' @param votes_knn_Rm A numeric value indicating the threshold for KNN voting when determining group-specific radii (default is 0.5).
#' @param worker_number An optional integer specifying the number of parallel workers to use. If NULL, it defaults to the number of available cores minus one.
#'
#' @return A list containing:
#' - `VV_D`: Matrix of distances between motifs.
#' - `VV_S`: Matrix of shifts between motifs.
#' - `k_knn`: The value of K used in KNN.
#' - `votes_knn_Rall`: Voting threshold for the global radius.
#' - `R_all`: The global radius determined from the clustering process.
#' - `hclust_res`: Result of hierarchical clustering (if applicable).
#' - `votes_knn_Rm`: Voting threshold for group-specific radius.
#' - `R_m`: Vector of group-specific radii for each cluster.
#' - All components from the input `filter_candidate_motifs_results`.
#'
#' @details This function performs the following steps:
#' 1. Sets up parallel jobs based on the specified `worker_number`.
#' 2. Prepares input data based on the type of distance measure used.
#' 3. Computes distances between motifs.
#' 4. Determines a global radius (`R_all`) using KNN classification.
#' 5. Clusters motifs and determines group-specific radii (`R_m`) for each cluster.
#'
#' @importFrom combinat combn
#' @importFrom stats as.dist quantile density na.omit as.dendrogram
#' @importFrom utils modifyList
#' @import ggplot2
#' @export
cluster_candidate_motifs <- function(filter_candidate_motifs_results,motif_overlap=0.6,
                                     k_knn=3,votes_knn_Rall=0.5,votes_knn_Rm=0.5,worker_number=NULL){
 
  ### set parallel jobs ###################################################################################
  core_number <- parallel::detectCores()
  # check worker number
  if(!is.null(worker_number)){
    if(!is.numeric(worker_number)){
      warning('Worker number not valid. Selecting default number.')
      worker_number=NULL
    }else{
      if((worker_number%%1!=0)|(worker_number<1)|(worker_number>core_number)){
        warning('Worker number not valid. Selecting default number.')
        worker_number=NULL
      }
    }
  }
  if(is.null(worker_number))
    worker_number <- core_number-1
  rm(core_number)
  if(worker_number>1){
    cl_search=parallel::makeCluster(worker_number,timeout=60*60*24*30)
    on.exit(parallel::stopCluster(cl_search))
  }else{
    cl_search=NULL
  }
  
  ### prepare input data ##################################################################################
  if(filter_candidate_motifs_results$diss=='d0_L2'){
    alpha=0
    use0=TRUE
    use1=FALSE
    Y=lapply(filter_candidate_motifs_results$Y0,function(y0) list(y0=y0,y1=NULL))
    V=lapply(filter_candidate_motifs_results$V0_clean,function(v0) list(v0=v0,v1=NULL))
  }
  if(filter_candidate_motifs_results$diss=='d1_L2'){
    alpha=1
    use0=FALSE
    use1=TRUE
    Y=lapply(filter_candidate_motifs_results$Y1,function(y1) list(y0=NULL,y1=y1))
    V=lapply(filter_candidate_motifs_results$V1_clean,function(v1) list(v0=NULL,v1=v1))
  }
  if(filter_candidate_motifs_results$diss=='d0_d1_L2'){
    alpha=filter_candidate_motifs_results$alpha
    use0=TRUE
    use1=TRUE
    Y=mapply(function(y0,y1) list(y0=y0,y1=y1),filter_candidate_motifs_results$Y0,filter_candidate_motifs_results$Y1,SIMPLIFY=FALSE)
    V=mapply(function(v0,v1) list(v0=v0,v1=v1),filter_candidate_motifs_results$V0_clean,filter_candidate_motifs_results$V1_clean,SIMPLIFY=FALSE)
  }
  w=filter_candidate_motifs_results$w
  max_gap=filter_candidate_motifs_results$max_gap
  d=ncol(filter_candidate_motifs_results$Y0[[1]])
  N=nrow(filter_candidate_motifs_results$D)
  K=ncol(filter_candidate_motifs_results$D)
  V_dom=lapply(filter_candidate_motifs_results$V0,function(v) rowSums(!is.na(v))!=0)
  V_length=unlist(lapply(V_dom,length))
  
  
  ### compute distances between motifs ####################################################################
  if(length(V)==1){
    VV_D=as.matrix(0)
    VV_S=as.matrix(1)
  }else{
    VV=combn(V,2,simplify=FALSE)
    VV=array(unlist(VV,recursive=FALSE),dim=c(2,length(VV)))
    VV_lengths=as.matrix(combn(V_length,2))
    VV_motif_overlap=floor(apply(VV_lengths,2,min)*motif_overlap)
    SD=mapply(.find_min_diss,VV[1,],VV[2,],VV_motif_overlap,
              MoreArgs=list(alpha=alpha,w=w,d=d,use0=use0,use1=use1),SIMPLIFY=TRUE)
    VV_D=matrix(0,nrow=length(V),ncol=length(V))
    VV_D[lower.tri(VV_D)]=SD[2,]
    VV_D=VV_D+t(VV_D) # matrix of distances
    VV_S=matrix(0,nrow=length(V),ncol=length(V))
    VV_S[lower.tri(VV_S)]=SD[1,]
    VV_S=VV_S+t(VV_S)+diag(1,nrow=length(V)) # matrix of shifts
  }
  
  ### determine a global radius Rall ######################################################################
  dataR=data.frame(P=as.vector(filter_candidate_motifs_results$P_clean),D=as.vector(filter_candidate_motifs_results$D_clean))
  if(max(dataR$D[dataR$P==1])<=min(dataR$D[dataR$P==0])){
    R_all=max(dataR$D[dataR$P==1])
  }else{
    D_new=seq(0,max(filter_candidate_motifs_results$D_clean),length.out=10000)
    pred_knn=class::knn(train=dataR$D,test=as.matrix(D_new),cl=dataR$P,k=k_knn,prob=TRUE)
    R_all=D_new[which(ifelse(pred_knn==1,attributes(pred_knn)$prob,1-attributes(pred_knn)$prob)<=votes_knn_Rall)[1]]
  }
  
  ### cluster motifs and determine a group-specific radius Rm ##############################################
  if(length(V)==1){
    hclust_res=NULL
    R_m=R_all
  }else{
    VV_dist=as.dist(VV_D)
    hclust_res=hclust(VV_dist,method='average') # hierarchical clustering based on motif-motif distances
    V_hclust=cutree(hclust_res,h=2*R_all) # cut at high 2*R_all
    n_hclust=max(V_hclust)
    
    R_m=rep(NA,n_hclust)
    for(i_hclust in seq_len(n_hclust)){
      index_i=which(V_hclust==i_hclust) # motifs in cluster i
      V_length_i=V_length[index_i]
      V_P_i=as.matrix(filter_candidate_motifs_results$P_clean[,index_i])
      V_D_i=as.matrix(filter_candidate_motifs_results$D_clean[,index_i])
      
      dataR=data.frame(P=as.vector(V_P_i),D=as.vector(V_D_i))
      if(max(dataR$D[dataR$P==1])<=min(dataR$D[dataR$P==0])){
        R_m[i_hclust]=max(dataR$D[dataR$P==1])
      }else{
        D_new=seq(0,max(V_D_i),length.out=10000)
        pred_knn=class::knn(train=dataR$D,test=as.matrix(D_new),cl=dataR$P,k=k_knn,prob=TRUE)
        R_m[i_hclust]=D_new[which(ifelse(pred_knn==1,attributes(pred_knn)$prob,1-attributes(pred_knn)$prob)<=votes_knn_Rm)[1]]
      }
    }
  }
  
  ### output ###############################################################################################
  return(c(list(VV_D=VV_D,VV_S=VV_S,k_knn=k_knn,votes_knn_Rall=votes_knn_Rall,R_all=R_all,hclust_res=hclust_res,votes_knn_Rm=votes_knn_Rm,R_m=R_m),filter_candidate_motifs_results)) 
}

#' @title High Dimensional simulated data for local clustering 
#'
#' @description
#' The functional dataset consists of 43 matrices for curves and an equal number 
#' for their derivatives of two columns and a variable number of rows ranging 
#' from a minimum of 1836 to a maximum of 43868. The data presents 
#' a lot of missing values (large gaps).
#' @name simulatedHighDimensional
#' @usage data(simulatedHighDimensional)
"simulatedHighDimensional"

---
title: "funMoDisco: Functional Motif Discovery"
author: "Niccolò Feresini"
date: "`r Sys.Date()`"
output:
  pdf_document:
    toc: true
    toc_depth: 2
  html_document:
    toc: true
    toc_depth: 2
vignette: > 
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{funMoDisco: Functional Motif Discovery}
  %\VignetteEngine{knitr::rmarkdown}
fontsize: 11pt
header-includes:
  - \usepackage{longtable}
  - \usepackage{tabularx}
  - \usepackage{geometry} 
  - \usepackage{booktabs}
  - \usepackage{xcolor}
---

<style>
  /* Hide the title, author, and date in the HTML output */
  h1 {
    display: none;
  }
</style>

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,    
  warning = FALSE     
)

# Load the package without startup messages
suppressPackageStartupMessages(library(funMoDisco))
```

## Introduction

The **`discoverMotifs`** function serves as the core of the package, offering a robust and efficient implementation of two advanced algorithms: **ProbKMA**(Cremona and Chiaromonte, 2020) and **FunBiAlign**(Di Iorio, Cremona,and Chiaromonte, 2023). Together, these algorithms provide a comprehensive solution for the detection and clustering of recurring patterns within functional data.
In addition to motif discovery, **`funMoDisco`** allows users to **simulate functional curves with embedded motifs**. The package provides several functions for generating synthetic functional data, which can be useful for testing and benchmarking the motif discovery algorithms. These simulated curves are customizable, allowing users to control the number, length, and complexity of the motifs.
By the end of this vignette, users will be guide through practical examples and equipped with a solid understanding of how to effectively utilize `funMoDisco` package for their pattern detection needs.

## Overview of discoverMotifs

The `discoverMotifs` function allows users to:

-   Choose to run **ProbKMA** multiple times with varying numbers of motifs (`K`) and minimum motif lengths (`c`), or **FunBiAlign** specifying the length (`portion_len`) and the minimum cardinality (`min_card`) of the motifs.
-   Perform clustering based on local alignments of curve segments.
-   Control the clustering process through a wide range of hyperparameters.

## Common Parameters

The following parameters are shared between both algorithms:

```{r table, echo=FALSE, results='asis'}
library(knitr)
library(kableExtra)

params <- data.frame(
  Parameter = c("Y0", "method", "stopCriterion", "name", "plot", "worker_number"),
  Description = c(
    "A list of N vectors (for univariate curves) or N matrices (for multivariate curves) representing the curves, where each curve is evaluated on a uniform grid.",
    "A character string specifying which method to use: either 'ProbKMA' or 'funBIalign'.",
    "A character string indicating the convergence criterion based on Bhattacharyya distance between memberships for 'ProbKMA', or ranking criteria for 'funBIalign'.",
    "A character string providing the name of the resulting folder.",
    "A logical value indicating whether to plot the motifs and results.",
    "An integer specifying the number of CPU cores to use for parallel computations. Defaults to cores minus one."
  ),
  Default = c("mandatory", "mandatory", "mandatory", "mandatory", "mandatory", "`detectCores() - 1`")
)

# Create the table with wrapped text and adjustable column widths
kable(params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "25em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "10em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

### Key Arguments for ProbKMA

Below is an overview of the key arguments for the `ProbKMA` algorithm in the `discoverMotifs` function:

```{r, message=FALSE, warning=FALSE,echo=FALSE}
library(knitr)

# Data for the table
params <- data.frame(
  Parameter = c("K", "c", "diss", "alpha", "Y1", "P0", "S0", "c_max", "w", "m", "iter_max", "quantile", 
                "tol", "iter4elong", "tol4elong", "max_elong", "trials_elong", "deltaJK_elong", "max_gap", 
                "iter4clean", "tol4clean", "quantile4clean", "return_options", "n_subcurves", "sil_threshold", 
                "set_seed", "seed", "exe_print", "transformed", "V_init", "n_init_motif"),
  Description = c(
    "A vector specifying the numbers of motifs to be tested.",
    "A vector specifying the minimum motif lengths to be tested.",
    "Dissimilarity. Possible choices are 'd0_L2', 'd1_L2', 'd0_d1_L2'.",
    "Parameter in [0,1] defining the relative weight of the curve's levels and derivatives. 'alpha'=0 means 'd0_L2', 'alpha'=1 means 'd1_L2'.",
    "A list of N vectors or matrices representing the derivative of the curves. Required if `diss='d0_d1_L2'`.",
    "A matrix specifying the initial membership probabilities. If not specified, it will be randomly generated.",
    "A matrix specifying the initial shift. If not specified, it will be randomly generated.",
    "An integer or a vector of K integers specifying the maximum motif lengths.",
    "Weight vector for the dissimilarity index across dimensions.",
    "Weighting exponent in the least-squares functional method (must be greater than 1).",
    "Maximum number of iterations allowed for the ProbKMA algorithm.",
    "Double specifying quantile probability when 'stopCriterion'=\"quantile\".",
    "Double specifying the tolerance level for the method; iteration stops if the stop criterion is less than 'tol'.",
    "Integer specifying the number of iterations after which motif elongation is performed. If 'iter4elong' > 'iter_max', no elongation is performed.",
    "Tolerance on the Bhattacharyya distance for motif elongation.",
    "Maximum elongation allowed in a single iteration, as a percentage of motif length.",
    "Integer specifying the number of elongation trials (equispaced) on each side of the motif in a single iteration.",
    "Maximum relative increase in the objective function allowed during motif elongation.",
    "Double specifying the maximum gap allowed in each alignment as a percentage of the motif length.",
    "Integer specifying number of iterations after which motif cleaning is performed. If 'iter4clean' > 'iter_max', no cleaning is performed.",
    "Tolerance on the Bhattacharyya distance for motif cleaning.",
    "Dissimilarity quantile used for motif cleaning.",
    "If 'TRUE', the options passed to the method are returned.",
    "Integer specifying the number of splitting subcurves used when the number of curves is equal to one.",
    "Double specifying the threshold value to filter candidate motifs.",
    "If 'TRUE', sets a random seed to ensure reproducibility.",
    "The random seed for initialization (used if set_seed=TRUE).",
    "If 'TRUE' and worker_number is equal to one, prints execution details for each iteration.",
    "A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure.",
    "A list of motif sets provided as specific initializations for clustering rather than using random initializations.",
    "The number of initial motif sets from `V_init` to be used directly as starting points in clustering."
  ),
  Default = c("mandatory", "mandatory", "mandatory", "mandatory", "`NULL`", "`matrix()`", "`matrix()`", "`Inf`", "`1`", "`2`", "`1e3`", "`0.25`", "`1e-8`", "`100`", "`1e-3`", "`0.5`", "`201`", "`0.05`", "`0.2`", "`50`", "`1e-4`", "`0.5`", "`TRUE`", "`10`", "`0.9`", "`FALSE`", "`1`", "`FALSE`", "`NULL`", "`NULL`", "`NULL`")
)

# Create the table with wrapped text and adjustable column widths
kable(params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "20em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "8em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

### Example Usage

Here is an example showing a possible use of `discoverMotifs` with **ProbKMA** :

```{r basic example of Probkma,eval = FALSE}
library(funMoDisco)

diss <- 'd0_d1_L2' 
alpha <- 0.5
# run probKMA multiple times (2x3x10=60 times)
K <- c(2,3) 
c <- c(61,51) 
n_init = 10 

data("simulated200") # load simulated data

results = funMoDisco::discoverMotifs(
  Y0 = simulated200$Y0,
  method = "ProbKMA",
  stopCriterion = "max",
  name = './results_ProbKMA_VectorData/',
  plot = TRUE,
  probKMA_options = list(
    Y1 = simulated200$Y1,
    K = K,
    c = c,
    n_init = n_init,
    diss = diss,
    alpha = alpha
  ),
  worker_number = NULL
)
```

In this scenario, all plots generated during the algorithm's execution will be saved in the folder specified by the 'name' parameter. Additionally, the function handles the entire post-processing phase, including filtering patterns and searching for occurrences within the curves, presenting both intermediate and final results along with their corresponding plots. If the user chooses to only perform the post-processing by adjusting parameters like 'sil_threshold,' it is sufficient to call the same discoverMotifs function with the updated parameters. The algorithm will automatically load the previously computed results (which are computationally expensive) and proceed with the post-processing, returning updated plots and results.

Below is an example of how to call `discoverMotifs` with a customized `V_init` and `transformed = TRUE`:

```{r example_V_init_transformed, eval = FALSE}
library(funMoDisco)

# Define parameters
c <- 5                
K <- 2                
n_init <- 2
diss <- 'd0_d1_L2' 
alpha <- 0.5

# Load sample data
data("simulated200")

motif1 <- list(v0 = matrix(runif(c * 4), nrow = c, ncol = 1))
motif2 <- list(v0 = matrix(runif(c * 4), nrow = c, ncol = 1))

# Optional: Include `v1` if using a dissimilarity measure that requires it
motif1$v1 <- matrix(runif(c * 4), nrow = c, ncol = 1)
motif2$v1 <- matrix(runif(c * 4), nrow = c, ncol = 1)

# Define V_init with multiple initial motif sets, matching `K` motifs 
# per initialization
V_init <- list(
  list(motif1, motif2),   # Initialization 1 with 2 motifs
  list(motif1, motif2)    # Initialization 2 with 2 motifs
)

# Run discoverMotifs 
results <- funMoDisco::discoverMotifs(
  Y0 = simulated200$Y0,
  method = "ProbKMA",
  stopCriterion = "max",
  name = './results_ProbKMA_VectorData/',
  plot = TRUE,
  probKMA_options = list(
    Y1 = simulated200$Y1, K = K, c = c, n_init = n_init,
    diss = diss, alpha = alpha, sil_threshold = 0.5,
    V_init = V_init,          
    transformed = TRUE        
  ),
  worker_number = NULL
)
```

### Key Arguments for funBIalign

Below is an overview of the key arguments for the `ProbKMA` algorithm in the `discoverMotifs` function:

```{r, message=FALSE, warning=FALSE,echo=FALSE}
library(knitr)

# Data for the table
small_params <- data.frame(
  Parameter = c("portion_len", "min_card", "cut_off"),
  Description = c(
    "An integer specifying the length of the curve portions to be aligned.",
    "An integer specifying the minimum cardinality of motifs, i.e., the minimum number of motif occurrences required.",
    "Used when `plot` is set to `TRUE`. A double that specifies the number of top-ranked motifs to keep based on the ranking criteria. In particular, all motifs that rank below the cut_off are retained."
  ),
  Default = c("mandatory", "mandatory", "`NULL`")
)

# Create the table with specified column widths for better fitting
kable(small_params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "18em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "8em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

### Example Usage

Here is an example showing a possible use of `discoverMotifs` with **funBIalign** :

```{r example_funbialign,eval = FALSE}
library(funMoDisco)

data("simulated200") # load simulated data

funBialignResult <- funMoDisco::discoverMotifs(
  Y0 = simulated200$Y0,
  method = "FunBIalign",
  stopCriterion = 'fMRS',
  name = './results_FunBialign',
  plot = TRUE,
  funBIalign_options = list(
    portion_len = 60,
    min_card = 3,
    cut_off = 1.0
  )
)
```

As previously discussed for 'ProbKMA', if the user intends to execute only the post-processing phase related to the re-ranking of discovered motifs, they can simply call the same function, specifying the updated re-ranking criterion and, if necessary, adjusting the new cut_off value.

## Motif Simulation

As previously noted, the package offers the capability to generate synthetic curves embedded with patterns. This functionality facilitates the testing of both algorithms and provides a reliable reference benchmark for performance evaluation.

The algorithm begins by generating random curves utilizing B-splines as the foundational tools. Subsequently, it incorporates either random or positional patterns into these curves. Finally, noise is introduced, which can manifest as either pointwise noise or noise applied to the expansion coefficients of the B-splines. This process effectively simulates real-world scenarios in which each measurement is associated with a degree of noise.

### Key Arguments for motifSimulationBuilder

'motifSimulationBuilder' represents the first function to be called. In particular, it represents the constructor of the S4 class 'motifSimulation'.

Below is an overview of the key arguments:

```{r, message=FALSE, warning=FALSE,echo=FALSE}
library(knitr)

# Data for the table
params <- data.frame(
  Parameter = c("N", "len", "mot_details", "norder", "coeff_min", "coeff_max", "dist_knots", "min_dist_motifs", "distribution"),
  Description = c(
    "The number of background curves to be generated.",
    "The length of the background curves.",
    "A list outlining the definitions of the motifs to be included. Each motif is characterized by its length, a set of coefficients that may be optionally specified, and the number of occurrences. These occurrences can be indicated either by specific positions within the curves or by a total count. In the latter case, the algorithm will randomly position the motifs throughout the curves.",
    "Integer specifying the order of the B-splines.",
    "Additive coefficients to be incorporated into the generation of coefficients for the background curves.",
    "Additive coefficients to be incorporated into the generation of coefficients for the background curves.",
    "Integer specifying the distance between two consecutive knots.",
    "Integer specifying the minimum distance between two consecutive motifs embedded in the same curve.",
    "Distribution from which the coefficients of the background curves are generated. You can choose between a uniform distribution or a beta distribution. Alternatively, you can pass a vector representing the empirical distribution from which you wish to sample."
  ),
  Default = c("mandatory", "mandatory", "mandatory", "3", "`-15`", "`15`", "`10`", "`'norder' * 'dist_knots'`", "`unif`")
)

# Create the table with specified column widths for better fitting
kable(params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "20em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "8em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

### Key Arguments for generateCurves

After calling the constructor of the class, it is then possible to generate the curves with the motifs embedded.

Below is an overview of the key arguments:

```{r, message=FALSE, warning=FALSE,echo=FALSE}
library(knitr)

# Data for the table
params <- data.frame(
  Parameter = c("object", "noise_type", "noise_str", "seed_background", "seed_motif", "only_der", "coeff_min_shift", "coeff_max_shift"),
  Description = c(
    "The S4 object first constructed.",
    "A string specifying whether to add pointwise error or coefficients ('pointwise' and 'coeff').",
    "A list corresponding to the number of motifs, specifying the structure of noise to be added for each motif. If 'pointwise' is chosen, the user can specify a list of vectors or matrices indicating the amount of noise for each motif. If 'coeff' is selected, a list of individual values or vectors can be provided.",
    "An integer specifying the seed for background curve generation.",
    "An integer specifying the seed for motif generation.",
    "If 'FALSE', a vertical shift is added to each motif instance.",
    "Minimum vertical shift.",
    "Maximum vertical shift."
  ),
  Default = c("mandatory", "mandatory", "mandatory", "`777`", "`43213`", "`TRUE`", "`-10`", "`10`")
)

# Create the table with specified column widths for better fitting
kable(params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "20em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "8em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

### Key Arguments for plot_motifs

This is the final function to be called. As indicated by its name, it generates summary plots. Each plot displays the background curve, the motif without noise, and the motif with noise highlighted within a shaded region.

Below is an overview of the key arguments:

```{r, message=FALSE, warning=FALSE,echo=FALSE}
library(knitr)

# Data for the table
params <- data.frame(
  Parameter = c("object", "curves", "path"),
  Description = c(
    "The S4 object first constructed.",
    "The result of the previous method.",
    "Path specifying the directory where the results will be saved."
  ),
  Default = c("mandatory", "mandatory", "mandatory")
)

# Create the table with specified column widths for better fitting
kable(params, format = "latex", booktabs = TRUE, longtable = TRUE, escape = TRUE) %>%
  kable_styling(latex_options = c("hold_position", "striped", "scale_down"), 
                position = "center", 
                font_size = 12) %>%
  column_spec(1, italic = TRUE , background = "#e2e3e5") %>%  # Header color for Parameter
  column_spec(2, width = "20em", background = "#e2e3e5") %>%  # Background for Description
  column_spec(3, width = "8em", background = "#e2e3e5") %>%  # Background for Default
  row_spec(0, bold = TRUE, color = "black", background = "#8DADB8") 
```

## Examples

The five main types of use are considered below.

### 0) Special case: No motifs

```{r no_motifs,message = FALSE, warning = FALSE}
library(funMoDisco)

mot_len <- 100
mot_details <- NULL # or list()

builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)

curves <- funMoDisco::generateCurves(builder) 

funMoDisco::plot_motifs(builder,
                        curves,
                        name = "plots_0",
                        path = getwd())
```

```{r display_pdf_0, echo=FALSE, out.width="100%",out.height="100%"}
knitr::include_graphics("plots_0.pdf")
```

### 1) Set the motif position and add pointwise noise

```{r motif_position_pointiwise_error,message = FALSE, warning = FALSE}
library(funMoDisco)

mot_len <- 100  

# Struct specifying the motif ID, the number of curves, and the relative knot position
motif_str <- rbind.data.frame(
  c(1, 1, 20),
  c(2, 1, 2),  
  c(1, 3, 1),
  c(1, 2, 1),
  c(1, 2, 15),
  c(1, 4, 1),
  c(2, 5, 1),
  c(2, 7, 1),
  c(2, 17, 1)
)

names(motif_str) <- c("motif_id", "curve", "start_break_pos")

mot1 <- list(
  "len" = mot_len,               # Length
  "coeffs" = NULL,              # Weights for the motif
  "occurrences" = motif_str %>% filter(motif_id == 1)
)

mot2 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = motif_str %>% filter(motif_id == 2)
)

mot_details <- list(mot1, mot2)

# MATRIX NOISE 
noise_str <- list(
  rbind(
    rep(2, 100),                      # Constant and identical
    c(rep(0.1, 50), rep(2, 50)),     # SD 0.1 first, SD 1 later
    c(rep(2, 50), rep(0.1, 50)),     # SD 1 first, 0.1 later
    c(seq(2, 0.1, len = 50), rep(0.1, 50))
  ),
  rbind(
    rep(0.0, 100),  
    rep(0.5, 100),  
    rep(1.0, 100),  
    rep(5.0, 100)
  )
)

builder <- funMoDisco::motifSimulationBuilder(
  N = 20,
  len = 300,
  mot_details,
  distribution = 'beta'
)

curves <- funMoDisco::generateCurves(
  builder,
  noise_type = 'pointwise',
  noise_str = noise_str
) 

funMoDisco::plot_motifs(builder,
                        curves,
                        name = "/plots_1",
                        path = getwd())
```

```{r display_pdf_1, echo=FALSE, out.width="100%",out.height="100%"}
knitr::include_graphics("plots_1.pdf")
```

### 2) Set the motif position and add coeff noise

```{r motif_position_coeff_error, message=FALSE, warning=FALSE}
library(funMoDisco)

mot_len <- 100  

# Struct specifying the motif ID, the number of curves, and the relative knot position
motif_str <- rbind.data.frame(
  c(1, 1, 20),
  c(1, 1, 2),  
  c(1, 3, 1),
  c(1, 2, 1),
  c(1, 2, 15),
  c(1, 4, 1),
  c(1, 5, 1),
  c(1, 7, 1),
  c(2, 17, 1)
)

names(motif_str) <- c("motif_id", "curve", "start_break_pos")

mot1 <- list(
  "len" = mot_len,               # Length
  "coeffs" = NULL,              # Weights for the motif
  "occurrences" = motif_str %>% filter(motif_id == 1)
)

mot2 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = motif_str %>% filter(motif_id == 2)
)

mot_details <- list(mot1, mot2)

# VECTOR NOISE 
noise_str <- list(
  c(0.1, 1.0, 5.0),
  c(0.0, 0.0, 0.0)
)

builder <- funMoDisco::motifSimulationBuilder(
  N = 20,
  len = 300,
  mot_details,
  distribution = 'beta'
)

curves <- funMoDisco::generateCurves(
  builder,
  noise_type = 'coeff',
  noise_str,
  only_der = FALSE
)

funMoDisco::plot_motifs(builder,
                        curves,
                        name = "plots_2",
                        path = getwd())
```

```{r display_pdf_2, echo=FALSE, out.width="100%",out.height="100%"}
knitr::include_graphics("plots_2.pdf")
```

### 3) Random motif position and add pointwse noise

```{r random_position_pointiwise_error,message = FALSE, warning = FALSE}
library(funMoDisco)

mot_len <- 100  

# Define motifs
mot1 <- list(
  "len" = mot_len,              # Length
  "coeffs" = NULL,             # Weights for the motif
  "occurrences" = 5
) 

mot2 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = 6
)

mot_details <- list(mot1, mot2)

# Define noise structure
noise_str <- list(
  rbind(rep(2, 100)),  
  rbind(rep(0.5, 100))
)

# Build motif simulation
builder <- funMoDisco::motifSimulationBuilder(
  N = 20,
  len = 300,
  mot_details,
  distribution = 'beta'
)

# Generate curves
curves <- funMoDisco::generateCurves(
  builder,
  noise_type = 'pointwise',
  noise_str,
  only_der = FALSE
)

# Plot motifs
funMoDisco::plot_motifs(builder,
                        curves,
                        name = "plots_3",
                        path = getwd())
```

```{r display_pdf_3, echo=FALSE, out.width="100%",out.height="100%"}
knitr::include_graphics("plots_3.pdf")
```

### 4) Random motif position and add coeff noise

```{r random_position_coeff_error,message = FALSE, warning = FALSE}
library(funMoDisco)

mot_len <- 100  

# Define motifs
mot1 <- list(
  "len" = mot_len,              # Length
  "weights" = NULL,             # Weights for the motif
  "occurrences" = 5
) 

mot2 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = 6
)

mot_details <- list(mot1, mot2)

# Define vector noise
noise_str <- list(
  c(0.1, 5.0, 10.0),
  c(0.1, 5.0, 10.0)
)

# Build motif simulation
builder <- funMoDisco::motifSimulationBuilder(
  N = 20,
  len = 300,
  mot_details,
  distribution = 'beta'
)

# Generate curves
curves <- funMoDisco::generateCurves(
  builder,
  noise_type = 'coeff',
  noise_str,
  only_der = FALSE
)

# Plot motifs
funMoDisco::plot_motifs(builder,
                        curves,
                        name = "plots_4",
                        path = getwd())
```

```{r display_pdf_4, echo=FALSE, out.width="100%",out.height="100%"}
knitr::include_graphics("plots_4.pdf")
```

### Additional functions

In addition to the functions previously described, the package includes a helper function that facilitates the direct transformation of the output from 'generateCurves' into a format suitable for 'discoverMotifs'. This function generates a comprehensive list that encompasses all curves, each containing the embedded patterns corresponding to various tested noise levels.

```{r to_motifDiscovery,eval = FALSE}
result <- funMoDisco::to_motifDiscovery(curves)
```

Additionally, a Shiny app is available, serving as a graphical user interface (GUI) that enables users to execute all the previously mentioned functions in a straightforward and intuitive manner. The app consistently provides summary plots, enhancing the user experience.

```{r shiny_app,eval = FALSE}
library(funMoDisco)

# Define motif structure
motif_str <- rbind.data.frame(
  c(1, 1, 20),
  c(1, 1, 2),
  c(1, 3, 1),
  c(1, 2, 1),
  c(1, 2, 15),
  c(1, 4, 1),
  c(1, 5, 1),
  c(1, 7, 1),
  c(2, 17, 1)
)

names(motif_str) <- c("motif_id", "curve", "start_break_pos")

# Define motifs
mot1 <- list(
  "len" = 100,                   # Length
  "weights" = NULL,              # Weights for the motif
  "appearance" = motif_str %>% filter(motif_id == 1)
)

mot2 <- list(
  "len" = 150,
  "weights" = NULL,
  "appearance" = motif_str %>% filter(motif_id == 2)
)

mot_details <- list(mot1, mot2)

# Define noise structure
noise_str <- list(
  rbind(rep(2, 100), c(rep(0.1, 50), rep(2, 50))),
  rbind(rep(0.0, 150), rep(5.0, 150))
)

# Run motif simulation app
funMoDisco::motifSimulationApp(noise_str, mot_details)
```

### Conclusion

The **`discoverMotifs`** function is a powerful tool for discovering functional motifs in complex datasets. With its flexibility, users can run multiple initializations, customize clustering parameters, simulate functional curves with motifs, and visualize the results in an intuitive way. Whether using **ProbKMA** or **funBIalign**, the `funMoDisco` package provides a robust solution for analyzing functional data and uncovering hidden patterns.

---
title: "Index of Vignettes for funMoDisco"
author: "Niccolò Feresini & Riccardo Lazzarini"
output:  
  rmarkdown::html_vignette:  
    toc: true  
    toc_depth: 2  
vignette: >  
  %\VignetteEncoding{UTF-8}  
  %\VignetteIndexEntry{Index of Vignettes for funMoDisco}  
  %\VignetteEngine{knitr::rmarkdown}  
---

# Vignettes

This package includes the following vignettes:
- [funMoDisco: Functional Motif Discovery](funMoDisco_vignette.Rmd)

# References

1. Cremona, M. A., and Chiaromonte, F. (2023). *Probabilistic K-means with Local Alignment for Clustering and Motif Discovery in Functional Data*. Journal of Computational and Graphical Statistics, 32(3), 1119-1130. Taylor & Francis. DOI: [10.1080/10618600.2022.2156522](https://doi.org/10.1080/10618600.2022.2156522).

2. Di Iorio, J., Cremona, M. A., and Chiaromonte, F. (2023). *funBIalign: A Hierarchical Algorithm for Functional Motif Discovery Based on Mean Squared Residue Scores*. arXiv:2306.04254. Available at [https://arxiv.org/abs/2306.04254](https://arxiv.org/abs/2306.04254).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_path_complete.R
\name{get_path_complete}
\alias{get_path_complete}
\title{Get Complete Paths from a Dendrogram}
\usage{
get_path_complete(minidend, window_data, min_card, worker_number)
}
\arguments{
\item{minidend}{A dendrogram structure from which to derive paths. This should be created from 
a hierarchical clustering result.}

\item{window_data}{A data frame or matrix containing the data associated with the nodes in the dendrogram. 
This data is used for path recommendations.}

\item{min_card}{An integer specifying the minimum number of leaves (or nodes) that must be present 
in a path for it to be considered valid.}

\item{worker_number}{An integer representing the number of worker nodes to be used for parallel processing.}
}
\value{
A list where each element contains the recommended paths for the corresponding node in the dendrogram. 
        Each path includes information about the nodes and their associated scores.
}
\description{
This function computes recommended paths from a dendrogram structure using parallel processing. 
It utilizes the `find_recommended_path` function to identify optimal paths based on a minimum 
cardinality constraint, distributing the computation across multiple worker nodes.
}
\details{
The function creates a cluster of worker nodes, loads necessary libraries, and exports required 
variables and functions to each worker. It then applies the `find_recommended_path` function 
in parallel to the leaves of the provided dendrogram, gathering results into a single list. 
Finally, the cluster is stopped, and the results are returned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probKMA_silhouette_plot.R
\name{probKMA_silhouette_plot}
\alias{probKMA_silhouette_plot}
\title{Plot Silhouette Index from probKMA Results}
\usage{
probKMA_silhouette_plot(silhouette_results, K, plot = TRUE)
}
\arguments{
\item{silhouette_results}{A list containing the results from the silhouette analysis, including:
- Silhouette indices for each motif.
- Motif identifiers.
- Curves associated with each motif.
- Average silhouette widths.
- Number of curves in each motif.}

\item{K}{An integer representing the number of motifs identified by the `probKMA` algorithm.}

\item{plot}{A logical value indicating whether to generate the plot. Default is `TRUE`.}
}
\value{
A list containing the following elements:
\item{silhouette}{A vector of silhouette indices for the motifs.}
\item{motifs}{A vector of motif identifiers.}
\item{curves}{A vector containing all curves associated with the motifs.}
\item{silhouette_average}{A vector of average silhouette widths for each motif.}
}
\description{
This function generates a bar plot displaying the adapted silhouette index based on the results from the `probKMA` algorithm. It visually represents the quality of the motifs identified by illustrating the average silhouette width for each motif. Additionally, it provides relevant information about the number of curves associated with each motif.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_accolites.R
\name{get_accolites}
\alias{get_accolites}
\title{Get Accolites for a Given Leaf Label}
\usage{
get_accolites(leaf_label, window_data, portion_len, multiple)
}
\arguments{
\item{leaf_label}{A character string representing the label of the leaf for which to find accolites.}

\item{window_data}{A data frame or matrix where the row names correspond to the labels of elements,
containing the data from which accolites will be extracted.}

\item{portion_len}{An integer specifying the total length of the portion used to define the accolites. 
The overlap is computed as half of this length.}

\item{multiple}{A logical indicating whether to check for multiple curves. If TRUE, the function 
filters out accolites that do not originate from the same curve as the specified leaf label.}
}
\value{
A character vector containing the labels of the identified accolites. 
        If no accolites are found, the function returns an empty vector.
}
\description{
This function retrieves the accolites (adjacent elements) of a specified leaf label from a dataset. 
Accolites are defined as the elements that overlap with the specified portion length around the leaf.
The function can filter accolites based on their origin curve in cases where multiple curves are present.
}
\details{
The function works as follows:
1. It calculates the index of the specified leaf label within the provided `window_data`.
2. Determines the range of indices representing the accolites by calculating the overlap based on `portion_len`.
3. Retrieves the corresponding leaf labels from the `window_data`.
4. If the `multiple` argument is TRUE, it checks if the accolites come from the same curve as the leaf label, 
   removing those that do not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{add_motif}
\alias{add_motif}
\title{Add Motif to Base Curve}
\usage{
add_motif(
  base_curve,
  mot_pattern,
  mot_len,
  dist_knots,
  mot_order,
  mot_weights,
  noise_str,
  only_der,
  coeff_min_shift,
  coeff_max_shift
)
}
\arguments{
\item{base_curve}{A list representing the base curve structure with coefficients and basis.}

\item{mot_pattern}{A data frame containing information about the motif patterns to add.}

\item{mot_len}{A matrix specifying the lengths of the motifs.}

\item{dist_knots}{A numeric value indicating the distance between knots for the motifs.}

\item{mot_order}{An integer specifying the order of the B-spline for the motifs.}

\item{mot_weights}{A list of numeric vectors containing weights for each motif.}

\item{noise_str}{A list of structures defining the noise to be added to motifs.}

\item{only_der}{A logical value indicating whether to add only derivatives.}

\item{coeff_min_shift}{A numeric value for the minimum shift to apply to coefficients.}

\item{coeff_max_shift}{A numeric value for the maximum shift to apply to coefficients.}
}
\value{
A list containing the updated base curve, background information, motifs with and without noise, and SNR data.
}
\description{
This function adds specified motifs to a base curve, adjusting coefficients and applying noise as needed. It also computes the 
signal-to-noise ratio (SNR) for the motifs.
}
\details{
Add Motif to Base Curve
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{.check_fits}
\alias{.check_fits}
\title{Check Fits}
\usage{
.check_fits(df, min_dist_motifs, len)
}
\arguments{
\item{df}{A data frame containing motif information with columns `start` and `end` indicating the positions of motifs.}

\item{min_dist_motifs}{A numeric value specifying the minimum distance required between motifs.}

\item{len}{A numeric value representing the total length of the curve.}
}
\value{
A logical value indicating whether the motifs fit within the curve and do not overlap.
}
\description{
This function checks if motifs fit within a specified curve length and whether there are overlaps between motifs. 
It returns TRUE if all conditions are met and FALSE otherwise.
}
\details{
Check Fits for Motifs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_nodes.R
\name{compare_nodes}
\alias{compare_nodes}
\title{Compare Nodes}
\usage{
compare_nodes(node_1, node_2)
}
\arguments{
\item{node_1}{A vector representing the first node whose elements will be compared against the accolites of `node_2`.}

\item{node_2}{A vector representing the second node, from which accolites will be derived for comparison.}
}
\value{
A logical value. The function returns `TRUE` if all elements of `node_1` are found in the accolites of `node_2`. It returns `FALSE` otherwise.
}
\description{
This function compares two nodes to determine if all elements of the first node (`node_1`) are present within the accolites of the second node (`node_2`). Accolites are defined as a set of elements retrieved from the `get_accolites` function, which operates on `node_1` within a specified window of data. The comparison is successful if all elements in `node_1` are found in the accolites of `node_2`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{.transform_to_matrix}
\alias{.transform_to_matrix}
\title{Transform to Matrix}
\usage{
.transform_to_matrix(input)
}
\arguments{
\item{input}{A numeric vector or matrix to be transformed.}
}
\value{
A matrix representation of the input.
}
\description{
This function transforms a numeric vector into a matrix. If the input is a single number, it creates a 1x1 matrix. 
If the input is a vector, it creates a 1xN matrix. If the input is already a matrix, it returns it unchanged.
}
\details{
Transform Input to Matrix


- If the input is neither a numeric vector nor a matrix, an error is raised.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_recommended_path.R
\name{find_recommended_path}
\alias{find_recommended_path}
\title{Find Recommended Path in a Tree Structure}
\usage{
find_recommended_path(minidend, window_data, min_card)
}
\arguments{
\item{minidend}{A tree structure representing the hierarchy of nodes, 
from which leaves will be derived for analysis.}

\item{window_data}{A matrix or data frame containing data associated with each node.}

\item{min_card}{An integer specifying the minimum number of leaves that must be present in a node 
for it to be considered for recommendation.}
}
\value{
A list containing:
        - \code{seed_path_info}: A data frame summarizing the valid seed nodes, their number of leaves, 
        heights, and recommended nodes.
        - \code{seed_path_list}: A list containing the names of elements for each node in the seed path.
        - \code{score_path_list}: A list of scores adjusted for each node in the seed path.
        - \code{recommended_node_labels}: A list of labels for the recommended nodes.
        - \code{recommended_node_scores}: A numeric vector containing scores for the recommended nodes.
}
\description{
This function identifies a recommended path in a tree structure based on user-defined minimum criteria. 
It filters nodes based on the number of leaves and calculates scores for potential recommendations.
}
\details{
The function follows these steps:
1. Extracts the leaves and heights of each node in the tree.
2. Filters nodes based on the minimum number of leaves specified by \code{min_card}.
3. Identifies nodes that are parents of the filtered nodes, marking them for deletion.
4. For each seed node, calculates crossing points with other seed nodes.
5. Calculates adjusted scores for nodes in the seed path using the C++ function \code{fMSR_adj}.
6. Recommends nodes based on calculated scores and returns a summary of results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domain.R
\name{.domain}
\alias{.domain}
\title{Domain Length Calculation for Curves}
\usage{
.domain(v, use0)
}
\arguments{
\item{v}{A list containing two elements:
\itemize{
  \item \code{v[[1]]}: A matrix representing the values of the curve (v(x)).
  \item \code{v[[2]]}: A matrix representing the values of the derivative of the curve (v'(x)).
}
Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curve.}

\item{use0}{A logical value indicating which element of \code{v} to consider:
\itemize{
  \item If \code{use0 = TRUE}, the function considers \code{v[[1]]} (the curve values).
  \item If \code{use0 = FALSE}, the function considers \code{v[[2]]} (the derivative values).
}}
}
\value{
A logical vector where each element is \code{TRUE} if at least one element in the corresponding row of the selected matrix (either \code{v[[1]]} or \code{v[[2]]}) is not NA, and \code{FALSE} otherwise.
}
\description{
Computes the length of the domain of a curve by checking the presence of non-NA values in the specified matrix.
The function returns a logical vector indicating which rows contain at least one non-NA element, allowing for an evaluation of the effective length of the domain.
}
\details{
The function evaluates each row of the specified matrix (either \code{v[[1]]} or \code{v[[2]]}) to determine if there is at least one non-NA entry. This is particularly useful for assessing the effective domain of curves where some values may be missing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probKMA_wrap.R
\name{probKMA_wrap}
\alias{probKMA_wrap}
\title{Wrapper for the Probabilistic K-means Algorithm (ProbKMA)}
\usage{
probKMA_wrap(
  Y0 = NULL,
  Y1 = NULL,
  P0 = matrix(),
  S0 = matrix(),
  standardize = FALSE,
  c_max = Inf,
  iter_max = 1000,
  iter4elong = 10,
  trials_elong = 10,
  return_options = TRUE,
  alpha = 0,
  max_gap = 0.2,
  quantile = 0.25,
  stopCriterion = "max",
  tol = 1e-08,
  tol4elong = 0.001,
  max_elong = 0.5,
  deltaJK_elong = 0.05,
  iter4clean = 50,
  tol4clean = 1e-04,
  m = 2,
  w = 1,
  seed = 1,
  K = 2,
  c = 40,
  quantile4clean = 1/K,
  exe_print = FALSE,
  set_seed = FALSE,
  diss = "d0_2",
  transformed = FALSE,
  V_init = NULL,
  align = TRUE,
  n_threads = 1
)
}
\arguments{
\item{Y0}{A matrix of functional data for the first set of observations.}

\item{Y1}{A matrix of functional data for the second set of observations.}

\item{P0}{A matrix representing the initial membership probabilities.}

\item{S0}{A matrix representing the initial shift parameters.}

\item{standardize}{A logical value indicating whether to standardize the data. Default is `FALSE`.}

\item{c_max}{Maximum number of motifs to extract. Default is `Inf`.}

\item{iter_max}{Maximum number of iterations for the algorithm. Default is 1000.}

\item{iter4elong}{Number of iterations for elongation. Default is 10.}

\item{trials_elong}{Number of trials for elongation. Default is 10.}

\item{return_options}{A logical value indicating whether to return additional options. Default is `TRUE`.}

\item{alpha}{A numeric value representing the weighting parameter. Default is 0.}

\item{max_gap}{Maximum allowable gap between motifs. Default is 0.2.}

\item{quantile}{Quantile to be used for cleaning. Default is 0.25.}

\item{stopCriterion}{Stopping criterion for the algorithm, can be 'max' or other specified values. Default is 'max'.}

\item{tol}{Tolerance for convergence. Default is 1e-8.}

\item{tol4elong}{Tolerance for elongation iterations. Default is 1e-3.}

\item{max_elong}{Maximum elongation allowed. Default is 0.5.}

\item{deltaJK_elong}{Increment for the elongation. Default is 0.05.}

\item{iter4clean}{Number of iterations for the cleaning process. Default is 50.}

\item{tol4clean}{Tolerance for the cleaning process. Default is 1e-4.}

\item{m}{Parameter controlling the clustering behavior. Default is 2.}

\item{w}{Weighting parameter for the dissimilarity measure. Default is 1.}

\item{seed}{Random seed for reproducibility. Default is 1.}

\item{K}{Number of motifs to extract. Default is 2.}

\item{c}{Minimum motif length. Default is 40.}

\item{quantile4clean}{Quantile used for the cleaning process. Default is 1/K.}

\item{exe_print}{A logical value indicating whether to print execution details. Default is `FALSE`.}

\item{set_seed}{A logical value indicating whether to set the random seed. Default is `FALSE`.}

\item{diss}{Dissimilarity measure to be used. Default is 'd0_2'.}

\item{transformed}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}

\item{V_init}{Initial values for the motifs. Default is `NULL`.}

\item{align}{A logical value indicating whether to align the curves. Default is `TRUE`.}

\item{n_threads}{Number of threads to use for parallel processing. Default is 1.}
}
\value{
A list containing:
\item{probKMA_results}{A list of results from the ProbKMA algorithm, including processed functional data and model parameters.}
\item{silhouette_results}{Results from silhouette analysis, indicating the quality of the clustering.}
}
\description{
This function serves as a wrapper for the Probabilistic K-means Algorithm (ProbKMA) to cluster functional data. It handles preprocessing, parameter setup, and execution of the core algorithm, returning the results along with silhouette analysis to assess the clustering quality.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ProbKMA-export.R
\docType{class}
\name{ProbKMA}
\alias{ProbKMA}
\title{ProbKMA Class}
\value{
A `ProbKMA` object from the C++ ProbKMA class.
}
\description{
The `ProbKMA` class is an R wrapper for the C++ implementation of the 
Probabilistic K-means Algorithm (ProbKMA) with local alignment. This class 
facilitates local clustering of functional data and functional motif discovery, 
as proposed in the paper 
`Probabilistic K-means with local alignment for clustering and motif discovery in functional data`, 
authored by Marzia A. Cremona and Francesca Chiaromonte.
}
\section{Constructor}{

Create a `ProbKMA` object using the following constructor:

\preformatted{
prok <- new(ProbKMA, data$Y, data$V, params, data$P0, data$S0, "H1")
}
}

\section{Parameters}{

\describe{
  \item{Y}{A list containing functional data and possibly derivatives.}
  \item{params}{An instance of the Parameters class, containing algorithm settings.}
  \item{P0}{A matrix representing the initial membership probabilities.}
  \item{S0}{A matrix representing the initial shift warping parameters.}
  \item{diss}{A character string specifying the dissimilarity measure. Possible choices are: 
    \itemize{
      \item `'d0_L2'`
      \item `'d1_L2'`
      \item `'d0_d1_L2'`
    }}
}
}

\section{Usage}{

You can access and modify the `ProbKMA` object with the following methods:
\describe{
  \item{Getters:}{
    \describe{
      \item{prok$get_parameters()}{Returns a list of parameters.}
      \item{prok$get_motifs()}{Returns a list containing the motifs found.}
    }
  }
  \item{Setters:}{
    \describe{
      \item{prok$set_P0(P)}{Sets the membership matrix.}
      \item{prok$set_S0(S)}{Sets the shift warping matrix.}
      \item{prok$set_parameters(param)}{Sets parameters field by passing a list of parameters.}
    }
  }
  \item{Initialize Motifs:}{
    \describe{
      \item{prok$reinit_motifs(c, d)}{Reinitializes (empty) K motifs with dimension c_k x d.}
    }
  }
  \item{Run ProbKMA algorithm:}{
    \describe{
      \item{prok$probKMA_run()}{Runs the algorithm.}
    }
  }
}
}

\author{
Niccolò Feresini and Riccardo Lazzarini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motifs_search_plot.R
\name{motifs_search_plot}
\alias{motifs_search_plot}
\title{Plot Motif Search Results}
\usage{
motifs_search_plot(
  motifs_search_results,
  ylab = ylab,
  freq_threshold = 5,
  top_n = "all",
  plot_curves = TRUE
)
}
\arguments{
\item{motifs_search_results}{A list containing the output from the `motifs_search` function. This
includes elements such as `V0`, `V1`, `V_frequencies`, `Y0`, `Y1`, `V_length`, `V_occurrences`,
`V_mean_diss`, and `R_motifs`, which store information about the detected motifs and their properties.}

\item{ylab}{A character string specifying the label for the y-axis in the plots. This label will be
appended with the dimension number to create individual titles for each plot. Default is an empty
string (`''`).}

\item{freq_threshold}{An integer indicating the minimum frequency a motif must have to be included
in the plots. Only motifs with a frequency equal to or greater than `freq_threshold` will be
visualized. Default value is `5`.}

\item{top_n}{Determines how many motifs to plot based on their frequency. If set to `'all'`, all
motifs meeting the `freq_threshold` will be plotted. If an integer is provided, only the top
`top_n` motifs with the highest frequencies will be displayed. Default is `'all'`.}

\item{plot_curves}{A logical value indicating whether to plot all underlying curves with colored
motifs highlighted. If `TRUE`, the function overlays motifs on the curves for better visualization.
Default is `TRUE`.}
}
\value{
The function does not return a value but generates plots visualizing the motifs and their occurrences
across different dimensions. It creates separate plots for each dimension and includes legends for
easy identification of motifs.
}
\description{
The `motifs_search_plot` function visualizes the results obtained from the `motifs_search` function.
It generates plots for detected motifs across multiple dimensions, displaying both the motifs and their
corresponding derivative curves (if available). Users can filter motifs based on frequency thresholds
and choose to display either all motifs or the top `n` motifs. Additionally, the function provides an
option to plot all underlying curves with colored motifs highlighted.
}
\details{
The `motifs_search_plot` function performs the following steps:
\enumerate{
  \item Validates input parameters, ensuring that the frequency threshold and `top_n` are appropriate.
  \item Selects motifs that meet the frequency threshold and, if specified, limits the number of motifs
    to the top `n`.
  \item For each dimension, it plots the motif centers (`V0`) and their derivatives (`V1`, if available).
  \item If `plot_curves` is `TRUE`, it overlays the motifs on the original curves, highlighting them with
    distinct colors.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_domain.R
\name{.select_domain}
\alias{.select_domain}
\title{.select_domain}
\usage{
.select_domain(v, v_dom, use0, use1)
}
\arguments{
\item{v}{A list containing two elements: 
\itemize{
  \item v[[1]]: A matrix representing \( v(x) \) with \( d \) columns.
  \item v[[2]]: A matrix representing \( v'(x) \) with \( d \) columns.
}}

\item{v_dom}{A boolean vector indicating the domain for `v0` and `v1`.}

\item{use0}{A boolean value indicating whether to select `v0`. If TRUE, `v0` is selected based on `v_dom`.}

\item{use1}{A boolean value indicating whether to select `v1`. If TRUE, `v1` is selected based on `v_dom`.}
}
\value{
A list containing the selected portions of `v0` and `v1` (if applicable), with dimensions adjusted accordingly.
}
\description{
This function selects the portion of a motif that is free from NA values based on a specified domain. 
It allows for the selection of two matrices, `v0` and `v1`, depending on the boolean flags provided.
}
\details{
Select Domain of a Motif
}
\author{
Marzia Angela Cremona & Francesca Chiaromonte
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diss_d0_d1_L2.R
\name{.diss_d0_d1_L2}
\alias{.diss_d0_d1_L2}
\title{Dissimilarity Index for Multidimensional Curves}
\usage{
.diss_d0_d1_L2(y, v, w, alpha, transform_y = FALSE, transform_v = FALSE)
}
\arguments{
\item{y}{A list of two matrices: 
\itemize{
  \item \code{y[[1]]}: Values of the curve at points in the domain (y(x)).
  \item \code{y[[2]]}: Values of the derivative of the curve at the same points (y'(x)).
}
Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curves.}

\item{v}{A list of two matrices: 
\itemize{
  \item \code{v[[1]]}: Values of the reference curve at points in the domain (v(x)).
  \item \code{v[[2]]}: Values of the derivative of the reference curve at the same points (v'(x)).
}
Each matrix should have \code{d} columns, where \code{d} is the dimensionality of the curves.}

\item{w}{A numeric vector of weights for the dissimilarity index in different dimensions. Each weight should be greater than 0.}

\item{alpha}{A numeric value (between 0 and 1) that specifies the weight coefficient between the L2 norm of the function (d0.L2) and the L2 norm of the derivative (d1.L2):
\itemize{
  \item \code{alpha = 0}: Only the levels (d0.L2) are considered.
  \item \code{alpha = 1}: Only the derivative information (d1.L2) is considered.
  \item Values between 0 and 1 provide a mixture of both.
}}

\item{transform_y}{A logical value indicating whether to normalize \code{y} to the range [0, 1] before applying the distance computation. Default is \code{FALSE}.}

\item{transform_v}{A logical value indicating whether to normalize \code{v} to the range [0, 1] before applying the distance computation. Default is \code{FALSE}.}
}
\value{
A numeric value representing the dissimilarity index between the curves defined by \code{y} and \code{v}.
}
\description{
Computes a Sobolev-type dissimilarity index for multidimensional curves based on a weighted combination of L2 norms of the function and its derivative. 
The distance is normalized on a common support, allowing for a comparison between curves considering both their levels and variations.
}
\details{
The dissimilarity index is calculated based on the following Sobolev-type distance:
\deqn{D = (1 - \alpha) \cdot d0.L2 + \alpha \cdot d1.L2}
where:
\itemize{
  \item \code{d0.L2}: L2 distance considering only the levels of the curves.
  \item \code{d1.L2}: L2 distance considering only the derivatives of the curves.
}

The function normalizes the inputs based on the specified flags to ensure that all features are comparable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{generate_curve_vector}
\alias{generate_curve_vector}
\title{Generate Curve Vector from FD Object}
\usage{
generate_curve_vector(fd_curve, step_by = 1, Lfdobj = 0)
}
\arguments{
\item{fd_curve}{A functional data object from which to generate the curve.}

\item{step_by}{A numeric value indicating the step size for generating the curve.}

\item{Lfdobj}{A numeric value indicating the order of the derivative to compute. Default is 0 (no derivative).}
}
\value{
A numeric vector representing the generated curve.
}
\description{
This function generates a curve vector from a functional data (fd) object, with a specified step size and optional derivative.
}
\details{
Generate Curve Vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discoverMotifs.R
\name{discoverMotifs}
\alias{discoverMotifs}
\title{Functional Motif Discovery}
\usage{
discoverMotifs(
  Y0,
  method,
  stopCriterion,
  name,
  plot,
  probKMA_options = list(),
  funBIalign_options = list(portion_len = NULL, min_card = NULL, cut_off = NULL),
  worker_number = NULL
)
}
\arguments{
\item{Y0}{A list containing N vectors (for univariate curves) or N matrices (for multivariate curves) representing the functional data.}

\item{method}{A character string specifying the motif discovery algorithm to use. Acceptable values are "ProbKMA" for Probabilistic K-means with Local Alignment and "funBIalign" for Functional Bi-directional Alignment.}

\item{stopCriterion}{A character string indicating the convergence criterion for the selected algorithm.}

\item{name}{A character string specifying the name of the output directory where results will be saved.}

\item{plot}{A logical value indicating whether to generate and save plots of the discovered motifs and clustering results.}

\item{probKMA_options}{A list of options specific to the ProbKMA algorithm.}

\item{funBIalign_options}{A list of options specific to the funBIalign algorithm.}

\item{worker_number}{An integer specifying the number of CPU cores to utilize for parallel computations.}
}
\value{
A list containing the discovered motifs and their corresponding statistics, tailored to the selected method:
\describe{
  \item{\code{motifs}}{A list of identified motifs, each containing the motif's representative curve, membership probabilities, and alignment information.}
  \item{\code{statistics}}{Detailed statistics for each motif, including measures such as silhouette scores, variance explained, and other relevant metrics that quantify the quality and significance of the discovered motifs.}
  \item{\code{parameters}}{The final parameters and configurations used during the motif discovery process, providing transparency and facilitating reproducibility of the results.}
  \item{\code{plots}}{If \code{plot = TRUE}, this component contains the generated plots visualizing the motifs and their distribution across the functional data.}
}
}
\description{
The `discoverMotifs` function facilitates the discovery of recurring patterns, or motifs, within functional data by employing two sophisticated algorithms: 
\code{ProbKMA} (Probabilistic K-means with Local Alignment) and \code{funBIalign}. 
These algorithms are designed to identify and cluster functional motifs across multiple curves, leveraging advanced clustering and alignment techniques to handle complex data structures.

\code{ProbKMA} integrates probabilistic clustering with local alignment strategies, enabling the detection of motifs that exhibit variability in both shape and position across different curves. 
This method is particularly adept at handling noisy data and motifs that may appear at varying scales or locations within the curves.

On the other hand, \code{funBIalign} utilizes hierarchical clustering based on mean squared residue scores to uncover motifs. 
This approach effectively captures the additive nature of functional motifs, considering both portion-specific adjustments and time-varying components to accurately identify recurring patterns.

By providing a flexible interface that accommodates different clustering paradigms, `discoverMotifs` empowers users to perform robust motif discovery tailored to their specific data characteristics and analytical requirements. 
Whether opting for the probabilistic and alignment-focused \code{ProbKMA} or the hierarchical and residue-based \code{funBIalign}, users can leverage these methods to extract meaningful and interpretable motifs from their functional datasets.
}
\details{
The `discoverMotifs` function dynamically switches between two advanced motif discovery algorithms based on the user's specification. 
Each algorithm employs distinct strategies to identify and cluster motifs within functional data, offering flexibility and adaptability to various analytical scenarios.
}
\section{Theoretical Background for ProbKMA}{


\code{ProbKMA} is inspired by methodologies prevalent in bioinformatics, particularly those involving local alignment techniques extended from high-similarity seeds. 
This algorithm combines fuzzy clustering approaches with local alignment strategies to effectively minimize a generalized least squares functional. 
The minimization process can incorporate both the levels and derivatives of the curves through a Sobolev-based distance metric, enhancing the algorithm's sensitivity to both shape and rate changes in the data.

Throughout its iterative process, \code{ProbKMA} refines motif centers, membership probabilities, and alignment shifts, making it highly effective for capturing complex motif structures and motifs distributed across multiple curves. 
This ensures that the discovered motifs are both representative and robust against variations and noise within the functional data.
}

\section{Theoretical Background for funBIalign}{


\code{funBIalign} models functional motifs as an additive combination of motif means, portion-specific adjustments, and time-varying components. 
The algorithm constructs a hierarchical dendrogram utilizing the generalized mean squared residue score (fMSR) to identify candidate motifs across curves.

A critical aspect of \code{funBIalign} is its post-processing step, which filters out redundant motifs and refines the final selection to ensure that only the most significant and representative motifs are retained. 
This hierarchical approach allows for a nuanced identification of motifs, capturing both broad and subtle patterns within the data.
}

\section{Common Parameters}{

The following parameters are common to both \code{ProbKMA} and \code{funBIalign} algorithms:
\describe{
  \item{\code{Y0}}{A list containing \code{N} vectors (for univariate curves) or \code{N} matrices (for multivariate curves) representing the functional data. Each curve is evaluated on a uniform grid, ensuring consistency across the dataset.}
  \item{\code{method}}{A character string specifying the motif discovery algorithm to use. Acceptable values are \code{"ProbKMA"} for Probabilistic K-means with Local Alignment and \code{"funBIalign"} for Functional Bi-directional Alignment.}
  \item{\code{stopCriterion}}{A character string indicating the convergence criterion for the selected algorithm. For \code{ProbKMA}, options include \code{"max"}, \code{"mean"}, or \code{"quantile"} based on the Bhattacharyya distance between memberships in successive iterations. For \code{funBIalign}, options are \code{"fMRS"} (functional Mean Squared Residue) or \code{"Variance"} to guide the ranking of motifs.}
  \item{\code{name}}{A character string specifying the name of the output directory where results will be saved. This facilitates organized storage and easy retrieval of analysis results.}
  \item{\code{plot}}{A logical value indicating whether to generate and save plots of the discovered motifs and clustering results. When set to \code{TRUE}, visualizations are produced to aid in the qualitative assessment of the motif discovery process.}
  \item{\code{worker_number}}{An integer specifying the number of CPU cores to utilize for parallel computations. By default, the function uses the total number of available cores minus one, optimizing computational efficiency without overloading the system.}
}
}

\section{ProbKMA Options}{

The following parameters are specific to the \code{ProbKMA} algorithm:
\describe{
  \item{\code{K}}{An integer or vector specifying the number of motifs to be discovered. It can be a single integer for uniform motif discovery or a vector for specifying different numbers of motifs.}
  \item{\code{c}}{An integer or vector indicating the minimum motif lengths. This ensures that each discovered motif meets a specified minimum length requirement, maintaining the integrity of motif structures.}
  \item{\code{c_max}}{An integer or vector specifying the maximum motif lengths, allowing control over the upper bounds of motif sizes to prevent excessively long motifs.}
  \item{\code{diss}}{A character string defining the dissimilarity measure to use. Possible values include \code{"d0_L2"}, \code{"d1_L2"}, and \code{"d0_d1_L2"}, which determine how the algorithm quantifies differences between motifs based on level and derivative information.}
  \item{\code{alpha}}{A numeric value between 0 and 1 that serves as a weight parameter between \code{d0_L2} and \code{d1_L2} when using \code{d0_d1_L2}. An \code{alpha} of 0 emphasizes \code{d0_L2}, while an \code{alpha} of 1 emphasizes \code{d1_L2}, allowing for balanced consideration of both metrics.}
  \item{\code{w}}{A numeric vector specifying the weight for the dissimilarity index across different dimensions. All values must be positive, enabling the algorithm to prioritize certain dimensions over others based on their relative importance.}
  \item{\code{m}}{A numeric value greater than 1 that acts as the weighting exponent in the least-squares functional method. This parameter influences the sensitivity of the algorithm to differences in motif alignment and membership probabilities.}
  \item{\code{iter_max}}{An integer specifying the maximum number of iterations allowed for the algorithm to converge. This prevents excessive computation time by limiting the number of optimization steps.}
  \item{\code{quantile}}{A numeric value representing the quantile probability used when \code{stopCriterion} is set to \code{"quantile"}. This determines the threshold for convergence based on the distribution of Bhattacharyya distances.}
  \item{\code{tol}}{A numeric value specifying the tolerance level for convergence. The algorithm stops iterating if the change in the stop criterion falls below this threshold, ensuring precise and stable convergence.}
  \item{\code{iter4elong}}{An integer indicating the number of iterations after which motif elongation is performed. If set to a value greater than \code{iter_max}, no elongation is performed. Motif elongation allows the algorithm to extend motifs to better fit the data.}
  \item{\code{tol4elong}}{A numeric value defining the tolerance on the Bhattacharyya distance for motif elongation. This parameter controls how much the objective function can increase during elongation, ensuring that motif extensions do not degrade the overall fit.}
  \item{\code{max_elong}}{A numeric value representing the maximum elongation allowed in a single iteration, expressed as a percentage of the motif length. This prevents excessive extension of motifs in any single step.}
  \item{\code{trials_elong}}{An integer specifying the number of elongation trials (equispaced) on each side of the motif in a single iteration. Multiple trials enhance the robustness of motif elongation by exploring various extension possibilities.}
  \item{\code{deltaJK_elong}}{A numeric value indicating the maximum relative increase in the objective function permitted during motif elongation. This ensures that elongation steps contribute positively to the motif fitting process.}
  \item{\code{max_gap}}{A numeric value defining the maximum gap allowed in each alignment as a percentage of the motif length. This parameter controls the allowable discontinuity between aligned motifs, maintaining coherence in motif placement.}
  \item{\code{iter4clean}}{An integer specifying the number of iterations after which motif cleaning is performed. If set to a value greater than \code{iter_max}, no cleaning is performed. Motif cleaning removes redundant or poorly fitting motifs to refine the final motif set.}
  \item{\code{tol4clean}}{A numeric value representing the tolerance on the Bhattacharyya distance for motif cleaning. This parameter determines the threshold for identifying and removing redundant motifs during the cleaning process.}
  \item{\code{quantile4clean}}{A numeric value specifying the dissimilarity quantile used for motif cleaning. This quantile determines which motifs are considered sufficiently dissimilar to be retained in the final set.}
  \item{\code{return_options}}{A logical value indicating whether to return the options passed to the \code{ProbKMA} method. When set to \code{TRUE}, users receive detailed information about the algorithm's configuration, facilitating transparency and reproducibility.}
  \item{\code{Y1}}{A list of derivative curves used if the dissimilarity measure \code{"d0_d1_L2"} is selected. These derivatives enhance the algorithm's ability to capture both shape and rate changes in the functional data.}
  \item{\code{P0}}{An initial membership matrix (N x K), where N is the number of curves and K is the number of clusters. If set to \code{NULL}, a random matrix is generated, initiating the probabilistic clustering process.}
  \item{\code{S0}}{An initial shift warping matrix (N x K). If set to \code{NULL}, a random matrix is generated to initialize the alignment process, allowing motifs to adapt to variations in the data.}
  \item{\code{n_subcurves}}{An integer specifying the number of splitting subcurves used when the number of curves is equal to one. This parameter allows the algorithm to handle single-curve datasets by dividing them into manageable segments for motif discovery.}
  \item{\code{sil_threshold}}{A numeric value representing the threshold to filter candidate motifs based on their silhouette scores. This ensures that only motifs with sufficient clustering quality are retained in the final results.}
  \item{\code{set_seed}}{A logical value indicating whether to set a random seed for reproducibility. When set to \code{TRUE}, the function initializes the random number generator to ensure consistent results across multiple runs.}
  \item{\code{seed}}{An integer specifying the random seed used for initialization when \code{set_seed} is \code{TRUE}. This parameter guarantees reproducibility of the clustering and alignment processes.}
  \item{\code{exe_print}}{A logical value determining whether to print execution details for each iteration. When set to \code{TRUE}, users receive real-time feedback on the algorithm's progress, aiding in monitoring and debugging.}
  \item{\code{V_init}}{A list of motif sets provided as specific initializations for clustering rather than using random initializations. The `V_init` parameter allows users to provide a set of motifs as starting points for the algorithm, instead of relying on random initialization. If `n_init` is specified as greater than the number of motifs given in `V_init`, the remaining initializations will be randomly generated. For example, if `n_init = 10` but only 5 motif sets are given in `V_init`, the algorithm will use these 5 initializations and generate an additional 5 randomly.}
  \item{\code{transformed}}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
  \item{\code{n_init_motif}}{The number of initial motif sets from `V_init` to be used directly as starting points in clustering. If `n_init_motif` is set to a value larger than the number of motifs provided in `V_init`, additional initializations will be generated randomly to meet the specified number. For example, if `n_init = 10` and `n_init_motif = 5` with only 3 motif sets in `V_init`, the algorithm will use these 3 sets and generate 7 additional random initializations.}
}
}

\section{funBIalign Options}{

The following parameters are specific to the \code{funBIalign} algorithm:
\describe{
  \item{\code{portion_len}}{An integer specifying the length of curve portions to align. This parameter controls the granularity of alignment, allowing the algorithm to focus on specific segments of the curves for motif discovery.}
  \item{\code{min_card}}{An integer representing the minimum cardinality of motifs, i.e., the minimum number of motif occurrences required for a motif to be considered valid. This ensures that only motifs with sufficient representation across the dataset are retained.}
  \item{\code{cut_off}}{A double that specifies the number of top-ranked motifs to keep based on the ranking criteria, facilitating focused visualization of the most significant motifs.
                        In particular, all motifs that rank below the cut_off are retained.}
}
}

\examples{
\donttest{
# Example 1: Discover motifs using ProbKMA

# Define dissimilarity measure and weight parameter
diss <- 'd0_d1_L2'
alpha <- 0.5

# Define number of motifs and their minimum lengths
K <- c(2, 3)
c <- c(61, 51)
n_init <- 3

# Load simulated data
data("simulated200")

# Perform motif discovery using ProbKMA
results <- funMoDisco::discoverMotifs(
  Y0 = simulated200$Y0,
  method = "ProbKMA",
  stopCriterion = "max",
  name = tempdir(),
  plot = FALSE,
  probKMA_options = list(
    Y1 = simulated200$Y1,
    K = K,
    c = c,
    n_init = n_init,
    diss = diss,
    alpha = alpha
  ),
  worker_number = 1
)

# Example 2: Discover motifs using funBIalign
results_funbialign <- funMoDisco::discoverMotifs(
  Y0 = simulated200$Y0,
  method = "funBIalign",
  stopCriterion = "fMSR",
  name = tempdir(),
  plot = FALSE,
  funBIalign_options = list(
    portion_len = 60,
    min_card = 3,
    cut_off = 1.0
  ),
  worker_number = 1
  ) 

}

}
\seealso{
\strong{ProbKMA}: 
\href{https://arxiv.org/pdf/1808.04773}{Probabilistic K-means with Local Alignment} \cr
\strong{funBIalign}: 
\href{https://arxiv.org/pdf/2306.04254}{Hierarchical Clustering with Mean Squared Residue Scores}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motifs_search.R
\name{motifs_search}
\alias{motifs_search}
\title{Motif Search in Curves}
\usage{
motifs_search(
  cluster_candidate_motifs_results,
  R_all = cluster_candidate_motifs_results$R_all,
  R_m = NULL,
  different_R_m_finding = FALSE,
  R_m_finding = NULL,
  use_real_occurrences = FALSE,
  length_diff = Inf,
  worker_number = NULL
)
}
\arguments{
\item{cluster_candidate_motifs_results}{A list containing the output from the `cluster_candidate_motifs` 
function. This list must include elements such as:
\describe{
  \item{Y0}{A list of matrices representing the original curves.}
  \item{Y1}{A list of matrices representing the derivatives of the curves (if applicable).}
  \item{V0_clean}{A list of candidate motifs derived from `Y0`.}
  \item{V1_clean}{A list of candidate motifs derived from `Y1` (if applicable).}
  \item{D_clean}{A matrix of dissimilarity measures between motifs and curves.}
  \item{P_clean}{A matrix indicating positive matches (e.g., presence of motifs in curves).}
  \item{hclust_res}{A hierarchical clustering object obtained from `hclust`.}
  \item{R_all}{A numeric value representing the global radius used for dendrogram cutting.}
  \item{w}{A numeric vector of weights for the dissimilarity index across different dimensions.}
  \item{transformed}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
  \item{max_gap}{A numeric value defining the maximum allowed gap in distances for cluster separation.}
  \item{k_knn}{An integer specifying the number of neighbors for K-Nearest Neighbors classification.}
  \item{votes_knn_Rm}{A numeric value defining the probability threshold for KNN-based radius determination.}
  \item{c}{A numeric vector specifying the minimum number of overlapping elements required for motif validation.}
}}

\item{R_all}{A numeric value representing the global radius used to cut the dendrogram, ensuring that 
clusters are at least twice this radius apart. This parameter defines the grouping of motifs into clusters.}

\item{R_m}{A numeric vector containing group-specific radii used to identify motif occurrences within each cluster. 
The length of this vector must match the number of clusters obtained by cutting the dendrogram at a height of `2 * R_all`. 
If `NULL`, the function automatically determines `R_m` for each group based on the distances between motifs 
within the same cluster and all curves.}

\item{different_R_m_finding}{A logical value indicating whether to use a different radius (`R_m_finding`) 
for finding motif occurrences compared to the initial radius (`R_m`). If `TRUE`, `R_m_finding` is used; 
otherwise, `R_m` is employed. This allows for separate tuning of motif occurrence detection.}

\item{R_m_finding}{A numeric vector containing group-specific radii used specifically for finding motif occurrences 
when `different_R_m_finding` is set to `TRUE`. The length of this vector must match the number of clusters obtained 
by cutting the dendrogram at a height of `2 * R_all`. If `NULL`, `R_m_finding` is determined automatically for 
each group based on distances between motifs within the same cluster and all curves.}

\item{use_real_occurrences}{A logical value indicating whether to compute real occurrences of candidate motifs 
within the curves. If `TRUE`, the function calculates actual frequencies and mean dissimilarities for motif 
selection, providing more accurate results at the cost of increased computation time. If `FALSE`, it uses 
approximate frequencies and mean dissimilarities for faster execution. Defaults to `FALSE`.}

\item{length_diff}{A numeric value specifying the minimum percentage difference in length required among 
motifs within the same group to retain multiple motifs. This parameter ensures diversity in motif selection 
by preventing motifs of similar lengths from being selected simultaneously. It is defined as a percentage 
relative to the length of the most frequent motif. Defaults to `Inf`, meaning no additional motifs are selected 
based on length differences.}

\item{worker_number}{An integer indicating the number of CPU cores to utilize for parallel processing. 
By default, the function uses one less than the total number of available cores (`detectCores() - 1`). 
Setting `worker_number = 1` forces the function to run sequentially without parallelization. If `NULL`, 
the function automatically determines the optimal number of workers based on the system's available cores.}
}
\value{
A list containing:
  \describe{
    \item{V0}{A list of selected motifs derived from `Y0`.}
    \item{V1}{A list of selected motifs derived from `Y1` (if applicable).}
    \item{V_length}{A numeric vector representing the real lengths of the selected motifs.}
    \item{V_occurrences}{A list detailing the occurrences of each selected motif within the curves.}
    \item{V_frequencies}{A numeric vector indicating the real frequencies of each selected motif.}
    \item{V_mean_diss}{A numeric vector representing the average dissimilarity of each selected motif.}
    \item{Y0}{A list of matrices corresponding to the original curves, as provided in `cluster_candidate_motifs_results`.}
    \item{Y1}{A list of matrices corresponding to the derivatives of the curves (if applicable), as provided in `cluster_candidate_motifs_results`.}
    \item{R_motifs}{A numeric vector containing the radii associated with each selected motif.}
  }
}
\description{
The `motifs_search` function identifies and ranks motifs within a set of curves based on their 
frequencies and dissimilarity measures. It processes candidate motifs clustered from hierarchical 
clustering results, selects optimal motifs within each cluster, and determines their occurrences 
in the original curves. The function supports parallel processing to enhance computational efficiency 
and offers flexibility in handling different dissimilarity metrics and motif selection criteria.
}
\details{
The `motifs_search` function operates through the following steps:
\enumerate{
  \item **Parallelization Setup**: Determines the number of worker cores to use based on `worker_number`. If `worker_number > 1`, 
    it initializes a cluster for parallel processing.
  \item **Input Preparation**: Depending on the dissimilarity metric (`d0_L2`, `d1_L2`, or `d0_d1_L2`), it prepares the data structures 
    `Y` and `V` for processing.
  \item **Dendrogram Cutting**: Cuts the hierarchical clustering dendrogram at a height of `2 * R_all` to define clusters of motifs.
  \item **Radius Determination**: If `R_m` or `R_m_finding` is not provided, the function calculates these radii for each cluster 
    based on motif distances and K-Nearest Neighbors (KNN) classification.
  \item **Candidate Motif Selection**: Depending on `use_real_occurrences`, the function either computes real occurrences and uses 
    actual frequencies and mean dissimilarities to select motifs, or it uses approximate measures for faster processing.
  \item **Motif Filtering**: Within each cluster, motifs are ranked based on their frequency and mean dissimilarity. Additional motifs 
    can be selected if their lengths differ sufficiently from the most frequent motif, as defined by `length_diff`.
  \item **Output Compilation**: The selected motifs and their associated properties are compiled into a comprehensive list for further analysis or visualization.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_constructor.R
\name{motifSimulationBuilder}
\alias{motifSimulationBuilder}
\title{Create motifSimulation Object}
\usage{
motifSimulationBuilder(
  N,
  len,
  mot_details,
  norder = 3,
  coeff_min = -15,
  coeff_max = 15,
  dist_knots = 10,
  min_dist_motifs = NULL,
  distribution = "beta",
  motif_seed = NULL
)
}
\arguments{
\item{N}{An integer specifying the number of background curves to be generated (mandatory).}

\item{len}{An integer specifying the length of the background curves (mandatory).}

\item{mot_details}{A list outlining the definitions of the motifs to be included. Each motif is characterized by its length, a set of coefficients that may be optionally specified, and the number of occurrences. These occurrences can be indicated either by specific positions within the curves or by a total count. In the latter case, the algorithm will randomly position the motifs throughout the curves (mandatory).}

\item{norder}{An integer specifying the order of the B-splines (default = 3).}

\item{coeff_min}{Additive coefficients to be incorporated into the generation of coefficients for the background curves (default = -15).}

\item{coeff_max}{Additive coefficients to be incorporated into the generation of coefficients for the background curves (default = 15).}

\item{dist_knots}{An integer specifying the distance between two consecutive knots (default = 10).}

\item{min_dist_motifs}{An integer specifying the minimum distance between two consecutive motifs embedded in the same curve (default = 'norder' * 'dist_knots').}

\item{distribution}{A character string specifying the distribution from which the coefficients of the background curves are generated. You can choose between a uniform distribution or a beta distribution. Alternatively, you can pass a vector representing the empirical distribution from which you wish to sample (default = "beta").}

\item{motif_seed}{Set the seed for motif b-spline coefficients generation (default = NULL, i.e., random)}
}
\value{
An object of class motifSimulation
}
\description{
It represents the constructor of the S4 class 'motifSimulation'..
}
\examples{
\donttest{
mot_len <- 100
motif_str <- rbind.data.frame(c(1, 1, 20),
                              c(2, 1, 2), 
                              c(1, 3, 1),
                              c(1, 2, 1),
                              c(1, 2, 15),
                              c(1, 4, 1),
                              c(2, 5, 1),
                              c(2, 7, 1),
                              c(2,17,1))

names(motif_str) <- c("motif_id", "curve","start_break_pos")

mot1 <- list("len" = mot_len, #length
             "coeffs" = NULL, # weights for the motif
             "occurrences" = motif_str \%>\% filter(motif_id == 1))

mot2 <- list("len" = mot_len,
             "coeffs" = NULL,
             "occurrences" = motif_str \%>\% filter(motif_id == 2))

mot_details <- list(mot1,mot2)

# MATRIX ERROR 
noise_str <- list(rbind(rep(2, 100)),
                  rbind(rep(0.0, 100)))

builder <- funMoDisco::motifSimulationBuilder(N = 20,len = 300, mot_details,
                                        distribution = 'beta')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_methods.R
\name{to_motifDiscovery,list-method}
\alias{to_motifDiscovery,list-method}
\title{to_motifDiscovery}
\usage{
\S4method{to_motifDiscovery}{list}(curves)
}
\arguments{
\item{curves}{A list coming from the generateCurves function.}
}
\value{
A list containing all curves formatted to be suitable for input into the discoverMotifs function.
}
\description{
Transforms the result of generateCurves into a format suitable for discoverMotifs
}
\examples{
\donttest{
builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details = NULL)
curves <- funMoDisco::generateCurves(builder)
formatted_curves <- to_motifDiscovery(curves)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{.resample}
\alias{.resample}
\title{Resample Vector}
\usage{
.resample(x, ...)
}
\arguments{
\item{x}{A vector to be resampled.}

\item{...}{Additional arguments passed to the sampling function.}
}
\value{
A resampled vector.
}
\description{
This function resamples a vector by randomly selecting indices. The number of samples can be specified.
}
\details{
Resample Vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padding.R
\name{padding}
\alias{padding}
\title{Pad a Matrix to a Specified Number of Rows}
\usage{
padding(dataMatrix, maxLen)
}
\arguments{
\item{dataMatrix}{A matrix to be padded. Each row represents a data point.}

\item{maxLen}{An integer specifying the desired number of rows in the matrix.}
}
\value{
A matrix padded with `NA` values up to the specified number of rows. 
If `dataMatrix` already has the desired number of rows or more, the original matrix is returned.
}
\description{
This function pads a matrix with `NA` values to ensure that the matrix reaches a specified number of rows (`maxLen`). 
If the matrix has fewer rows than the desired length, additional rows of `NA` are appended to the matrix. 
Otherwise, the matrix is returned unchanged.
}
\details{
The function compares the number of rows in `dataMatrix` with `maxLen`. If the matrix has fewer rows, 
it pads the matrix with `NA` values until it reaches the specified number of rows. 
No changes are made if `dataMatrix` has rows equal to or greater than `maxLen`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_methods.R
\name{plot_motifs,motifSimulation-method}
\alias{plot_motifs,motifSimulation-method}
\title{Plot Embedded Motifs in Functional Curves}
\usage{
\S4method{plot_motifs}{motifSimulation}(object, curves, name, path)
}
\arguments{
\item{object}{An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object contains all necessary parameters and configurations used during curve and motif generation (mandatory).}

\item{curves}{The output list from the `generateCurves` function, containing the generated functional curves with embedded motifs. This parameter provides the data to be visualized (mandatory).}

\item{name}{Name of the output file.}

\item{path}{A character string specifying the directory path where the generated plots will be saved. The function will save the plots in this directory, allowing for organized storage and easy access to visual results (mandatory).}
}
\value{
The function does not return any value but generates and saves plots of the functional curves with embedded motifs in the specified directory. Each plot visually represents the motifs within the curves, aiding in the qualitative assessment of motif embedding.
}
\description{
The `plot_motifs` function visualizes the results generated by the `generateCurves` function. It provides comprehensive plots of functional curves with embedded motifs, allowing users to inspect the placement and characteristics of each motif within the curves. This visualization is crucial for validating the correctness of motif embedding and for gaining insights into the distribution and variability of motifs across different curves.

The function supports saving the generated plots to a specified directory, facilitating the creation of reports or the sharing of visual results. By plotting motifs in distinct colors or styles, `plot_motifs` ensures that overlapping motifs and their respective curves remain distinguishable, enhancing the clarity and interpretability of the visualizations.

This plotting utility is an essential tool within the `funMoDisco` package, aiding users in the exploratory analysis of simulated functional data and the assessment of motif detection algorithms.
}
\examples{
\donttest{
# Example: Plotting motifs in generated curves
# Assume 'builder' has been created and 'curves' have been generated using generateCurves
mot_details <- NULL
builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
curves <- funMoDisco::generateCurves(builder)

# Specify the directory to save plots
plots_name <- "plots_1"  

# Generate and save the plots
funMoDisco::plot_motifs(builder, curves, plots_name, path = tempdir())
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulated200.R
\docType{data}
\name{simulated200}
\alias{simulated200}
\title{Simulated data for local clustering}
\format{
An object of class \code{list} of length 2.
}
\usage{
data(simulated200)
}
\description{
18 curves of length 200 (corresponding to 201 evaluation points), each
containing exactly one motif of length 60 (corresponding to 61 evaluation
points), with noise level sigma=0.1
Curves 1-6, 14-16 contain one occurrence of motif 1
Curves 7-13, 17-18 contain one occurrence of motif 2
The data have been generated using a B-spline basis of order 3 and knots at
distance 10, following the simulation procedure presented in Cremona and
Chiaromonte (Comparison with non-sparse and sparse functional clustering
simulation section).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{initialChecks}
\alias{initialChecks}
\title{Initial Checks for ProbKMA}
\usage{
initialChecks(Y0, Y1, P0, S0, params, diss, V_init)
}
\arguments{
\item{Y0}{A list of matrices or vectors representing the curves.}

\item{Y1}{A list of matrices or vectors representing the derivatives of the curves.}

\item{P0}{A numeric matrix representing initial membership probabilities. Rows correspond to curves, and columns correspond to clusters.}

\item{S0}{A numeric matrix representing the initial shift matrix.}

\item{params}{A list containing various parameters for ProbKMA, including:
\itemize{
   \item \code{standardize} Logical indicating whether to standardize the curves.
   \item \code{K} Number of motifs.
   \item \code{c} Minimum motif length.
   \item \code{c_max} Maximum motif length.
   \item \code{iter_max} Maximum number of iterations.
   \item \code{quantile} Quantile value for stopping criterion.
   \item \code{alpha} A numeric value related to the dissimilarity measure.
   \item \code{w} Weights used in the algorithm.
   \item \code{stopCriterion} Stopping criterion for the algorithm.
   \item \code{m} Weighting exponent.
   \item \code{tol} Tolerance level for stopping criteria.
   \item \code{iter4elong} Maximum iterations for elongation.
   \item \code{tol4elong} Tolerance for elongation.
   \item \code{max_elong} Maximum elongation allowed.
   \item \code{trials_elong} Number of trials for elongation.
   \item \code{deltaJK_elong} Threshold for elongation.
   \item \code{max_gap} Maximum gap allowed.
   \item \code{iter4clean} Number of iterations for cleaning.
   \item \code{tol4clean} Tolerance for cleaning.
   \item \code{quantile4clean} Quantile for cleaning.
   \item \code{return_options} Options for returning results.
   \item \code{seed} Seed for random number generation.
   \item \code{exe_print} Boolean to control printing of execution messages.
   \item \code{set_seed} Boolean to control whether to set a random seed.
   \item{\code{transformed}} A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.
   \item \code{n_threads} Number of threads for parallel processing.
 }}

\item{diss}{A character string indicating the type of dissimilarity measure to be used. 
Possible values are: \code{'d0_L2'}, \code{'d1_L2'}, \code{'d0_d1_L2'}.}

\item{V_init}{A list containing initial values for the clusters. If provided, it must match the expected structure based on \code{K}.}
}
\value{
A list containing:
\item{FuncData}{A list of processed curves and derivatives after performing the checks.}
\item{Parameters}{A list of validated parameters ready for use in initializing the ProbKMA object.}
}
\description{
This function performs various input checks on the parameters provided by the user to ensure they are valid 
for running the ProbKMA algorithm. It verifies the structure and content of the input data, including 
curves, derivatives, initial membership probabilities, shift matrices, and various parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probKMA_plot.R
\name{probKMA_plot}
\alias{probKMA_plot}
\title{Plot the Results of probKMA}
\usage{
probKMA_plot(
  probKMA_results,
  plot,
  ylab = "",
  sil_avg = NULL,
  cleaned = FALSE,
  transformed = FALSE
)
}
\arguments{
\item{probKMA_results}{A list containing the output from the `probKMA` function. This list should include elements such as:
\describe{
  \item{Y0}{A list of matrices representing the original curves.}
  \item{Y1}{A list of matrices representing the derivatives of the curves (if applicable).}
  \item{V0}{A list of motifs.}
  \item{V1}{A list of derived motifs (if applicable).}
  \item{P}{A matrix indicating motif memberships across curves.}
  \item{S}{A matrix indicating motif start positions in curves.}
  \item{S_clean}{A matrix indicating cleaned motif start positions (if applicable).}
  \item{P_clean}{A matrix indicating cleaned motif memberships (if applicable).}
  \item{V0_clean}{A list of cleaned motifs (if applicable).}
  \item{V1_clean}{A list of derived cleaned motifs (if applicable).}
  \item{iter}{An integer indicating the number of iterations performed.}
  \item{J_iter}{A numeric vector recording the objective function value at each iteration.}
  \item{BC_dist_iter}{A numeric vector recording the Bhattacharyya distance between memberships at each iteration.}
}}

\item{plot}{A logical flag indicating whether to produce the plots. If `TRUE`, the function generates all relevant plots. If `FALSE`, no plots are produced.}

\item{ylab}{A character vector of length `d`, providing labels for the y-axis in each dimension. Defaults to an empty string (`''`) for all dimensions.}

\item{sil_avg}{A numeric vector containing the average silhouette scores for each embedded motif. This parameter is used to annotate the plots with silhouette information. Defaults to `NULL`, meaning no silhouette scores are displayed.}

\item{cleaned}{A logical value indicating whether to plot only the cleaned motifs (`TRUE`) or all motifs (`FALSE`). When set to `TRUE`, the function highlights motifs that have been cleaned based on predefined criteria. Defaults to `FALSE`.}

\item{transformed}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
}
\value{
The function generates a series of plots visualizing:
\itemize{
  \item \strong{Motifs with Matched Curves}: Displays the original curves with embedded motifs overlaid. If `cleaned = TRUE`, only cleaned motifs are highlighted.
  \item \strong{Memberships}: Shows bar plots representing the membership scores of each motif across all curves.
  \item \strong{Objective Function}: Plots the progression of the objective function (`J_iter`) over iterations to illustrate convergence.
  \item \strong{Bhattacharyya Distance}: Plots the Bhattacharyya distance (`BC_dist_iter`) between memberships over iterations to assess similarity.
}
No value is returned; the function is used solely for its side effects of generating visualizations.
}
\description{
The `probKMA_plot` function visualizes the results obtained from the `probKMA` analysis. It generates a series of plots
including motif memberships across different curves, the progression of the objective function over iterations,
and the Bhattacharyya distance between memberships. Depending on the parameters, it can plot both original
and cleaned motifs across multiple dimensions, providing insights into the embedding and characteristics of
identified motifs.
}
\details{
The `probKMA_plot` function performs the following operations:
\enumerate{
  \item **Motif Visualization**:
    \itemize{
      \item Plots the original curves (`Y0`) with embedded motifs (`V0`). If derivatives (`Y1` and `V1`) are available, additional plots are generated for them.
      \item When `cleaned = TRUE`, the function highlights only the cleaned motifs (`V0_clean` and `V1_clean`), providing a clearer view of significant motifs.
      \item Utilizes color coding and legends to differentiate between different motifs and their instances.
    }
  \item **Memberships**:
    \itemize{
      \item Generates bar plots showing the membership scores (`P` or `P_clean`) of each motif across all curves.
      \item Provides a visual representation of how strongly each motif is associated with different curves.
    }
  \item **Objective Function and Bhattacharyya Distance**:
    \itemize{
      \item Plots the objective function (`J_iter`) over the iterations to demonstrate the optimization process.
      \item Plots the Bhattacharyya distance (`BC_dist_iter`) to measure the similarity between motif memberships across iterations.
    }
}

The function is designed to handle multiple dimensions (`d`) and can accommodate both original and derivative data if provided. 
It also supports the visualization of cleaned motifs, which are motifs that have been refined based on specific criteria to ensure quality and relevance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_candidate_motifs_plot.R
\name{cluster_candidate_motifs_plot}
\alias{cluster_candidate_motifs_plot}
\title{cluster_candidate_motifs_plot}
\usage{
cluster_candidate_motifs_plot(
  cluster_candidate_motifs_results,
  ylab = "",
  R_all = cluster_candidate_motifs_results$R_all,
  R_m = NULL,
  ask = TRUE
)
}
\arguments{
\item{cluster_candidate_motifs_results}{A list containing the output of the
`cluster_candidate_motifs` function, which includes clustering results and
distances between motifs and curves.}

\item{ylab}{A character string specifying the label for the y-axis in the
plots. Default is an empty string.}

\item{R_all}{A numeric value representing the global radius. This is used
to cut the dendrogram, requiring groups to be more than twice `R_all`
apart. The default is set to the `R_all` component of the input results.}

\item{R_m}{A numeric vector with group-specific radii. The length of this
vector must match the number of clusters obtained by cutting the dendrogram
at height `2 * R_all`. If set to NULL, the radius `R_m` is determined
within each group based on the distances between motifs of the same group
and all curves.}

\item{ask}{A logical value indicating whether the user should be prompted
before a new plot is drawn. Default is TRUE.}
}
\description{
This function generates plots to visualize the results of the
`cluster_candidate_motifs` function. It provides insights into the
distances between motifs and curves, group-specific radii, and displays
aligned motifs for each cluster.
}
\details{
This function produces:
- Histograms of distances between all motifs and all curves, highlighting
the global radius.
- A dendrogram of motifs cut at `2 * R_all`, which allows visualization of
the clustering structure.
- Histograms of distances for motifs within specific clusters, comparing
motifs from curves with and without the motifs.
- Plots of aligned motifs for each cluster, showing their positions and
distributions.
- A scatter plot displaying the approximate average distance against the
approximate frequency for motifs in each cluster.
}
\author{
Marzia Angela Cremona & Francesca Chiaromonte
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_methods.R
\name{generateCurves}
\alias{generateCurves}
\title{Generate Functional Curves with Embedded Motifs}
\usage{
generateCurves(
  object,
  noise_type = NULL,
  noise_str = NULL,
  seed_background = NULL,
  only_der = FALSE,
  coeff_min_shift = -10,
  coeff_max_shift = 10
)
}
\arguments{
\item{object}{An S4 object of class `motifSimulation` that has been previously constructed using the `motifSimulationBuilder` function. This object encapsulates all necessary parameters and configurations for curve and motif generation (mandatory).}

\item{noise_type}{A character string specifying the type of noise to add to the curves. Acceptable values are `'pointwise'` for adding noise directly to data points or `'coeff'` for perturbing the coefficients of the basis functions (mandatory).}

\item{noise_str}{A list detailing the structure and magnitude of the noise to be added for each motif. 
\itemize{
  \item If `noise_type` is `'pointwise'`, `noise_str` should contain vectors or matrices indicating the noise level for each motif.
  \item If `noise_type` is `'coeff'`, `noise_str` should include individual values or vectors representing the noise to be applied to the coefficients.
}
This parameter allows fine-grained control over the noise characteristics applied to each motif (mandatory).}

\item{seed_background}{An integer value setting the seed for the random number generator used in background curve generation. This ensures reproducibility of the background curves. Default is `777`.}

\item{only_der}{A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `FALSE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `TRUE`.}

\item{coeff_min_shift}{A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.}

\item{coeff_max_shift}{A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.}
}
\value{
A list containing the following components:
\itemize{
  \item \strong{basis}: The basis functions used to represent the curves.
  \item \strong{background}: A list containing the coefficients and the background curves without any motifs.
  \item \strong{no_noise}: A list containing the coefficients and the background curves with embedded motifs but without added noise.
  \item \strong{with_noise}: A list containing the noise structure and the curves with embedded motifs and added noise.
  \item \strong{SNR}: A list of Signal-to-Noise Ratio (SNR) metrics calculated for each motif within each curve, useful for assessing the quality of motif embedding.
}
}
\description{
The `generateCurves` function is designed to create synthetic functional data by embedding predefined motifs into background curves. This is particularly useful for testing and benchmarking motif discovery and clustering algorithms in functional data analysis. By allowing the incorporation of various noise types and controlled motif placements, the function provides a flexible framework for simulating realistic scenarios where motifs may or may not be noisy.

The function supports two types of noise addition:
\itemize{
  \item \strong{Pointwise Noise}: Adds noise directly to the data points of the curves, simulating random fluctuations or measurement errors.
  \item \strong{Coefficient Noise}: Perturbs the coefficients of the basis functions used to represent the curves, allowing for smoother variations and controlled distortions.
}

Additionally, `generateCurves` allows for the specification of vertical shifts to motifs, enabling the simulation of motifs appearing at different baseline levels within the curves. The function ensures that all generated subcurves meet the minimum motif length requirement, maintaining the integrity of the embedded motifs.

This function is integral to the `funMoDisco` package's motif simulation capabilities, providing users with the ability to create complex functional datasets tailored to their specific research or testing needs.
}
\examples{
\donttest{
# Example 0: Special case with no motifs
mot_len <- 100
mot_details <- NULL  # or list()
builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
curves <- funMoDisco::generateCurves(builder)

# Example 1: Set the motif position and add pointwise noise
# Define motif positions and their respective curves
motif_str <- rbind.data.frame(
  c(1, 1, 20),
  c(2, 1, 2),
  c(2, 7, 1),
  c(2,17,1)
)
names(motif_str) <- c("motif_id", "curve", "start_break_pos")

# Define motif details
mot1 <- list(
  "len" = mot_len, 
  "coeffs" = NULL, 
  "occurrences" = motif_str \%>\% filter(motif_id == 1)
)
mot2 <- list(
  "len" = mot_len, 
  "coeffs" = NULL, 
  "occurrences" = motif_str \%>\% filter(motif_id == 2)
)
mot_details <- list(mot1, mot2)

# Define noise structure for pointwise noise
noise_str <- list(
  rbind(rep(2, 100), rep(c(rep(0.1, 50), rep(2, 50)), 1)),
  rbind(rep(0.0, 100), rep(0.5, 100))
)

# Build the simulation object
builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')

# Generate curves with pointwise noise
curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str = noise_str)

# Example 2: Set the motif position and add coefficient noise
# Define noise structure for coefficient noise
noise_str <- list(c(0.1, 1.0, 5.0), c(0.0, 0.0, 0.0))

# Generate curves with coefficient noise without vertical shifts
curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)

# Example 3: Random motif positions and add pointwise noise
mot1 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = 5
)
mot2 <- list(
  "len" = mot_len,
  "coeffs" = NULL,
  "occurrences" = 6
)
mot_details <- list(mot1, mot2)

# Define noise structure for pointwise noise
noise_str <- list(
  rbind(rep(2, 100)),
  rbind(rep(0.5, 100))
)

# Build the simulation object
builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details, distribution = 'beta')

# Generate curves with pointwise noise and vertical shifts
curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str, only_der = FALSE)

# Example 4: Random motif positions and add coefficient noise
# Define noise structure for coefficient noise
noise_str <- list(c(0.1, 5.0, 10.0), c(0.1, 5.0, 10.0))

# Generate curves with coefficient noise and vertical shifts
curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{generate_background_curve}
\alias{generate_background_curve}
\title{Generate Background Curve}
\usage{
generate_background_curve(len, dist_knots, norder, weights, add_noise)
}
\arguments{
\item{len}{An integer indicating the length of the curve to generate.}

\item{dist_knots}{A numeric value indicating the distance between knots.}

\item{norder}{An integer specifying the order of the B-spline.}

\item{weights}{A numeric vector containing the coefficients for the B-spline.}

\item{add_noise}{A logical value indicating whether to add Gaussian noise to the curve.}
}
\value{
A list containing the generated basis, coefficients, and curve values with or without noise.
}
\description{
This function generates a background curve using B-splines with specified parameters, including knots, order, and weights. 
Optionally, additive noise can be added to the curve.
}
\details{
Generate Background Curve
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_candidate_motifs.R
\name{cluster_candidate_motifs}
\alias{cluster_candidate_motifs}
\title{Cluster Candidate Motifs}
\usage{
cluster_candidate_motifs(
  filter_candidate_motifs_results,
  motif_overlap = 0.6,
  k_knn = 3,
  votes_knn_Rall = 0.5,
  votes_knn_Rm = 0.5,
  worker_number = NULL
)
}
\arguments{
\item{filter_candidate_motifs_results}{A list containing results from filtering candidate motifs, including various components like `Y0`, `Y1`, `V0_clean`, `V1_clean`, `D_clean`, and more, which are essential for the clustering process.}

\item{motif_overlap}{A numeric value representing the minimum proportion of overlap required between motifs to be considered similar (default is 0.6).}

\item{k_knn}{An integer specifying the number of nearest neighbors to consider when determining the global radius (default is 3).}

\item{votes_knn_Rall}{A numeric value indicating the threshold for KNN voting when determining the global radius (default is 0.5).}

\item{votes_knn_Rm}{A numeric value indicating the threshold for KNN voting when determining group-specific radii (default is 0.5).}

\item{worker_number}{An optional integer specifying the number of parallel workers to use. If NULL, it defaults to the number of available cores minus one.}
}
\value{
A list containing:
- `VV_D`: Matrix of distances between motifs.
- `VV_S`: Matrix of shifts between motifs.
- `k_knn`: The value of K used in KNN.
- `votes_knn_Rall`: Voting threshold for the global radius.
- `R_all`: The global radius determined from the clustering process.
- `hclust_res`: Result of hierarchical clustering (if applicable).
- `votes_knn_Rm`: Voting threshold for group-specific radius.
- `R_m`: Vector of group-specific radii for each cluster.
- All components from the input `filter_candidate_motifs_results`.
}
\description{
This function clusters candidate motifs based on their distances and computes group-specific radii for motif clusters. It utilizes K-nearest neighbors (KNN) for determining a global radius and evaluates overlaps among motifs. The function supports parallel computation for efficiency.
}
\details{
This function performs the following steps:
1. Sets up parallel jobs based on the specified `worker_number`.
2. Prepares input data based on the type of distance measure used.
3. Computes distances between motifs.
4. Determines a global radius (`R_all`) using KNN classification.
5. Clusters motifs and determines group-specific radii (`R_m`) for each cluster.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mapply_custom.R
\name{.mapply_custom}
\alias{.mapply_custom}
\title{Custom mapply Function for Parallel Processing}
\usage{
.mapply_custom(
  cl,
  FUN,
  ...,
  MoreArgs = NULL,
  SIMPLIFY = TRUE,
  USE.NAMES = TRUE
)
}
\arguments{
\item{cl}{A cluster object created by `makeCluster`. If `NULL`, the classical `mapply` function is used.}

\item{FUN}{A function to apply to the arguments.}

\item{...}{Arguments to be passed to `FUN`. These should be vectors of equal length.}

\item{MoreArgs}{A list of additional arguments to be passed to `FUN`.}

\item{SIMPLIFY}{A logical value indicating whether to simplify the result if possible. Default is `TRUE`.}

\item{USE.NAMES}{A logical value indicating whether to use names from the first argument. Default is `TRUE`.}
}
\value{
A vector or list containing the results of applying the function `FUN` to the provided arguments. 
        The output type depends on the value of `SIMPLIFY`.
}
\description{
A wrapper function that conditionally applies either the classical `mapply` function 
or `parallel::clusterMap` based on whether a cluster object is provided. 
If the number of clusters is not null, it applies `clusterMap` for parallel execution; 
otherwise, it defaults to the classical `mapply`.
}
\details{
This function is useful for switching between parallel and non-parallel execution 
based on the availability of a cluster, allowing for more flexible and efficient code.
}
\author{
Marzia Angela Cremona & Francesca Chiaromonte
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{.generate_coefficients}
\alias{.generate_coefficients}
\title{Generate Coefficients}
\usage{
.generate_coefficients(
  motif_i,
  distrib,
  dist_knots,
  norder,
  coeff_min,
  coeff_max,
  motif_seed
)
}
\arguments{
\item{motif_i}{A list representing the motif structure that includes the length (`len`).}

\item{distrib}{A string or numeric vector indicating the distribution from which to generate coefficients. 
Accepted values are "unif", "beta", or a numeric vector.}

\item{dist_knots}{A numeric value indicating the distance between knots.}

\item{norder}{An integer specifying the order of the B-spline.}

\item{coeff_min}{A numeric value indicating the minimum coefficient value for uniform or beta distributions.}

\item{coeff_max}{A numeric value indicating the maximum coefficient value for uniform or beta distributions.}

\item{motif_seed}{An integer value setting the seed for the random number generator used in motif generation. This ensures reproducibility of the motif embedding process. Default is `43213`.}
}
\value{
A modified motif structure containing the generated coefficients.
}
\description{
This function generates a vector of coefficients for a motif based on the specified distribution. 
It can sample coefficients from a numeric vector, a uniform distribution, or a beta distribution.
}
\details{
Generate Coefficients for Motif


- For uniform distribution, coefficients are generated within the specified range defined by `coeff_min` and `coeff_max`.
- For beta distribution, coefficients are scaled to the desired range using the specified minimum and maximum values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probKMA_silhouette_filter.R
\name{probKMA_silhouette_filter}
\alias{probKMA_silhouette_filter}
\title{Filter Motifs from probKMA Results Based on Silhouette and Size Thresholds}
\usage{
probKMA_silhouette_filter(
  probKMA_results,
  silhouette,
  sil_threshold = 0.5,
  size_threshold = 2
)
}
\arguments{
\item{probKMA_results}{A list representing the output of the `probKMA` function with `return_options = TRUE`. This output includes the motifs, dissimilarity matrices, and membership matrices required for filtering.}

\item{silhouette}{A list output from the `probKMA_silhouette` function, which provides silhouette scores for each motif.}

\item{sil_threshold}{A numeric value representing the threshold for the average silhouette index. Motifs with a silhouette index greater than or equal to this value will be retained. Default is 0.5.}

\item{size_threshold}{An integer representing the minimum number of curves (size) in a motif cluster. Motifs with a cluster size greater than or equal to this value will be retained. Default is 2.}
}
\value{
A list containing the filtered results:
\item{V0_clean}{Filtered motifs.}
\item{V1_clean}{Derivatives of the filtered motifs.}
\item{D}{Dissimilarity matrix.}
\item{D_clean}{Filtered dissimilarity matrix after cleaning motifs.}
\item{P}{Membership matrix.}
\item{P_clean}{Filtered membership matrix after cleaning motifs.}
\item{c}{Filtered minimum motif lengths.}
\item{K}{Vector containing the number of motifs repeated by the filtered motifs.}
}
\description{
This function filters the motifs identified by the `probKMA` algorithm based on a threshold for the average silhouette index and a minimum size criterion. Motifs that meet or exceed both the silhouette index threshold and the minimum number of curves (size) are retained. The function returns a cleaned version of the input data, including the filtered motifs, their derivatives, and associated matrices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{add_error_to_motif}
\alias{add_error_to_motif}
\title{Add Additive Error to Motif}
\usage{
add_error_to_motif(or_y, noise_str, start_point, end_point, k)
}
\arguments{
\item{or_y}{A numeric vector representing the original motif values.}

\item{noise_str}{A string or numeric vector indicating the structure of the noise to be added.}

\item{start_point}{A numeric vector indicating the starting points of motifs.}

\item{end_point}{A numeric vector indicating the ending points of motifs.}

\item{k}{An integer representing the current iteration or motif index.}
}
\value{
A numeric vector of the motif values with added noise.
}
\description{
This function adds an additive noise to a given motif based on specified parameters. The noise is generated as a percentage of the standard deviation of the motif.
}
\details{
Add Error to Motif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funMoDisco-pkg.R
\docType{package}
\name{funMoDisco-package}
\alias{funMoDisco}
\alias{funMoDisco-package}
\title{funMoDisco: Motif Discovery in Functional Data}
\description{
Efficiently implementing two complementary methodologies for discovering motifs in functional data: ProbKMA and FunBIalign. Cremona and Chiaromonte (2023) "Probabilistic K-means with Local Alignment for Clustering and Motif Discovery in Functional Data" \doi{10.1080/10618600.2022.2156522} is a probabilistic K-means algorithm that leverages local alignment and fuzzy clustering to identify recurring patterns (candidate functional motifs) across and within curves, allowing different portions of the same curve to belong to different clusters. It includes a family of distances and a normalization to discover various motif types and learns motif lengths in a data-driven manner. It can also be used for local clustering of misaligned data. Di Iorio, Cremona, and Chiaromonte (2023) "funBIalign: A Hierarchical Algorithm for Functional Motif Discovery Based on Mean Squared Residue Scores" \doi{10.48550/arXiv.2306.04254} applies hierarchical agglomerative clustering with a functional generalization of the Mean Squared Residue Score to identify motifs of a specified length in curves. This deterministic method includes a small set of user-tunable parameters. Both algorithms are suitable for single curves or sets of curves. The package also includes a flexible function to simulate functional data with embedded motifs, allowing users to generate benchmark datasets for validating and comparing motif discovery methods.
}
\author{
\strong{Maintainer}: Jacopo Di Iorio \email{jacopo.di.iorio@emory.edu}

Authors:
\itemize{
  \item Marzia Angela Cremona
  \item Francesca Chiaromonte
  \item Niccolò Feresini
  \item Riccardo Lazzarini
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_candidate_motifs.R
\name{filter_candidate_motifs}
\alias{filter_candidate_motifs}
\title{Filter Candidate Motifs}
\usage{
filter_candidate_motifs(
  find_candidate_motifs_results,
  sil_threshold = 0.5,
  size_threshold = 2,
  K = find_candidate_motifs_results$K,
  c = find_candidate_motifs_results$c
)
}
\arguments{
\item{find_candidate_motifs_results}{Output from the \code{find_candidate_motif} function, which contains the results of the motif discovery process.}

\item{sil_threshold}{A numeric value representing the threshold for the average silhouette index. 
Values should be between -1 and 1.}

\item{size_threshold}{An integer representing the threshold for the size of the motif, defined as the number of curves in the cluster. 
Should be at least 1.}

\item{K}{A vector containing the numbers of motifs that must be considered. 
This should be a subset of \code{find_candidate_motifs_results$K}.}

\item{c}{A vector with the minimum motif lengths that must be considered. 
This should be a subset of \code{find_candidate_motifs_results$c}.}
}
\value{
A list containing the filtered candidate motifs and some ProbKMA options:
\item{V0_clean}{A vector of candidate motifs after filtering.}
\item{V1_clean}{A vector of derived candidate motifs after filtering.}
\item{D_clean}{A matrix of distances of candidate motifs from the curves after filtering.}
\item{P_clean}{A matrix of probabilities of membership of the candidate motifs after filtering.}
\item{Y0}{The original input data for the first curve.}
\item{Y1}{The original input data for the derivative of the curve.}
\item{diss}{The dissimilarity matrix used in motif discovery.}
\item{alpha}{The weight parameter for the Sobolev distance.}
\item{w}{Weights for the dissimilarity index across different dimensions.}
\item{max_gap}{Maximum allowable gap between curves.}
}
\description{
Filters the candidate motifs based on a specified threshold for the average silhouette index and 
a threshold for the size of the curves in the motif. This function is useful for refining the set 
of candidate motifs by removing those that do not meet the defined criteria.
}
\details{
This function checks the provided thresholds for silhouette indices and motif sizes, 
ensuring they are valid before filtering the motifs. If any parameters are invalid, 
default values from the \code{find_candidate_motifs_results} are used.

The function then loads the necessary data files for each combination of motif numbers and sizes, 
filtering them using the silhouette criteria. The resulting motifs are sorted by their length 
in descending order, and the filtered results are returned in a structured list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recommend_node.R
\name{recommend_node}
\alias{recommend_node}
\title{Recommend Node from a Numeric Vector}
\usage{
recommend_node(node)
}
\arguments{
\item{node}{A numeric vector representing scores or metrics. Must contain at least two elements.}
}
\value{
An integer index indicating the recommended node based on the analysis of the input vector.
}
\description{
This function analyzes a numeric vector and recommends an index based on the characteristics of the vector. 
If the vector is strictly increasing, it suggests stopping before the maximum growth rate. 
If the vector is not strictly increasing, it recommends the index of the minimum value.
}
\details{
Recommend Node Function


- If the input vector has fewer than two elements, the function defaults to returning an index of 1.
- The function first checks if the input is a numeric vector. If not, an error is thrown.
- The function calculates the differences between successive elements and identifies whether the vector is strictly increasing.
- In the case of a strictly increasing vector, it returns the index of the maximum growth rate.
- For non-increasing vectors, it returns the index of the minimum value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_occurrences.R
\name{.find_occurrences}
\alias{.find_occurrences}
\title{Find Occurrences of a Motif}
\usage{
.find_occurrences(v, Y, R, alpha, w, c_k, use0, use1, transformed = FALSE)
}
\arguments{
\item{v}{A list containing two matrices:
- \code{v0}: The first set of motif values.
- \code{v1}: The second set of motif values.
Each matrix should have \( d \) columns corresponding to the dimensions.}

\item{Y}{A list of \( N \) lists, each containing two matrices:
- \code{Y0}: The first set of curve values.
- \code{Y1}: The second set of curve values.
Each matrix should have \( d \) columns corresponding to the dimensions.}

\item{R}{A numeric value representing the maximum allowed dissimilarity.}

\item{alpha}{A numeric value that serves as a weight coefficient between the L2 norms 
of the two sets of motifs when using the dissimilarity function \code{diss_d0_d1_L2}.}

\item{w}{A numeric vector of weights for the dissimilarity index across different dimensions.
All weights must be positive (\( w > 0 \)).}

\item{c_k}{An integer specifying the minimum length of the intersection of the supports 
of the shifted motif and the curves.}

\item{use0}{A logical value indicating whether to use the first component of the curves (i.e., \( Y0 \) and \( v0 \)).}

\item{use1}{A logical value indicating whether to use the second component of the curves (i.e., \( Y1 \) and \( v1 \)).}

\item{transformed}{A logical value indicating whether to normalize the curve segments to the interval [0,1] before applying the dissimilarity measure. Setting `transformed = TRUE` scales each curve segment between 0 and 1, which allows for the identification of motifs with consistent shapes but different amplitudes. This normalization is useful for cases where motif occurrences may vary in amplitude but have similar shapes, enabling better pattern recognition across diverse data scales.}
}
\value{
A matrix with three columns:
        - \code{curve}: The ID of the curve where the motif was found.
        - \code{shift}: The optimal shift at which the motif occurs.
        - \code{diss}: The dissimilarity value associated with the match.
        If no occurrences are found, an empty matrix is returned.
}
\description{
Finds occurrences of a specified motif in a set of curves, where the dissimilarity is lower than a specified threshold \( R \). 
The function compares the motif against curves represented in multiple dimensions and returns the details of matching occurrences.
}
\details{
The function systematically checks each curve in the provided list against the specified motif 
to identify positions where the dissimilarity does not exceed the defined threshold \( R \). 
It handles multidimensional curves and can selectively consider different components of the motifs and curves.
}
\author{
Marzia Angela Cremona & Francesca Chiaromonte
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_utilities.R
\name{.transform_list}
\alias{.transform_list}
\title{Transform List Structure}
\usage{
.transform_list(lst, noise_str)
}
\arguments{
\item{lst}{A list containing elements to be transformed.}

\item{noise_str}{A string indicating the noise structure to apply.}
}
\value{
The transformed list with updated structures.
}
\description{
This function modifies a list of structures by checking for the presence of a "with_noise" field. 
If absent, it creates a new sub-list containing relevant fields.
}
\details{
Transform List Structure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_minidend.R
\name{get_minidend}
\alias{get_minidend}
\title{Generate Minimum Dendrogram from Hierarchical Clustering}
\usage{
get_minidend(adj_fMSR)
}
\arguments{
\item{adj_fMSR}{A numeric matrix or a distance object representing the dissimilarity matrix 
computed from functional data. This matrix is used to perform hierarchical clustering.}
}
\value{
A dendrogram object representing the clustered data. The dendrogram is cut at the identified height,
        resulting in a tree structure that can be used for further analysis or visualization.
}
\description{
This function generates a minimum dendrogram by performing hierarchical clustering on an input distance matrix. 
The function identifies an optimal height cut based on the largest gap in cluster heights and returns the resulting 
dendrogram, allowing for further analysis of clustering structures.
}
\details{
The function performs the following steps:
1. Uses `fastcluster::hclust` to perform hierarchical clustering on the provided dissimilarity matrix using 
   the "complete" method.
2. Extracts the heights of the clusters and identifies the largest gap in heights to determine the optimal 
   cut position for the dendrogram.
3. Cuts the dendrogram at the identified height and returns the lower part of the cut.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulatedHighDimensional.R
\docType{data}
\name{simulatedHighDimensional}
\alias{simulatedHighDimensional}
\title{High Dimensional simulated data for local clustering}
\format{
An object of class \code{list} of length 2.
}
\usage{
data(simulatedHighDimensional)
}
\description{
The functional dataset consists of 43 matrices for curves and an equal number 
for their derivatives of two columns and a variable number of rows ranging 
from a minimum of 1836 to a maximum of 43868. The data presents 
a lot of missing values (large gaps).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_classes.R
\docType{class}
\name{motifSimulation-class}
\alias{motifSimulation-class}
\title{motifSimulationS4Class}
\description{
The `motifSimulation` class is an S4 class designed for simulating functional data with embedded motifs. This class is essential for modeling various aspects of the data, such as the number of curves, motif details, curve characteristics, and knot-based spline definitions. It allows users to generate realistic synthetic functional data to test and benchmark motif discovery algorithms like ProbKMA and funBIalign.
}
\section{Slots}{

\describe{
\item{\code{N}}{A numeric value representing the number of curves to be simulated. 
This parameter controls the number of functional curves generated in the simulation.}

\item{\code{mot_details}}{A list containing details of the motifs to be embedded within the curves. 
Each motif detail specifies attributes such as motif shape, location, and frequency of occurrence across the simulated curves.}

\item{\code{motifs_in_curves}}{A list specifying which curves contain motifs and the positions where they are embedded. 
This slot allows precise control over the placement of motifs in the generated curves, enabling flexible motif-to-curve assignments.}

\item{\code{distribution}}{A character string or numeric value representing the distribution of the weights for the motifs. 
This slot defines the distribution used to generate the weight coefficients for the motifs, influencing their amplitude in the simulated data. 
Accepted values include any distribution supported in R or a custom vector provided by the user.}

\item{\code{dist_knots}}{A numeric value indicating the distance between knots in the spline representation of the curves. 
This parameter is crucial for defining the smoothness of the generated curves and the precision of the spline-based motif embedding process.}

\item{\code{len}}{A numeric value representing the length of the generated curves. 
This parameter determines the number of time points or the granularity of the data, allowing for high-resolution simulations.}

\item{\code{norder}}{A numeric value specifying the order of the B-spline used in the functional data representation. 
Higher-order splines provide smoother representations of the curves, while lower-order splines offer more flexible curve shapes.}

\item{\code{coeff_min}}{A numeric value specifying the minimum coefficient value for the spline-based curve generation. 
This controls the lower bound of the weight coefficients applied to the basis functions, influencing the range of curve amplitudes.}

\item{\code{coeff_max}}{A numeric value specifying the maximum coefficient value for the spline-based curve generation. 
This controls the upper bound of the weight coefficients applied to the basis functions, setting the maximum amplitude for the curves.}

\item{\code{min_dist_motifs}}{A numeric value indicating the minimum distance between motifs in the simulated curves. 
This ensures that motifs are not placed too closely, preserving their distinctiveness and reducing overlap during the simulation.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_parents.R
\name{get_parents}
\alias{get_parents}
\title{Get Parent Nodes from a Given Node}
\usage{
get_parents(node, node_list)
}
\arguments{
\item{node}{A vector representing a single node whose parents are to be found. 
It is expected to contain the elements that may be present in the parent nodes.}

\item{node_list}{A list of vectors, where each vector represents a node in the hierarchical structure. 
The function will search through these nodes to find parents of the specified node.}
}
\value{
A numeric vector containing the indices of the parent nodes in the `node_list` that include the specified node.
}
\description{
This function identifies the parent nodes of a given node in a hierarchical structure. 
It checks which nodes in the provided list contain the specified node, thereby determining 
its parent nodes.
}
\details{
The function works by applying a logical check across the `node_list`. It returns the indices of all nodes 
that contain all elements of the specified node vector. If no parent nodes are found, the function will return 
an empty integer vector.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_min_diss.R
\name{.find_min_diss}
\alias{.find_min_diss}
\title{Find Minimum Dissimilarity}
\usage{
.find_min_diss(
  y,
  v,
  alpha,
  w,
  c_k,
  d,
  use0,
  use1,
  transform_y = FALSE,
  transform_v = FALSE
)
}
\arguments{
\item{y}{A list containing two matrices:
- \code{y0}: The first set of curve values \(y(x)\).
- \code{y1}: The second set of curve values \(y'(x)\).
Each matrix should have \(d\) columns corresponding to the dimensions.}

\item{v}{A list containing two matrices:
- \code{v0}: The first set of curve values \(v(x)\).
- \code{v1}: The second set of curve values \(v'(x)\).
Each matrix should have \(d\) columns corresponding to the dimensions.}

\item{alpha}{A numeric weight coefficient that balances the contributions of 
the L2 norms of the two curve sets.}

\item{w}{A numeric vector of weights for the dissimilarity index across different dimensions.
All weights must be positive (\(w > 0\)).}

\item{c_k}{An integer specifying the minimum length of the intersection of the supports 
of the shifted \(y\) and \(v\).}

\item{d}{An integer indicating the dimensionality of the curves.}

\item{use0}{A logical value indicating whether to use the first component of the curves (i.e., \(y0\) and \(v0\)).}

\item{use1}{A logical value indicating whether to use the second component of the curves (i.e., \(y1\) and \(v1\)).}

\item{transform_y}{A logical value indicating whether to normalize \(y\) to the range \([0,1]\) 
before calculating the distance.}

\item{transform_v}{A logical value indicating whether to normalize \(v\) to the range \([0,1]\) 
before calculating the distance.}
}
\value{
A numeric vector containing:
        - The optimal shift that minimizes the dissimilarity.
        - The minimum dissimilarity value found.
}
\description{
Finds the shift warping that minimizes dissimilarity between multidimensional curves. 
The function operates on curves represented by two sets of data, each containing multiple dimensions.
}
\details{
This function computes the shift warping between the provided multidimensional curves by 
examining various shifts and calculating the corresponding dissimilarity. The user can control 
which components of the curves to include in the calculation and whether to normalize the data.

The function returns both the optimal shift and the minimal dissimilarity, which can be used 
to assess the similarity between the two sets of curves under the specified constraints.
}
\author{
Marzia Angela Cremona & Francesca Chiaromonte
}
